<?php

/*************************************************************
 * Secure Imbibe Login (Sil) system controller
 * @author    S M Kessner <limbercode@gmail.com>
 * @copyright 2010-2018 T2
 * @link      http://limbercode.com
 *************************************************************/
class silController extends controller {

  public $auth;
  public $notif;
  public $trans;
  public $text;
  private $userName = null;                    // from POST
  private $userPassword = null;                // from POST
  private $verificationLoginCode = null;       // from POST
  private $verificationRegisterCode = null;    // from POST
  private $rememberMe = null;                  // from POST or COOKIE

  /**
   * Initial function to load
   */
  protected function init() {

    global $auth;

    $this->setDoNotRenderPartOfTemplate(array("header"));
    $this->addCssIncludes("themes/".basic::$cfg['template']['theme']."/css/auth.css");
    $this->addJsIncludes("js/ajaxPerfect.js");
    $this->addJsIncludes("js/auth.js");
    $this->translations("sil");    // load the text translations file
    $this->unsetOperator();

    if($auth) {        // if auth is already loaded
      $this->auth = $auth;
    }
    else {            // if auth is not loaded
      $this->auth = basic::load("auth","core");
    }
    $this->trans = basic::load("trans","core");
    $this->notif = basic::load("notif","core");
    $this->notif->loadNotifs($file = "auth");
  }

  /**
   * Before action - the function that comes before all the functions in the controller class
   */
  public function beforeAction() {
  }    // we cant insert "showAuthAlert()" in here because we have to show the alert also on none login pages

  /**
   * Check if user is logged into the system
   * @return boolean
   */
  public function checkIfLogged() {

    if($this->auth->checkIfLogged()) {
      echo true;
    }
    else return false;
  }

  /**
   * Index - the default action - route redirected "login"
   * @return string - the view content
   */
  public function index() {

    $this->view->auth = $this->auth;

    if($this->auth->checkIfLogged()) {
      if($_SERVER['REQUEST_URI']=="/login") {
        $this->auth->redirectInside($address = "logged",$notifNum = 7030);
      }
      else {
        $this->auth->redirectInside($address = "/");
      }   // if site home page
    }

    $this->addCssIncludes("css/auth/swipe.css");
    $this->addJsIncludes("js/auth/swipe.js");
    $this->addJsIncludes("js/auth/login-checkIt.js");

    $this->view->currentAction = "login";
    $this->view->authFileName  = "/".basic::$cfg['site']['directory']['main'].lang::getCurrentLanguageForUrls()."sil/check";    // ?&chock=1

    if(router::isLocalhost()) {
      $this->view->authFileName = router::$localHostNextDir.$this->view->authFileName;
    }

    if(basic::$cfg['auth']['allowNotifs']==1) {
      $notifNum = Auth::catchAuthAlert($CatchableAlerts = array(
        7004,
        7008,
        7011
      ),$unCatchableAlerts = "");    // catch alerts
      //	if(!$notifNum) $notifNum = 7005;	// if no active alert, show this alert...
    }

    $this->view->loginVia = "";
    foreach(basic::$cfg['auth']['login']['loginVia'] as $value) {
      $this->view->loginVia .= $value."/";
    }

    $this->view->loginVia = trim($this->view->loginVia,"/");
    $this->view->captchaShow       = "";
    $this->view->rememberMeChecked = "";
    $this->captchaSensitivity      = "";

    if($this->auth->getCookieNow($name = "rememberMe")) {
      $this->view->rememberMeChecked = "checked";
    }

    $this->captchaSensitive("login");
    if(!basic::$cfg['auth']['attemptsAllowed']) {
      $this->view->triesLeft = "unlimited";
    }
    else {
      $loginAttemptsCookie = $this->auth->getCookieNow($name = "loginAttempts");
      if($loginAttemptsCookie) {
        $this->view->triesLeft = (int)(basic::$cfg['auth']['attemptsAllowed'] - $loginAttemptsCookie);
      }
      else {
        $this->view->triesLeft = (int)(basic::$cfg['auth']['attemptsAllowed']);
      }
    }

    $cookieMethodFound  = "";
    $sessionMethodFound = "";
    foreach(basic::$cfg['auth']['dataDictionary'] as $key => $value) {
      if(strchr(basic::$cfg['auth']['dataDictionary'][$key]['method'],"cookie")) {
        $cookieMethodFound = 1;
      }
      if(strchr(basic::$cfg['auth']['dataDictionary'][$key]['method'],"session")) {
        $sessionMethodFound = 1;
      }
    }

    if($cookieMethodFound==1 && $sessionMethodFound==1) {
      //	$this->view->set("methodText",$this->T("Based on session and cookies"));	// T2: sets a variable $methodText
      $this->view->methodText = $this->T("Based on session and cookies");            // T2: sets $this->methodText
    }
	elseif($cookieMethodFound==1) {
      $this->view->methodText = $this->T("Based on cookies only");
    }
	elseif($sessionMethodFound==2) {
      $this->view->methodText = $this->T("Based on session only");
    }

    if(basic::$cfg['auth']['login']['allowCaptcha']==1 && basic::$cfg['auth']['captcha']['verificationTriesLimit']<=$this->auth->getCookieNow($name = "verificationLoginTries")) {
      $delay = basic::$cfg['auth']['captcha']['verificationWaitDelay'];
      if(basic::$cfg['auth']['allowNotifs']==1) {
        $this->notif->genNotifByNum($file = "auth",7011,array($delay,$delay));
      }
    }    // too many verificationLoginTries attempts to login (captcha)	// have to be here because we change the cookie on the captcha script
    else {
      if(basic::$cfg['auth']['allowNotifs']==1) {
        $externalVar = "";
        if(isset($notifNum)) {
          if(in_array(7004,$notifNum)) {
            $externalVar = $this->trans->seconds2Human(basic::$cfg['auth']['triple']);
          }
          if(in_array(7008,$notifNum)) {
            $externalVar = $this->view->captchaSensitivity;
          }
          if($notifNum && count($notifNum)>=1) {
            $this->notif->genNotifByNum($file = "auth",$notifNum,$externalVar);    // issue alerts
          }
        }
      }
    }    // regular catch alert

    return $this->view();
  }

  /**
   * Captcha sensitive - generate the text for sensitivity (private functions are not used as page)
   * @param string $type - currently unused
   */
  private function captchaSensitive($type) {

    if($type=="login") {
      $this->view->allowCaptcha   = basic::$cfg['auth']['login']['allowCaptcha'];
      $this->captchaActiveAfter   = basic::$cfg['auth']['login']['captchaActiveAfter'];
      $this->captchaActiveAfterCS = $this->auth->getCookieNow($name = "loginAttempts");
    }
    if($type=="register") {
      $this->view->allowCaptcha   = basic::$cfg['auth']['register']['allowCaptcha'];
      $this->captchaActiveAfter   = basic::$cfg['auth']['register']['captchaActiveAfter'];
      $this->captchaActiveAfterCS = $this->auth->getCookieNow($name = "registerAttempts");
    }
    if($type=="forgotMyPassword") {
      $this->view->allowCaptcha   = basic::$cfg['auth']['forgotMyPassword']['allowCaptcha'];
      $this->captchaActiveAfter   = basic::$cfg['auth']['forgotMyPassword']['captchaActiveAfter'];
      $this->captchaActiveAfterCS = $this->auth->getCookieNow($name = "forgotMyPasswordAttempts");
    }

    if($this->view->allowCaptcha==1) {
      if($this->captchaActiveAfter<=$this->captchaActiveAfterCS) {
        $this->view->captchaShow = true;
      }
      else {
        $this->view->captchaShow = false;
      }

      if(basic::$cfg['auth']['captcha']['verificationKeySensitive']==1) {
        $this->captchaSensitivity = "(".$this->T("key sensitive").")";
      }
    }
    else {
      $this->view->captchaShow = false;
    }

    $this->view->captchaSensitivity = "";
    if($this->view->allowCaptcha==1 && $this->view->captchaShow==true) {
      if(basic::$cfg['auth']['captcha']['verificationKeySensitive']==1) {
        $this->view->captchaSensitivity = "<br>".$this->T("+ captcha is key sensitive");
      }
      else {
        $this->view->captchaSensitivity = "<br>".$this->T("+ captcha is not key sensitive");
      }
    }
  }

  /**
   * Check - validate login attempt
   * @param $userName - from POST
   * @param $userPassword - from POST
   * @param $verificationLoginCode - from POST
   * @param $rememberMe - from POST
   * @throws ReflectionException
   */
  public function check($userName,$userPassword,$verificationLoginCode,$rememberMe) {

    $this->auth->validation = basic::load("validation","functions");

    $this->auth->userName              = isset($userName['POST']) ? $this->auth->validation->validate($userName['POST'],"varchar",80,"required","userName") : false;
    $this->auth->userPassword          = isset($userPassword['POST']) ? $this->auth->validation->validate($userPassword['POST'],"varchar",80,"required","userPassword") : false;
    $this->auth->verificationLoginCode = isset($verificationLoginCode['POST']) ? $this->auth->validation->validate($verificationLoginCode['POST'],"varchar",80,"required","verificationLoginCode") : false;

    if(basic::$cfg['auth']['allowRememberMe']==1) {
      $this->auth->rememberMe = isset($rememberMe['POST']) ? $this->auth->validation->validate($rememberMe['POST'],"int",1,"optional","rememberMe") : false;
    }

    // Some programs (like Form1 Bilder) check $_POST array to see if parameters passed
    // So need to clear password protector variables
    unset($_POST);

    $this->auth->loginAttemptStart();            // check captcha code + token + userName
    $validUser = $this->checkUser();             // db checkup
    $this->auth->loginAttemptEnd($validUser);    // if user is valid set the connection
  }

  /**
   * Encrypt password - (private functions are not used as page)
   * @param string $password - to encrypt
   * @return string - encrypted password
   * @throws ReflectionException
   */
  private function encryptPassword($password) {

    $encryption        = basic::load("encryption","functions");
    $encryptedPassword = $encryption->encode($password);

    return $encryptedPassword;
  }

  /**
   * Decrypt password - (private functions are not used as page)
   * @param string $password - to decrypt
   * @return string - decrypted password
   * @throws ReflectionException
   */
  private function decryptPassword($password) {

    $encryption        = basic::load("encryption","functions");
    $decryptedPassword = $encryption->decode($password);

    return $decryptedPassword;
  }

  /**
   * Encrypt bridge code - (private functions are not used as page)
   * @return string - encrypted code for url
   * @throws ReflectionException
   */
  private function encryptBridgeCode() {

    $encryption = basic::load("encryption","functions");
    $bridgeCode = $encryption->encryptForUrl(rand().time().basic::$cfg['auth']['register']['encryptPassword']);

    return $bridgeCode;
  }

  /**
   * Check user - check if userName and userPassword are correct and user is active (private functions are not used as page)
   * @return boolean
   * @throws ReflectionException
   */
  private function checkUser() {

    $includingData = array(
      "isActive"  => "1",
      "isDeleted" => "0"
    );

    $existUserId     = "";
    $alternatelyData = [];
    $isPasswordTrue  = false;

    foreach(basic::$cfg['auth']['login']['loginVia'] as $key => $value) {
      if(in_array($value,basic::$cfg['auth']['login']['loginVia'])) {
        $alternatelyData[$value] = $this->auth->userName;
      }
    }

    $checkExist = $this->model->checkUserExistenceDetails($includingData,$alternatelyData);
    if(!empty($checkExist)) {
      $existUserId = $checkExist[0]['leadId'];
    }

    if(!empty($checkExist[0]['password'])) {
      $decryptedPasswordFromDb = $this->decryptPassword($checkExist[0]['password']);

/*      p(111111111111111);
      p($this->auth->userPassword);
      p($decryptedPasswordFromDb);
      die();*/

      if($decryptedPasswordFromDb==$this->auth->userPassword) {
        $isPasswordTrue = true;
      }
    }

    if($isPasswordTrue==true) {
      $loggedUserId        = $checkExist[0]['leadId'];
      $loggedUserName      = $checkExist[0]['userName'];
      $loggedEmail         = $checkExist[0]['email'];
      $loggedPhone         = $checkExist[0]['phone'];
      $loggedUserAuthLevel = $checkExist[0]['userAuthLevel'];

      $this->auth->setUserAsLogged($loggedUserId,$loggedUserName,$loggedEmail,$loggedPhone,$loggedUserAuthLevel);
      $this->model->updateUserLog($loggedUserId,"lastSuccessLogin");
    }
    else {
      $this->model->updateUserLog($existUserId,"lastFailedLogin");

      return false;
    }
  }

  /**
   * holding - the page that holds faulty users
   * @return string - the view content
   */
  public function holding() {

    if(basic::$cfg['auth']['allowNotifs']==1) {
      $notifNum = Auth::catchAuthAlert();
    }

    $delay = "";
    if(in_array(7012,$notifNum)) {
      $delay = $this->auth->attemptsDelay();
    }    // holding because of attempts limit

    if(in_array(7011,$notifNum)) {
      $delay = $this->auth->verificationTriesDelay();
    }    // holding because of verificationTries limit

    if($delay<=0) {
      $this->auth->redirectInside($address = "login",$notifNum = 7005);
    }    // if no delay - redirect back to login screen

    $delayText = $this->trans->seconds2Human($delay);

    if(basic::$cfg['auth']['allowNotifs']==1) {
      if($notifNum) {
        $this->notif->genNotifByNum($file = "auth",$notifNum,array(
          basic::$cfg['auth']['attemptsAllowed'],
          $delayText
        ));
      }
    }

    //	$this->view->doNotRenderPartOfTemplate = array('headers');
    return $this->view();
  }

  /**
   * Site is down - the page that displays when site is down
   * @param $error - from GET
   * @return string - the view content
   */
  public function siteIsDown($error) {

    $error = isset($error['GET']) ? $error['GET'] : false;

    if(basic::$cfg['site']['isActive']==1 && !$error) {    // error - is mysql connection error (wow)
      Header("Location: /");
      die();
    }

    return $this->view();
  }

  /**
   * Logout - the page that displays when user logged out
   * @return string - the view content
   */
  public function logout() {

  //  session_destroy();    // delete the session	// problem that this will also deletes csrfToken..

    $this->addCssIncludes("css/auth/swipe.css");
    $this->addJsIncludes("js/auth/swipe.js");
    $this->addJsIncludes("js/auth/login-checkIt.js");

    $this->auth->revokeAll();
    if(basic::$cfg['auth']['allowNotifs']==1) {
      $this->notif->genNotifByNum($file = "auth",7015);
    }

    if(basic::$cfg['auth']['logoutPath']) {
      if(basic::$cfg['auth']['logoutPath']=="/") {
        basic::$cfg['auth']['logoutPath'] = "";
      }    // we don't need leading "/" here, just redirect
      $this->auth->redirectInside($address = basic::$cfg['auth']['logoutPath'],$notifNum = 7030);
    }
    else {
      return $this->view();
    }
  }

  /**
   * Logged - the page that displays when user is logged
   * @param $redirectPost - from GET
   * @return string - the view content
   */
  public function logged($redirectPost) {

    $this->setDoNotRenderPartOfTemplate(array());   // put back the header and the footer

    if($this->auth->loggedIn==false) {
      $this->auth->restrictedSite();
      $this->auth->redirectInside($address = "");   // if user is not loggedIn but came to error 7030, redirect to home page
    }    // if we are not logged we must check all the cookies for validation

    $redirectPost = isset($redirectPost['GET']) ? $this->auth->validation->validate($redirectPost['GET'],"xss",80,"required","redirectPost") : false;

    if($redirectPost) {    // happens only when: basic::$cfg['auth']['allowRedirect'] = 0; - dont know why... - but this is the fix...
      ?>
		<script type="text/javascript">
					setTimeout("location.href='/<?=basic::$cfg["site"]["directory"]?><?=$redirectPost['GET']?>'",4000);
		</script>
      <?php

      if(basic::$cfg['auth']['allowNotifs']==1) {
        $this->notif->genNotifByNum($file = "auth",$notifNum = 7016,array(basic::$cfg['site']['directory']['main']));        // issue alerts
      }

      $this->setDoNotRenderPartOfTemplate(array("header","footer","clean","mainEvent"));
    }

    return $this->view();
  }

  /**
   * Timer - prolonging time on site, the real function is on auth class
   * @return string - the view content
   */
  public function timer() {

    $this->auth->timer();
  }

  /**
   * Insufficient - the page that displays when user have insufficient auth level to view a page
   * @return string - the view content
   */
  public function insufficient() {

    $this->setDoNotRenderPartOfTemplate(array("header","footer"));

    return $this->view();
  }

  /**
   * Register - the site
   * @return string - the view content
   */
  public function register() {

    if($this->auth->checkIfLogged()) {
      $this->auth->redirectInside($address = "logged",$notifNum = 7030);
    }

    $this->addJsIncludes(array('js/auth/login-register.js','js/auth/login-checkIt.js'));

    $this->view->cfg           = basic::$cfg;
    $this->view->cfgAuth       = basic::$cfg['auth'];
    $this->view->currentAction = "register";
    $this->view->createUser    = "/".basic::$cfg['site']['directory']['main'].lang::getCurrentLanguageForUrls()."sil/createUser";    // ?&chock=1
    $this->view->language      = lang::getDefaultLanguage()==lang::$currentLanguage ? "" : lang::$currentLanguage;

    if(basic::$cfg['auth']['allowNotifs']==1) {
      $notifNum = Auth::catchAuthAlert($catchableAlerts = array(7004,7008,7024,7025),$unCatchableAlerts = "");

      $externalVar = "";
      if(is_array($notifNum)) {
        if(in_array(7024,$notifNum) && in_array("userName",basic::$cfg['auth']['register']['require'])) {
          $externalVar = basic::$cfg['auth']['register']['minUserName'];
          $this->notif->genNotifByNum($file = "auth",7024,$externalVar);        // issue alerts
        }    // issuing the specific alert because we want to use specific $externalVar

        if(in_array(7025,$notifNum)) {
          $externalVar = basic::$cfg['auth']['register']['minPassword'];
          $this->notif->genNotifByNum($file = "auth",7025,$externalVar);    // issue alerts
        }    // issuing the specific alert because we want to use specific $externalVar

        $this->captchaSensitive("register");
        if(in_array(7004,$notifNum)) {
          $externalVar = $this->trans->seconds2Human(basic::$cfg['auth']['triple']);
        }
        if(in_array(7008,$notifNum)) {
          $externalVar = $this->view->captchaSensitivity;
        }
        if($notifNum) {
          $this->notif->genNotifByNum($file = "auth",$notifNum,$externalVar);    // issue alerts
        }
      }
    }

    $this->view->userNameC  = "";
    $this->view->titleC     = "";
    $this->view->nameC      = "";
    $this->view->firstNameC = "";
    $this->view->lastNameC  = "";
    $this->view->emailC     = "";
    $this->view->phoneC     = "";

    //	if(isset($_COOKIE['userName'])) $this->view->set("userNameC",$_COOKIE['userName']);
    if(isset($_COOKIE['userName-R'])) {
      $this->view->userNameC = $_COOKIE['userName-R'];
    }
    if(isset($_COOKIE['title'])) {
      $this->view->titleC = $_COOKIE['title'];
    }
    if(isset($_COOKIE['name'])) {
      $this->view->nameC = $_COOKIE['name'];
    }
    if(isset($_COOKIE['firstName'])) {
      $this->view->firstNameC = $_COOKIE['firstName'];
    }
    if(isset($_COOKIE['lastName'])) {
      $this->view->lastNameC = $_COOKIE['lastName'];
    }
    if(isset($_COOKIE['email'])) {
      $this->view->emailC = $_COOKIE['email'];
    }
    if(isset($_COOKIE['phone'])) {
      $this->view->phoneC = $_COOKIE['phone'];
    }

    return $this->view();
  }

  /**
   * Create user - on the db
   * @param $userName - from POST
   * @param $userPassword - from POST
   * @param $userPasswordRepeat - from POST
   * @param $title - from POST
   * @param $name - from POST
   * @param $firstName - from POST
   * @param $lastName - from POST
   * @param $email - from POST
   * @param $phone - from POST
   * @param $verificationRegisterCode - from POST
   * @param $approveUserAgreement - from POST
   * @param $subscription - from POST
   * @throws ReflectionException
   */
  public function createUser($userName,$userPassword,$userPasswordRepeat,$title,$name,$firstName,$lastName,$email,$phone,$verificationRegisterCode,$approveUserAgreement,$subscription) {

    if($this->auth->checkIfLogged()) {
      $this->auth->redirectInside($address = "logged",$notifNum = 7030);
    }

    $this->view->cfg     = basic::$cfg;
    $this->view->cfgAuth = basic::$cfg['auth'];
    //	$this->view->createUser = "/".basic::$cfg['site']['directory']['main']."login/createUser";	// ?&chock=1

    $this->auth->validation = basic::load("validation","functions");

    $createUserData = array();

    $attemptCS = $this->auth->getCookieNow("registerAttempts");
    $this->auth->setCookieNow("registerAttempts",$data = (int)$attemptCS + 1,$till = "",$domain = "");

    if(!empty($userName['POST'])) {
      $createUserData['userName'] = $this->auth->validation->validate($userName['POST'],"varchar",80,"required","userName");
      setcookie("userName-R",$createUserData['userName'],time() + basic::$cfg['auth']['globalTimeout'],"/");        // set cookies for new users
    }
    if(!empty($userPassword['POST'])) {
      $createUserData['password'] = $this->auth->validation->validate($userPassword['POST'],"varchar",80,"required","userPassword");
    }
    if(!empty($userPasswordRepeat['POST'])) {
      $userPasswordRepeater = $this->auth->validation->validate($userPasswordRepeat['POST'],"varchar",80,"required","userPasswordRepeat");
    }
    if(!empty($title['POST'])) {
      $createUserData['title'] = $this->auth->validation->validate($title['POST'],"int",3,"optional","userPasswordRepeat");
      setcookie("title",$createUserData['title'],time() + basic::$cfg['auth']['globalTimeout'],"/");        // set cookies for new users
    }
    if(!empty($name['POST'])) {
      $originalName                = $this->auth->validation->validate($name['POST'],"varchar",160,"optional","name");
      $name                        = preg_replace('/\s+/'," ",$originalName);    // remove multiple spaces
      $name                        = explode(" ",$name);
      $createUserData['firstName'] = $name[0];
      $createUserData['lastName']  = $name[1];
      setcookie("name",$originalName,time() + basic::$cfg['auth']['globalTimeout'],"/");        // set cookies for new users
    }
    if(!empty($firstName['POST'])) {
      $createUserData['firstName'] = $this->auth->validation->validate($firstName['POST'],"varchar",80,"optional","firstName");
      setcookie("firstName",$createUserData['firstName'],time() + basic::$cfg['auth']['globalTimeout'],"/");        // set cookies for new users
    }
    if(!empty($lastName['POST'])) {
      $createUserData['lastName'] = $this->auth->validation->validate($lastName['POST'],"varchar",80,"optional","lastName");
      setcookie("lastName",$createUserData['lastName'],time() + basic::$cfg['auth']['globalTimeout'],"/");        // set cookies for new users
    }
    if(!empty($email['POST'])) {
      $createUserData['email'] = $this->auth->validation->validate($email['POST'],"varchar",80,"optional","email");
      setcookie("email",$createUserData['email'],time() + basic::$cfg['auth']['globalTimeout'],"/");        // set cookies for new users
    }
    if(!empty($phone['POST'])) {
      $createUserData['phone'] = $this->auth->validation->validate($phone['POST'],"int",20,"optional","phone");
      setcookie("phone",$createUserData['phone'],time() + basic::$cfg['auth']['globalTimeout'],"/");        // set cookies for new users
    }
    if(!empty($verificationRegisterCode['POST'])) {
      $this->auth->verificationRegisterCode = $this->auth->validation->validate($verificationRegisterCode['POST'],"varchar",80,"optional","verificationRegisterCode");
    }
    if(!empty($approveUserAgreement['POST']) && basic::$cfg['auth']['register']['mustApprove']==1) {
      $createUserData['isApprovedUserAgreement'] = $this->auth->validation->validate($approveUserAgreement['POST'],"int",1,"required","isApprovedUserAgreement");
    }

    $createUserSubscriptionData['subscription'] = array();
    if(!empty($subscription['POST'])) {
      $createUserSubscriptionData['subscription'] = $this->auth->validation->validate($subscription['POST'],"int",1,"optional","subscription");
    }

    if(!$this->auth->validation->isValidated()) {
      $this->auth->redirectInside($address = "register",$notif = 7600);
    }

    unset($_POST);

    if(empty($userPasswordRepeater)) {
      $this->auth->redirectInside($address = "",$notifNum = 7050);
    }    // no post made - so dump user

    $createUserData['userAuthLevel'] = basic::$cfg['auth']['register']['initialAuthLevel'];

    $validRegistration = $this->auth->registerAttempt($createUserData,$userPasswordRepeater);    // generate alerts in case of invalid register info

    // check Availability
    $AvailabilityUserName = "";
    $AvailabilityEmail    = "";
    $AvailabilityPhone    = "";

    if(!empty($createUserData['userName'])) {
      $AvailabilityUserName = $this->model->checkUserExistence(array("userName" => $createUserData['userName']));
    }

    if(!empty($createUserData['email'])) {
      $AvailabilityEmail = $this->model->checkUserExistence(array("email" => $createUserData['email']));
    }

    if(!empty($createUserData['phone'])) {
      $AvailabilityPhone = $this->model->checkUserExistence(array("phone" => $createUserData['phone']));
    }

    if($AvailabilityUserName!==false && in_array("userName",basic::$cfg['auth']['register']['require'])) {
      $this->auth->redirectInside($address = "register",$notif = 7035);
    }

    if($AvailabilityEmail!==false && in_array("email",basic::$cfg['auth']['register']['require'])) {
      $this->auth->redirectInside($address = "register",$notif = 7036);
    }

    if($AvailabilityPhone!==false && in_array("phone",basic::$cfg['auth']['register']['require'])) {
      $this->auth->redirectInside($address = "register",$notif = 7037);
    }

    if(basic::$cfg['auth']['register']['encryptPassword']) {
      $createUserData['password'] = $this->encryptPassword($password = $createUserData['password']);
    }

    if(basic::$cfg['auth']['register']['mustActivate']) {        // send activation mail to user's email
      // we don't have to activate user right now
      $createUserData['actionTypeId'] = 100001;
      $createUserData['isActive']       = 0;
    }
    else {
      $createUserData['actionTypeId'] = 100000;
      $createUserData['isActive']       = 1;
    }

    $createUserData['createdByUserId'] = 0;
    $createdUserId            = "";

    if($validRegistration==true) {
      $createdUserId = $this->model->createUser($createUserData,$createUserSubscriptionData);    // create the user on the db
    }

    if(isset($createdUserId['lastInsertId']) && $createdUserId['lastInsertId']!==false) {
    }
    else {
      $this->auth->redirectInside($address = "register",$notif = 7027);
    }    // if user was not created

    if(basic::$cfg['auth']['register']['mustActivate']) {
      $activationCode = $this->encryptBridgeCode();

      $vars                   = $createUserData;
      $vars['activationLink'] = lang::getCurrentLanguageForUrls()."userActivation?&code=".$activationCode;

      $userName = !empty($createUserData['userName']) ? $createUserData['userName'] : "";
      $email    = !empty($createUserData['email']) ? $createUserData['email'] : "";

      $mail = basic::load("mail","functions");
      $sent = $mail->sendMail($messageId = 76001,$bodyFile = "userActivation",$vars,$subject = basic::$cfg['site']['name']." - ".$this->T("User activation")."",$fromName = basic::$cfg['site']['name'],$fromAddress = basic::$cfg['mail']['defaultAddress'],$replayAddress = basic::$cfg['mail']['defaultAddress'],$toName = $userName,$toAddress = $email);

      if($sent!==false) {        // if mail was sent
        $createActivationData['codeType']     = 1;        // 1 - activationCode
        $createActivationData['code']         = $activationCode;
        $createActivationData['email']        = $createUserData['email'];
        $createActivationData['createdByUserId']       = $createdUserId['lastInsertId'];
        $createActivationData['actionTypeId'] = 100002;
        $createdCode                          = $this->model->createActivationCode($createActivationData);    // create the activation code on the db

        setcookie("userName-R","",time() - 3600,"/");
        setcookie("title","",time() - 3600,"/");
        setcookie("name","",time() - 3600,"/");
        setcookie("firstName","",time() - 3600,"/");
        setcookie("lastName","",time() - 3600,"/");
        setcookie("email","",time() - 3600,"/");
        setcookie("phone","",time() - 3600,"/");        // remove all "register" cookies because user already registered

        if(!$createdCode) {        // if code was not created
          $this->auth->redirectInside($address = "register",$notif = 7029);
        }
      }
      else {        // if mail was not sent
        $this->auth->redirectInside($address = "register",$notif = 7028);
      }

      $this->auth->redirectInside($address = "haveToActivate");
    }

    $this->auth->redirectInside($address = "userCreated");
  }

  /**
   * Have to activate - user that is not activated yet -> activation mail was sent to user -> user have to activate the account
   * @return string - the view content
   */
  public function haveToActivate() {

    $timed = $this->trans->seconds2Human(basic::$cfg['auth']['bridgeTimeout']['activation']);
    $this->set("timed",$timed);

    return $this->view();
  }

  /**
   * User created - the page that displays when user created
   * @return string - the view content
   */
  public function userCreated() {

    $this->auth->setCookieNow($name = "registerAttempts",$data = "",$till = -3600,$domain = "");

    return $this->view();
  }

  /**
   * User activation - the page that displays when user created account -> activation mail send to user -> user have to activate the account
   * @param $code - from GET
   * @return string - the view content
   * @throws ReflectionException
   */
  public function userActivation($code) {

    if($this->auth->checkIfLogged()) {
      $this->auth->redirectInside($address = "logged",$notifNum = 7030);
    }

    $this->auth->validation = basic::load("validation","functions");

    $userActivationCode = isset($code['GET']) ? $this->auth->validation->validate($code['GET'],"varchar",255,"optional","code") : false;

    if(!$userActivationCode) {
      $this->auth->redirectInside($address = "",$notifNum = 7050);
    }    // no post made - so dump user

    $validActivation   = $this->model->checkBridgeCode($userActivationCode);        // checks if the code valid
    $user['firstName'] = "*Invalid*";    // just a reset
    $user['lastName']  = "*Invalid*";    // just a reset
    if(isset($_COOKIE['userActivatedFirstName'])) {
      $user['firstName'] = $_COOKIE['userActivatedFirstName'];
    }
    if(isset($_COOKIE['userActivatedLastName'])) {
      $user['lastName'] = $_COOKIE['userActivatedLastName'];
    }
    $turnUserActive = "";

    if($validActivation) {
      if($validActivation['codeType']==1 && $validActivation['createdByUserId'] && strtotime($validActivation['createdAt'])<time() + basic::$cfg['auth']['bridgeTimeout']['activation'] && $validActivation['executed']!=1) {        // if it's userActivation type and date is not over

        $updateBridgeCodeExecuted = $this->model->updateBridgeCodeExecuted($userActivationCode,1,100004,$validActivation['createdByUserId'],$validActivation['leadId']);    // have to turn the user Active

        if($updateBridgeCodeExecuted) {
          $turnUserActive = $this->model->turnUserActive($validActivation['createdByUserId'],$actionTypeId = 100003);    // have to turn the user Active
          $user           = $this->model->getUserDetails($validActivation['createdByUserId']);    // fetching info for the message

          setcookie("userActivated",$userActivationCode,time() + basic::$cfg['auth']['globalTimeout'],"/");    // so we will know user already activated
          setcookie("userActivatedFirstName",$user['firstName'],time() + basic::$cfg['auth']['globalTimeout'],"/");    // so we will know user already activated
          setcookie("userActivatedLastName",$user['lastName'],time() + basic::$cfg['auth']['globalTimeout'],"/");    // so we will know user already activated
        }
      }
    }

    if($turnUserActive || (isset($_COOKIE['userActivated']) && $_COOKIE['userActivated']==$userActivationCode)) {
      $this->set("valid",true);
      $this->set("user",$user);

      $user = $this->model->getUserDetails($validActivation['createdByUserId']);    // fetching info for the message
      $mail = basic::load("mail","functions");
      $vars = array("firstName" => $user['firstName']);
      $sent = $mail->sendMail($messageId = 76002,$bodyFile = "newUserWelcome",$vars,$subject = $this->T("Welcome to").": ".basic::$cfg['site']['name'],$fromName = basic::$cfg['site']['name'],$fromAddress = basic::$cfg['mail']['defaultAddress'],$replayAddress = basic::$cfg['mail']['defaultAddress'],$toName = $user['firstName'],$toAddress = $user['email']);

      $vars      = array(
        "id"        => $validActivation['createdByUserId'],
        "firstName" => $user['firstName'],
        "lastName"  => $user['lastName'],
        "email"     => $user['email']
      );
      $sentAdmin = $mail->sendMailToAdmin($messageId = 85000,$bodyFile = "newUserCreatedToAdmin",$vars,$subject = $this->trans->T("New user created on:")." ".basic::$cfg['site']['name'].", env: ".ucfirst(basic::$cfg['site']['environment']));
    }        // validate ActivationCode is still active
    else {
      $this->set("valid",false);
    }

    return $this->view();
  }

  /**
   * Forgot my password
   * @return string - the view content
   */
  public function forgotMyPassword() {

    if($this->auth->checkIfLogged()) {
      $this->auth->redirectInside($address = "logged",$notifNum = 7030);
    }

    $this->view->cfg              = basic::$cfg;
    $this->view->cfgAuth          = basic::$cfg['auth'];
    $this->view->askPasswordReset = "/".basic::$cfg['site']['directory']['main'].lang::getCurrentLanguageForUrls()."sil/askPasswordReset";    // ?&chock=1
    $this->view->currentAction    = "forgotMyPassword";
    $this->captchaSensitive("forgotMyPassword");

    if(basic::$cfg['auth']['allowNotifs']==1) {
      $notifNum = Auth::catchAuthAlert($catchableAlerts = array(7004,7008),$unCatchableAlerts = "");

      $externalVar = "";
      if(is_array($notifNum)) {
        if(in_array(7004,$notifNum)) {
          $externalVar = $this->trans->seconds2Human(basic::$cfg['auth']['triple']);
        }
        if(in_array(7008,$notifNum)) {
          $externalVar = $this->view->captchaSensitivity;
        }
        if($notifNum) {
          $this->notif->genNotifByNum($file = "auth",$notifNum,$externalVar);    // issue alerts
        }
      }
    }

    return $this->view();
  }

  /**
   * Ask password reset
   * @param $userName - from POST
   * @param $email - from POST
   * @param $phone - from POST
   * @param $verificationForgotMyPasswordCode - from POST
   * @return string - the view content
   * @throws ReflectionException
   */
  public function askPasswordReset($userName,$email,$phone,$verificationForgotMyPasswordCode) {

    if($this->auth->checkIfLogged()) {
      $this->auth->redirectInside($address = "logged",$notifNum = 7030);
    }

    $this->view->cfgAuth = basic::$cfg['auth'];
    $this->view->cfg     = basic::$cfg;

    $attemptCS = $this->auth->getCookieNow($name = "forgotMyPasswordAttempts");
    if(!$attemptCS) {
      $attemptCS = 0;
    }
    $this->auth->setCookieNow($name = "forgotMyPasswordAttempts",$data = (int)($attemptCS + 1),$till = "",$domain = "");

    $this->auth->validation = basic::load("validation","functions");

    if(in_array("userName",basic::$cfg['auth']['forgotMyPassword']['forgotMyPasswordVia'])) {
      if(isset($userName['POST']) && $userName['POST']) {
        $checkData['userName'] = $this->auth->validation->validate($userName['POST'],"varchar",80,"optional","userName");
      }
    }
    if(in_array("email",basic::$cfg['auth']['forgotMyPassword']['forgotMyPasswordVia'])) {
      if(isset($email['POST']) && $email['POST']) {
        $checkData['email'] = $this->auth->validation->validate($email['POST'],"varchar",80,"optional","email");
      }
    }
    if(in_array("phone",basic::$cfg['auth']['forgotMyPassword']['forgotMyPasswordVia'])) {
      if(isset($phone['POST']) && $phone['POST']) {
        $checkData['phone'] = $this->auth->validation->validate($phone['POST'],"varchar",80,"optional","phone");
      }
    }
    if(isset($verificationForgotMyPasswordCode['POST'])) {
      $this->auth->verificationForgotMyPasswordCode = $this->auth->validation->validate($verificationForgotMyPasswordCode['POST'],"varchar",80,"optional","verificationForgotMyPasswordCode");
    }

    if(!$this->auth->validation->isValidated()) {
      $this->auth->redirectInside($address = "forgotMyPassword",$notif = 7610);
    }

    $checkData['isActive']  = 1;
    $newBridgeCodeCreated = false;
    unset($_POST);

    if(empty($checkData['userName']) && empty($checkData['email']) && empty($checkData['phone'])) {
      $this->auth->redirectInside($address = "forgotMyPassword",$notifNum = 7034);
    }    // no post made - so dump user

    $this->auth->forgotMyPasswordAttempt();                     // check captcha code + token
    $validId = $this->model->checkUserExistenceDetails($checkData);    // check if the user exist on the db

    if($validId==true) {    // send password reset link
      if(!empty($validId[0]['leadId'])) {
        $userId = $validId[0]['leadId'];

        $checkBridgeCode = $this->model->checkBridgeCodeExist(array("createdByUserId" => $userId));

        if($checkBridgeCode['codeType']==2 && $checkBridgeCode['createdByUserId'] && strtotime($checkBridgeCode['createdAt'])<time() + basic::$cfg['auth']['bridgeTimeout']['activation'] && $checkBridgeCode['executed']!=1) {        // if it's userActivation type and date is not over
          $resetPasswordCode = $checkBridgeCode['code'];    // no need to send new code - the old code is still available for action...
        }
        else {
          $newBridgeCodeCreated = true;
          $resetPasswordCode    = $this->encryptBridgeCode();
        }
      }
      else {
        $newBridgeCodeCreated = true;
        $resetPasswordCode    = $this->encryptBridgeCode();
      }

      $vars                      = $validId[0];
      $vars['passwordResetLink'] = lang::getCurrentLanguageForUrls()."userPasswordReset?&code=".$resetPasswordCode;

      $mail = basic::load("mail","functions");
      $sent = $mail->sendMail($messageId = 76000,$bodyFile = "userPasswordReset",$vars,$subject = basic::$cfg['site']['name']." - ".$this->T("User password reset request")."",$fromName = basic::$cfg['site']['name'],$fromAddress = basic::$cfg['mail']['defaultAddress'],$replayAddress = basic::$cfg['mail']['defaultAddress'],$toName = $validId[0]['userName'],$toAddress = $validId[0]['email']);

      if($sent!==false) {        // if mail was sent
        $createPasswordResetData['codeType']     = 2;        // 1 - resetUserPassword
        $createPasswordResetData['code']         = $resetPasswordCode;
        $createPasswordResetData['email']        = $validId[0]['email'];
        $createPasswordResetData['createdByUserId']       = $validId[0]['leadId'];
        $createPasswordResetData['actionTypeId'] = 100010;

        if($newBridgeCodeCreated===true) {
          $createdCode = $this->model->createActivationCode($createPasswordResetData);    // create the activation code on the db
          if(!$createdCode) {        // if code was not created
            $this->auth->redirectInside($address = "forgotMyPassword",$notif = 7031);
          }
        }
      }
      else {        // if mail was not sent
        $this->auth->redirectInside($address = "forgotMyPassword",$notif = 7032);
      }
    }
    else {                    // don't send password reset link
      //	$this->set("valid",false);
    }

    $timed = $this->trans->seconds2Human(basic::$cfg['auth']['bridgeTimeout']['passwordReset']);
    $this->set("timed",$timed);

    return $this->view();    // return that: mail was sent even if we didn't sent it
  }

  /**
   * User password reset - on db
   * @param $code - from GET
   * @return string - the view content
   * @throws ReflectionException
   */
  public function userPasswordReset($code) {

    if($this->auth->checkIfLogged()) {
      $this->auth->redirectInside($address = "logged",$notifNum = 7030);
    }

    $this->auth->validation = basic::load("validation","functions");
    $this->view->language   = lang::getDefaultLanguage()==lang::$currentLanguage ? "" : lang::$currentLanguage;

    $passwordResetCode = isset($code['GET']) ? $this->auth->validation->validate($code['GET'],"varchar",255,"optional","code") : false;

    if(!$passwordResetCode) {
      $this->auth->redirectInside($address = "",$notifNum = 7050);
    }    // no post made - so dump user

    $validPasswordReset = $this->model->checkBridgeCode($passwordResetCode);        // checks if the code valid
    //	$user['firstName'] = "*Invalid*";	// just a reset
    //	$user['lastName'] = "*Invalid*";	// just a reset
    $activePasswordReset = "";

    /*    p(22222222222);
        p($validPasswordReset);*/

    if($validPasswordReset) {
      if($validPasswordReset['codeType']==1 && $validPasswordReset['createdByUserId'] && strtotime($validPasswordReset['createdAt'])<time() + basic::$cfg['auth']['bridgeTimeout']['passwordReset'] && $validPasswordReset['executed']!=1) {        // if it's userActivation type and date is not over

        $updateBridgeCodeExecuted = $this->model->updateBridgeCodeExecuted($passwordResetCode,1,100004,$validPasswordReset['createdByUserId'],$validPasswordReset['leadId']);    // have to turn the user Active

        if($updateBridgeCodeExecuted) {
          $turnUserActive = $this->model->turnUserActive($validPasswordReset['createdByUserId'],$actionTypeId = 100003);    // have to turn the user Active
          $user           = $this->model->getUserDetails($validPasswordReset['createdByUserId']);    // fetching info for the message
          setcookie("userActivated",$passwordResetCode,time() + basic::$cfg['auth']['globalTimeout'],"/");    // so we will know user already activated
        }

        $activePasswordReset = true;
      }
    }

    if(!$activePasswordReset) {
      $this->addJsIncludes(array('js/auth/login-register.js','js/auth/login-checkIt.js'));

      $this->view->cfg     = basic::$cfg;
      $this->view->cfgAuth = basic::$cfg['auth'];

      $this->view->passwordReset = "/".basic::$cfg['site']['directory']['main'].lang::getCurrentLanguageForUrls()."sil/savePasswordReset";    // ?&chock=1

      $this->view->code = "";
      if(isset($code['GET'])) {
        $this->view->passwordResetCode = $code['GET'];
      }

      return $this->view();
    }    // validate passwordResetCode is still active
    else {
      return $this->loadView("sil/userPasswordResetInvalid");
    }
  }

  /**
   * Save password reset - on db
   * @param $userPassword
   * @param $userPasswordRepeat
   * @param $passwordResetCode
   * @return string - the view content
   * @throws ReflectionException
   */
  public function savePasswordReset($userPassword,$userPasswordRepeat,$passwordResetCode) {

    if($this->auth->checkIfLogged()) {
      $this->auth->redirectInside($address = "logged",$notifNum = 7030);
    }

    $this->view->cfg        = basic::$cfg;
    $this->view->cfgAuth    = basic::$cfg['auth'];
    $this->auth->validation = basic::load("validation","functions");

    $updateUserPassword['userPassword'] = isset($userPassword['POST']) ? $this->auth->validation->validate($userPassword['POST'],"varchar",80,"required","userPassword") : false;

    $updateUserPassword['userPasswordRepeat'] = isset($userPasswordRepeat['POST']) ? $this->auth->validation->validate($userPasswordRepeat['POST'],"varchar",80,"required","userPasswordRepeat") : false;

    $passwordResetCode = isset($passwordResetCode['POST']) ? $this->auth->validation->validate($passwordResetCode['POST'],"varchar",255,"required","passwordResetCode") : false;

    unset($_POST);

    if(!$updateUserPassword['userPassword'] || !$updateUserPassword['userPasswordRepeat'] || !$passwordResetCode) {
      $this->auth->redirectInside($address = "",$notifNum = 7050);
    }    // no post made - so dump user

    if($updateUserPassword['userPassword']!=$updateUserPassword['userPasswordRepeat']) {
      $this->auth->redirectInside($address = "userPasswordReset?&code=".$passwordResetCode,$notif = 7026);
    }

    $this->auth->userPasswordResetAttempt();        // checks token
    $validPasswordReset = $this->model->checkBridgeCode($passwordResetCode);    // checks if the code valid
    //	$user['firstName'] = "*Invalid*";	// just a reset
    //	$user['lastName'] = "*Invalid*";	// just a reset
    //	$turnUserActive = "";

    if($validPasswordReset) {
      if($validPasswordReset['codeType']==2 && $validPasswordReset['createdByUserId'] && strtotime($validPasswordReset['createdAt'])<time() + basic::$cfg['auth']['bridgeTimeout']['passwordReset'] && $validPasswordReset['executed']!=1) {        // if it's resetUserPassword type and date is not over

        $updateBridgeCodeExecuted = $this->model->updateBridgeCodeExecuted($passwordResetCode,2,100012,$validPasswordReset['createdByUserId'],$validPasswordReset['leadId']);    // have to turn the user Active

        if($updateBridgeCodeExecuted) {
          $updateUserPassword['encryptedUserPassword'] = $this->encryptPassword($updateUserPassword['userPassword']); // encrypt password

          $saveNewUserPassword = $this->model->saveNewUserPassword($validPasswordReset['createdByUserId'],$updateUserPassword['encryptedUserPassword'],$actionTypeId = 100011);        // have to save the new password
          $user                = $this->model->getUserDetails($validPasswordReset['createdByUserId']);    // fetching info for the message
          setcookie("userActivated",$passwordResetCode,time() + basic::$cfg['auth']['globalTimeout'],"/");    // so we will know user already activated

          $this->model->updateUserLog($validPasswordReset['createdByUserId'],"lastPasswordReset");
          $this->auth->redirectInside($address = "login",$notifNum = 7033);    // user can now login
        }
      }
    }
  }

  /**
   * Captcha image loader
   * @param $verificationSubject - from GET
   * @throws ReflectionException
   */
  public function captcha($verificationSubject) {

    $auth = basic::load("auth","core");

    $verificationTriesCSN = "";

    $this->auth->validation = basic::load("validation","functions");
    $verificationSubject    = isset($verificationSubject['GET']) ? $this->auth->validation->validate($verificationSubject['GET'],"varchar",80,"required","verificationSubject") : false;

    if(empty($verificationSubject)) {
      echo "Invalid captcha!";
      die();
    }

    if(basic::$cfg['auth']['login']['allowCaptcha']==1 && $verificationSubject=="login") {
      $verificationTriesCSN = "verificationLoginTries";
      $verificationCodeCSN  = "verificationLoginCode";
    }
    if(basic::$cfg['auth']['register']['allowCaptcha']==1 && $verificationSubject=="register") {
      $verificationTriesCSN = "verificationRegisterTries";
      $verificationCodeCSN  = "verificationRegisterCode";
    }
    if(basic::$cfg['auth']['forgotMyPassword']['allowCaptcha']==1 && $verificationSubject=="forgotMyPassword") {
      $verificationTriesCSN = "verificationForgotMyPasswordTries";
      $verificationCodeCSN  = "verificationForgotMyPasswordCode";
    }

    if(!$verificationTriesCSN) {
      die("Captcha \"subject\" Error!");
    }

    $verificationTriesCS   = $auth->getCookieNow($name = $verificationTriesCSN);
    $timeLimitVerification = explode("-",$verificationTriesCS);

    if(basic::$cfg['auth']['login']['allowCaptcha']!=1 || (isset($timeLimitVerification[1]) && $timeLimitVerification[1]>=time())) {

      // Output the image to browser
      header("Content-type: image/png");
      header("Expires: Sun,1 Jan 2000 12:00:00 GMT");
      header("Last-Modified: ".gmdate("D,d M Y H:i:s")."GMT");
      header("Cache-Control: no-store,no-cache,must-revalidate");
      header("Cache-Control: post-check=0,pre-check=0",false);
      header("Pragma: no-cache");

      $file = "images/captchaDelay.png";
      $fd   = fopen($file,"r");
      fpassthru($fd);
    }    // do not make new captcha if verificationTriesLimit if overdue
    else {
      if($timeLimitVerification[0]=="T") {
        $auth->setCookieNow($name = $verificationTriesCSN,$data = 1,$till = -3600,$domain = "");
      }    // reset the counter to 1

      basic::load("captcha","functions",array($auth,$verificationTriesCSN,$verificationCodeCSN));
    }
  }

  /**
   * Check user availability
   * @param string $typeOfCheck - userName/email/phone
   * @param string $valueOfCheck - the value to check
   * @return string - the view content on ajax
   * @throws ReflectionException
   */
  public function checkUserAvailability($param0,$typeOfCheck = "",$valueOfCheck = "") {    // must remain ="" for ajax purposes

    $isAjax = myHelpers::isAjax();

    if(!$isAjax || !$typeOfCheck || !$valueOfCheck) {    // that means that user is not using ajax page
      $this->auth->redirectInside($address = "",$notifNum = 7050);

      return false;
    }

    $validToken = $this->auth->checkToken();

    if($validToken===false) {
      return $this->T("No valid token - you must to refresh the page before submit");
    }    // redirect to the "register" page

    $this->auth->validation = basic::load("validation","functions");
    $valueOfCheck           = $this->auth->validation->validate($valueOfCheck['POST'],$type = "varchar",80,"optional","valueOfCheck");
    //	p($valueOfCheck);
    if(!$valueOfCheck) {
      return false;
    }    // if no value given or didn't pass validation

    if($typeOfCheck['POST']=="userName") {
      $checkData['userName'] = $valueOfCheck;
    }
	elseif($typeOfCheck['POST']=="email") {
      $checkData['email'] = $valueOfCheck;
    }
	elseif($typeOfCheck['POST']=="phone") {
      $checkData['phone'] = $valueOfCheck;
    }
    else {
      echo $this->T("Error with given Availability data");
    }

    $userExist = $this->model->checkUserExistence($checkData);    // check if exists

    if($userExist!==false) {
      if($typeOfCheck['POST']=="userName") {
        echo $this->T("User \"\$externalVar\" is not available - please choose other",$valueOfCheck);
      }

      if($typeOfCheck['POST']=="email") {
        echo $this->T("Email \"\$externalVar\" is already registered",$valueOfCheck);
      }

      if($typeOfCheck['POST']=="phone") {
        echo $this->T("Phone \"\$externalVar\" is already registered",$valueOfCheck);
      }
    }
  }

  /**
   * After action - the function that comes before all the functions in the controller class
   */
  public function afterAction() {

    session_write_close();
  }

  /*	public function redirect() {

  if(!headers_sent()) {
    header("Expires: Mon, 26 Jul 1990 05:00:00 GMT");
    header("Last-Modified: ".gmdate("D, d M Y H:i:s")." GMT");
    header("Cache-Control: no-store, no-cache, must-revalidate, max-age=-1");
    header("Cache-Control: post-check=0, pre-check=0", false);
    header("Pragma: no-cache");
  }

  return $this->view();
}*/

  /**
   * Get user details
   * @param $userId
   * @return mixed
   */
  private function getUserDetails($userId) {

    $user = $this->model->getUserDetails($userId);    // fetching info for the message

    return $user;
  }

  /**
   * Personal Info Edit
   * @return string - the view content
   */
  public function personalInfo() {

    $this->authLevelNeeded(1);

    //   $validation = basic::load("validation","functions");

    $userDetails = $this->getUserDetails($this->auth->getUserLoggedUserId());

    $this->view->editPersonalInfo = "/".basic::$cfg['site']['directory']['main'].lang::getCurrentLanguageForUrls()."sil/editPersonalInfo";
    $this->view->language         = lang::getDefaultLanguage()==lang::$currentLanguage ? "" : lang::$currentLanguage;

    $this->view->userName  = $userDetails['userName'];
    $this->view->email     = $userDetails['email'];
    $this->view->phone     = $userDetails['phone'];
    $this->view->title     = $userDetails['title'];
    $this->view->firstName = $userDetails['firstName'];
    $this->view->lastName  = $userDetails['lastName'];

    return $this->view();
  }
}

