<?php

/*************************************************************
 * silModel class
 * - Sil connection with the db
 * - checks and validate users on the db
 * @author    S M Kessner <limbercode@gmail.com>
 * @copyright 2010-2018 T2
 * @link      http://limbercode.com
 *************************************************************/
class silModel extends model {

  /**
   * Initial function to load
   */
  protected function init() {
  }

  /**
   * Check if user exists on db according to the data given
   * @param array $data - containing userName/password/active/deleted and any other things that have to validate
   * @return array $checkExistence - containing all the information about the user
   * @throws ReflectionException
   */
  public function checkUserExistence($data) {

    $checkExistence = $this->db->checkExist($table = basic::getConfig("db","main","tablePrefix")."users",$data);

    return $checkExistence;
  }

  /**
   * Check if user exists on db according to the data given
   * @param array $includingData - containing active/deleted and any other things that have to validate
   * @param string|array $alternatelyData - containing userName/email/phone and any other things that have to validate
   * @return array $checkExistence - containing all the information about the user
   * @throws ReflectionException
   */
  public function checkUserExistenceDetails($includingData,$alternatelyData="") {

    $andData = "";
    $orData = "";
    $params = "";
    $sql = "";

    if(!empty($includingData) && is_array($includingData)) {
      foreach($includingData as $key => $value) {
//    $andData .= $key." = ? AND ";
        $andData .= $key." = :".$key." AND ";
      }

      $andData = stringManipulation::trimEnd("AND ",$andData);
    }

    if(!empty($alternatelyData) && is_array($alternatelyData)) {
      foreach($alternatelyData as $key => $value) {
//    $orData .= $key." = ? OR ";
        $orData .= $key." = :".$key." OR ";
      }

      $orData = stringManipulation::trimEnd("OR ",$orData);
    }

    if($andData && $orData) {
      $params = array_merge($includingData,$alternatelyData);
      $sql = "SELECT * FROM ".basic::getConfig("db","main","tablePrefix")."users WHERE (".$andData.") AND (".$orData.")";
    }
    else {
      if($andData) {
        $params = $includingData;
        $sql = "SELECT * FROM ".basic::getConfig("db","main","tablePrefix")."users WHERE ".$andData;
      }
    }

    $checkExistence = $this->db->fetchCustom($sql,$params,"fullString");

    return $checkExistence;
  }

  /**
   * Find last userId
   * @return mixed
   * @throws ReflectionException
   */
  public function findLastUserId() {

    $params = array();
    $sel    = $this->db->prepare("
			SELECT MAX(leadId) as lastUserId FROM ".basic::getConfig("db","main","tablePrefix")."users
		");
    $sel->execute($params);
    $sel->setFetchMode(PDO::FETCH_ASSOC);

    $result = $sel->fetch();

    return $result['lastUserId'];
  }

  /**
   * Create a new user on the db
   * @param array $createData - containing userName/password and any other things that have to be saved
   * @param array $createUserSubscriptionData - containing subscription data
   * @return array $createdUser - containing: numOfRowsEffected, lastInsertId, queryString (of the row inserted)
   * @throws ReflectionException
   */
  public function createUser($createData,$createUserSubscriptionData) {

    $createdUser = $this->db->insert($table = basic::getConfig("db","main","tablePrefix")."users",$createData,$advancedLead = true);
    $createdUserSubscription = "";

    if($createdUser) {
      if(!empty($createUserSubscriptionData)) {
        foreach($createUserSubscriptionData['subscription'] as $key => $value) {
          if($value!==false) {
            $createDataSub = array(
              "createdByUserId" => $createdUser['lastInsertId'],
              "isActive" => $value,
              "type"   => $key
            );

            $createdUserSubscription = $this->db->insert($table = basic::getConfig("db","main","tablePrefix")."users_subscriptions",$createDataSub,$advancedLead = true);
          }
        }
      }

      return $createdUser;
    }
    else {
      return false;
    }
  }

  /**
   * Create activation code on the db
   * @param array $createData - containing codeType, code, email, userId, actionTypeId
   * @return boolean - if succeed true
   * @throws ReflectionException
   */
  public function createActivationCode($createData) {

    $createdCode = $this->db->insert($table = basic::getConfig("db","main","tablePrefix")."bridge_codes",$createData,$advancedLead = true);

    if($createdCode) {
      return true;
    }
    else {
      return false;
    }
  }

  /**
   * Check bridge code exist?
   * @param $checkData  - userName/email/phone
   * @return array - containing: leadId, codeType, userId, createdAt, email, executed
   * @throws ReflectionException
   */
  public function checkBridgeCodeExist($checkData) {

    $checkData = $this->db->find($table = basic::getConfig("db","main","tablePrefix")."bridge_codes",$col = array(
      "leadId",
      "codeType",
      "createdByUserId",
      "createdAt",
      "code",
      "email",
      "executed"
    ),$where = $checkData,false,"leadId desc",1);

    if(isset($checkData[0])) {
      return $checkData[0];
    }
    else {
      return false;
    }
  }

  /**
   * Check bridge code
   * @param string $activationCode - activation code
   * @return array - containing: leadId, codeType, createdByUserId, createdAt, email, executed
   * @throws ReflectionException
   */
  public function checkBridgeCode($activationCode) {

    $checkData = $this->db->find($table = basic::getConfig("db","main","tablePrefix")."bridge_codes",$col = array(
      "leadId",
      "codeType",
      "createdByUserId",
      "createdAt",
      "email",
      "executed"
    ),$where = array("code" => $activationCode),false,"leadId desc",1);

    if(isset($checkData[0])) {
      return $checkData[0];
    }
    else {
      return false;
    }
  }

  /**
   * Update bridge code executed
   * @param string $activationCode - activation code
   * @param string $codeType - code type
   * @param string $actionTypeId - action type id
   * @param string $userId - user id that created the bridge code
   * @param string $leadId - lead id
   * @return boolean - if executed true
   * @throws ReflectionException
   */
  public function updateBridgeCodeExecuted($activationCode,$codeType,$actionTypeId,$userId,$leadId) {

    $preData['actionTypeId'] = $actionTypeId;
    $preData['createdByUserId']       = $userId;
    $preData['executed']     = 1;

    $updateBridgeCodeExecuted = $this->db->update($table = basic::getConfig("db","main","tablePrefix")."bridge_codes",$preData,$where = array(
      "code"     => $activationCode,
      "codeType" => $codeType
    ),$advancedLeadId = $leadId);        // mark that code was activated - and we don't have to do that again for this code

    if($updateBridgeCodeExecuted['numOfRowsEffected']>0) {
      return true;
    }
    else {
      return false;
    }
  }

  /**
   * Turn user active
   * @param string $userId - user id
   * @param string $actionTypeId - action type id
   * @return boolean - if turn to true
   * @throws ReflectionException
   */
  public function turnUserActive($userId,$actionTypeId) {

    $turnUserActive = $this->db->update($table = basic::getConfig("db","main","tablePrefix")."users",$data = array(
      "actionTypeId" => $actionTypeId,
      "isActive"       => 1
    ),$where = array("leadId" => $userId));        // make the necessary changes on the db

    if($turnUserActive['numOfRowsEffected']>0) {
      return true;
    }
    else {
      return false;
    }
  }

  /**
   * Get user details
   * @param string $userId - user id
   * @return array $checkData - containing: userName, email, phone, firstName, lastName
   * @throws ReflectionException
   */
  public function getUserDetails($userId) {

    $checkData = $this->db->find($table = basic::getConfig("db","main","tablePrefix")."users",$col = array(
      "userName",
      "email",
      "phone",
      "title",
      "firstName",
      "lastName"
    ),$where = array("leadId" => $userId));

    if($checkData) {
      return $checkData[0];
    }
    else {
      return false;
    }
  }

  /**
   * Save new user password
   * @param string $userId - user id
   * @param string $userNewPassword - user new password
   * @param string $actionTypeId - action type i
   * @return boolean - if password changed true
   * @throws ReflectionException
   */
  public function saveNewUserPassword($userId,$userNewPassword,$actionTypeId) {

    $preData['createdByUserId']       = $userId;
    $preData['actionTypeId'] = $actionTypeId;
    $preData['password']     = $userNewPassword;

    basic::$cfg['advanced']['autoIncrementLeadIdBy'] = basic::getConfig("auth","register","actionIdLeap");    // replace autoIncrementLeadIdBy

    $saveNewUserPassword = $this->db->update($table = basic::getConfig("db","main","tablePrefix")."users",$preData,$where = array("leadId" => $userId),$advancedLeadId = $userId);        // make the necessary changes on the db

    if($saveNewUserPassword['numOfRowsEffected']>0) {
      return true;
    }
    else {
      return false;
    }
  }

  /**
   * Update user log - update the last time it (the option) happened
   * @param int - $userId
   * @param string - $option - can be: lastSuccessLogin|lastFailedLogin|lastPasswordReset
   * @return bool
   * @throws ReflectionException
   */
  public function updateUserLog($userId,$option) {

    $sqlDate          = date("Y-m-d H:i:s",time());
    $preData[$option] = $sqlDate;

    $updatelastOption = $this->db->update($table = basic::getConfig("db","main","tablePrefix")."users",$preData,$where = array(
      "createdByUserId" => $userId
    ));

    if($updatelastOption['numOfRowsEffected']>0) {
      return true;
    }
    else {
      return false;
    }
  }
}