<?php

/*************************************************************
 * Rudiment theme class - minimal theme
 * @author    S M Kessner <limbercode@gmail.com>
 * @copyright 2010-2017 T2
 * @link      http://limbercode.com
 *************************************************************/
class rudimentTheme extends template {

	public $templatePlan;
	public $topPanelR;
	public $showSidePanelR;
	public $operatorsInsideR;
	public $topPanelL;
	public $showSidePanelL;
	public $operatorsInsideL;

	/**
	 * Constructor for rudiment theme - do not allow constructor here cz it will disable the template constructor
	 * @param string $controller
	 * @param string $action
	 * @param string $architectureType
	 */
	public function __construct($controller,$action,$architectureType) {

		parent::__construct($controller,$action,$architectureType);
	}

	/**
	 * Template plan
	 * @return array - the files that will be used at this theme
	 */
	public function templatePlan() {

		return $this->templatePlan = array(
			"metaHead",
			"header",
			"operatorLeft",
			"mainEvent",
			"operatorRight",
			"footer",
			"closeTags"
		);
	}

	/**
	 * Template plan cleaned - clean design template
	 * @return array - the files that will be used at this theme
	 */
	public function templatePlanCleaned() {

		return $this->templatePlan = array(
			"metaHead",
			"mainEvent",
			"closeTags"
		);
	}

	/**
	 * Js includes
	 * @return array - the js files that will be used at this theme
	 */
	public function jsIncludes() {

		if(myHelpers::isEnv($env = "prod")) {          // will work only on prod
			//	$jsInclude[] = "http://ajax.googleapis.com/ajax/libs/jquery/1.4.4/jquery.min.js";
		}

    $jsInclude[] = "js/tip.js";
    $jsInclude[] = "js/cookies.js";
    $jsInclude[] = "js/ajaxPerfect.js";
		$jsInclude[] = "js/auth.js";                   // is essential if we use ajaxInterval for auth

    if(count($this->operatorsToActivate)>0) {
      $jsInclude[] = "themes/".$this->currentTemplate."/js/operatorsCfg.js";
      $jsInclude[] = "js/operators.js";
    }

		return $jsInclude;
	}

	/**
	 * Css includes
	 * @return array - the css files that will be used at this theme
	 */
	public function cssIncludes() {

		if(isset($this->operatorsToActivate['position'])) {        // putting the right media query css (for operators)
			$cssMediaForOperators = "themes/".$this->currentTemplate."/css/mediaForOperators.css";
			$cssInclude[] = $cssMediaForOperators;
		}

		$cssInclude[] = "css/effects.css";
		$cssInclude[] = "themes/".$this->currentTemplate."/css/operators.css";
		$cssInclude[] = "themes/".$this->currentTemplate."/css/main.css";
		$cssInclude[] = "themes/".$this->currentTemplate."/css/media.css";
		$cssInclude[] = "css/pageAuthVer.css";      // for auth class

		return $cssInclude;
	}

	/**
	 * Js
	 * @return string - the js script that will be used in this theme
	 */
	public function js() {

		//	return $js = "!window.jQuery && document.write(unescape('%3Cscript src=\"http://ajax.googleapis.com/ajax/libs/jquery/1.4.4/jquery.min.js\"%3E%3C/script%3E'))";
	}

	/**
	 * Css
	 * @return string - the css script that will be used in this theme
	 */
	public function css() {

		return $css = "";
	}

	/**
	 * Header - like a controller for middle file, $this->aaa will remain $this->aaa on header
	 */
	public function header() {

		$this->cssIncludes[] = "themes/".$this->currentTemplate."/css/header.css";
	//	$this->jsIncludes[]  = "themes/".$this->currentTemplate."/js/header.js";
	}

  /**
   * Operator right - like a controller for operatorRight file, $this->aaa will remain $this->aaa on operator right
   */
  public function operatorRight() {

    $style = "";

    if(!isset($this->operatorsToActivate['position']) || array_search("right",$this->operatorsToActivate['position'])===false) {
      $style .= "<style>
					#operatorsR, #sidePanelR {display:none;}
					.mainContent {margin-right:0px;}
				</style>";
    }    // searching if operatorRight panel have to appear - and if not - remove it

    if(!isset($this->themePlan) || array_search("header",$this->themePlan)===false) {
      $style .= "<style>
					#operatorsR {top:0px !important;}
					#sidePanelR {top:60px !important;}
				</style>";
    }    // searching if header is loaded - and if not - we have to change the position of the right operators

    if(isset($_COOKIE['operatorsR']) && $_COOKIE['operatorsR']=="closed" && isset($this->operatorsToActivate['position']) && array_search("right",$this->operatorsToActivate['position'])==true) {
      $style .= "<style>
					/*.mainContent {margin-right:0;}*/
					/*#topPanelR {opacity:0;}*/
				</style>";

      $this->topPanelR        = "extractTopPanelR";
      $this->showSidePanelR   = "attractSidePanelR";
      $this->operatorsInsideR = "slideAndFadeChanged";
    }    // cookies can be managed from js also - this will be second attempt

    return $style;
  }

  /**
   * Middle - like a controller for middle file, $this->aaa will remain $this->aaa on middle
   */
  public function middle() {
  }

  /**
   * Operator left - like a controller for operatorLeft file, $this->aaa will remain $this->aaa on operator left
   */
  public function operatorLeft() {

    $style = "";

    if(!isset($this->operatorsToActivate['position']) || array_search("left",$this->operatorsToActivate['position'])===false) {
      $style .= "<style>
					#operatorsL, #sidePanelL {display:none;}
					.mainContent {margin-left:0px;}
				</style>";
    }    // searching if operatorLeft panel have to appear - and if not - remove it

    if(!isset($this->themePlan) || array_search("header",$this->themePlan)===false) {
      $style .= "<style>
					#wrapper {margin-top:0px !important;}
					#sidePanelL {top:60px !important;}
			</style>";
    }    // searching if header is loaded - and if not - we have to change the position of the left operators

    if(isset($_COOKIE['operatorsL']) && $_COOKIE['operatorsL']=="closed" && isset($this->operatorsToActivate['position']) && array_search("left",$this->operatorsToActivate['position'])==true) {
      $style .= "<style>
					/*.mainContent {margin-left:0;}*/
					/*#topPanelL {opacity:0;}*/
				</style>";

      $this->topPanelL        = "extractTopPanelL";
      $this->showSidePanelL   = "attractSidePanelL";
      $this->operatorsInsideL = "slideAndFadeChanged";
    }    // cookies can be managed from js also - this will be second attempt

    return $style;
  }

	/**
	 * Footer - like a controller for header file, $this->aaa will remain $this->aaa on footer
	 */
	public function footer() {

		$this->cssIncludes[] = "themes/".$this->currentTemplate."/css/footer.css";
    $this->jsIncludes[]  = "themes/".$this->currentTemplate."/js/footer.js";
	}

	/**
	 * Operator box - the frame operators sites
	 */
  public function operatorBox($title,$operatorName,$operatorViewFile,$num,$side) {    // operators content

    global $numOfOperators,$sideReset;

    $convertedTitle = bin2hex($title);
    $coni           = "expander-$convertedTitle";

    if(!isset($sideReset)) {
      $sideReset = "";
    }
    if($side!=$sideReset) {
      $numOfOperators = 0;
    }
    $sideReset = $side;

    $numOfOperators = $numOfOperators + 1;

    if(!isset($coniHref)) {
      $coniHref = "";
    }    // just a reset

    $slideAndFadeChanged = "";            // just a reset
    if(isset($_COOKIE['expander-'.$side.'-'.$operatorName]) && $_COOKIE['expander-'.$side.'-'.$operatorName]=="closed") {
      $slideAndFadeChanged = "slideAndFadeChanged";
    }

    echo "<div class=\"operator\">
				<div class=\"operatorTitle\">
					$num - $title
				</div>
				$coniHref <a class=\"toggleOperator\" id=\"$side-$operatorName\" title=\"".$this->T("Expand or fold")."\"><b>&#8660;</b></a>
				<div id=\"expander-$side-$operatorName\" class=\"clear slideAndFade ".$slideAndFadeChanged."\">
					<div class=\"expander\"></div>";

    if(isset($this->loadedOperators->$operatorName)) {
      //	p($this->loadedOperators->$operatorName,555555);
      $operatorController = $this->loadedOperators->$operatorName;    // retrieve the operator classes
      $putOperatorContent = new operatorsView($operatorName,$operatorController);
    }

    echo "	</div>
			</div>";
  }

  /**
   * Notif - the way we pop up the notifications
   * @param string $text - text to display
   * @param string $notifType - css to use for this alert
   * @param string $link - link inside alert
   * @param string $linkText - link text inside alert
   * @param string $more - more usability for future function
   * @param string /integer/boolean $breakOut - if we break the alert to normal html text
   * @return string - the output
   */
  static function notif($text,$notifType = "option",$link = "",$linkText = "",$more = "",$breakOut = "") {

    global $allowAjaxSaving,$notif;

    $allowAjaxSaving = 1;

    if($notifType=="warning") {
      error_log("Issued Alert popup - ".stripslashes($text));
    }

//	if(headers_sent()) {	// have to check with ajax too

    $lifeSpan = 5000;

    if(isset($more['lifeSpan'])) {
      $lifeSpan = $more['lifeSpan'];
    }

    if($link) {
      $link = "&nbsp;<a href=\"$link\">$linkText</a>";
    }
    if($more=="seconed") {
      $second = "2";
    }
    else $second = "";

    $position = "";
    if($notifType=="efficiency") {
      $position = "position:'bottom-right',";
    }

    if($allowAjaxSaving==1 && $breakOut!=1) {    // $breakOut - for it to always display this massage in regular html and not js
      //  header: '".$notifType."',	// to put inside: $.jGrowl(
      //	$output = "<script type=\"text/javascript\">$.jGrowl('".$text.$link."',{".$position." theme:'".$notifType."',life:".$lifeSpan."});tops();</script>\n";

      $output = "<div class=\"notifUiNum bozoNotification alert ui-state-highlight ui-corner-all ".$notifType."\" data-notifNum=\"\" data-notifType=\"".$notifType."\" data-notifLifeSpan=\"".$lifeSpan."\" data-notifPosition=\"\" data-notifTheme=\"\" style=\"display:none;\">
							<button class=\"bozoClose\">×</button>
							<div class=\"bozoHeader\"></div>
							<div class=\"bozoMessage\">
								".$text.$link."
							</div>
						</div>";        // display:none; must stay to prevent showing before page is up with css
    }
    else {
      $output = "<div id=\"alert-noti$second\">";

      //	$output .= openTable();

      $output .= "<span class=\"$notifType\">$text$link</span><span class=\"close\"><a href=\"javascript:animatedcollapse.toggle('alert-noti$second');\" title=\"close\">X</a>aa</span>";

      //	$output .= closeTable();

      $output .= "<p></p>";
      $output .= "</div>";
    }    // a box on the page

    if(headers_sent() || myHelpers::isAjax()) {
      echo $output;
    }
    else {
      $notif .= $output;

      return $notif;
    }    // for sending the alert at the end of the render
  }

  /**
   * Js notification - send notification from ajax
   * @return string
   */
  static function jsNotif() {

    $script = "<script>function notification(text,type) {

	//	alert('No connection');
	var alert = '<div class=\"notifUiNum bozoNotification alert ui-state-highlight ui-corner-all red\" data-notifNum=\"\" data-notifType=\"+type+\" data-notifLifeSpan=\"99999\" data-notifPosition=\"\" data-notifTheme=\"\" style=\"display:none;\"><button class=\"bozoClose\">×</button><div class=\"bozoHeader\"></div><div class=\"bozoMessage\">'+text+'</div></div>';

	document.body.innerHTML = document.body.innerHTML + alert;
}</script>";        // display:none; must stay to prevent showing before page is up with css

    return $script;
  }
}

