<?php

/*************************************************************
 * The system second starting point:
 * - start the session
 * - load necessary classes
 * - starting the router and dispatch
 * @author    S M Kessner <limbercode@gmail.com>
 * @copyright 2010-2018 T2
 * @link      http://limbercode.com
 *************************************************************/

header_remove('X-Powered-By');  // PHP 5.3+  // Security risk remover
@ini_set('expose_php','off');   // Security risk remover

require_once(LABYARDPATH."functions".DS."includes.php");    // include the includes functions - T2

basic::loadCfg("phpConfig");
basic::loadCfg("errorControlConfig");
basic::loadCfg("templateConfig");
basic::loadCfg("dbConfig");
basic::loadCfg("authConfig");

$sessions = basic::load("sessions","functions");
$sessions::startSession();

/*if(session_id()==="") {    // Ensure we have session    // we have to start it before logConfig load - cz it have $_SESSION
	session_start();
}*/

basic::loadCfg("logConfig");
basic::loadCfg("mailConfig");
basic::loadCfg("notifConfig");
basic::load("myHelpers","core");                        // loads the myHelpers function
basic::load("lang","core");                             // loads the language function
basic::load("stringManipulation","functions");          // loads the stringManipulation function

if(basic::$cfg['errorControl']['allow']==1) {
	$errorControl          = basic::load("errorControl","core");
	$errorControlContainer = $errorControl->showContainer();
}

if(basic::$cfg['errorControl']['dbUi']['allow']) {
	$dbInformant          = basic::load("dbInformant","core");
	$dbInformantContainer = $dbInformant->showContainer();
}

if(basic::$cfg['log']['infoUi']['allow']) {
	$log          = basic::load("log","core");
	$logContainer = $log->showContainer();
}

if(basic::$cfg['notif']['allow']) {
	$notifcation    = basic::load("notif","core");
	$notifContainer = $notifcation->showNotifContainer();
}

if($_SERVER['REQUEST_URI']=="/public/") {    // do not allow to write "public" in the uri
	Header("Location: /");
	die();
}
if(substr($_SERVER['REQUEST_URI'],0,7)=="/public") {
//	echo "problem with VirtualHost... or \basic::$cfg['site']['directory']['main']";
//	die();
}

require_once(LABYARDPATH."mvc".DS."router".".class.php");
$router = new router($route);          // register route - from index.php file
router::isPost();                      // load router::$isPost into memory in essential if someone will do unset($_POST);

$auth = basic::load("auth","core");    // loads the auth function

//	$auth->restrictedSite();

if($auth->isDevUser()) {    // loads the MemoryUsageInformation class
	if(!stringManipulation::startsWith($haystack = $_SERVER["REQUEST_URI"],$needle = "/".basic::$cfg['site']['directory']['api']) && !myHelpers::isAjax()) {
		$memAndTime = basic::load("memoryUsageAndTiming","functions");
	}
}

spl_autoload_register(function ($className) {    // auto load any classes that are required

	$valid = file_exists($classFile = LABYARDPATH.$className.".class.php");        // check root directory of library

	if(!$valid) {    // if we cannot find any, then find library/core directory
		$valid = file_exists($classFile = LABYARDPATH."core".DS.$className.".class.php");
	}
	if(!$valid) {    // if we cannot find any, then find library/mvc directory
		$valid = file_exists($classFile = LABYARDPATH."mvc".DS.$className.".class.php");
	}

	$preDir = APPLICATIONPATH;      // loads regular mvc
	if(stringManipulation::startsWith($haystack = $_SERVER["REQUEST_URI"],$needle = "/advanced")) {
		$preDir = ADVANCEDPATH;     // loads advanced mvc
	}
	elseif(stringManipulation::startsWith($haystack = $_SERVER["REQUEST_URI"],$needle = "/api")) {
		$preDir = APIPATH;          // loads api mvc
	}

	if(!$valid) {    // if we cannot find any, then find architecture/application/controllers directory
		$valid = file_exists($classFile = $preDir."controllers".DS.$className.".php");
	}
	if(!$valid) {    // if we cannot find any, then find architecture/application/models directory
		$valid = file_exists($classFile = $preDir."models".DS.$className.".php");
	}

	if($valid) {    // if we have valid file, then include it
		require_once($classFile);
	}
	else {
		//	echo "Error loading class ".$classFile;
		/* Error Generation Code Here */
	}
});

myHelpers::removeMagicQuotes();        // remove the magic quotes
myHelpers::unregisterGlobals();        // unregistered globals

$router->dispatch();                   // finally we dispatch the output

if($auth->isDevUser()) {    // prints the MemoryUsageInformation output
	if(!stringManipulation::startsWith($haystack = $_SERVER["REQUEST_URI"],$needle = "/".basic::$cfg['site']['directory']['api']) && !myHelpers::isAjax()) {
		$memAndTime->printMemoryUsageInformation();     // causing this element of html to be outside of body - but it's only for DEV users and not on LIVE environment - so we can ignore this
	}
}

session_write_close();        // close session to speed up the concurrent connections
// http://php.net/manual/en/function.session-write-close.php

$userDefinedClasses = array_filter(get_declared_classes(),function ($className) {
	return !call_user_func(array(new ReflectionClass($className),'isInternal'));
});

foreach($userDefinedClasses as $key => $value) {
	$vars = get_class_vars($value);
	if(!empty($vars)) {
		$userDefinedClassesPlusVarsAndMethods[$value]['vars'] = $vars;
	}

	$methods = getThisClassMethods($value);
	if(!empty($methods)) {
		$userDefinedClassesPlusVarsAndMethods[$value]['methods'] = $methods;
	}
}

function getThisClassMethods($class) {

	$array1 = get_class_methods($class);
	if($parent_class = get_parent_class($class)) {
		$array2 = get_class_methods($parent_class);
		$array3 = array_diff($array1,$array2);
	}
	else {
		$array3 = $array1;
	}

	return ($array3);
}

