<?php

/*************************************************************
 * PDO mysql database helper class
 * - use of server sessions
 * @author    S M Kessner <limbercode@gmail.com>
 * @copyright 2010-2018 T2
 * @link      http://limbercode.com
 *************************************************************/
class pdoHelper {

  public $pdo;
  public $startTime;
  public static $pdoConnection;
  public static $log = array();
  public $prepared;
  private $table;
  private $tablePast;
  private $columns;
  private $column;
  private $where;
  private $join;
  private $orderBy;
  private $limit;
  private $customSql;
  private $customizedFunction;
  private $queryString;
  private $params;
  private $isAdvanced;
  private $advancedForMysql;

  private $implodedColumns;
  private $implodedValues;
  private $columnsSqlString;
  private $columnsOnDuplicateSqlString;
  private $columnsSet;
  private $tableProperties;
  private $characterSet;
  private $characterSetMedium;
  private $characterSetFull;
  private $engine;
  private $columnsVariables;
  private $columnsValues;
  private $whereData;
  private $whereValues;
  private $whereColumns;
  private $whereValuesWithLike;
  private $whereValuesEquivalent;
  private $whereMarkEquivalent;
  private $whereMarkWithLike;
  private $whereFoundLeadId;
  private $whereSqlString;
  private $pdoConnectionFailed;

  private $fetchResult;
  private $fetchAllResult;
  private $queryResult;
  private $errorOnQuery;

  private $rowCountResult;
  private $setFetchModeResult;
  private $numOfRowsEffected;
  private $lastInsertId;

  private $calledFromFunction;
  private $calledFromClass;
  private $calledFromFile;
  private $calledFromLine;

  static private $queries;

  /**
   * Constructor for pdo helper - creates the db connection
   */
  public function __construct() {

    if(basic::getConfig("db","main","activate")!=true) {
      if($this->pdoConnectionFailed==false) {
        $errorControl = basic::load("errorControl","core");
        $errorControl->sendMailNotificationError("pdoHelper is not activated 1!",$fatal = true,$dontLogOnDb = 1);
        $this->pdoConnectionFailed = true;
      }

      echo "<p><span style=\"color:orange;\">**</span> mysql/pdoHelper is not activated 1! <span style=\"color:red;\">**</span></p>";
      return false;
    }
  }

  public function __call($method,$arguments) {    // will call in start of each function, other functions must be "protected"

    if(method_exists($this,$method)) {
      if(basic::getConfig("db","main","activate")==true) {
        $this->makeConnection();    // is to prevent the pdoHelper to load even if we don't use mysql in our current page
        if(self::$pdoConnection==false) {
          return false;
        }

        self::startLog();
        return call_user_func_array(array($this,$method),$arguments);
      }
      else {
        echo "<p><span style=\"color:orange;\">**</span> mysql/pdoHelper is not activated 2! <span style=\"color:red;\">**</span></p>";
        return false;
      }
    }
  }

  public function makeConnection() {

    if(!self::$pdoConnection && strpos($_SERVER['REQUEST_URI'],'siteIsDown')!=true && $this->pdoConnectionFailed==false) {
      try {
        $this->engine = "InnoDB";
//        $this->characterSet = "utf8mb4_general_ci";
        $this->characterSet = "utf8";
//        $this->characterSet = "utf8mb4";
        $this->characterSetMedium = "utf8_general_ci";
//        $this->characterSetMedium = "utf8mb4_general_ci";
        $this->characterSetFull = "utf8 COLLATE utf8_general_ci";
//        $this->characterSetFull = "utf8mb4 COLLATE utf8mb4_general_ci";

        $options = array(
          PDO::ATTR_ERRMODE => PDO::ERRMODE_EXCEPTION,
          1002              => "SET NAMES '".$this->characterSet."';"
          // PDO::MYSQL_ATTR_INIT_COMMAND = 1002  // 1002 fixing bug on php 7.2
        );

        if(!$this->pdo) {
          $driver       = $this->sanitizePdoVariable(basic::getConfig("db","main","driver"),true);
          $host         = $this->sanitizePdoVariable(basic::getConfig("db","main","hostname"),true);
          $databaseName = $this->sanitizeTableName(basic::getConfig("db","main","database"));
          $username     = $this->sanitizePdoVariable(basic::getConfig("db","main","username"),true);
          $password     = $this->sanitizePdoVariable(basic::getConfig("db","main","password"),true,"!@#$%^&*()[] ,.<>|-=+");

          try {
            $this->pdo = new PDO($driver.":host=".$host.";dbname=".$databaseName,$username,$password,$options);
          }
          catch(PDOException $e) {
            //   die('Error: '.$e->getMessage().' Code: '.$e->getCode());

            if($e->getCode()=="1049") {   // we have to load without database - that will be created later on: installSystemTables()
              $this->pdo = new PDO($driver.":host=".$host.";",$username,$password,$options);
              $this->charset();

              self::$pdoConnection = true;
            }

            if(self::$pdoConnection==false) {
              $notif = basic::load("notif","core");
              $notif->genNotifByNum($file = "core",$notifNum = 84100,$externalVar = "No mysql configured, No database selected ".$e->getMessage());

              echo "<p><span style=\"color:orange;\">**</span> mysql/pdoHelper is not activated 4! ".$e->getMessage()." <span style=\"color:red;\">**</span></p>";
              //      return false;
            }
          }

          if(!empty($this->pdo)) {
            $this->charset();
          }
        }

        self::$pdoConnection = true;
      }
      catch(PDOException $e) {
        $this->pdoConnectionFailed = true;      // to prevent loop
        $errorControl              = basic::load("errorControl","core");
        $errorControl->sendMailNotificationError($e->getMessage(),$fatal = true,$dontLogOnDb = 1);

        if(myHelpers::isEnv($env = "dev")) {    // the system can function even if no db connection
          echo "error ".$e->getMessage();

          $notif = basic::load("notif","core");
          $notif->genNotifByNum($file = "core",$notifNum = 84100,$externalVar = $e->getMessage());
        }
        else {    // the system can't function if no db connection
          if($_SERVER['REQUEST_URI']!="/".lang::getCurrentLanguageForUrls()."siteIsDown?&error=84100") {
            Header("Location: /".lang::getCurrentLanguageForUrls()."siteIsDown?&error=84100");
            die();
          }
          else {
            echo "<br> Site is currently down.";
          }
        }
      }
    }
  }

  /**
   * Charset settings
   */
  private function charset() {

    $this->pdo->exec("SET CHARACTER SET ".$this->characterSet);
    $this->pdo->setAttribute(PDO::ATTR_ERRMODE,PDO::ERRMODE_EXCEPTION);
    //     $this->pdo->setAttribute(PDO::ATTR_EMULATE_PREPARES, false);
    //    $this->pdo->query('SET NAMES gbk');
    $this->pdo->query("set names ".$this->characterSetFull);
  }

  /**
   * Prepare - if called like: $sel = $this->db->prepare("SELECT * FROM table");
   * @param $sql
   * @return mixed
   */
  protected function prepare($sql) {  // a fix to prevent the need to load the connection even if we are not using mysql

    try {
      if($this->pdoConnectionFailed!==false) {

        self::$queries[] = $sql;

        $this->setFetchModeResult = "";
        $this->fetchResult        = "";
        $this->fetchAllResult     = "";
        $this->queryResult        = "";
        $this->rowCountResult     = "";

        $this->queryString = $sql;
        $this->tracePosition();

        /*				$stack                    = debug_backtrace();
                $this->calledFromFunction = $stack[5]['function'];
                $this->calledFromClass    = $stack[5]['class'];
                $this->calledFromFile     = $stack[4]['file'];
                $this->calledFromLine     = $stack[4]['line'];*/

        $this->prepared = $this->pdo->prepare($sql);

        return $this;
      }
    }
    catch(PDOException $e) {
      echo "<p><span style=\"color:orange;\">**</span> pdoHelper \"prepare\" problem number 1 <span style=\"color:red;\">**</span></p>";
      //	die();
      throw $e;
    }
  }

  // fetch,bindParam,bindColumn,bindValue,rowCount,fetchColumn,fetchAll,fetchObject,errorCode,errorInfo,setAttribute,getAttribute,columnCount,getColumnMeta,setFetchMode,nextRowset,closeCursor,debugDumpParams

  public function setFetchMode($values = array()) {

    try {
      if($this->prepared) {
        if(!$this->setFetchModeResult) {
          $this->setFetchModeResult = $this->prepared->setFetchMode($values);

          return $this->setFetchModeResult;
        }
      }
      else {
        echo "<p><span style=\"color:orange;\">**</span> pdoHelper \"setFetchMode\" problem number 2 <span style=\"color:red;\">**</span></p>";
        die();
      }
    }
    catch(PDOException $e) {
      echo "<p><span style=\"color:orange;\">**</span> pdoHelper \"setFetchMode\" problem number 3 <span style=\"color:red;\">**</span></p>";
      //	die();
      throw $e;
    }

    return $this->setFetchModeResult;
  }

  public function fetch($type = null) {

    try {
      if($this->prepared) {
        if(!$this->fetchResult) {
          $this->fetchResult = $this->prepared->fetch($type);
          $this->queryResult = $this->fetchResult;

          if(!$this->pdo->lastInsertId()) {   // that means we do not use insert/update/delete and we won't have an error with fetchAll

            $this->tracePosition();

            if(!$this->customizedFunction) {    // put only after $this->tracePosition();
              $this->customizedFunction = "userDefined";
            }

            /*						$stack                    = debug_backtrace();
                        $this->calledFromFunction = $stack[4]['function'];
                        $this->calledFromClass    = $stack[4]['class'];
                        $this->calledFromFile     = $stack[3]['file'];
                        $this->calledFromLine     = $stack[3]['line'];

                        if(!$this->customizedFunction) {
                          $stack                    = debug_backtrace();
                          $this->calledFromFunction = $stack[1]['function'];
                          $this->calledFromClass    = $stack[1]['class'];
                          $this->calledFromFile     = $stack[0]['file'];
                          $this->calledFromLine     = $stack[0]['line'];

                          $this->customizedFunction = "userDefined";
                        }*/

            $this->endLog();
          }       // fetchable
          else {
          }       // unfetchable

          return $this->fetchResult;
        }
      }
      else {
        echo "<p><span style=\"color:orange;\">**</span> pdoHelper \"fetch\" problem number 4 <span style=\"color:red;\">**</span></p>";
        die();
      }
    }
    catch(PDOException $e) {
      echo "<p><span style=\"color:orange;\">**</span> pdoHelper \"fetch\" problem number 5 <span style=\"color:red;\">**</span></p>";
      //	die();
      throw $e;
    }

    return $this->fetchResult;
  }

  public function fetchAll($type = null) {

    try {
      if($this->prepared) {
        if(!$this->fetchAllResult) {
          //	$this->fetchAllResult = $this->prepared->fetchAll(PDO::FETCH_ASSOC);

          if($this->setFetchModeResult) {
            $setFetchModeResult = $this->setFetchModeResult;
          }
          else {
            $setFetchModeResult = PDO::FETCH_ASSOC;
          }

          $this->fetchAllResult = $this->prepared->fetchAll($type);
          $this->queryResult    = $this->fetchAllResult;

          if(!$this->pdo->lastInsertId()) {   // that means we do not use insert/update/delete and we won't have an error with fetchAll

            if(!$this->customizedFunction) {
              $this->tracePosition();

              /*							$stack                    = debug_backtrace();
                            $this->calledFromFunction = $stack[1]['function'];
                            $this->calledFromClass    = $stack[1]['class'];
                            $this->calledFromFile     = $stack[0]['file'];
                            $this->calledFromLine     = $stack[0]['line'];*/

              $this->customizedFunction = "userDefined";
            }

            $this->endLog();
          }       // fetchable
          else {
          }       // unfetchable
        }

        return $this->fetchAllResult;
      }
      else {
        echo "<p><span style=\"color:orange;\">**</span> pdoHelper \"fetchAll\" problem number 6 <span style=\"color:red;\">**</span></p>";
        die();
      }
    }
    catch(PDOException $e) {
      echo "<p><span style=\"color:orange;\">**</span> pdoHelper \"fetchAll\" problem number 7 <span style=\"color:red;\">**</span></p>";
      //	die();
      throw $e;
    }

    return $this->fetchAllResult;
  }

  public function rowCount() {

    try {
      if($this->prepared) {
        if(!$this->rowCountResult) {
          $this->rowCountResult = $this->prepared->rowCount();

          return $this->rowCountResult;
        }
      }
      else {
        echo "<p><span style=\"color:orange;\">**</span> pdoHelper \"rowCount\" problem number 8 <span style=\"color:red;\">**</span></p>";
        die();
      }
    }
    catch(PDOException $e) {
      echo "<p><span style=\"color:orange;\">**</span> pdoHelper \"rowCount\" problem number 9 <span style=\"color:red;\">**</span></p>";
      //	die();
      throw $e;
    }

    return $this->rowCountResult;
  }

  public function execute($params = array()) {

    try {
      if($this->prepared) {
        if(!$params) {
          $params = [];
        }
        $this->params = $params;
        $execute      = $this->prepared->execute($params);

        return $execute;
      }
      else {
        echo "<p><span style=\"color:orange;\">**</span> pdoHelper \"execute\" problem number 10 <span style=\"color:red;\">**</span></p>";
        die();
      }
    }
    catch(PDOException $e) {
      $this->tracePosition();
      $this->errorOnQuery = true;

      if($this->isAdvanced) {
        $this->advancedForMysql->rollBack($this);
      }
      else {
        $this->rollBack($e);
      }

      $error = $e->getMessage();

      $this->endLog($error);
      trigger_error($error);
    }
  }

  private function tracePosition() {

    $stack = debug_backtrace();

    $functionNum = "";
    $classNum    = "";
    $fileNum     = "";
    $lineNum     = "";

    if(isset($stack[1])) {
      if($stack[1]['class']=="pdoHelper" && $stack[1]['function']=="execute") {
        $functionNum = 2;
        $classNum    = 2;
        $fileNum     = 1;
        $lineNum     = 1;
      }

      if($stack[1]['class']=="pdoHelper" && $stack[1]['function']=="prepare") {
        $functionNum = 6;
        $classNum    = 6;
        $fileNum     = 5;
        $lineNum     = 5;

        if($stack[2]['function']=="__call") {
          $functionNum = 3;
          $classNum    = 3;
          $fileNum     = 2;
          $lineNum     = 2;
        }

        if($stack[3]['function']!="call_user_func_array") {
          $functionNum = 4;
          $classNum    = 4;
          $fileNum     = 3;
          $lineNum     = 3;
        }
      }

      if($stack[1]['class']=="pdoHelper" && $stack[1]['function']=="fetch") {
        $functionNum = 5;
        $classNum    = 5;
        $fileNum     = 4;
        $lineNum     = 4;

        if(!$this->customizedFunction) {
          $functionNum = 2;
          $classNum    = 2;
          $fileNum     = 1;
          $lineNum     = 1;
        }
      }

      if($stack[1]['class']=="pdoHelper" && $stack[1]['function']=="fetchAll") {
        $functionNum = 2;
        $classNum    = 2;
        $fileNum     = 1;
        $lineNum     = 1;
      }

      if($stack[1]['class']=="pdoHelper" && $stack[1]['function']=="update") {
        $functionNum = 4;
        $classNum    = 4;
        $fileNum     = 4;
        $lineNum     = 4;
      }
    }

    if($functionNum) {
      $this->calledFromFunction = $stack[$functionNum]['function'];
    }

    if($classNum) {

      //	$sss = debug_backtrace();

      /*			if(!isset($stack[$classNum]['class'])) {
              log::log(1314,"hhhhhhhhhhhhhere");
            }

            log::log(1314,$classNum);
            log::log(1314,$stack[1]['class']);
            log::log(1314,$stack[1]['function']);
            log::log(1314,$sss);*/

      $this->calledFromClass = $stack[$classNum]['class'];
    }

    if($fileNum) {
      if(isset($stack[$fileNum]['file'])) {
        $this->calledFromFile = $stack[$fileNum]['file'];
      }
    }

    if($lineNum) {
      if(isset($stack[$fileNum]['line'])) {
        $this->calledFromLine = $stack[$fileNum]['line'];
      }
    }
  }

  /**
   * Pdo - if called like: $this->db->pdo();
   * you can then use the pdo statements as: $this->db->pdo->prepare("SELECT * FROM table");
   * @return mixed
   */
  protected function pdo() {  // a fix to prevent the need to load the connection even if we are not using mysql

    if($this->pdoConnectionFailed!==false) {
      return $this->pdo;
    }
  }

  /**
   * Begin a transaction
   */
  protected function inTransaction() {

    return $this->pdo->inTransaction();
  }

  /**
   * Begin a transaction
   */
  protected function beginTransaction() {

    //   if(!$this->inTransaction()) {
    $this->pdo->setAttribute(PDO::ATTR_AUTOCOMMIT,0);
    $this->pdo->beginTransaction();
    //   }
  }

  /**
   * Commit the transaction
   */
  protected function commit() {

    //  if($this->inTransaction()) {
    $this->pdo->commit();
    $this->pdo->setAttribute(PDO::ATTR_AUTOCOMMIT,1);
    //  }
  }

  /**
   * Rollback the transaction
   */
  protected function rollback($e) {

    if($this->inTransaction()) {
      $this->pdo->rollBack();
      $this->pdo->setAttribute(PDO::ATTR_AUTOCOMMIT,1);

      $this->endLog();
      $this->tracePosition();

      $this->errorOnQuery = true;
      trigger_error($e->getMessage());
    }
  }

  /**
   * Use database
   * @param $databaseName
   */
  public function useDatabase($databaseName) {

    try {
      $sql = "USE ".$databaseName;
      $sel = $this->prepare($sql);
      $sel->execute();
    }
    catch(PDOException $e) {
      print_r("DB ERROR: ".$e->getMessage());
    }
  }

  /**
   * Create database
   * @param $databaseName
   * @return mixed
   * @throws ReflectionException
   */
  protected function createDatabase($databaseName) {

    $databaseName = $this->sanitizeTableName($databaseName);

    try {
      $sql                  = "CREATE DATABASE IF NOT EXISTS ".$databaseName;
      $returnCreateDatabase = $this->prepare($sql);
      $returnCreateDatabase = $returnCreateDatabase->execute();
    }
    catch(PDOException $e) {
      print_r("DB ERROR: ".$e->getMessage());
    }

    $this->endLog();

    return $returnCreateDatabase;
  }

  /**
   * can be UNSAFE function in case of an SQL ATTACK
   * Fetch custom - custom query ,joining multiple table, arithmetic etc
   * @param string $sql custom query
   * @param array $params associative/dynamic array
   * @param string $pdoStringMethod - can be: questionMark/fullString
   * @return array record set
   * @throws ReflectionException
   */
  protected function fetchCustom($sql,$params = null,$pdoStringMethod = "questionMark") {

    $this->customizedFunction = "fetchCustom";
    $params                   = $this->sanitizePdoVariable($params,false,"@#$%^&");
    $this->customSql          = $sql;
    $data                     = [];

    if($params!==null) {
      if($pdoStringMethod=="questionMark") {
        $data = array_values($params);
      }
      else if($pdoStringMethod=="fullString") {
        $data = $params;
      }
    }

    $sel = $this->prepare($sql);

    if($params!==null) {
      $sel->execute($data);
    }
    else {
      $sel->execute();
    }

    $sel->setFetchMode(PDO::FETCH_OBJ);
    $this->queryResult = $this->prepared->fetchAll(PDO::FETCH_ASSOC);

    $this->endLog();

    return $this->queryResult;
  }

  /**
   * fetch fetch column - multiple rows
   * @param string $table table name
   * @param string|array $columns specific column selection
   * @param string $returnMode
   * @return array record set
   * @throws ReflectionException
   */
  protected function fetchColumn($table,$columns,$returnMode = "array") {

    $this->customizedFunction = "fetchColumn";
    $this->getTable($table);
    $this->getColumns($columns);

    $sql = "SELECT ".$this->implodedColumns." FROM ".$this->table;
    $sel = $this->prepare($sql);
    $sel->execute();
    $returnModePdo = "";

    if($returnMode=="array") {
      $returnModePdo = PDO::FETCH_ASSOC;
    }
    if($returnMode=="object") {
      $returnModePdo = PDO::FETCH_OBJ;
    }

    $sel->setFetchMode($returnModePdo);
    $this->queryResult = $sel->prepared->fetchAll($returnModePdo);

    $this->endLog();

    return $this->queryResult;
  }

  /**
   * Check if there is exist data
   * @param string $table - table name
   * @param string|array $where - list of data to find
   * @return boolean - true or false
   * @throws ReflectionException
   */
  protected function checkExist($table,$where) {

    $this->customizedFunction = "checkExist";
    $this->table              = $this->getTable($table);
    $this->where              = $this->getWhere($where,"equivalent");
    $this->columns            = $this->getColumns("*");;

    $sql = "SELECT ".$this->implodedColumns." FROM ".$this->table." WHERE ".$this->whereSqlString;
    $sel = $this->prepare($sql);
    $sel->execute($this->whereValuesEquivalent);
    $sel->setFetchMode(PDO::FETCH_ASSOC);
    $this->numOfRowsEffected = $sel->rowCount();
    //	$result = $this->prepared->fetchAll(PDO::FETCH_ASSOC);
    $this->endLog();

    if($sel->rowCount()>0) {
      $this->queryResult = 1;

      return true;
    }
    else {
      $this->queryResult = 0;

      return false;
    }
  }

  /**
   * Check if table exist
   * @param string $table
   * @param bool|string $database
   * @return bool
   * @throws ReflectionException
   */
  protected function checkIfTableExist($table,$database = false) {

    $this->customizedFunction = "checkIfTableExist";
    $this->table              = $this->getTable($table);

    $database = !empty($database) ? $database : basic::getConfig("db","main","database");
    $database = $this->sanitizeTableName($database);

    $sql = "SELECT count(*) as count FROM information_schema.tables WHERE table_schema = '".$database."' AND table_name = '".$this->table."'";

    $selectCheckIfTableExist = $this->prepare($sql);
    $selectCheckIfTableExist->execute();
    $selectCheckIfTableExist->setFetchMode(PDO::FETCH_OBJ);
    $checkIfTableExist = $selectCheckIfTableExist->fetch();
    $this->queryResult = $checkIfTableExist->count==1 ? true : false;

    $this->endLog();

    return $this->queryResult;
  }

  protected function countTableRows($table,$where = false,$type = "equivalent",$join = "") {

    $this->customizedFunction = "countTableRows";
    $this->getTable($table);
    $this->getWhere($where,$type);
    $this->getJoin($join);
    $params = "";

    if($type=="equivalent") {
      $params = $this->whereValuesEquivalent;
    }
    if($type=="withLike") {
      $params = $this->whereValuesWithLike;
    }

    $sql = "SELECT count(*) as totalRowsOnTable FROM ".$this->table." ".$this->join." ".$this->where;
    $sel = $this->prepare($sql);
    $sel->execute($params);
    $this->fetchResult = $sel->prepared->fetchAll(PDO::FETCH_ASSOC);
    $this->queryResult = $this->fetchResult[0]['totalRowsOnTable'];

    $this->endLog();

    return $this->queryResult;
  }

  /**
   * Find exact data
   * @param string|array $table table name - example: array(xx_items=>i); - means: xx_items as i
   * @param string|array $columns column name
   * @param bool|string|array $where where condition
   * @param bool|array $join
   * @param bool|array $orderBy example: array("columnName"=>"desc");   // can be multiple
   * @param bool|string|array $limit example: array(10=>10); // offset[limit] // or: "string" that will be the limit
   * @param string $type - equivalent/withLike
   * @param string $returnMode
   * @return array|object record set
   * @throws ReflectionException
   * @example: $this->db->find($table = array(basic::getConfig("db","main","tablePrefix")."items" => "i"),$col = array(
   * "i.*,L.priceRangeStartLocal,L.priceRangeEndLocal,L.currencyLocal,L.deliveryTimeLocal,f.features",
   * ),$where = array(
   * "i.leadId"  => $itemId,
   * $keyActive  => $valueActive,
   * "i.deleted" => 0
   * ),
   * $join = array(
   * array(
   * "LEFT JOIN",
   * array(basic::getConfig("db","main","tablePrefix")."items_local_info" => "L"),
   * array("i.leadId" => "L.leadId")
   * ),
   * array(
   * "LEFT JOIN",
   * array(basic::getConfig("db","main","tablePrefix")."items_full_spec" => "f"),
   * array("i.leadId" => "f.leadId")
   * )
   * )
   * );
   */
  protected function find($table,$columns,$where = false,$join = false,$orderBy = false,$limit = false,$type = "equivalent",$returnMode = "array") {    // $columns,$where - can be array also

    $this->customizedFunction = "find";
    $this->getTable($table);
    $this->getColumns($columns);
    $this->getWhere($where,$type);
    $this->getOrderBy($orderBy);
    $this->getLimit($limit);
    $this->getJoin($join);
    $params = "";

    if($type=="equivalent") {
      $params = $this->whereValuesEquivalent;
    }
    if($type=="withLike") {
      $params = $this->whereValuesWithLike;
    }

    $sql = "SELECT ".$this->implodedColumns." FROM ".$this->table." ".$this->join." ".$this->where." ".$this->orderBy." ".$this->limit;
    $sel = $this->prepare($sql);
    $sel->execute($params);
    $returnModePdo = "";

    if($returnMode=="array") {
      $returnModePdo = PDO::FETCH_ASSOC;
    }
    if($returnMode=="object") {
      $returnModePdo = PDO::FETCH_OBJ;
    }

    $this->queryResult = $sel->prepared->fetchAll($returnModePdo);

    $this->endLog();

    return $this->queryResult;
  }

  /**
   * @param string $table table name
   * @param $createData
   * @param bool $isAdvanced
   * @return array|mixed|string
   * @throws ReflectionException
   */
  protected function create($table,$createData,$isAdvanced = false) {

    $this->customizedFunction = "create";
    $this->isAdvanced         = $isAdvanced;

    if($this->isAdvanced) {
      $storeCurrentTableName  = $this->table;      // fix issue that $this->table become the _past table and more problems from notif class
      $this->advancedForMysql = basic::load("advancedForMysql","core");
      $createData             = $this->advancedForMysql->addAdvancedDataToCreateData($createData,$this->characterSetMedium,$this->characterSetFull,$this->engine);
      $this->table            = $storeCurrentTableName;
    }

//    $this->prepareColumnsInto($preData,$data);
    $this->prepareNewColumns($createData);
    $this->getTable($table);
//    $this->beginTransaction();
    $returnCreateTable = "";

    try {
      $sql               = "CREATE TABLE IF NOT EXISTS ".$this->table." (".$this->columnsSet.") ".$this->tableProperties;
      $returnCreateTable = $this->prepare($sql);
      $returnCreateTable = $returnCreateTable->execute();
//      $this->commit();
    }
    catch(PDOException $e) {
      $this->rollback($e);
    }

    //  $this->numOfRowsEffected = $insert->rowCount();
    // $queryString             = $insert->prepared->queryString;

    if($returnCreateTable==true) {
      $returnCreateTable = array(
        "params"  => $this->columnsSet." - ".$this->tableProperties,
        "preData" => $createData
      );
    }

    $this->endLog();

    return $returnCreateTable;
  }

  /**
   * insert data to table
   * @param string $table table name
   * @param array $preData associative array 'column_name'=>'val'
   * @param boolean $isAdvanced - mark if it's part of advanced system
   * @return array - numOfRowsEffected,lastInsertId,queryString
   * @throws ReflectionException
   */
  protected function insert($table,$preData,$isAdvanced = false) {

    $this->customizedFunction = "insert";
    $this->getTable($table);
    $this->isAdvanced = $isAdvanced;
    $insert           = "";

    if($this->isAdvanced) {
      $storeCurrentTableName  = $this->table;      // fix issue that $this->table become the _past table and more problems from notif class
      $this->advancedForMysql = basic::load("advancedForMysql","core");
      $preData                = $this->advancedForMysql->addAdvancedDataToPreData($this,$this->table,$preData);
      $this->table            = $storeCurrentTableName;
    }

    $data = [];
    if($preData) {
      $data = array_values($preData);
    }

    $this->prepareColumnsInto($preData,$data);
    //   $this->beginTransaction();

    try {
      $sql                = "INSERT INTO ".$this->table." (".$this->columnsVariables.") VALUES (".$this->columnsValues.")";
      $insert             = $this->prepare($sql);
      $return             = $insert->execute($data);
      $this->lastInsertId = $this->pdo->lastInsertId();
      //     $this->commit();
    }
    catch(PDOException $e) {
      $this->rollback($e);
    }

    $this->numOfRowsEffected = $insert->rowCount();
    $queryString             = $insert->prepared->queryString;

    if($return==true) {
      $return = array(
        "numOfRowsEffected" => $insert->rowCount(),
        "lastInsertId"      => $this->lastInsertId,
        "queryString"       => $queryString,
        "params"            => $data,
        "preData"           => $preData
      );
    }

    $this->endLog();

    return $return;
  }

  /**
   * update record
   * @param string $table table name
   * @param array $preData associative array 'col'=>'val'
   * @param string|array $where associative array 'col'=>'val'
   * @param boolean $isAdvanced - for advanced.class purposes
   * @return array - numOfRowsEffected,queryString
   * @throws ReflectionException
   */
  protected function update($table,$preData,$where,$isAdvanced = false) {

    $this->customizedFunction = "update";
    $this->getTable($table);
    $this->getWhere($where);
    $this->isAdvanced = $isAdvanced;
    $actionIdLeap     = "";        // just a reset

    if($this->isAdvanced) {
      $storeCurrentTableName  = $this->table;      // fix issue that $this->table become the _past table and more problems from notif class
      $this->advancedForMysql = basic::load("advancedForMysql","core");
      $this->advancedForMysql->getLastActionIdAndPosition($this,$table,$preData);   // the leap in actionId that have to be done
      list($preData,$actionIdLeap) = $this->advancedForMysql->update($this,$this->table,$this->whereFoundLeadId,$preData);
      $this->table = $storeCurrentTableName;
    }

    $data = [];
    if($preData!==null) {
      $data = array_values($preData);
    }

    $this->prepareColumnsSet($preData);
    $this->getWhere($where);  // it is imperative to load that again
    //  $this->beginTransaction();
    $update = "";

    try {
      $sql    = "UPDATE ".$this->table." SET ".$actionIdLeap." ".$this->columnsSqlString." WHERE ".$this->whereSqlString;
      $update = $this->prepare($sql);
      $data   = array_merge($data,$this->whereData);
      $return = $update->execute($data);
      //    $this->commit();
    }
    catch(PDOException $e) {
      $this->rollback($e);

      // have to roll back the _past table save also ...... with advanced $this->lastActionId
      // can't get lastInsertId there cz there is no auto increment

      //	p($this->advancedForMysql->lastActionId,222222);

      //    $this->endLog();
      //    $this->tracePosition();

      /*			$stack = debug_backtrace();
            $this->calledFromFunction = $stack[3]['function'];
            $this->calledFromClass    = $stack[3]['class'];
            $this->calledFromFile     = $stack[3]['file'];
            $this->calledFromLine     = $stack[3]['line'];*/

      //   $this->errorOnQuery = true;
      //   trigger_error($e->getMessage());
    }

    $this->numOfRowsEffected = $update->rowCount();
    $queryString             = $update->prepared->queryString;

    $this->endLog();

    ///////////////////////// VITAL = remove all advanced _past from endLog()
// so this wont be shown:  SELECT count(*) as count FROM information_schema.tables WHERE table_schema = 'limberCode' AND table_name = 't2_items_local_info_past'

    if(!empty($return) && $return==true) {
      return array(
        "return"            => $return,
        "numOfRowsEffected" => $update->rowCount(),
        "queryString"       => $queryString,
        "params"            => $data,
        "preData"           => $preData
      );

      return $return;
    }
  }

  /**
   * Insert or update
   * @param $table
   * @param $preData
   * @param bool $isAdvanced
   * @return array
   * @throws ReflectionException
   */
  protected function insertOrUpdate($table,$preData,$isAdvanced = false) {

    $this->customizedFunction = "insertOrUpdate";
    $this->getTable($table);
    $this->isAdvanced = $isAdvanced;
    $return           = "";

    if($this->isAdvanced) {
      $storeCurrentTableName  = $this->table;      // fix issue that $this->table become the _past table and more problems from notif class
      $this->advancedForMysql = basic::load("advancedForMysql","core");
      $preData                = $this->advancedForMysql->addAdvancedDataToPreData($this,$this->table,$preData);
      $this->table            = $storeCurrentTableName;
    }

    $data = [];
    if($preData!==null) {
      $data = array_values($preData);
    }

    $this->prepareColumnsInto($preData,$data);
    $this->prepareColumnsSet($preData);
    $this->prepareColumnsOnDuplicate($preData);
    //   $this->beginTransaction();
    $insertOrUpdate = "";

    try {
      $sql                = "INSERT INTO ".$this->table." (".$this->columnsVariables.") VALUES (".$this->columnsValues.") ON DUPLICATE KEY UPDATE ".$this->columnsOnDuplicateSqlString;
      $insertOrUpdate     = $this->prepare($sql);      // can be replaced with: link='77' instead of link=VALUES(link)
      $return             = $insertOrUpdate->execute($data);
      $this->lastInsertId = $this->pdo->lastInsertId();
      //    $this->commit();
    }
    catch(PDOException $e) {
      $this->rollback($e);
    }

    $this->numOfRowsEffected = $insertOrUpdate->rowCount();
    $queryString             = $insertOrUpdate->prepared->queryString;

    if($return==true) {
      $return = array(
        "numOfRowsEffected" => $insertOrUpdate->rowCount(),
        "lastInsertId"      => $this->lastInsertId,
        "queryString"       => $queryString,
        "params"            => $data,
        "preData"           => $preData
      );
    }   // get the last inserted id

    $this->endLog();

    return $return;
  }

  /**
   * Add new column
   * @param $table
   * @param $columnName
   * @param $dataType
   * @param $lengthSet
   * @param $addOn
   * @param $after
   * @return array
   * @throws ReflectionException
   */
  protected function addColumn($table,$columnName,$dataType,$lengthSet,$addOn,$after = false) {

    $this->customizedFunction = "addColumn";
    $this->getTable($table);
    $columnName = $this->sanitizePdoVariable($columnName,true);
    $dataType   = $this->sanitizePdoVariable($dataType,true);
    $lengthSet  = $this->sanitizePdoNumber($lengthSet) ? " (".$lengthSet.") " : "";
    $addOn      = $this->sanitizePdoVariable($addOn,""," ");
    $putAfter   = "";

    if(!empty($after)) {
      $after    = $this->sanitizePdoVariable($after,true);
      $putAfter = "AFTER `".$after."`";
    }

    //  $this->beginTransaction();
    $this->columns = array($columnName);

    try {
      $sql       = "ALTER TABLE ".$this->table." ADD COLUMN `".$columnName."` ".$dataType." ".$lengthSet." ".$addOn." ".$putAfter;
      $addColumn = $this->prepare($sql);
      $return    = $addColumn->execute();
      //    $this->commit();
    }
    catch(PDOException $e) {
      $this->rollback($e);
      //   $this->endLog();
      //   $this->tracePosition();

      //   $this->errorOnQuery = true;
      //   trigger_error($e->getMessage());
    }

    $queryString = $addColumn->prepared->queryString;

    $this->endLog();

    if(!empty($return) && $return==true) {
      return array(
        "return"            => $return,
        "numOfRowsEffected" => "",
        "queryString"       => $queryString,
        "params"            => "",
        "preData"           => ""
      );

      return $return;
    }
  }

  /**
   * Delete column
   * @param $table
   * @param $columnName
   * @return array
   * @throws ReflectionException
   */
  protected function deleteColumn($table,$columnName) {

    $this->customizedFunction = "deleteColumn";
    $this->getTable($table);
    $columnName = $this->sanitizePdoVariable($columnName,true);

    //  $this->beginTransaction();
    $this->columns = array($columnName);

    try {
      $sql          = "ALTER TABLE ".$this->table." DROP COLUMN `".$columnName."`";
      $deleteColumn = $this->prepare($sql);
      $return       = $deleteColumn->execute();
      //    $this->commit();
    }
    catch(PDOException $e) {
      $this->rollback($e);
      //   $this->endLog();
      //   $this->tracePosition();

      //   $this->errorOnQuery = true;
      //   trigger_error($e->getMessage());
    }

    $queryString = $deleteColumn->prepared->queryString;

    $this->endLog();

    if(!empty($return) && $return==true) {
      return array(
        "return"            => $return,
        "numOfRowsEffected" => "",
        "queryString"       => $queryString,
        "params"            => "",
        "preData"           => ""
      );

      return $return;
    }
  }

  /**
   * Change column
   * @param $table
   * @param $oldColumnName
   * @param $newColumnName
   * @param $dataType
   * @param $lengthSet
   * @param $addOn
   * @param $after
   * @return array
   * @throws ReflectionException
   */
  protected function changeColumn($table,$oldColumnName,$newColumnName,$dataType,$lengthSet,$addOn,$after = false) {

    $this->customizedFunction = "changeColumn";
    $this->getTable($table);
    $oldColumnName = $this->sanitizePdoVariable($oldColumnName,true);
    $newColumnName = $this->sanitizePdoVariable($newColumnName,true);
    $dataType      = $this->sanitizePdoVariable($dataType,true);
    $lengthSet     = $this->sanitizePdoNumber($lengthSet) ? " (".$lengthSet.") " : "";    // can be number, but can be: "10,4" for float too
    $addOn         = $this->sanitizePdoVariable($addOn,""," ");
    $putAfter      = "";

    if(!empty($after)) {
      $after    = $this->sanitizePdoVariable($after,false);
      $putAfter = "AFTER `".$after."`";
    }

    //  $this->beginTransaction();
    $this->columns = array($oldColumnName,$newColumnName);

    try {
      $sql    = "ALTER TABLE ".$this->table." CHANGE COLUMN `".$oldColumnName."` `".$newColumnName."` ".$dataType.$lengthSet." ".$addOn." ".$putAfter;
      $change = $this->prepare($sql);
      $return = $change->execute();
      //    $this->commit();
    }
    catch(PDOException $e) {
      $this->rollback($e);
      //   $this->endLog();
      //   $this->tracePosition();

      //   $this->errorOnQuery = true;
      //   trigger_error($e->getMessage());
    }

    $queryString = $change->prepared->queryString;

    $this->endLog();

    if(!empty($return) && $return==true) {
      return array(
        "return"            => $return,
        "numOfRowsEffected" => "",
        "queryString"       => $queryString,
        "params"            => "",
        "preData"           => ""
      );

      return $return;
    }
  }

  /**
   * Rename table name
   * @param $table
   * @param $newTableName
   * @return array
   * @throws ReflectionException
   */
  protected function renameTableName($table,$newTableName) {

    $this->customizedFunction = "renameTableName";
    $newTableName             = $this->getTable($newTableName,true);
    $this->getTable($table);

    //  $this->beginTransaction();

    try {
      $sql    = "RENAME TABLE ".$this->table." TO ".$newTableName;
      $update = $this->prepare($sql);
      $return = $update->execute();
      //    $this->commit();
    }
    catch(PDOException $e) {
      $this->rollback($e);
      //   $this->endLog();
      //   $this->tracePosition();

      //   $this->errorOnQuery = true;
      //   trigger_error($e->getMessage());
    }

    $queryString = $update->prepared->queryString;

    $this->endLog();

    if(!empty($return) && $return==true) {
      return array(
        "return"            => $return,
        "numOfRowsEffected" => "",
        "queryString"       => $queryString,
        "params"            => "",
        "preData"           => ""
      );

      return $return;
    }
  }

  /**
   * Retrieve row from past
   * @param $table
   * @param $leadId
   * @param $actionId
   * @return mixed
   * @throws ReflectionException
   */
  protected function retrieveRowFromPast($table,$leadId,$actionId) {

    $this->customizedFunction = "retrieveRowFromPast";
    $this->advancedForMysql   = basic::load("advancedForMysql","core");
    $this->getTable($table);

    $return = $this->advancedForMysql->retrieveRowFromPast($this,$this->table,$leadId,$actionId);

    $this->endLog();

    return $return['return'];
  }

  /**
   * Drop table
   * @param string $table - table name
   * @return array|string
   * @throws ReflectionException
   */
  protected function drop($table) {

    $this->customizedFunction = "drop";
    $this->getTable($table);
    //  $this->beginTransaction();
    $drop   = "";
    $return = "";

    try {
      if($this->table) {
        $sql = "DROP TABLE ".$this->table;

        $drop   = $this->prepare($sql);
        $return = $drop->execute();
        //      $this->commit();
      }
    }
    catch(PDOException $e) {
      $this->rollback($e);
    }

    if($return==true) {
      $queryString = $drop->prepared->queryString;

      $return = array(
        "queryString" => $queryString,
      );
    }

    $this->endLog();

    return $return;
  }

  /**
   * Delete record
   * @param string $table table name
   * @param string $where column name for condition (commonly primary key column name)
   * @param int $id key value
   * @return array|string
   * @throws ReflectionException
   */
  protected function delete($table,$where,$id = "") {

    $this->customizedFunction = "delete";
    $this->getTable($table);

    if(!empty($id)) {
      $id    = $this->sanitizePdoNumber($id);
      $where = array("leadId" => $id);
    }

    $this->getWhere($where,"equivalent");

    /////////// have to store data in _past table - why it's missing?

    //  $this->beginTransaction();
    $delete = "";
    $return = "";

    try {
      if($this->table && $this->where) {
        $sql = "DELETE FROM ".$this->table." WHERE ".$this->whereSqlString;

        $delete = $this->prepare($sql);
        $return = $delete->execute($this->whereValuesEquivalent);
        //      $this->commit();
      }
    }
    catch(PDOException $e) {
      $this->rollback($e);
    }

    if($return==true) {
      $queryString = $delete->prepared->queryString;

      $return = array(
        "numOfRowsEffected" => $delete->rowCount(),
        "queryString"       => $queryString,
        "params"            => $where,
      );
    }

    $this->endLog();

    return $return;
  }

  /**
   * Get Enum values
   * @param string $table - table name
   * @param string $column
   * @return array|string
   * @throws ReflectionException
   */
  protected function getEnumValues($table,$column) {

    $this->customizedFunction = "getEnumValues";
    $this->getTable($table);
    $this->column = $this->getColumns($column);

    /*		$sql = "SELECT COLUMN_TYPE
            FROM information_schema.COLUMNS
            WHERE TABLE_NAME = '".$this->table."'
            AND COLUMN_NAME = '".$this->column."'";*/

    $sql = "SHOW COLUMNS FROM {$this->table} WHERE Field = '{$this->column}'";

    $sel = $this->prepare($sql);
    $sel->execute();
    $result = $sel->fetch(PDO::FETCH_ASSOC);

    $enum = "";
    if(isset($result['Type'])) {
      preg_match('/^enum\(\'(.*)\'\)$/',$result['Type'],$matches);
      if(!empty($matches[1])) {
        $enum = explode("','",$matches[1]);
      }
    }

    $this->endLog();

    return $enum;
  }

  /**
   * @param $table
   * @param bool $returnOnly - not setting the $this->table
   * @return bool|string|string[]|null
   */
  private function getTable($table,$returnOnly = false) {

    if(is_array($table)) {
      reset($table);
      $tableName = key($table);
      $as        = $table[$tableName];

      $table       = $this->sanitizeTableName($tableName,true);
      $as          = $this->sanitizeTableName($as,true);
      $returnTable = $table." as ".$as;
    }
    else {
      $returnTable = $this->sanitizeTableName($table,true);
    }

    if($returnOnly!==false) {
      return $returnTable;
    }
    else {
      $this->table = $returnTable;

      return $this->table;
    }
  }

  /**
   * @param $columns
   * @return array|string|string[]|null
   * @throws ReflectionException
   */
  private function getColumns($columns) {

    $this->columns = $this->sanitizePdoVariable($columns,true,array(
      "*",
      "(",
      ")",
      " ",
      ",",
    ));

    $this->implodeColumns();

    return $this->columns;
  }

  /**
   * @param $where
   * @param string $type - equivalent/withLike
   * @return array|string|string[]|null
   * @throws ReflectionException
   */
  private function getWhere($where,$type = "equivalent") {

    $this->where = $where;
    $this->getDataOutOfWhere();
    $this->implodeWhere($type);

    if($this->whereSqlString) {
      $this->where = "WHERE ".$this->whereSqlString;
    }

    return $this->where;
  }

  /**
   * @param $orderBy
   * @return array|string|string[]|null
   * @throws ReflectionException
   */
  private function getOrderBy($orderBy) {

    if(!empty($orderBy)) {
      if(is_array($orderBy)) {
        $this->orderBy = $this->sanitizePdoVariable($orderBy,true);
        $orderByOutput = "";

        foreach($this->orderBy as $key => $value) {
          if(strtolower($value)=="desc" || strtolower($value)=="asc") {
            $orderByOutput .= $key." ".$value.", ";
          }
        }

        $this->orderBy = "ORDER BY ".trim($orderByOutput,", ");
      }
      else {
        $orderBy          = explode(" ",$orderBy);
        $orderByColumn    = !empty($orderBy[0]) ? $this->sanitizePdoVariable($orderBy[0],true) : "";
        $orderByDirection = !empty($orderBy[1]) ? $this->sanitizePdoVariable($orderBy[1],true) : "";

        if(!empty($orderByColumn) && (strtolower($orderByDirection)=="desc" || strtolower($orderByDirection)=="asc")) {
          $this->orderBy = "ORDER BY ".trim($orderByColumn)." ".trim($orderByDirection);
        }
      }
    }
    else {
      $this->orderBy = "";    // converts from array to string
    }

    return $this->orderBy;
  }

  /**
   * @param $limit
   * @return array|string|string[]|null
   * @throws ReflectionException
   */
  private function getLimit($limit) {

    $this->limit = $this->sanitizePdoVariable($limit,true);

    if(!empty($this->limit)) {
      $limit = "";

      if(is_array($this->limit)) {
        reset($this->limit);
        $offset               = !empty(key($this->limit)) ? key($this->limit) : "0";
        $offset               = $this->sanitizePdoNumber($offset);
        $this->limit[$offset] = $this->sanitizePdoNumber($this->limit[$offset]);

        if((!empty($offset) || $offset==="0") && !empty($this->limit[$offset])) {
          $limit = "LIMIT ".$offset.", ".$this->limit[$offset];
        }
      }
      else {
        $this->limit = $this->sanitizePdoNumber($this->limit);

        if($this->limit) {
          $limit = "LIMIT $this->limit";
        }
      }

      $this->limit = $limit;
    }

    return $this->limit;
  }

  /**
   * @param $join
   * @return string
   * @throws ReflectionException
   */
  private function getJoin($join) {

    basic::load("arrayManipulation","functions");
    $joinDepth = arrayManipulation::arrayDepth($join,2);

    $joinAddOn = "";
    if($joinDepth==1) {
      if(!empty($join) && is_array($join) && !empty($join[0]) && !empty($join[1]) && !empty($join[2])) {
        $joinAddOn .= $this->runOnJoin($join[0],$join[1],$join[2]);
      }
    }
    if($joinDepth==2) {
      if(!empty($join) && is_array($join)) {
        foreach($join as $key => $value) {
          if(!empty($value[0]) && !empty($value[1]) && !empty($value[2])) {
            $joinAddOn .= $this->runOnJoin($value[0],$value[1],$value[2]);
          }
        }
      }
    }

    $this->join = $joinAddOn;

    return $this->join;
  }

  private function runOnJoin($joinType,$joinTable,$joinOn) {

    $joinTypeReturn  = $this->sanitizePdoVariable($joinType,true,array(" "));
    $joinTableReturn = $this->getTable($joinTable,true);
    $joinOnReturn    = "";

    if(!empty($joinOn) && is_array($joinOn)) {
      reset($joinOn);
      $joinFirst  = $this->sanitizePdoVariable(key($joinOn));
      $joinSecond = $this->sanitizePdoVariable($joinOn[$joinFirst]);

      $joinOnReturn = "ON ".$joinFirst."=".$joinSecond;
    }

    $return = $joinTypeReturn." ".$joinTableReturn." ".$joinOnReturn." ";

    return $return;
  }

  private function prepareNewColumns($createData) {

    $this->columnsSet = "";

    if(!empty($createData['columns']) && is_array($createData['columns'])) {
      foreach($createData['columns'] as $key => $value) {
        $columnType = "";
        if(!empty($value[0])) {
          $columnType = $value[0];
        }
        if(!empty($value['dataType'])) {
          $columnType = $value['dataType'];
        }

        $columnLengthSet = "";
        if(!empty($value[1])) {
          $columnLengthSet = $value[1];
        }
        if(!empty($value['lengthSet'])) {
          $columnLengthSet = $value['lengthSet'];
        }
        $columnLengthSet = !empty($columnLengthSet) ? " (".strtoupper($columnLengthSet).") " : "";

        $columnAddOn = "";
        if(!empty($value[2])) {
          $columnAddOn = $value[2];
        }
        if(!empty($value['addOn'])) {
          $columnAddOn = $value['addOn'];
        }
        $columnAddOn = !empty($columnAddOn) ? $columnAddOn : "";

        if($columnType) {
          $this->columnsSet .= " `".$key."` ".$columnType." ".$columnLengthSet." ".$columnAddOn.", ";
        }
        else {
          return "error creating table 11223";
        }
      }
    }

    if(!empty($createData['indexes'])) {
      if(is_array($createData['indexes'])) {
        foreach($createData['indexes'] as $key2 => $value2) {
          $this->columnsSet .= " ".$createData['indexes'][$key2].", ";
        }
      }
      else {
        $this->columnsSet .= " ".$createData['indexes'].", ";
      }
    }

    $this->columnsSet = substr($this->columnsSet,0,-2);   // remove last comma ","

    if(!empty($createData['characterSet'])) {
      $this->characterSet = $createData['characterSet'];
    }

    if(!empty($createData['engine'])) {
      $this->engine = $createData['engine'];
    }

    $this->tableProperties = " CHARACTER SET ".$this->characterSet;
    $this->tableProperties .= " ENGINE=".$this->engine;

    if(!empty($createData['autoIncrement'])) {
      $this->tableProperties .= " AUTO_INCREMENT=".$createData['autoIncrement'];
    }
  }

  /**
   * Implode columns
   * @return string
   */
  private function implodeColumns() {

    if(is_array($this->columns)) {        // example for $columns: array("codeType","email")
      $this->implodedColumns = implode(', ',$this->columns);
    }
    else {                          // example for $columns: "*"
      $this->implodedColumns = $this->columns;
    }

    return $this->implodedColumns;
  }

  /**
   * Implode Where
   * @param string $type - equivalent/withLike
   * @return mixed
   */
  private function implodeWhere($type = "equivalent") {

    $rule      = "";
    $whereMark = "";

    if($type=="equivalent") {
      $whereMark = $this->whereMarkEquivalent;
      $rule      = " AND ";
    }
    if($type=="withLike") {
      $whereMark = $this->whereMarkWithLike;
      $rule      = " OR ";
    }

    if(is_array($this->where)) {
      $jum = count($this->where);
    }
    else {
      $jum = 0;
    }

    if($jum>1) {
      $this->whereSqlString = implode(" ".$rule." ",$whereMark);
    }
    else {
      $this->whereSqlString = implode("",$whereMark);
    }

    return $this->implodedColumns;
  }

  /**
   * Get data out of where
   * @return array
   * @throws ReflectionException
   */
  private function getDataOutOfWhere() {

    $this->whereData             = [];
    $this->whereColumns          = [];
    $this->whereValues           = [];
    $this->whereValuesEquivalent = [];
    $this->whereValuesWithLike   = [];
    $this->whereMarkEquivalent   = [];
    $this->whereMarkWithLike     = [];
    $this->whereFoundLeadId      = "";

    if(is_array($this->where)) {      // example for $this->where: array("detectedIp"=>11,"modifiedAt"=>222)
      $this->where     = $this->sanitizePdoVariable($this->where,true,array("@#-"));
      $this->whereData = array_values($this->where);

      foreach($this->whereData as $key) {
        $this->whereValues[] = $key;
      }

      $this->whereColumns = array_keys($this->where);
    }
    else {                      // example for $this->where: "modifiedAt=yoyo"
      if(isset($this->whereColumns[0])) {
        $whereParts  = explode("=",$this->where);
        $whereColumn = !empty($whereParts[0]) ? $this->sanitizePdoVariable($whereParts[0],true,array("@#-")) : "";
        $whereValue  = !empty($whereParts[1]) ? $this->sanitizePdoVariable($whereParts[1],true,array("@#-")) : "";

        if(!empty($whereColumn) && !empty($whereValue)) {
          $this->whereColumns[0] = trim($whereColumn);
          $this->whereValues[0]  = trim($whereValue);
          $this->whereData       = array($this->whereValues[0]);
        }
      }
    }

    $tableUsingAs = "";
    if(strpos(strtolower($this->table),' as ')!==false) {
      $tableUsingAs = true;
    }

    /*    $valueHaveDot = "";

        if(strpos($value,'.')!==false) {
          $valueHaveDot = true;
        }
        $valueWithoutDot = preg_replace('/.*?\./',"",$value);*/

    /*    $arr = array(1,2,3,4);
        $isIndexed = array_values($this->where) === $this->where;

        p($this->where);
        p($isIndexed);*/

//    $whereIsSequential = "";
//    basic::load("arrayManipulation","functions");
//    if(arrayManipulation::arrayType($this->where)) {
//      echo"ffffffffffffffff";
//      $whereIsSequential = true;
//    }

    foreach($this->whereColumns as $key => $value) {
      if($this->whereColumns[$key]=="leadId") {
        $this->whereFoundLeadId = $this->whereValues[$key];
      }

      if($tableUsingAs || !empty($this->columnsSqlString)) {   // we can't use "dot" in Pdo prepared statement so we will use "?" - example: JOIN ... WHERE i.leadId = ?
        $keys                          = $value."=?";
        $keysWithLike                  = $value." LIKE ?";
        $this->whereValuesEquivalent[] = $this->whereValues[$key];
        $this->whereValuesWithLike[]   = '%'.$this->whereValues[$key].'%';
      }
      else {
        $keys                                = $value." = :".$value;
        $keysWithLike                        = $value." LIKE :".$value;
        $this->whereValuesEquivalent[$value] = $this->whereValues[$key];
        $this->whereValuesWithLike[$value]   = '%'.$this->whereValues[$key].'%';
      }

      $this->whereMarkEquivalent[] = $keys;
      $this->whereMarkWithLike[]   = $keysWithLike;
    }

    return array(
      $this->whereData,
      $this->whereColumns,
      $this->whereValues,
      $this->whereValuesEquivalent,
      $this->whereValuesWithLike,
      $this->whereMarkEquivalent,
      $this->whereMarkWithLike,
      $this->whereFoundLeadId
    );
  }

  /**
   * @param array $data
   * @return array
   * @throws ReflectionException
   */
  private function prepareColumnsAndValues($data) {

    $data                  = $this->sanitizePdoVariable($data,true);
    $this->implodedColumns = implode(", ",array_keys($data));
    $this->implodedValues  = "'".implode("', '",$data)."'";

    return array($this->implodedColumns,$this->implodedValues);
  }

  /**
   * Prepare columns set
   * @param $preData
   * @return string
   * $columnsSqlString - will output something like this: leadId=?, actionId=?, level1Id=?, level2Id=?, level3Id=? ...
   */
  private function prepareColumnsSet($preData) {

    $columnsSet = array_keys($preData);
    $mark       = array();
    foreach($columnsSet as $column) {
      $mark[] = $column."=?";
    }
    $this->columnsSqlString = implode(', ',$mark);

    return $this->columnsSqlString;
  }

  /**
   * @param $preData
   * @return string
   * $columnsOnDuplicateSqlString - will output something like this: leadId=VALUES(leadId), actionId=VALUES(actionId), level1Id=VALUES(level1Id), level2Id=VALUES(level2Id) ...
   */
  private function prepareColumnsOnDuplicate($preData) {

    $columnsSet = array_keys($preData);
    $mark       = array();
    foreach($columnsSet as $column) {
      $mark[] = $column."=VALUES(".$column.")";
    }
    $this->columnsOnDuplicateSqlString = implode(', ',$mark);

    return $this->columnsOnDuplicateSqlString;
  }

  /**
   * Prepare columns into
   * @param $preData
   * @param $data
   * @return array
   * $columnsVariables - will output something like this: leadId, actionId, level1Id, level2Id, level3Id ...
   * $columnsValues - will output something like this: ?, ?, ?, ?, ? ...
   */
  private function prepareColumnsInto($preData,$data) {

    // grab keys
    $this->columns          = array_keys($preData);
    $this->columnsVariables = implode(', ',$this->columns);

    // grab values and change it's value
    $mark = array();
    foreach($data as $key) {
      $keys   = '?';
      $mark[] = $keys;
    }

    /*		foreach($data as $key => $value) {
          if(empty($data[$key])) {
            $data[$key] = 0;
          }
        }       // can fix issue when PDO get stacked */

    $this->columnsValues = implode(', ',$mark);

    return array($this->columnsVariables,$this->columnsValues);
  }

  /**
   * Destruct
   */
  public function __destruct() {

    $this->pdo = null;
  }

  public function startLog() {

    $this->startTime = microtime(true);
  }

  /**
   * End log
   * @param string $error if any
   * @throws ReflectionException
   */
  protected function endLog($error = "") {

    //	$query = (array)$this->prepared;
    if($this->customizedFunction=="create" || $this->customizedFunction=="insert" || $this->customizedFunction=="insertOrUpdate" || $this->customizedFunction=="update" || $this->customizedFunction=="delete") {
      $result = array();
    }
    else {
      //	$result = $this->prepared->fetchAll(PDO::FETCH_ASSOC);
    }

    $info['prepareInfo']['customizedFunction'] = $this->customizedFunction;

    if($this->customizedFunction!="userDefined") {
      $info['prepareInfo']['table']   = $this->table;
      $info['prepareInfo']['columns'] = $this->columns;
      $info['prepareInfo']['where']   = $this->where;
    }

    $info['prepareInfo']['params']             = $this->params;
    $info['prepareInfo']['queryString']        = $this->queryString;
    $info['executedData']['numOfRowsEffected'] = $this->numOfRowsEffected;
    $info['executedData']['lastInsertId']      = $this->lastInsertId;

    if($this->queryResult) {
      $info['executedData']['return'] = $this->queryResult;
    }
    else {
      //	$info['executedData']['return'] = $result;
      $info['executedData']['return'] = "";
    }

    $info['locationInfo']['calledFromFunction'] = $this->calledFromFunction;
    $info['locationInfo']['calledFromClass']    = $this->calledFromClass;
    $info['locationInfo']['calledFromFile']     = $this->calledFromFile;
    $info['locationInfo']['calledFromLine']     = $this->calledFromLine;

    if($this->customSql) {
      $data['customSql'] = $this->customSql;
    }

    if($this->errorOnQuery) {
      $info['executedData']['error'] = $error;
    }

    $this->where              = "";
    $this->customSql          = "";
    $this->columnsVariables   = "";
    $this->customizedFunction = "";
    $this->numOfRowsEffected  = "";
    $this->lastInsertId       = "";

    $timing      = microtime(true) - $this->startTime;
    $dbInformant = Basic::load("dbInformant","core");

    if(isset(basic::$cfg['errorControl']['dbQueries']['logMethod']) && (in_array("file",basic::$cfg['errorControl']['dbQueries']['logMethod']) || in_array("fileJson",basic::$cfg['errorControl']['dbQueries']['logMethod']))) {

      Log::logToFile($fileName = basic::$cfg['errorControl']['dbQueries']['logLocation'],$data = $this->queryString.", params:".print_r($this->params,true));
    }

    if(isset($_GET['route']) || basic::$cfg['errorControl']['dbUi']['dbUiOnPost']==1 || (basic::$cfg['errorControl']['dbUi']['dbUiOnPost']==0 && !router::isPost()) || myHelpers::isAjax()) {       // if we doing redirect we don't need to show this dbUi - it will stuck if so
      $dbInformant->showDbUi($this->queryString,$this->params,round($timing * 100,3),$info);
    }

    $this->queryString  = "";
    $this->params       = "";
    $this->errorOnQuery = "";
  }

  /**
   * Sanitize table name - usually used for table/database name - can't start with number or underscore
   * @param $value
   * @return string|string[]|null
   */
  private function sanitizeTableName($value) {

    if(!preg_match('/^[a-zA-Z].*/',$value)) {
      return "(: disallowedTableName :)";
    }

    $returnValue = preg_replace('/[^a-zA-Z0-9_]*/','',$value);

    return $returnValue;
  }

  /**
   * @param $value
   * @param bool $disAllowedWords
   * @param bool $allowedChars
   * @return string|string[]|null
   * @throws ReflectionException
   */
  private function sanitizeVariable($value,$disAllowedWords = false,$allowedChars = false) {

    $value = htmlspecialchars($value);    // can avoid bit 0x0101 stuff here
    $value = stripslashes($value);
    $value = trim($value);

    if(is_array($allowedChars)) {
      $extraAllowedChars = implode("",$allowedChars);
    }
    else {
      $extraAllowedChars = $allowedChars;
    }

    if($disAllowedWords==true) {
      $disAllowedWords = array(
        "select ",
        "join ",
        "delete ",    // " " is not by mistake - we have "deleted" column on advance system
        "show ",
        "drop ",
        "describe ",
        "table ",
        "insert ",
        "update ",
        "create ",    // " " is not by mistake - we have "createdByUserId" column on advance system
        "into ",
        "join ",
        "union ",
      );  // for example: count(*) as totalRowsOnTable
    }

    if(is_array($disAllowedWords)) {
      basic::load("stringManipulation","functions");

      foreach($disAllowedWords as $key => $disAllow) {
        if(stringManipulation::stringContains($value,$disAllowedWords[$key])) {
          return "(: disallowedWord :)";
        }
      }
    }

    $returnValue = preg_replace('/[^a-zA-Z0-9_.'.$extraAllowedChars.']*/','',$value);

    return $returnValue;
  }

  /**
   * @param $data
   * @param bool $disAllowedWords
   * @param bool $allowedChars
   * @return array|string|string[]|null
   * @throws ReflectionException
   */
  private function sanitizePdoVariable($data,$disAllowedWords = false,$allowedChars = false) {

    if(is_array($data)) {
      $returnValue = [];

      foreach($data as $key => $value) {
        $returnValue[$key] = $this->sanitizeVariable($value,$disAllowedWords,$allowedChars);
      }
    }
    else {
      $returnValue = $this->sanitizeVariable($data,$disAllowedWords,$allowedChars);
    }

    return $returnValue;
  }

  /**
   * @param $data
   * @return int|string
   */
  private function sanitizePdoNumber($data) {

    $returnValue = "(: disallowedInteger :)";
    $data        = htmlspecialchars($data);    // can avoid bit 0x0101 stuff here
    $data        = stripslashes($data);
    $data        = trim($data);

    $data = str_replace("x","",strtolower($data));
    $data = str_replace("-","",$data);

    if(ctype_digit($data) || is_int($data)) {
      $returnValue = $data;
    }

    if(empty($data)) {
      return $data;
    }

    return $returnValue;
  }
}

