<?php

/*************************************************************
 * Advanced system class for files
 * - configure the db handling system
 * @author    S M Kessner <limbercode@gmail.com>
 * @copyright 2010-2018 T2
 * @link      http://limbercode.com
 *************************************************************/
class advancedForFiles {

  private $advancedTempDirectory;
  private $advancedDeletedDirectory;
  private $advancedDataDirectory;
  private $advancedArchiveDataDirectory;
  public $advancedsystemDirectory;
  private $notif;

  /**
   * Constructor for advanced
   */
  function __construct() {

    $this->advancedTempDirectory        = SOURCEPATH."data".DS."advancedBlueprints".DS."temp";        // maybe: need to change to impermanent
    $this->advancedDeletedDirectory     = SOURCEPATH."data".DS."advancedBlueprints".DS."deleted";     // maybe: need to change to impermanent
    $this->advancedDataDirectory        = SOURCEPATH."data".DS."advancedBlueprints";
    $this->advancedArchiveDataDirectory = SOURCEPATH."data".DS."advancedBlueprints".DS."archive";
    $this->advancedsystemDirectory      = SOURCEPATH."data".DS."advancedBlueprints".DS."system";
    $this->notif                        = basic::load("notif","core");
    $this->notif->loadNotifs($file = "advanced");

    basic::loadCfg("advancedConfig");
  }

  /**
   * Get advanced data archived blueprints
   * @return mixed
   */
  public function getBlueprintsFromDir($dir = "") {

    $pullDir = "";
    if($dir=="active" || $dir=="") {
      $pullDir = $this->advancedDataDirectory;
    }
    if($dir=="archived" || $dir=="archive") {
      $pullDir = $this->advancedArchiveDataDirectory;
    }
    if($dir=="tmp" || $dir=="temp") {
      $pullDir = $this->advancedTempDirectory;
    }
    if($dir=="delete" || $dir=="deleted") {
      $pullDir = $this->advancedDeletedDirectory;
    }
    if($dir=="system" || $dir=="system") {
      $pullDir = $this->advancedsystemDirectory;
    }

    if(!empty($pullDir)) {
      return myHelpers::getFilesInDirectory($pullDir);
    }
  }

  /**
   * Get Blueprints data
   * @param string $type
   * @return array
   */
  public function getBlueprintsData($type = "") {

    $advancedFiles = $this->getBlueprintsFromDir($type);
    $advancedData  = [];

    if(!empty($advancedFiles)) {
      $n = 0;

      foreach($advancedFiles as $file) {
        if(!empty($file['extension']) && $file['extension']=="json") {
          if(!empty($file['basename']) && !empty($file['filename']) && !empty($file['size'])) {
            $n++;
            if($type=="active") {
              $advancedData[$n] = $this->loadBlueprint($file['basename']);
            }
            if($type=="archive") {
              $advancedData[$n] = $this->loadArchivedBlueprint($file['basename']);
            }

            $blueprintFilename         = $file['filename'];
            $validateBlueprintFileName = $this->validateBlueprintFileName($blueprintFilename,$type);

            if($validateBlueprintFileName!==true) {
              $wrongNaming = $advancedData[$n]['blueprintName'];
              unset($advancedData[$n]);
              $this->notif->genNotifByNum("advanced",9110,array($blueprintFilename,$wrongNaming));
            }
            else {
              $validateBlueprintName = $this->validateBlueprintName($advancedData[$n]['blueprintName'],$blueprintFilename,$type);

              if($validateBlueprintName!==true) {
                $wrongNaming = $advancedData[$n]['blueprintName'];
                unset($advancedData[$n]);
                $this->notif->genNotifByNum("advanced",9110,array($blueprintFilename,$wrongNaming));
              }
              else {
                $fixNaming = $this->fileNameAccordingToBlueprint($advancedData[$n]['blueprintName'],$blueprintFilename,$type);

                if($fixNaming!==true) {
                  $this->notif->genNotifByNum("advanced",9111,array(
                    $blueprintFilename,
                    $advancedData[$n]['blueprintName']
                  ));
                }

                $advancedData[$n]['size'] = $file['size'];
                //    $advancedData[$n]['createdDateTime'] = date("d-m-Y H:i:s",$file['modified']);
                $advancedData[$n]['createdDateTime'] = $file['modified'];    // for sorting: arraySortByColumn
              }
            }
          }
        }
      }

      return $advancedData;
    }
  }

  /**
   * Get Blueprints name
   * @param string $type
   * @return array
   */
  public function getBlueprintsName($type = "") {

    $advancedFiles = $this->getBlueprintsFromDir($type);
    $advancedNames = [];
    $n             = 0;

    if(!empty($advancedFiles)) {
      foreach($advancedFiles as $file) {
        if(!empty($file['extension']) && $file['extension']=="json") {
          if(!empty($file['basename']) && !empty($file['filename']) && !empty($file['size'])) {
            $n++;
            $advancedNames[$n] = $file['filename'];
          }
        }
      }

      return $advancedNames;
    }
  }

  /**
   * Store advanced data blueprint
   * @param $file
   * @param $data
   * @return mixed
   */
  public function storeBlueprint($file,$data) {

    $stored = myHelpers::storeJsonFile($file,$lastDir = $this->advancedDataDirectory,$data);

    if($stored) {
      return $stored;
    }

    $this->notif->genNotifByNum("advanced",9202,$lastDir.DS.$file.".json");
  }

  /**
   * Archive blueprint
   * @param $file - the filename
   * @param $action - archive/unArchive
   * @return bool|string
   */
  public function archiveBlueprint($file,$action) {

    if($action=="archive") {
      $dirAndFile = myHelpers::fixDirAndFile($file,$ext = "json",$lastDir = $this->advancedDataDirectory);
    }
    else {
      $dirAndFile = myHelpers::fixDirAndFile($file,$ext = "json",$lastDir = $this->advancedArchiveDataDirectory);
    }

    $dir    = explode(DS,$dirAndFile);
    $sliced = array_slice($dir,0,-1);
    $dir    = implode(DS,$sliced);        // take the dir without the file name

    if($action=="unArchive") {
      $dir = $this->advancedDataDirectory;
    }
    else {
      $dir = $this->advancedArchiveDataDirectory;

      if(!is_dir($dir)) {
        mkdir($dir);
      }
    }

    /*    if(file_exists($old_name)
    &&
            ((!file_exists($new_name)) || is_writable($new_name))) {
          rename($old_name, $new_name);
        }*/

    if(file_exists($dir.DS.$file.".".$ext)) {
      $this->notif->genNotifByNum("advanced",9008,$dir.DS.$file.".".$ext);
    }

    if(!file_exists($dirAndFile) || !rename($dirAndFile,$dir.DS.$file.".".$ext)) {
      $this->notif->genNotifByNum("advanced",9007,$dirAndFile);
    }

    return true;
  }

  /**
   * Export blueprint
   * @param $file - the filename
   * @param $from - archive/main
   * @return string
   */
  public function exportBlueprint($file,$from) {

    if($from=="archive") {
      $lastDir = $this->advancedArchiveDataDirectory;
    }
    else {
      $lastDir = $this->advancedDataDirectory;
    }

    if(is_array($file)) {
      $exportedParts    = array();
      $exportedFileName = "";

      foreach($file as $key => $value) {
        $exportedFileName = myHelpers::fixDirAndFile($value,$ext = "json",$lastDir);

        if(file_exists($exportedFileName)) {
          $fileContents  = file_get_contents($exportedFileName);
          $decodedJson   = json_decode($fileContents,true);
          $exportedParts = array_merge($exportedParts,$decodedJson);
        }
        else {
          $this->notif->genNotifByNum("advanced",9211,$exportedFileName);

          return false;
        }
      }

      if(!empty($exportedParts)) {
        $filename = "exported-".time();
        $dir      = $this->advancedTempDirectory;

        if(!is_dir($dir)) {
          mkdir($dir);
        }

        $combinedJson = myHelpers::storeJsonFile($filename,$lastDir = $dir,$exportedParts);
        $exportedFile = $dir.DS.$filename.".json";

        if($combinedJson) {
          if(file_exists($exportedFile)) {
            header("Pragma: public",true);
            header("Expires: 0"); // set expiration time
            header("Cache-Control: must-revalidate, post-check=0, pre-check=0");
            header("Content-Type: application/force-download");
            header("Content-Type: application/octet-stream");
            header("Content-Type: application/download");
            header("Content-Disposition: attachment; filename=".basename($exportedFile));
            header("Content-Transfer-Encoding: binary");
            header("Content-Length: ".filesize($exportedFile));
            die(file_get_contents($exportedFile));
          }
          else {
            $this->notif->genNotifByNum("advanced",9212,$exportedFile);

            return false;
          }
        }
      }
      else {
        $this->notif->genNotifByNum("advanced",9213,$exportedFileName);

        return false;
      }
    }
    else {
      $exportedFile = myHelpers::fixDirAndFile($file,$ext = "json",$lastDir);

      if(file_exists($exportedFile)) {
        header("Pragma: public",true);
        header("Expires: 0"); // set expiration time
        header("Cache-Control: must-revalidate, post-check=0, pre-check=0");
        header("Content-Type: application/force-download");
        header("Content-Type: application/octet-stream");
        header("Content-Type: application/download");
        header("Content-Disposition: attachment; filename=".basename($exportedFile));
        header("Content-Transfer-Encoding: binary");
        header("Content-Length: ".filesize($exportedFile));
        die(file_get_contents($exportedFile));
      }
      else {
        $this->notif->genNotifByNum("advanced",9214,$exportedFile);

        return false;
      }
    }
  }

  public function importingBlueprint($fileParams) {

    $type         = $fileParams['importingFile']['type'];
    $fileTempName = $fileParams['importingFile']['tmp_name'];
    $decodedJson  = "";
    $imported     = "";

    if($type!="application/json") {
      return "This is not json file!";
    }

    if(file_exists($fileTempName)) {
      $fileContents = file_get_contents($fileTempName);
      $decodedJson  = json_decode($fileContents,true);

      if(!empty($decodedJson)) {
        foreach($decodedJson as $key => $value) {
          $blueprintName = $value['blueprintName'];

          if(!empty($blueprintName)) {
            $imported = myHelpers::storeJsonFile($blueprintName,$lastDir = $this->advancedDataDirectory,[$value]);
          }
        }
      }
      else {
        return "Failed importing json file! structure is wrong.";
      }
    }

    if($imported) {
      return $decodedJson;
    }
    else {
      return "Failed importing json file!";
    }
  }

  public function permanentDeleteBlueprint($file) {

    $originalFile = $file;
    $file         = myHelpers::fixDirAndFile($file,$ext = "json",$lastDir = $this->advancedArchiveDataDirectory);
    $dir          = $this->advancedDeletedDirectory;
    $fileMoveTo   = myHelpers::fixDirAndFile($originalFile,$ext = "json",$lastDir = $this->advancedDeletedDirectory);

    if(!is_dir($dir)) {
      mkdir($dir);
    }

    if(file_exists($file)) {
      if(rename($file,$fileMoveTo)) {
        return true;
      }
    }

    return false;
  }

  /**
   * Check if Blueprint exist
   * @param $file
   * @return mixed
   */
  public function checkIfBlueprintExist($file) {

//    $isFileExist = myHelpers::checkIfFileExistCaseInsensitive($file,$lastDir = $this->advancedDataDirectory);

//    if($isFileExist) {
    $jsonDecoded = myHelpers::loadJsonFile($file,$lastDir = $this->advancedDataDirectory);

    if($jsonDecoded) {
      return true;
    }

//    }

    return false;
  }

  /**
   * @param $file
   * @param $newFileName
   * @return bool
   */
  public function renameBlueprintName($file,$newFileName) {

    $jsonDecoded = myHelpers::loadJsonFile($file,$lastDir = $this->advancedDataDirectory);

    if($jsonDecoded) {
      $jsonDecoded['blueprintName'] = $newFileName;

      $stored = myHelpers::storeJsonFile($file,$lastDir = $this->advancedDataDirectory,[$jsonDecoded]);

      if($stored) {
        $path = $this->advancedDataDirectory;
        rename($path.DS.$file.".json",$path.DS.$newFileName.".json");

        return $stored;
      }
    }

    return false;
  }

  /**
   * Load advanced data Blueprint
   * @param $blueprintName
   * @return mixed
   */
  public function loadBlueprint($blueprintName) {

    $jsonDecoded = myHelpers::loadJsonFile($blueprintName,$lastDir = $this->advancedDataDirectory);

    if($jsonDecoded) {
      return $jsonDecoded;
    }

    $this->notif->genNotifByNum("advanced",9201,$lastDir.DS.$blueprintName);
  }

  /**
   * Load archived advanced data Blueprint
   * @param $blueprintName
   * @return mixed
   */
  public function loadArchivedBlueprint($blueprintName) {

    $jsonDecoded = myHelpers::loadJsonFile($blueprintName,$lastDir = $this->advancedArchiveDataDirectory);

    if($jsonDecoded) {
      return $jsonDecoded;
    }

    $this->notif->genNotifByNum("advanced",9201,$lastDir.DS.$blueprintName);
  }

  /**
   * Load archived advanced data Blueprint
   * @param $blueprintName
   * @return mixed
   */
  public function loadSystemBlueprint($blueprintName) {

    $jsonDecoded = myHelpers::loadJsonFile($blueprintName,$lastDir = $this->advancedsystemDirectory);

    if($jsonDecoded) {
      return $jsonDecoded;
    }

    $this->notif->genNotifByNum("advanced",9201,$lastDir.DS.$blueprintName);
  }

  /**
   * Load advanced data Blueprint
   * @param $blueprintName
   * @return mixed
   */
  public function getBlueprintFields($blueprintName) {

    $jsonDecoded = myHelpers::loadJsonFile($blueprintName,$lastDir = $this->advancedDataDirectory);

    if(!empty($jsonDecoded['fields'])) {
      foreach($jsonDecoded['fields'] as $key => $value) {
        if(!empty($value['standard']['name'])) {
          $jsonDecodedData[] = $value['standard']['name'];
        }
      }
    }

    if(isset($jsonDecodedData)) {
      return $jsonDecodedData;
    }

    $this->notif->genNotifByNum("advanced",9201,$lastDir.DS.$blueprintName);
  }

  /**
   * Filename according to Blueprint - will fix filename with json blueprint name
   * @param $blueprintName
   * @param $filename
   * @param $type
   * @return bool
   */
  public function fileNameAccordingToBlueprint($blueprintName,$filename,$type) {

    if($type=="archive") {
      $path = $this->advancedArchiveDataDirectory;
    }
    else {
      $path = $this->advancedDataDirectory;
    }

    if($blueprintName!=$filename) {
      rename($path.DS.$filename.".json",$path.DS.$blueprintName.".json");

      return false;
    }

    return true;
  }

  /**
   * Validate Blueprint fileName
   * @param $filename
   * @param $type
   * @return bool
   */
  public function validateBlueprintFileName($filename,$type) {

    if($type=="archive") {
      $path = $this->advancedArchiveDataDirectory;
    }
    else {
      $path = $this->advancedDataDirectory;
    }

    $dir = $this->advancedTempDirectory;

    if(!is_dir($dir)) {
      mkdir($dir);
    }

    if(!preg_match('/^[a-zA-Z][A-Za-z0-9_]*$/',$filename)) {
      rename($path.DS.$filename.".json",$this->advancedTempDirectory.DS.$filename.".json");

      return false;
    }   // invalid file name or filename - can't start with number or underscore

    return true;
  }

  public function validateBlueprintName($blueprintName,$filename,$type) {

    if($type=="archive") {
      $path = $this->advancedArchiveDataDirectory;
    }
    else {
      $path = $this->advancedDataDirectory;
    }

    $dir = $this->advancedTempDirectory;

    if(!is_dir($dir)) {
      mkdir($dir);
    }

    if(!preg_match('/^[a-zA-Z][A-Za-z0-9_]*$/',$blueprintName)) {
      rename($path.DS.$filename.".json",$this->advancedTempDirectory.DS.$filename.".json");

      return false;
    }   // invalid Blueprint name or filename - can't start with number or underscore

    return true;
  }
}

