<?php

/*************************************************************
 * Advanced system class for Mysql
 * - configure the db handling system
 * @author    S M Kessner <limbercode@gmail.com>
 * @copyright 2010-2018 T2
 * @link      http://limbercode.com
 *************************************************************/
class advancedForMysql {

  private $pdoHelper;
  private $table;
  private $tablePast;
  private $leadId;
  private $actionId;
  private $rollBack;
  private $notif;
  public $lastActionId;
  public $lastOrderingPosition;

  /**
   * Constructor for advanced
   */
  function __construct() {

    $this->notif = basic::load("notif","core");
    $this->notif->loadNotifs($file = "advanced");

    basic::loadCfg("advancedConfig");
  }

  /**
   * Get lastActionId
   * @param $pdoHelper
   * @param $table
   * @param $preData
   * @throws ReflectionException
   */
  public function getLastActionIdAndPosition($pdoHelper,$table,$preData) {

    $this->table = $table;

    basic::loadCfg("advancedConfig");
    $this->pdoHelper = $pdoHelper;

    $lastActionId = $this->pdoHelper->pdo->prepare("SELECT MAX(actionId) as lastActionId, MAX(orderingPosition) as lastOrderingPosition FROM ".$this->table);
    $lastActionId->execute();
    $last                       = $lastActionId->fetch(PDO::FETCH_ASSOC);
    $this->lastActionId         = $last['lastActionId'];
    $this->lastOrderingPosition = $last['lastOrderingPosition'];
  }

  /**
   * Add advanced data to preData
   * @param $pdoHelper
   * @param $preData
   * @return array
   * @throws ReflectionException
   */
  public function addAdvancedDataToPreData($pdoHelper,$table,$preData) {

    $this->table = $table;

    basic::loadCfg("advancedConfig");
    $this->pdoHelper = $pdoHelper;
    $advancedData    = $this->preDataForSqlRowNew();

    $this->getLastActionIdAndPosition($pdoHelper,$table,$preData);

    /*		p($this->table,4444);

        $lastActionId = $this->pdoHelper->pdo->prepare("SELECT MAX(actionId) as lastActionId FROM ".$this->table);
        $lastActionId->execute();
        $last                = $lastActionId->fetch(PDO::FETCH_ASSOC);
        $this->lastActionId = $last['lastActionId'];

    p($this->lastActionId,1111);
    p("SELECT MAX(actionId) as lastActionId FROM ".$this->table,2222);*/

    $actionIdLeap                = $this->lastActionId + basic::getConfig("advanced","autoIncrementActionIdBy");
    $orderingPositionLeap        = $this->lastOrderingPosition + 1;
    $preData['actionId']         = $actionIdLeap;    // manual increment of actionId
    $preData['orderingPosition'] = $orderingPositionLeap;    // manual increment of orderingPosition

    $autoIncrementTidBy       = $this->pdoHelper->pdo->prepare("SET @@auto_increment_increment = ".basic::getConfig("advanced","autoIncrementLeadIdBy").", @@auto_increment_offset = ".basic::$cfg['advanced']['autoIncrementLeadIdBy']."");
    $autoIncrementTidByReturn = $autoIncrementTidBy->execute();

    foreach($advancedData as $key => $value) {    // add the date/ip and stuff to $preData if it holds no data
      if(!isset($preData[$key])) {
        $preData[$key] = $advancedData[$key];
      }
    }

    return $preData;
  }

  /**
   * Pre data for sql row new - preData that will be used if no other parameters given
   * for creating new row in the db
   */
  public function preDataForSqlRowNew() {

    $sqlDate = date("Y-m-d H:i:s",time());

    $preData['level1Id']   = basic::getConfig("advanced","level1Id");
    $preData['level2Id']   = basic::getConfig("advanced","level2Id");
    $preData['level3Id']   = basic::getConfig("advanced","level3Id");
    $preData['detectedIp'] = myHelpers::getUserIp();
    $preData['createdAt']  = $sqlDate;
    $preData['modifiedAt'] = "";
    $preData['isActive']   = 1;

    $auth                       = basic::load("auth","core");
    $userId                     = $auth->getUserLoggedUserId();
    $preData['createdByUserId'] = ($userId) ? $userId : 0;

    return $preData;
  }

  /**
   * Pre data for sql row update - preData that will be used if no other parameters given
   * for updating row in the db
   */
  public function preDataForSqlRowUpdate() {

    $sqlDate = date("Y/m/d H:i:s",time());

    //	$preData['level1Id'] = $this->cfgAdvanced['level1Id'];
    //	$preData['level2Id'] = $this->cfgAdvanced['level2Id'];
    //	$preData['level3Id'] = $this->cfgAdvanced['level3Id'];		// not suppose to replace that, only if user is doing it from other levelId
    $preData['detectedIp']   = myHelpers::getUserIp();
    $preData['modifiedAt']   = $sqlDate;
    $preData['actionTypeId'] = "";        // must be reset - before new data enters (so we won't forget to delete it)
    $preData['exceptional']  = "";        // must be reset - before new data enters (so we won't forget to delete it)

    /*		if($userId) {
          $preData['userId'] = $userId;
        }
        else {		// not yet implemented*/
    //	global $auth;

    $auth                        = basic::load("auth","core");
    $preData['modifiedByUserId'] = $auth->getUserLoggedUserId();

    //	}

    return $preData;
  }

  /**
   * Export entries
   * @param $tableName
   * @return string
   * @throws Exception
   */
  public function exportEntries($tableName) {

    require_once(OUTSOURCEPATH."mysql".DS."mysqldump".DS."Mysqldump.php");

    $dumpSettingsDefault = array(
      'include-tables' => array($tableName)
    );

    $dumper = new Mysqldump('mysql:host='.basic::getConfig("db","main","hostname").';dbname='.basic::getConfig("db","main","database"),basic::getConfig("db","main","username"),basic::getConfig("db","main","password"),$dumpSettingsDefault);

    if(!is_dir(IMPERMANENTPATH."mysqlDumps")) {
      mkdir(IMPERMANENTPATH."mysqlDumps");
    }

    $exportedFile = IMPERMANENTPATH."mysqlDumps".DS.$tableName."_".time().'.sql';
    $dumper->start($exportedFile);

    if(file_exists($exportedFile)) {
      header("Pragma: public",true);
      header("Expires: 0"); // set expiration time
      header("Cache-Control: must-revalidate, post-check=0, pre-check=0");
      header("Content-Type: application/force-download");
      header("Content-Type: application/octet-stream");
      header("Content-Type: application/download");
      header("Content-Disposition: attachment; filename=".basename($exportedFile));
      header("Content-Transfer-Encoding: binary");
      header("Content-Length: ".filesize($exportedFile));
      die(file_get_contents($exportedFile));
    }
    else {
      return "Failed exporting mysql file!";
    }
  }

  /**
   * Add advanced data to createData - preData that will be used if no other parameters given
   * @param $createData
   * @param $characterMedium
   * @param $characterSetFull
   * @param $engine
   * @return array
   */
  public function addAdvancedDataToCreateData($createData,$characterMedium,$characterSetFull,$engine) {

    $preData = array(
      "columns"       => array(
        "leadId"           => array("int","11","UNSIGNED NOT NULL AUTO_INCREMENT"),
        "actionId"         => array("int","13","UNSIGNED NOT NULL"),
        "level1Id"         => array("int","11","UNSIGNED NOT NULL"),
        "level2Id"         => array("int","11","UNSIGNED NOT NULL"),
        "level3Id"         => array("int","11","UNSIGNED NOT NULL"),
        "orderingPosition" => array("int","11","UNSIGNED NOT NULL"),
        "createdByUserId"  => array("int","11","UNSIGNED NOT NULL"),
        "modifiedByUserId" => array("int","11","UNSIGNED NOT NULL"),
        "detectedIp"       => array("varchar","20","NOT NULL COLLATE '".$characterMedium."'"),
        "createdAt"        => array("datetime","","NOT NULL"),
        "modifiedAt"       => array("datetime","","NOT NULL"),
        "actionTypeId"     => array("int","11","UNSIGNED NOT NULL"),
        "exceptional"      => array("varchar","40","NOT NULL COLLATE '".$characterMedium."'"),
        "isActive"         => array("tinyint","1","NOT NULL"),
        "isDeleted"        => array(
          "dataType"  => "tinyint",
          "lengthSet" => "1",
          "addOn"     => "NOT NULL",
        ),
      ),
      "indexes"       => array(
        "PRIMARY KEY (`leadid`) USING BTREE",
      ),
      "characterSet"  => $characterSetFull,
      "engine"        => $engine,   // or MyISAM
      "autoIncrement" => "10",
    );

    //   UNIQUE INDEX `id` (`id`) USING BTREE,
//	INDEX `Index 1` (`id`) USING BTREE

    if(!empty($createData['columns'])) {
      $preData['columns'] = array_merge($preData['columns'],$createData['columns']);
    }

    if(!empty($createData['engine'])) {
      $preData['engine'] = $createData['engine'];
    }

    $createData = $preData;

    return $createData;
  }

  /**
   * Create past table - checks first if past table already exists, if not, it will clone the table to past, then it will remove auto increment leadId on past table. it will run all over the past table indexes and remove them 1 by 1. after that it will create PRIMARY index on actionId
   */
  public function createPastTable() {

    $checkIfTablePastExist = $this->pdoHelper->checkIfTableExist($this->tablePast);

    if($checkIfTablePastExist!==true) {
      //    $this->pdoHelper->beginTransaction();
      $selectCreateTablePast = $this->pdoHelper->pdo->prepare("CREATE TABLE IF NOT EXISTS ".$this->tablePast." LIKE ".$this->table);
      $returnCreateTablePast = $selectCreateTablePast->execute();
      //    $this->pdoHelper->commit();

      //    $this->pdoHelper->beginTransaction();
      $selectIndexes = $this->pdoHelper->pdo->prepare("SHOW INDEX FROM ".$this->tablePast);
      $selectIndexes->execute();
      $selectIndexes->setFetchMode(PDO::FETCH_OBJ);
      $returnIndexesData = $selectIndexes->fetchAll();
      //    $this->pdoHelper->commit();

      $weAlreadyHaveIndexActionId = false;
      foreach($returnIndexesData as $indexesData) {
        if($indexesData->Column_name!="actionId") {
          if($indexesData->Column_name=="leadId") {
            //          $this->pdoHelper->beginTransaction();
            $removeAutoIncrement = $this->pdoHelper->pdo->prepare("ALTER TABLE ".$this->tablePast." CHANGE leadId leadId INT(11) UNSIGNED NOT null");
            $removeAutoIncrement->execute();
            //          $this->pdoHelper->commit();
          }

          if($indexesData->Key_name=="PRIMARY") {
            //          $this->pdoHelper->beginTransaction();
            $dropPraimaryTablePast = $this->pdoHelper->pdo->prepare("ALTER TABLE ".$this->tablePast." DROP PRIMARY KEY");
            $dropPraimaryTablePast->execute();
            //          $this->pdoHelper->commit();
          }
          else {
            //          $this->pdoHelper->beginTransaction();
            $dropIndexTablePast = $this->pdoHelper->pdo->prepare("ALTER TABLE ".$this->tablePast." DROP INDEX link");
            $dropIndexTablePast->execute();
            //          $this->pdoHelper->commit();
          }
        }
        else {
          $weAlreadyHaveIndexActionId = true;
        }
      }

      //    $this->pdoHelper->commit();

      if($weAlreadyHaveIndexActionId!==true) {
        //		$addPrimaryTablePast = $this->pdoHelper->pdo->prepare("ALTER TABLE ".$this->tablePast." ADD PRIMARY KEY (actionId)");
        //		$addPrimaryTablePast->execute();
        // removed cz caused problems
      }
    }
  }

  public function retrieveRowFromPast($pdoHelper,$table,$leadId,$actionId) {        // clone the row to _past table

    $this->pdoHelper = $pdoHelper;
    $this->table     = $table;
    $this->leadId    = $leadId;
    $this->actionId  = $actionId;
    $this->tablePast = $table."_past";

    $getPastInfo = $this->pdoHelper->find($table."_past","*",array("leadId" => $leadId,"actionId" => $actionId));

    $pastData                 = [];
    $pastData['level1Id']     = $getPastInfo[0]['level1Id'];
    $pastData['level2Id']     = $getPastInfo[0]['level2Id'];
    $pastData['level3Id']     = $getPastInfo[0]['level3Id'];
    $pastData['actionTypeId'] = $getPastInfo[0]['actionTypeId'];
    $pastData['exceptional']  = $getPastInfo[0]['exceptional'] = "Retrieved from past actionId: ".$getPastInfo[0]['actionId'];
    $pastData['isActive']     = $getPastInfo[0]['isActive'];
    $pastData['isDeleted']    = $getPastInfo[0]['isDeleted'];

    $fromHereThereIsLiveData = "";
    foreach($getPastInfo[0] as $key => $value) {
      if($fromHereThereIsLiveData==true) {
        $pastData[$key] = $getPastInfo[0][$key];
      }

      if($key=="isDeleted") {     // symbolize the end of advanced data and where the real data begins
        $fromHereThereIsLiveData = true;
      }
    }

    $updateLeadId = $this->pdoHelper->update($table,$pastData,$where = array("leadId" => $leadId),$advancedLead = true);

    return $updateLeadId;
  }

  /**
   * Clone row to past
   * @return boolean
   */
  public function cloneRowToPast() {        // clone the row to _past table

    $this->createPastTable();
    $data = array($this->leadId);
    $sql  = "INSERT INTO ".$this->tablePast." (SELECT * FROM ".$this->table." WHERE leadId=?)";
    //  $this->pdoHelper->beginTransaction();
    $sel    = $this->pdoHelper->pdo->prepare($sql);
    $return = $sel->execute($data);

    //  $this->pdoHelper->commit();

    return $return;
    /*		if($this->whereFoundLeadId) {
          $currentDataParam  = array($this->whereFoundLeadId);
          $selectCurrentData = "SELECT * FROM t2_items WHERE leadId=?";
          $selectCurrentData = $this->pdoHelper->pdo->prepare($selectCurrentData);
          $selectCurrentData->execute($currentDataParam);
          $selectCurrentData->setFetchMode(PDO::FETCH_ASSOC);
          $currentData           = $selectCurrentData->fetch();
          $currentData['leadId'] = null;  // to prevent: Integrity constraint violation: 1062 Duplicate entry '?' for key 'PRIMARY'
        }

        $this->prepareColumnsAndValues($currentData);
        $this->getTablePast($table);

        $sel    = $this->pdoHelper->pdo->prepare("INSERT INTO ".$this->tablePast." (".$this->imploadedColumns.") VALUES (".$this->imploadedValues.")");
        $return = $sel->execute();

        return $return;*/
  }

  public function update($pdoHelper,$table,$leadId,$preData) {

    $this->pdoHelper = $pdoHelper;
    $this->table     = $table;
    $this->tablePast = $table."_past";
    $this->leadId    = $leadId;

    basic::loadCfg("advancedConfig");
    $actionIdLeap = "actionId = (SELECT max(actionId)) + ".basic::getConfig("advanced","autoIncrementActionIdBy").",";
    $this->cloneRowToPast();
    $advancedData = $this->preDataForSqlRowUpdate();

    foreach($advancedData as $key => $value) {    // add the date/ip and stuff to $preData if it holds no data
      if(!isset($preData[$key])) {
        $preData[$key] = $advancedData[$key];
      }
    }

    return array($preData,$actionIdLeap);
  }

  public function rollBack($pdoHelper) {

    //	$this->pdoHelper->delete($this->tablePast,$where="actionId",$this->lastActionId);

    //  if($this->pdoHelper->inTransaction()) {
    //   $this->pdoHelper->beginTransaction();
    //  }

    try {
      if(!$this->rollBack) {      // prevent loop
        $this->pdoHelper = $pdoHelper;
        $this->rollBack  = 1;
        $this->tablePast = $this->table ? $this->table : $this->table."_past";
        $sql             = "DELETE FROM ".$this->tablePast." WHERE actionId=?";

        $this->pdoHelper->pdo->setAttribute(PDO::ATTR_AUTOCOMMIT,0);
        $this->pdoHelper->pdo->beginTransaction();

        $sel = $this->pdoHelper->prepare($sql);
        $sel->execute(array($this->lastActionId));
        $this->pdoHelper->commit();
      }
    }
    catch(PDOException $e) {
      $this->pdoHelper->rollback();
    }
  }

  public function getAllDataFromBlueprint($pdoHelper,$blueprintName) {

    $this->pdoHelper = $pdoHelper;
    $this->table     = basic::getConfig("db","main","tablePrefix").$blueprintName;
    $getInfo         = $this->pdoHelper->find($this->table,"*");

    return $getInfo;
  }

  public function getSingleDataFromBlueprint($pdoHelper,$blueprintName,$leadId) {

    $this->pdoHelper = $pdoHelper;
    $this->table     = basic::getConfig("db","main","tablePrefix").$blueprintName;
    $getInfo         = $this->pdoHelper->find($this->table,"*",$where = array("leadId" => $leadId));

    return $getInfo;
  }

  public function getSearchedDataFromBlueprint($pdoHelper,$blueprintName,$where,$orderBy,$limit,$type = "equivalent",$returnMode = "object") {

    $this->pdoHelper = $pdoHelper;
    $this->table     = basic::getConfig("db","main","tablePrefix").$blueprintName;
    $getInfo         = $this->pdoHelper->find($this->table,"*",$where,"",$orderBy,$limit,$type,$returnMode);

    return $getInfo;
  }

  public function getRepeater($pdoHelper,$holderBlueprint,$assetBlueprint,$holderId) {



  }
}
