<?php

/*************************************************************
 * Auth for Secure Imbibe Login (Sil) system core
 * @author    S M Kessner <limbercode@gmail.com>
 * @copyright 2010-2018 T2
 * @link      http://limbercode.com
 *************************************************************/
class auth {

	public $loggedIn = null;
	public $authLevelNeeded;
	public $userName = null;                    // from POST
	public $userPassword = null;                // from POST
	public $verificationLoginCode = null;       // from POST
	public $verificationRegisterCode = null;    // from POST
	public $rememberMe = null;                  // from POST or COOKIES
	private $cfg;
	private $cfgAuth;
	private $dontAuthUrl;
	private $loggedUserId = null;               // if logged in
	private $loggedUserName = null;             // if logged in
	private $loggedUserAuthLevel = null;        // if logged in
	private $pastCookie = null;                 // when allowShove
	private $pastSession = null;                // when allowShove
	private $regenerate = false;                // to prevent multi regenerating
	private $csrfToken = false;

	/**
	 * Constructor for auth
	 */
	function __construct() {

		$this->dontAuthUrl = array(
			"login",
			"holding",
			"register",
			"sil/createUser",
			"forgotMyPassword",
			"userAgreement",
			"sil/resetPassword",
			"sil/apiLogin",
			"sil/check",
			"userActivation",
			"haveToActivate",
			"insufficient",
			"logged",
			"logout",
			"redirect",
			"siteIsDown",
			"userCreated",
			"timer",
			"sil/askPasswordReset",
			"userPasswordReset",
			"sentPasswordReset",
			"sil/savePasswordReset",
			"sil/checkUserAvailability",
			"favicon.ico",
			"sil/captcha",
			"download/now",
			"cache.manifest"
		);    // holds all of "sil" addresses (or most of them)

		$this->setTempAddress();        // if site is restricted
		$this->setTokenFromInside();    // set token on login/register page only

		if($_SERVER['PHP_SELF']!=basic::getPublicDirectory().lang::getCurrentLanguageForUrls()."captcha/captcha.image.php" && $_SERVER['REQUEST_URI']!=basic::getPublicDirectory().lang::getCurrentLanguageForUrls()."timer") {        // prevent this data to go to captcha or timer

      $dontAuthUrl = [];

			foreach($this->dontAuthUrl as $key => $value) {
				$dontAuthUrl[$key] = basic::getPublicDirectory().lang::getCurrentLanguageForUrls().$value;
			}    // add the directory startup...

			if($this->checkIfLogged()) {
				if($this->getCookieNow($name = "createdAt") + basic::$cfg['auth']['prolongDelay']<=time()) {
					$this->prolongTimeout();    // prolong time on site
				}    // prevent prolong time if user is refreshing after less then x seconds

				if(!headers_sent()) {    // prevent caching
					header("Expires: Mon, 26 Jul 1990 05:00:00 GMT");
					header("Last-Modified: ".gmdate("D, d M Y H:i:s")." GMT");
					header("Cache-Control: no-store, no-cache, must-revalidate, max-age=-1");
					header("Cache-Control: post-check=0, pre-check=0",false);
					header("Pragma: no-cache");
				}
			}
			else {
				if(!headers_sent()) {    // prevent caching
					if(stringManipulation::stringContainsArray($_SERVER['REQUEST_URI'],$dontAuthUrl)) {    // if it's logout we need to logout immediately
						header("Expires: Mon, 26 Jul 1990 05:00:00 GMT");
						header("Last-Modified: ".gmdate("D, d M Y H:i:s")." GMT");
						header("Cache-Control: no-store, no-cache, must-revalidate, max-age=-1");
						header("Cache-Control: post-check=0, pre-check=0",false);
						header("Pragma: no-cache");
					}
					else {    // if it's regular link in the site we want to save the page on cache
						header("Expires: ".gmdate('D, d M Y H:i:s',time() + 60 * 60 * 24)." GMT");    // expires after 24hr
					}
				}
			}

			$this->authLevelNeeded = basic::$cfg['auth']['restrictedSite'];
			$this->restrictedSite();
			$this->checkIfHaveAuthLevel();
		}
	}

	/**
	 * Is dev user?
	 * @return bool
	 */
	public function isDevUser() {

		if(myHelpers::isEnv($env = "dev") || $this->getUserLoggedAuthLevel()>=10) {
			return true;
		}

		return false;
	}

	/**
	 * Set token from inside - set token only on predestined pages ($setTokenDirectories)
	 */
	public function setTokenFromInside() {

		$setTokenDirectories = array("login","logout","register","forgotMyPassword","userPasswordReset");

		foreach($setTokenDirectories as $key => $value) {
			$setTokenDirectoriesUrl[$key] = basic::getPublicDirectory().lang::getCurrentLanguageForUrls().$value;
		}    // add the directory startup...

		if(stringManipulation::stringContainsArray($_SERVER['REQUEST_URI'],$setTokenDirectories)) {
			$this->setToken();    // set token on login/register... page only
		}
	}

	/**
	 * Set token
	 */
	public function setToken() {

		$this->setCookieNow($name = "token",$data = time(),$till = "",$domain = "");
	}

	/**
	 * Set auth level needed
	 * @param integer $num - 0 to 5 - configured on authConfig ($cfgAuth['authLevel'])
	 */
	public function setAuthLevelNeeded($num) {

		$this->authLevelNeeded = $num;
	}

	/**
	 * Get user logged userId
	 * @return integer - get current userId (if logged in)
	 */
	public function getUserLoggedUserId() {

		return $this->loggedUserId;
	}

	/**
	 * Get user logged authLevel
	 * @return integer - get current authLevel (if logged in)
	 */
	public function getUserLoggedAuthLevel() {

		return $this->loggedUserAuthLevel;
	}

	/**
	 * Set temp address - set the cookie: tempAddress - if irrelevant address, it wont be on cookie made
	 */
	public function setTempAddress() {

		if(myHelpers::isAjax()) {
			return false;
		}

		basic::load("lang","core");
		foreach($this->dontAuthUrl as $key => $value) {
			$dontAuthUrl[$key] = "/".basic::$cfg['site']['directory']['main'].lang::getCurrentLanguageForUrls().$value;
		}    // add the directory startup...

		$dontAuthUrl[] = "api/";
		$dontAuthUrl[] = "captcha.image.php";
		$dontAuthUrl[] = ".jpg";
		$dontAuthUrl[] = ".png";
		$dontAuthUrl[] = ".js";
		$dontAuthUrl[] = ".css";
		$dontAuthUrl[] = ".ico";
		$dontAuthUrl[] = "timer";
		$dontAuthUrl[] = "checkIfLogged";
		$dontAuthUrl[] = "index.php";
		$dontAuthUrl[] = "contactUs/mailSent";
		$dontAuthUrl[] = "userActivation";
		$dontAuthUrl[] = "haveToActivate";

		if(!class_exists("myHelpers") || !class_exists("stringManipulation")) {
			basic::load("myHelpers","core");                // for captcha only
			basic::load("stringManipulation","functions");    // for captcha only
		}

		if(!stringManipulation::stringContainsArray($_SERVER['REQUEST_URI'],$dontAuthUrl)) {
			$this->setCookieNow($name = "tempAddress",$data = $_SERVER['REQUEST_URI'],$till = "",$domain = "");
		}    // set tempAddress
	}

	/**
	 * Catch auth alert - returns the alerts numbers
	 * @param string $catchableAlerts - alerts that can be catch - if left blank means all alerts will be catch
	 * @param string $unCatchableAlerts - alerts that can't be catch - will be ignored - if left blank means no alerts will be catch
	 * @return integer - alert number
	 */
	static function catchAuthAlert($catchableAlerts = "",$unCatchableAlerts = "") {

		$notifNum = array();

		if(isset($_REQUEST['authAlert'])) {
			$notifNum[] = $_REQUEST['authAlert'];
		}        // catches both get and post

		foreach($notifNum as $key => $value) {
			$notifNum = explode(",",$value);
		}    // if we have more then 1 alert in same request

		if(isset($_COOKIE['authAlert'])) {
			$notifNum[] = $_COOKIE['authAlert'];
			//	if(!headers_sent()) {
			setcookie("authAlert","",time() - 3600,"/");    // deleting the cookie
			//	}
		}
		if(isset($_SESSION['authAlert'])) {
			$notifNum[] = $_SESSION['authAlert'];
			unset($_SESSION['authAlert']);                // deleting the session
		}

		if($catchableAlerts) {
			if(!is_array($catchableAlerts)) {
				if(in_array($catchableAlerts,$notifNum)) {
					return $notifNum = array($catchableAlerts);
				}
				else {
					return false;
				}
			}
			else {
				foreach($notifNum as $key => $value) {
					if(!in_array($value,$catchableAlerts)) {
						unset($notifNum[$key]);
					}
				}
			}
		}    // catch only allowed alerts - add only wanted alerts

		if($unCatchableAlerts) {
			if(!is_array($unCatchableAlerts)) {
				if(false!==$pos = array_search($unCatchableAlerts,$notifNum)) {
					//	$pos=array_search($unCatchableAlerts,$notifNum);
					unset($notifNum[$pos]);
				}
			}
			else {
				foreach($notifNum as $key => $value) {
					if(in_array($value,$unCatchableAlerts)) {
						unset($notifNum[$key]);
					}
				}
			}
		}    // catch only allowed alerts - remove unwanted alerts

		return $notifNum;
	}

	/**
	 * Restricted site - if in site cfg $cfgAuth['restrictedSite'] > 0, we have to check certain cookies/session to check if user logged in
	 */
	public function restrictedSite() {

		if(basic::$cfg['site']['isActive']!=1 && $this->loggedUserAuthLevel<10) {    // if site is down for maintenance
			if(basic::$cfg['site']['isActive'] && basic::$cfg['site']['isActive']!=1) {
				Header("Location: ".lang::getCurrentLanguageForUrls().basic::$cfg['site']['isActive']."");
				die();
			}
			else {
				if($_SERVER['REQUEST_URI']!=basic::getPublicDirectory().lang::getCurrentLanguageForUrls()."siteIsDown") {
					$this->redirectInside($address = "siteIsDown");
				}
			}

			return false;
		}

		if(!stringManipulation::stringContains($_SERVER['REQUEST_URI'],"sil/captcha")) {
			if($_SERVER['REQUEST_URI']!=basic::getPublicDirectory().lang::getCurrentLanguageForUrls()."holding") {

				$timeLimitAttempt = explode("-",$this->getCookieNow($name = "loginAttempts"));

				if(isset($timeLimitAttempt[1]) && $timeLimitAttempt[1]>=time()) {
					$this->setCookieNow($name = "verificationLoginTries",$data = "",$till = -3600,$domain = "");
					$this->redirectInside($address = "holding",$notifNum = 7012);
				}    // too many attempts to login
				elseif(isset($timeLimitAttempt[1]) && $timeLimitAttempt[1]<time()) {
					$this->setCookieNow($name = "loginAttempts",$data = "",$till = -3600,$domain = "");
				}    // reset the timer

				if(basic::$cfg['auth']['login']['allowCaptcha']==1) {        // if captcha is activated
					$timeLimitVerification = explode("-",$this->getCookieNow($name = "verificationLoginTries"));

					if(isset($timeLimitVerification[1]) && $timeLimitVerification[1]>=time()) {
						$this->redirectInside($address = "holding",$notifNum = 7011);
					}    // too many verificationLoginTries attempts to login (captcha)
					elseif(isset($timeLimitVerification[1]) && $timeLimitVerification[1]<time()) {
						$this->setCookieNow($name = "verificationLoginTries",$data = "",$till = -3600,$domain = "");
					}    // reset the timer
				}
			}    // to prevent endless loop
		}

		$notifsArray = $this->catchAuthAlert();
		if((empty($notifsArray)) && ($_SERVER['REQUEST_URI']!=basic::getPublicDirectory().lang::getCurrentLanguageForUrls()."login" && $_SERVER['REQUEST_URI']!=basic::getPublicDirectory().lang::getCurrentLanguageForUrls()."sil/check" && $_SERVER['REQUEST_URI']!=basic::getPublicDirectory().lang::getCurrentLanguageForUrls()."sil/logout") || ($_SERVER['REQUEST_URI']==basic::getPublicDirectory().lang::getCurrentLanguageForUrls()."logged" && basic::$cfg['auth']['alertMethod']!=2)) {

			$this->rememberMe = $this->getCookieNow($name = "rememberMe");

			if($this->getCookieNow($name = "address") && !$this->getCookieNow($name = "validation")) {
				if(basic::$cfg['auth']['intervalCheck'] && !$this->rememberMe && $_SERVER['REQUEST_URI']==basic::getPublicDirectory().lang::getCurrentLanguageForUrls()."timer") {
					echo "<div id=\"stopAjaxInterval\"></div>";        // for later to be targeted by js
					die();
				}    // that means that timer is trying to add intervals but $cfgAuth['timeout'] has been reached

				$this->setCookieNow($name = "address",$data = "",$till = -3600,$domain = "");

				if(basic::$cfg['auth']['restrictedSite']) {
					$this->revokeAll();
					$this->redirectInside($address = "login",$notifNum = 7003);
				}
				else {
					$this->revokeAll();
				}
			}    // that means: user was logged till now, the validation cookie is missing

			if($_SERVER['REQUEST_URI']!="/".basic::$cfg['site']['directory']['main'].lang::getCurrentLanguageForUrls()."undefined" && $_SERVER['REQUEST_URI']!=basic::getPublicDirectory().lang::getCurrentLanguageForUrls()."login") {
				if(!$this->getCookieNow($name = "connected") && !$this->getCookieNow($name = "createdAt") && $this->getCookieNow($name = "validation")) {

					if(basic::$cfg['auth']['restrictedSite']) {
						if($_SERVER['REQUEST_URI']==basic::getPublicDirectory().lang::getCurrentLanguageForUrls()."timer") {
							die();
						}

						$this->revokeAll();
						$this->redirectInside($address = "login",$notifNum = 7002);    // just connect because it's first time
					}
					else {
						$this->revokeAll();

						$revoked = 1;
						if($_SERVER['REQUEST_URI']==basic::getPublicDirectory().lang::getCurrentLanguageForUrls()."timer") {
							die();
						}
					}
				}    // means session is missing

				if(!$this->checkIpOnCookie() && $this->getCookieNow($name = "validation")) {
					$this->revokeAll();
					$this->redirectInside($address = "login",$notifNum = 7057);
				}    // means that user is trying to connect from multiple locations with the same validation cookie

				if($this->loggedIn===null) {
					$this->checkIfLogged();
				}
				if(!$this->loggedIn) {    // if user is not logged
					foreach($this->dontAuthUrl as $key => $value) {
						$this->dontAuthUrlPublic[$key] = basic::getPublicDirectory().lang::getCurrentLanguageForUrls().$value;       // dontAuthUrlPublic - don't know what it for
					}    // add the directory startup...

					if(!stringManipulation::stringContainsArray($_SERVER['REQUEST_URI'],$this->dontAuthUrl)) {
						if(!$this->getCookieNow($name = "validation")) {
							if(basic::$cfg['auth']['restrictedSite']) {
								$this->redirectInside($address = "login",$notifNum = 7005);    // just connect because it's first time
							}
							else {
								$this->revokeAll();
							}
						}
						else {
							if(!isset($revoked)) {
								$this->redirectInside($address = "login",$notifNum = 7018);    // validation cookie been messed or something
							}
						}
					}
				}
				else {        // if user already logged - we need to prolong the timeout
					if(basic::$cfg['auth']['intervalCheck'] && !$this->rememberMe) {
						if(!$this->checkTimer()) {
							$this->revokeAll();

							if(basic::$cfg['auth']['restrictedSite']) {
								$this->redirectInside($address = "login",$notifNum = 7004);
							}
						}    // means that timer is not found - no activity - maybe user closed the browser more then $cfgAuth['triple'] time
						else {
							$this->setCookieNow($name = "timer",$data = time() + basic::$cfg['auth']['triple'],$till = "",$domain = "");
						}    // prolong timer
					}    // means that timer is activated

					if($_SERVER['REQUEST_URI']!=basic::getPublicDirectory().lang::getCurrentLanguageForUrls()."timer") {
						if($this->getCookieNow($name = "createdAt") + basic::$cfg['auth']['prolongDelay']<=time()) {
							$this->prolongTimeout();
						}    // prevent prolong time if user is refreshing after less then x seconds
					}    // if timer activated no need to prolong validation timeOut
				}
			}    // if we are not on login path/controller
		}    // to prevent endless loop
		//	genNotifByNum($file="core",$num=81150,$externalVar="",$controller="");	// not suppose to see this error
	}

	/**
	 * Revoke all - when user logged out or in any other case we revoke user credentials
	 */
	public function revokeAll() {    // revoke all important cookies - user will be disconnected in any case

		$this->setCookieNow($name = "address",$data = "",$till = -3600,$domain = "");    // added lately
		$this->setCookieNow($name = "connected",$data = "",$till = -3600,$domain = "");
		$this->setCookieNow($name = "createdAt",$data = "",$till = -3600,$domain = "");
		$this->setCookieNow($name = "validation",$data = "",$till = -3600,$domain = "");
		$this->setCookieNow($name = "userId",$data = "",$till = -3600,$domain = "");
		$this->setCookieNow($name = "userAuthLevel",$data = "",$till = -3600,$domain = "");

		if(basic::$cfg['auth']['allowRememberMe']==1 && $this->rememberMe) {
			$this->setCookieNow($name = "rememberMe",$data = 1,$till = "",$domain = "");
		}
		else {
			$this->setCookieNow($name = "rememberMe",$data = "",$till = -3600,$domain = "");
		}

		if(basic::$cfg['auth']['intervalCheck'] && !$this->rememberMe) {
			$this->setCookieNow($name = "timer",$data = "",$till = -3600,$domain = "");
		}

		if(basic::$cfg['auth']['tokenCheck']==1) {
			$this->setTokenFromInside();
		}
	}

	/**
	 * Prolong timeout - when in cfg $cfgAuth['intervalCheck']==1 and $cfgAuth['noProlong']==0, we have to prolong time of user on the site (for ajax use and regular use)
	 */
	public function prolongTimeout() {

		if(basic::$cfg['auth']['noProlong']!=1) {
			$userAgent = "";
			if(isset($_SERVER['HTTP_USER_AGENT'])) {
				$userAgent = strtolower(filter_var($_SERVER['HTTP_USER_AGENT'],FILTER_SANITIZE_STRING));
			}

			$until = "";
			if(basic::$cfg['auth']['allowRememberMe']==1 && $this->rememberMe) {
				$until = $this->rememberMe - time();    // give the time to prolong to..
			}        // when we prolonging time although we don't need to change the validation timing or any other thing, we are changing the validation code and regenerating session key

			$userIdCS        = $this->getCookieNow($name = "userId");
			$userNameCS      = $this->getCookieNow($name = "userName");
			$userAuthLevelCS = $this->getCookieNow($name = "userAuthLevel");
			$timeNow         = time();
			$validCookie     = $userNameCS.basic::$cfg['auth']['hashKey'].$userIdCS.$timeNow.$userAgent;
			$encryptMethod   = basic::$cfg['auth']['encryptMethod'];

			$this->validation = $encryptMethod($validCookie);    // have to save this data for later use - so it wont mess with sessions...
			$this->setCookieNow($name = "createdAt",$data = $timeNow,$till = $until,$domain = "");
			$this->setCookieNow($name = "userId",$data = $userIdCS,$till = $until,$domain = "");
			$this->setCookieNow($name = "userName",$data = $userNameCS,$till = $until,$domain = "");
			$this->setCookieNow($name = "userAuthLevel",$data = $userAuthLevelCS,$till = $until,$domain = "");
			$this->setCookieNow($name = "validation",$data = $this->validation,$till = $until,$domain = "");
		}
	}        // if user already logged and have some activity on the site we need to prolong his timeout

	/**
	 * Check redirect url - check $_SERVER['REQUEST_URI'] ---> public problem when switching to virtualHost
	 * @param string $dir - the path address
	 * @return boolean
	 * @example $this->checkRedirectUrl("login/check");
	 */
	public function checkRedirectUrl($dir) {

		$optionWithoutVirtualHost = basic::getPublicDirectory().lang::getCurrentLanguageForUrls();
		$optionWithVirtualHost    = basic::getPublicDirectory().lang::getCurrentLanguageForUrls().basic::$cfg['site']['directory']['main'];

		if($_SERVER['REQUEST_URI']!=$optionWithoutVirtualHost.$dir && $_SERVER['REQUEST_URI']!=$optionWithVirtualHost.$dir) {
			return true;
		}

		return false;
	}

	/**
	 * Verification tries delay - check how much delay left for user, after that the verification tries will be reset
	 * @return integer $delay - the delay left
   * @throws ReflectionException
	 * @example $this->auth->verificationTriesDelay();
	 */
	public function verificationTriesDelay() {

		$delay             = "";
		$verificationTries = $this->getCookieNow($name = "verificationLoginTries");
		if($verificationTries) {
			$timeLimitVerification = explode("-",$verificationTries);
			if(isset($timeLimitVerification[1])) {
				$delay = $timeLimitVerification[1] - time();
			}
		}

		return $delay;
	}

	/**
	 * Attempts delay - check how much delay left for user, after that the attempts will be reset
	 * @return integer $delay - the delay left
   * @throws ReflectionException
	 * @example $this->auth->attemptsDelay();
	 */
	public function attemptsDelay() {

		$delay            = "";
		$timeLimitAttempt = explode("-",$this->getCookieNow($name = "loginAttempts"));
		if(isset($timeLimitAttempt[1])) {
			$delay = $timeLimitAttempt[1] - time();
		}

		return $delay;
	}

	/**
	 * Check if have authLevel
	 * @example $this->checkIfHaveAuthLevel();
	 */
	public function checkIfHaveAuthLevel() {

		//	$this->login();		// T2: somehow its not letting it pass -------- when logging out ----------------------------
		//	$this->login(1);

		foreach($this->dontAuthUrl as $key => $value) {
			$this->dontAuthUrlPublic[$key] = basic::getPublicDirectory().lang::getCurrentLanguageForUrls().$value;       // dontAuthUrlPublic - don't know what it for
		}    // add the directory startup...

		if(!stringManipulation::stringContainsArray($_SERVER['REQUEST_URI'],$this->dontAuthUrl)) {
			if($this->checkIfLogged()===true) {    // logged in
				if($this->checkAuthLevel()!==false) {
					return false;
				}
			}
			else {
				if($this->authLevelNeeded) {    // if we need auth level of some kind here
					//	$this->setCookieNow($name = "tempAddress",$data = $_SERVER['REQUEST_URI'],$till = "",$domain = "");
					$this->setTempAddress();

          $isAjax = myHelpers::isAjax();

          if($isAjax) {
            header("Location: /login");   // force full screen on ajax - for login to show correctly
          }

					if(!stringManipulation::stringContainsArray($_SERVER['REQUEST_URI'],$this->dontAuthUrl)) {

					  p($_SERVER['REQUEST_URI']);
					  p($this->dontAuthUrl);

						$this->redirectInside($address = "login",$notif = "7019");
					}
				}
			}
		}
	}

	/**
	 * Check authLevel - if it's sufficient
	 * @return boolean
   * @throws ReflectionException
	 * @example $auth->checkAuthLevel();
	 */
	public function checkAuthLevel() {

		if($this->authLevelNeeded<=$this->getCookieNow($name = "userAuthLevel")) {    // ok - can pass
			return true;
		}
		else {    // nope - auth level is insufficient
			if($_SERVER['REQUEST_URI']!=basic::getPublicDirectory().lang::getCurrentLanguageForUrls()."insufficient" && $_SERVER['REQUEST_URI']!=basic::getPublicDirectory().lang::getCurrentLanguageForUrls()."login" && !stringManipulation::stringContains($_SERVER['REQUEST_URI'],"/".basic::$cfg['site']['directory']['main'].lang::getCurrentLanguageForUrls()."login")) {
				if($this->checkIfLogged()===true) {
					$this->redirectInside($address = "insufficient");
				}
				else {
					//	$this->setCookieNow($name = "tempAddress",$data = $_SERVER['REQUEST_URI'],$till = "",$domain = "");
					$this->setTempAddress();
					$this->redirectInside($address = "login",$notif = "7019");
				}    // must log in
			}

			return false;
		}
	}

	/**
	 * Check if logged
	 * @return boolean
   * @throws ReflectionException
	 * @example $this->auth->checkIfLogged();
	 */
	public function checkIfLogged() {

		$ip = $_SERVER['REMOTE_ADDR'];

		$userAgent = "";
		if(isset($_SERVER['HTTP_USER_AGENT'])) {
			$userAgent = strtolower(filter_var($_SERVER['HTTP_USER_AGENT'],FILTER_SANITIZE_STRING));
		}

		$userName      = $this->getCookieNow($name = "userName");
		$userId        = $this->getCookieNow($name = "userId");
		$userAuthLevel = $this->getCookieNow($name = "userAuthLevel");




/*    p(33333333);
    p($this->loggedUserId);*/

		$validCookie = $userName.basic::$cfg['auth']['hashKey'].$userId.$this->getCookieNow($name = "createdAt").$userAgent;

		if(!isset($this->validation)) {
			$this->validation = "";
		}
		$encryptMethod = basic::$cfg['auth']['encryptMethod'];

		if($this->getCookieNow($name = "connected")==$ip && ($this->getCookieNow($name = "validation")==$encryptMethod($validCookie) || $this->validation==$encryptMethod($validCookie))) {
			$this->loggedUserName      = $userName;
			$this->loggedUserId        = $userId;
			$this->loggedUserAuthLevel = $userAuthLevel;
			$this->loggedIn            = true;

			return true;    // means logged in
		}

		return false;    // means not logged in
	}

	/**
	 * Redirect inside - redirect and generate alerts
	 * @param string $address - path inside the site to redirect to, like: sil/login
	 * @param string /integer $notifNum - alert number
   * @throws ReflectionException
	 * @example $this->redirectInside($address="login",$notif="7019");
	 */
	public function redirectInside($address,$notifNum = "") {    // redirect to target page with alert

		if(!$address) {
			$address = "/";     // we will redirect to home page for now

			/*			echo "no addres to redirect!!! problem...";
						die();*/
		}    // auth error 7050 currently uses blank address

		if(basic::$cfg['auth']['alertMethod']==1 || basic::$cfg['auth']['alertMethod']==3 || basic::$cfg['auth']['alertMethod']==4) {    // redirect by get
			$notifGet = "";

			if(basic::$cfg['auth']['allowNotifs']==1 && $notifNum) {
				if(basic::$cfg['auth']['alertMethod']==1) {
					if(stringManipulation::stringContains($address,"?")) {
						$startingSign = "&";
					}
					else {
						$startingSign = "?";
					}

					$notifGet = $startingSign."authAlert=".$notifNum;
				}

				if(basic::$cfg['auth']['alertMethod']==3) {
					setcookie("authAlert",$notifNum,time() + 3600,"/");
				}

				if(basic::$cfg['auth']['alertMethod']==4) {
					$_SESSION['authAlert'] = $notifNum;
				}
			}

			$uriToCheck = substr($_SERVER['REQUEST_URI'],0,strpos($_SERVER['REQUEST_URI'],"?"));
			if(!$uriToCheck) {
				$uriToCheck = $_SERVER['REQUEST_URI'];
			}

			if($uriToCheck!="/".$address && lang::getCurrentLanguageForUrls().$address.$notifGet) {
				$routeTo = "/".lang::getCurrentLanguageForUrls().$address.$notifGet;
				$routeTo = str_replace("//","/",$routeTo);

				if(class_exists("router")) {
					if(router::isLocalhost()) {
						$routeTo = router::$localHostNextDir.$routeTo;
					}
				}

        Header("Location: ".$routeTo);
				die();
			}
		}
		elseif(headers_sent()) {
			echo "Headers already sent!!! problem...";
			die();
		}

		if(basic::$cfg['auth']['alertMethod']==2) {    // redirect by post
			$notifPost = "";
			if($notifNum) {
				$notifPost = array("authAlert" => $notifNum);
			}

			if($notifNum==7014) {    // means user logged - need to redirect it because we have a problem that cookies are gone
				if(basic::$cfg['auth']['allowRedirect']==1) {
					$notifPost = array("authAlert" => $this->getCookieNow($name = "tempAddress"));
				}
				else {
					$notifPost = array("authAlert" => "logged");
				}
			}
			redirectPost($url = $address,$data = $notifPost);
//	$notifPost must be "authAlert" because we check it in: restrictedSite() ... but somehow it works without it ...
		}
	}

	/**
	 * User password reset attempt - check if token exists and if not redirect it + generate alert
	 * @example $this->auth->userPasswordResetAttempt();
	 */
	public function userPasswordResetAttempt() {

		$validToken = $this->checkToken();
		if($validToken===false) {
			$this->redirectInside($address = "login",$notif = "7007");    // alert: 7007, no token
		}    // redirect to the "login" page
	}

	/**
	 * Forgot my password attempt - check if token exists and if not redirect it + generate alerts
	 * @example $this->auth->forgotMyPasswordAttempt();
	 */
	public function forgotMyPasswordAttempt() {

		$validToken = $this->checkToken();
		if($validToken===false) {
			$this->redirectInside($address = "forgotMyPassword",$notif = "7007");    // alert: 7007, no token
		}    // redirect to the "forgotMyPassword" page

		if(basic::$cfg['auth']['forgotMyPassword']['allowCaptcha'] && basic::$cfg['auth']['forgotMyPassword']['captchaActiveAfter']<$this->getCookieNow($name = "forgotMyPasswordAttempts")) {
			$checkCode = $this->captchaVerifyCode($at = "ForgotMyPassword");    // get "false" to pass

			if($checkCode==7008) {
				$this->redirectInside($address = "forgotMyPassword",$notif = 7008);    // alert: 7008, errorVerificationLoginCode
			}
			if($checkCode==7009) {
				$this->redirectInside($address = "forgotMyPassword",$notif = 7009);    // alert: 7009, errorVerificationLoginCodeBlank
			}
			if($checkCode==7010) {
				$this->redirectInside($address = "forgotMyPassword",$notif = 7010);    // alert: 7010, errorVerificationTimeout
			}
			if($checkCode==7011) {
				$this->redirectInside($address = "holding",$notif = 7011);    // alert: 7011, errorFlood
			}
		}    // if allowCaptcha and captchaActiveAfter <= attempts

		return true;
	}

	/**
	 * Register attempt - check if token exists and if not redirect it + generate alerts in case of invalid register info
	 * @param array $createData - the user typed data
	 * @param string $userPasswordRepeat - the user typed userPasswordRepeat
	 * @return boolean if user typed all data as supposed to
   * @throws ReflectionException
	 * @example $this->auth->registerAttempt($createUserData,$userPasswordRepeat);
	 */
	public function registerAttempt($createData,$userPasswordRepeat) {

		$notifs = array();

		$validToken = $this->checkToken();
		if($validToken===false) {
			$this->redirectInside($address = "register",$notif = "7007");    // alert: 7007, no token
		}    // redirect to the "login" page

		if(basic::$cfg['auth']['register']['allowCaptcha'] && basic::$cfg['auth']['register']['captchaActiveAfter']<$this->getCookieNow($name = "registerAttempts")) {
			$checkCode = $this->captchaVerifyCode($at = "Register");    // get "false" to pass

			if($checkCode==7008) {
				$this->redirectInside($address = "register",$notif = 7008);    // alert: 7008, errorVerificationLoginCode
			}
			if($checkCode==7009) {
				$this->redirectInside($address = "register",$notif = 7009);    // alert: 7009, errorVerificationLoginCodeBlank
			}
			if($checkCode==7010) {
				$this->redirectInside($address = "register",$notif = 7010);    // alert: 7010, errorVerificationTimeout
			}
			if($checkCode==7011) {
				$this->redirectInside($address = "holding",$notif = 7011);    // alert: 7011, errorFlood
			}
		}    // if allowCaptcha and captchaActiveAfter <= attempts

		if(basic::$cfg['auth']['register']['mustApprove']==1 && (!isset($createData['isApprovedUserAgreement']) || $createData['isApprovedUserAgreement']!=1)) {
			$notifs[] = 7020;
		}

		if(in_array("name",basic::$cfg['auth']['register']['require']) && (!isset($createData['firstName']) || !$createData['firstName']) || strlen($createData['firstName'])<3) {
			$notifs[] = 7021;
		}

		if(in_array("email",basic::$cfg['auth']['register']['require']) && (!isset($createData['email']) || !$createData['email'])) {
			$notifs[] = 7022;
		}

		if(in_array("phone",basic::$cfg['auth']['register']['require']) && (!isset($createData['phone']) || !$createData['phone'])) {
			$notifs[] = 7023;
		}

		if(in_array("userName",basic::$cfg['auth']['register']['require'])) {
      if((!isset($createData['userName']) || !$createData['userName']) || strlen($createData['userName'])<basic::$cfg['auth']['register']['minUserName']) {
        $notifs[] = 7024;
      }
    }

		if(!isset($createData['password']) || !$createData['password'] || strlen($createData['password'])<basic::$cfg['auth']['register']['minPassword']) {
			$notifs[] = 7025;
		}

		if(!isset($userPasswordRepeat) || !$userPasswordRepeat || $userPasswordRepeat!=$createData['password']) {
			$notifs[] = 7026;
		}

		$notifs = array_filter($notifs);
		if(!empty($notifs)) {
			$notifsImploded = implode(",",$notifs);
			$this->redirectInside($address = "register",$notif = $notifsImploded);
		}    // issuing all alerts together so all of them will show
		else {
			return true;
		}
	}

	/**
	 * Login attempt start
	 */
	public function loginAttemptStart() {

		$validToken = $this->checkToken();

		if($validToken===false) {
			$this->setInValidConnection();    // add attempts number
			$this->redirectInside($address = "login",$notif = "7007");    // alert: 7007, no token
		}    // redirect to the "login" page

		if(!$this->userName) {
			$this->setInValidConnection();    // add attempts number
			$this->redirectInside($address = "login",$notif = "7006");    // alert: 7006, no userName entered
		}

		if(basic::$cfg['auth']['login']['allowCaptcha'] && basic::$cfg['auth']['login']['captchaActiveAfter']<=$this->getCookieNow($name = "loginAttempts")) {
			$checkCode = $this->captchaVerifyCode($at = "Login");    // get "false" to pass

			if($checkCode) {
				$this->setInValidConnection();    // add attempts number
			}

			if($checkCode==7008) {
				$this->redirectInside($address = "login",$notif = 7008);    // alert: 7008, errorVerificationLoginCode
			}
			if($checkCode==7009) {
				$this->redirectInside($address = "login",$notif = 7009);    // alert: 7009, errorVerificationLoginCodeBlank
			}
			if($checkCode==7010) {
				$this->redirectInside($address = "login",$notif = 7010);    // alert: 7010, errorVerificationTimeout
			}
			if($checkCode==7011) {
				$this->redirectInside($address = "holding",$notif = 7011);    // alert: 7011, errorFlood
			}
		}    // if allowCaptcha and captchaActiveAfter <= attempts
	}

	/**
	 * Login attempt end - continue login attempt
	 * @param boolean $validUser
   * @throws ReflectionException
	 */
	public function loginAttemptEnd($validUser) {

		if($validUser!==false) {
			if(basic::$cfg['auth']['allowMultiUser']!=1) {
				basic::load('sessions','functions');

				$userAlreadyLogged       = "";
				$currentSessionsOnServer = sessions::allSessions("userId");

				foreach($currentSessionsOnServer as $key => $value) {
					if($currentSessionsOnServer[$key]['userId'][1]==$this->loggedUserId) {
						if($currentSessionsOnServer[$key]['userId'][0]>=time()) {
							$userAlreadyLogged = 1;
						}    // if session of 'userId' is still alive	// if user will press on rememberMe it will have a prob
						else {
							sessions::unlinkSessionId($key);
						}    // unset the session id of previous connection (on other computer/browser)
					}
				}    // check if user already logged

				if($userAlreadyLogged==1) {
					$this->redirectInside($address = "login",$notif = 7017);
				}
			}

			$this->setValidConnection();
			$tempAddress = $this->getCookieNow($name = "tempAddress");

			if($tempAddress && basic::$cfg['auth']['allowRedirect']) {
				//	setcookie("authAlert","7016",time()+basic::$cfg['auth']['globalTimeout'],"/");	// setting the cookie to show the alert on the redirected page
				$this->redirectInside($address = $tempAddress,$notif = 7014);        // redirect through a web page
				//	Header("Location: redirect");	die();			 // redirect through a web page
				//	Header("Location: ".$tempAddress);	die();		 // alert: logged and redirected immediately
			}   // redirect to the last known address
			else {
				$this->redirectInside($address = "logged",$notif = 7014);    // alert: 7014, logged
			}    // redirect to the "logged" page
		}
		else {
			$this->setInValidConnection();    // add attempts number
			$this->redirectInside($address = "login",$notif = "7001");    // alert: 7001, incorrect user/pass
		}    // redirect back to login screen
	}

	/**
	 * Set user as logged - set user as logged
	 * @param integer $loggedUserId
	 * @param string $loggedUserName
	 * @param integer $loggedUserAuthLevel
	 * @return boolean
	 */
	public function setUserAsLogged($loggedUserId,$loggedUserName,$loggedEmail,$loggedPhone,$loggedUserAuthLevel) {    // set variable if user is valid

    $loginViaProperty = "";
    $loginVia = basic::$cfg['auth']['login']['loginVia'];

    if(in_array("phone",$loginVia)) {
      $loginViaProperty = $loggedPhone;
    }

    if(in_array("email",$loginVia)) {
      $loginViaProperty = $loggedEmail;
    }

    if(in_array("userName",$loginVia)) {
      $loginViaProperty = $loggedUserName;
    }

		if($loggedUserId && $loginViaProperty && $loggedUserAuthLevel) {
			$this->loggedUserId        = $loggedUserId;
			$this->loggedUserName      = $loginViaProperty;
			$this->loggedUserAuthLevel = $loggedUserAuthLevel;

			return true;
		}
	}

	/**
	 * Timer - activating ajax login check periodically
	 */
	public function AddOnBody() {

		if(!myHelpers::isAjax()) {        // if it's ajax we don't need another timer on page
			$return = $this->putAjaxTimer();
			$return .= $this->putPageAuthVer();

			return $return;
		}
	}

	/**
	 * Timer - activating ajax login check periodically
	 */
	public function timer() {

		$this->setCookieNow($name = "timer",$data = time() + basic::$cfg['auth']['triple'],$till = "",$domain = "");

		$createdAtCS = (int)$this->getCookieNow($name = "createdAt");

		if($createdAtCS + basic::$cfg['auth']['prolongDelay']<=time()) {
			if($createdAtCS + basic::$cfg['auth']['triple']>time()) {        // way to know if user connected
				$this->prolongTimeout();    // prolong time on site
			}
		}    // prevent prolong time if user is refreshing after less then x seconds

		if(basic::$cfg['site']['environment']=="dev") {
			echo basic::$cfg['auth']['intervalCheck']." : ";
			echo time();
		}
		die();
	}

	/**
	 * Put ajax timer - the timer js script to put on html
	 */
	public function putAjaxTimer() {

		if(!stringManipulation::startsWith($haystack = $_SERVER['REQUEST_URI'],$needle = "/".basic::$cfg['site']['directory']['api'])) {    // not api url
			if(basic::$cfg['auth']['allowRememberMe']==1) {
				$this->rememberMe = $this->getCookieNow($name = "rememberMe");
			}

			$return = "";

			if($this->checkIfLogged()===true && basic::$cfg['auth']['intervalCheck'] && !$this->rememberMe) {
				if(basic::$cfg['site']['environment']=="dev") {
					$return .= "
					<div id=\"timerCheck\">
						Ajax timer HERE
					</div>
					";
				}

				$return .= "
					<script>
						ajaxInterval('".(basic::$cfg['auth']['intervalCheck'] * 1000)."');
					</script>
				";        // activate the intervalCheck	// interval are set in milliseconds - that's why it's *1000

				return $return;
			}
		}
	}

	/**
	 * Check timer - if it's valid
	 * @return boolean
   * @throws ReflectionException
	 */
	public function checkTimer() {

		if($this->getCookieNow($name = "timer")>time()) {
			return true;
		}

		return false;
	}

	/**
	 * Put page auth version - tag that's define if page ver is for logged user or for NOT logged user
	 * if the answer does not match - page will be refreshed and prevent from user to see things he shouldn't
   * @return string
   * @throws ReflectionException
   */
	public function putPageAuthVer() {
		if(!stringManipulation::startsWith($haystack = $_SERVER['REQUEST_URI'],$needle = "/".basic::$cfg['site']['directory']['api']) && !stringManipulation::stringContainsArray($_SERVER['REQUEST_URI'],$this->dontAuthUrl)) {    // not api url
			if(basic::$cfg['auth']['ajaxCheckOnPageLoad']==1) {
				if($this->checkIfLogged()===true) {
					$pageAuthVer = "verified on level ".$this->getCookieNow($name = "userAuthLevel")." (needed: ".$this->authLevelNeeded.")";
				}
				else {
					$pageAuthVer = "unVerified";
				}

				$hide             = "";
				$devShow          = "";
				$checkIfLoggedUrl = "<span class='hide' id='checkIfLoggedUrl' data-checkIfLoggedUrl='".lang::url("sil/checkIfLogged")."'></span>";
				if(basic::$cfg['site']['environment']=="dev" || $this->isDevUser()) {
					$devShow = "pageAuthVer: $pageAuthVer";
				}
				else {
					$hide = "hide";
				}

				$siteNotActive = "";     // just a reset
				if(basic::$cfg['site']['isActive']!=1) {
					$siteNotActive = "<a id=\"siteNotActive\" class=\"unVerified ".$hide."\" title=\"Site is not active\">
						Site is not active
					</a>";
				}
				$return = $checkIfLoggedUrl."<a id=\"pageAuthVer\" class=\"".$pageAuthVer." ".$hide."\" title=\"Is this page is auth version?\nMay show if user logged in\" data-ver=\"".$pageAuthVer."\">
						".$devShow.$siteNotActive."
					</a>
					<script>
						checkIfNeedToResfresh();
					</script>
				";

				return $return;
			}
		}
	}

	/**
	 * Set valid connection
	 */
	public function setValidConnection() {    // set cookies/sessions for valid user connection

		$tillTime = "";
		if($this->rememberMe==1) {
			$tillTime = basic::$cfg['auth']['dataDictionary']['rememberMe']['timeToLive'];
		}    // prolong time if rememberMe is checked

		$userAgent = "";
		if(isset($_SERVER['HTTP_USER_AGENT'])) {
			$userAgent = strtolower(filter_var($_SERVER['HTTP_USER_AGENT'],FILTER_SANITIZE_STRING));
		}

		$self               = filter_var($_SERVER['PHP_SELF'],FILTER_SANITIZE_STRING);
		$posLastBreakerSelf = strrpos($self,"/");
		$afterBreakerSelf   = substr($self,$posLastBreakerSelf + 1);
		$posFirstQwSelf     = strrpos($afterBreakerSelf,"?");
		if($posFirstQwSelf) {
			$selfScriptName = substr($afterBreakerSelf,0,$posFirstQwSelf);
		}
		else $selfScriptName = $afterBreakerSelf;

		$ip       = $_SERVER['REMOTE_ADDR'];
		$createdAt = time();

		// set session if password was validated
		$this->setCookieNow($name = "connected",$data = $ip,$till = $tillTime,$domain = "");
		$this->setCookieNow($name = "createdAt",$data = $createdAt,$till = $tillTime,$domain = "");
		$this->setCookieNow($name = "token",$data = "",$till = -3600,$domain = "");

		// set cookie if password was validated
		$this->setCookieNow($name = "userId",$data = $this->loggedUserId,$till = $tillTime,$domain = "");
		$this->setCookieNow($name = "userName",$data = $this->loggedUserName,$till = $tillTime,$domain = "");
		$this->setCookieNow($name = "userAuthLevel",$data = $this->loggedUserAuthLevel,$till = $tillTime,$domain = "");
		$encryptMethod = basic::$cfg['auth']['encryptMethod'];
		$this->setCookieNow($name = "validation",$data = $encryptMethod($this->loggedUserName.basic::$cfg['auth']['hashKey'].$this->loggedUserId.$createdAt.$userAgent),$till = $tillTime,$domain = "");

		if(basic::$cfg['auth']['allowRememberMe']==1 && $this->rememberMe==1) {
			$this->setCookieNow($name = "rememberMe",$data = time() + $tillTime,$till = $tillTime + 200,$domain = "");
		}
		else {
			$this->setCookieNow($name = "rememberMe",$data = "",$till = -3600,$domain = "");
		}

		if($selfScriptName && $selfScriptName!=basic::$cfg['auth']['authFileName'] && $self) {
			$this->setCookieNow($name = "address",$data = $_SERVER['REQUEST_URI'],$till = $tillTime,$domain = "");
		}

		if(basic::$cfg['auth']['intervalCheck'] && $this->rememberMe!=1) {
			$this->setCookieNow($name = "timer",$data = time() + basic::$cfg['auth']['triple'],$till = "",$domain = "");
		}

		if($this->getCookieNow($name = "loginAttempts")) {
			$this->setCookieNow($name = "loginAttempts",$data = "",$till = -3600,$domain = "");
		}    // delete attempts

		if($this->getCookieNow($name = "verificationLoginTries")) {
			$this->setCookieNow($name = "verificationLoginTries",$data = "",$till = -3600,$domain = "");
		}

		$this->loggedIn = true;

		return true;
	}

	/**
	 * Set inValid connection - set cookies/sessions for invalid user connection
	 */
	public function setInValidConnection() {

		$attemptCS        = $this->getCookieNow($name = "loginAttempts");
		$timeLimitAttempt = explode("-",$attemptCS);

		if(((int)$attemptCS + 1)>=basic::$cfg['auth']['attemptsAllowed'] && basic::$cfg['auth']['attemptsAllowed']) {
			$attempts = "T-".(time() + basic::$cfg['auth']['attemptsWaitDelay']);    // "T" will define timeLimit
			$this->setCookieNow($name = "loginAttempts",$data = $attempts,$till = "",$domain = "");
			$this->setCookieNow($name = "verificationLoginTries",$data = "",$till = -3600,$domain = "");
			$this->redirectInside($address = "holding",$notifNum = 7012);
		}
		else {
			$this->setCookieNow($name = "loginAttempts",$data = (int)$attemptCS + 1,$till = "",$domain = "");
		}

		return false;
	}

	/**
	 * Set cookie now - set cookies/sessions configured at $cfgAuth['dataDictionary']
	 * @param string $name - name of cookie/session
	 * @param string $data - value of cookie/session
	 * @param string $till - timeout of cookie/session
	 * @param string $domain - domain of cookie/session
   * @throws ReflectionException
	 * @example $this->setCookieNow($name="verification".$at."Code",$data="",$till=-3600,$domain="");
	 */
	public function setCookieNow($name,$data,$till,$domain) {

		if(!$domain) {
			$domain = "/";
		}
		$cookieBlower  = "";
		$sessionBlower = "";

		if(basic::$cfg['auth']['dataDictionary'][$name]['encrypt']==1) {
			$encryption = basic::load("encryption","functions");
		}

		if(basic::$cfg['auth']['allowShove']==1) {
			$sessions = basic::load("sessions","functions");
			$sessions::startSession();

			if(!headers_sent() && !$this->regenerate) {    // so we won't regenerate every time we ask to set cookie/session
				$sessions::regenerateSession();
				$this->regenerate = true;

//			It mainly helps prevent session fixation attacks. Session fixation attacks is where a malicious user tries to exploit the vulnerability in a system to fixate (set) the session ID (SID) of another user. By doing so, they will get complete access as the original user and be able to do tasks that would otherwise require authentication.

//			To prevent such attacks, assign the user a new session ID using session_regenerate_id() when he successfully signs in (or for every X requests). Now only he has the session ID, and your old (fixated) session ID is no longer valid.
			}

			if(basic::$cfg['auth']['dataDictionary'][$name]['method']=="cookie") {
				$cookiesDataFresh['data'] = "";
				if(basic::$cfg['auth']['allowEncryptHoleShubang']==1) {
					$cookiesDataFresh['data'] = $encryption->decode($_COOKIE['data']);
				}
				else {
					if(isset($_COOKIE['data'])) {
						$cookiesDataFresh['data'] = $_COOKIE['data'];
					}
				}

				preg_match_all('/=\*{([^\|]+)\|([^(}\*=]+)}\*=([^=\*{]+)/',$cookiesDataFresh['data'],$matchesC);

				foreach($matchesC[1] as $key => $value) {        // we make $cookieHolder that is: $_COOKIE['data'] expanded
					foreach(basic::$cfg['auth']['dataDictionary'] as $key2 => $value2) {
						if($matchesC[1][$key]==basic::$cfg['auth']['dataDictionary'][$key2]['short']) {
							if($matchesC[2][$key]>time()) {
								$cookieHolder[$key2]['short']      = $matchesC[1][$key];
								$cookieHolder[$key2]['timeToLive'] = $matchesC[2][$key];
								$cookieHolder[$key2]['data']       = $matchesC[3][$key];
							}
							else {
								unset($cookieHolder[$key2]);
							}    // if cookie have to timing is over
						}
					}
				}    // export the data in $_COOKIE['data'] to small pieces

				if($this->pastCookie) {
					foreach($this->pastCookie as $key => $value) {
						foreach(basic::$cfg['auth']['dataDictionary'] as $key2 => $value2) {
							if($key==$key2) {
								//	if($cookieHolder[$key2]['timeToLive']>time()) {
								$cookieHolder[$key2]['short']      = basic::$cfg['auth']['dataDictionary'][$key]['short'];
								$cookieHolder[$key2]['timeToLive'] = $this->pastCookie[$key]['till'];
								$cookieHolder[$key2]['data']       = $this->pastCookie[$key]['data'];
								/*								}
																else {
																	unset($cookieHolder[$key2]);
																}*/
							}
						}
					}
				}
			}

			if(basic::$cfg['auth']['dataDictionary'][$name]['method']=="session") {

				$sessionDataFresh['data'] = "";
				if(isset($_SESSION['data'])) {
					if(basic::$cfg['auth']['allowEncryptHoleShubang']==1) {
						$sessionDataFresh['data'] = $encryption->decode($_SESSION['data']);
					}
					else {
						$sessionDataFresh['data'] = $_SESSION['data'];
					}
				}

				preg_match_all('/=\*{([^\|]+)\|([^(}\*=]+)}\*=([^=\*{]+)/',$sessionDataFresh['data'],$matchesS);

				foreach($matchesS[1] as $key => $value) {        // we make $sessionHolder that is: $_SESSION['data'] expanded
					foreach(basic::$cfg['auth']['dataDictionary'] as $key2 => $value2) {
						if($matchesS[1][$key]==basic::$cfg['auth']['dataDictionary'][$key2]['short']) {
							if($matchesS[2][$key]>time()) {
								$sessionHolder[$key2]['short']      = $matchesS[1][$key];
								$sessionHolder[$key2]['timeToLive'] = $matchesS[2][$key];
								$sessionHolder[$key2]['data']       = $matchesS[3][$key];
							}
							else {
								unset($sessionHolder[$key2]);
							}    // if session have to timing is over
						}
					}
				}    // export the data in $_SESSION['data'] to small pieces

				if($this->pastSession) {
					foreach($this->pastSession as $key => $value) {
						foreach(basic::$cfg['auth']['dataDictionary'] as $key2 => $value2) {
							if($key==$key2) {
								//		if($sessionHolder[$key2']['timeToLive']>time() || 1==1) {
								$sessionHolder[$key2]['short']      = basic::$cfg['auth']['dataDictionary'][$key]['short'];
								$sessionHolder[$key2]['timeToLive'] = $this->pastSession[$key]['till'];
								$sessionHolder[$key2]['data']       = $this->pastSession[$key]['data'];
								/* 								}
																else {
																	unset($sessionHolder[$key2']);
																} */
							}
						}
					}
				}
			}

			if(isset($cookieHolder)) {
				foreach($cookieHolder as $key2 => $value2) {
					if($key2!=$name) {
						if($cookieHolder[$key2]['timeToLive']>time()) {
							//			if($sessionHolder[$name]['data']) {
							$cookieBlower .= "=*{".$cookieHolder[$key2]['short']."|".$cookieHolder[$key2]['timeToLive']."}*=".$cookieHolder[$key2]['data']."";
							$didNewC      = 1;
							//			}
						}
						else {
							unset($cookieHolder[$key2]);
						}
					}
				}    // old data cookie information
			}

			if(isset($sessionHolder)) {
				foreach($sessionHolder as $key2 => $value2) {
					if($key2!=$name) {
						if($sessionHolder[$key2]['timeToLive']>time()) {
							//			if($sessionHolder[$name]['data']) {
							$sessionBlower .= "=*{".$sessionHolder[$key2]['short']."|".$sessionHolder[$key2]['timeToLive']."}*=".$sessionHolder[$key2]['data']."";
							$didNewS       = 1;
							//			}
						}
						else {
							unset($sessionHolder[$key2]);
						}
						//	}
					}
				}    // old data session information
			}

			if(basic::$cfg['auth']['dataDictionary'][$name]['encrypt']==1 && basic::$cfg['auth']['allowEncryptHoleShubang']!=1) {
				$data = $encryption->encode($data);
			}

			foreach(basic::$cfg['auth']['dataDictionary'] as $key4 => $value4) {
				if($key4==$name) {
					if(!$till || $till>0) {
						if($till) {
							$untilA = time() + $till;
						}
						else $untilA = time() + basic::$cfg['auth']['dataDictionary'][$key4]['timeToLive'];

						if(basic::$cfg['auth']['dataDictionary'][$key4]['method']=="cookie") {
							if(!isset($cookieHolder[$key4]) || !$cookieHolder[$key4] || $cookieHolder[$key4]['timeToLive']>time()) {
								$cookieBlower .= "=*{".basic::$cfg['auth']['dataDictionary'][$key4]['short']."|".$untilA."}*=".$data."";
								$didNewC      = 1;
							}
						}
						if(basic::$cfg['auth']['dataDictionary'][$key4]['method']=="session") {
							if(!isset($sessionHolder[$key4]) || !$sessionHolder[$key4] || $sessionHolder[$key4]['timeToLive']>time()) {
								$sessionBlower .= "=*{".basic::$cfg['auth']['dataDictionary'][$key4]['short']."|".$untilA."}*=".$data."";
								$didNewS       = 1;
							}
						}
					}
				}
			}    // new data cookie information

			if(!$till) {
				$untilB = time() + basic::$cfg['auth']['dataDictionary'][$name]['timeToLive'];
			}
			else $untilB = $till;

			if(basic::$cfg['auth']['allowEncryptHoleShubang']==1) {
				$cookieBlower  = $encryption->encode($cookieBlower);
				$sessionBlower = $encryption->encode($sessionBlower);
			}

			if(basic::$cfg['auth']['dataDictionary'][$name]['method']=="cookie") {
				if(!headers_sent()) {
					setcookie("data",$cookieBlower,$untilB,$domain);
				}

				$this->pastCookie[$name]['data']   = $data;
				$this->pastCookie[$name]['till']   = $untilB;
				$this->pastCookie[$name]['domain'] = $domain;
			}
			if(basic::$cfg['auth']['dataDictionary'][$name]['method']=="session") {
				$_SESSION['data']                   = $sessionBlower;
				$this->pastSession[$name]['data']   = $data;
				$this->pastSession[$name]['till']   = $untilB;
				$this->pastSession[$name]['domain'] = $domain;
			}
		}
		else {
			foreach(basic::$cfg['auth']['dataDictionary'] as $key => $value) {
				if($name==$key) {
					if($till) {
						$until = time() + $till;
					}
					else $until = time() + basic::$cfg['auth']['dataDictionary'][$key]['timeToLive'];

					if(basic::$cfg['auth']['dataDictionary'][$name]['encrypt']==1) {
						$data = $encryption->encode($data);
					}

					if(basic::$cfg['auth']['dataDictionary'][$name]['method']=="cookie") {
						if(!headers_sent()) {
							setcookie($name,$data,$until,$domain);
							if($till>0) {
								$this->setCSParameter($name,$data);
							}
							else $this->setCSParameter($name,"");

							//	if($data) {
							$this->setCSParameter($name,$data);
							//	}
						}
					}
					if(basic::$cfg['auth']['dataDictionary'][$name]['method']=="session") {

						if(!$till) {
							$till = basic::$cfg['auth']['dataDictionary'][$name]['timeToLive'];
						}

						$sessions = basic::load("sessions","functions");
						$sessions::startSession();

						if(!headers_sent() && !$this->regenerate) {    // so we won't regenerate every time we ask to set cookie/session

							$sessions::regenerateSession();

							$this->regenerate = true;
						}

						if($till>0) {
							$_SESSION[$name] = $till + time()."|".$data;
							$this->setCSParameter($name,$data);
						}
						else {
							unset($_SESSION[$name]);
							$this->setCSParameter($name,"");
						}
					}
				}
			}
		}
	}

	/**
	 * Set CS parameter - set cookies/sessions as class parameter
	 * @param string $name - name of cookies/session
	 * @param string $value - value of cookies/session
	 * @example $this->setCSParameter($name,$data);
	 */
	public function setCSParameter($name,$value) {

		$nameCS        = $name."CS";
		$this->$nameCS = $value;
	}

	/**
	 * Get cookie now - get cookies/sessions configured at $cfgAuth['dataDictionary']
	 * @param string $name - name of cookie/session
   * @return bool|mixed|string - value of cookie/session
   * @throws ReflectionException
	 * @example $this->getCookieNow($name="userId");
	 */
	public function getCookieNow($name) {

		$sessions = basic::load("sessions","functions");
		$sessions::startSession();
    $encryption = "";
    $cookiesData = "";
    $sessionData = "";

		if(basic::$cfg['auth']['dataDictionary'][$name]['encrypt']==1) {
			$encryption = basic::load("encryption","functions");
		}

		$dataGet                  = "";
		$cookiesDataFresh['data'] = "";
		$sessionDataFresh['data'] = "";
		if(basic::$cfg['auth']['allowShove']==1) {
			if(basic::$cfg['auth']['allowEncryptHoleShubang']==1) {
				$cookiesDataFresh['data'] = $encryption->decode($_COOKIE['data']);
				$sessionDataFresh['data'] = $encryption->decode($_SESSION['data']);
			}
			else {
				if(isset($_COOKIE['data'])) {
					$cookiesDataFresh['data'] = $_COOKIE['data'];
				}
				if(isset($_SESSION['data'])) {
					$sessionDataFresh['data'] = $_SESSION['data'];
				}
			}

			preg_match_all('/=\*{([^\|]+)\|([^(}\*=]+)}\*=([^=\*{]+)/',$cookiesDataFresh['data'],$matchesC);

			foreach($matchesC[1] as $key2 => $value2) {
				foreach(basic::$cfg['auth']['dataDictionary'] as $key3 => $value3) {
					if($matchesC[1][$key2]==basic::$cfg['auth']['dataDictionary'][$key3]['short']) {
						if($matchesC[2][$key2]>time()) {
							$cookiesData[$key3] = $matchesC[3][$key2];
						}    // is timeToLive is greater then current time?
					}
				}
			}    // export the data in $_COOKIE['data'] to many cookies

			preg_match_all('/=\*{([^\|]+)\|([^(}\*=]+)}\*=([^=\*{]+)/',$sessionDataFresh['data'],$matchesS);

			foreach($matchesS[1] as $key2 => $value2) {
				foreach(basic::$cfg['auth']['dataDictionary'] as $key3 => $value3) {
					if($matchesS[1][$key2]==basic::$cfg['auth']['dataDictionary'][$key3]['short']) {
						if($matchesS[2][$key2]>time()) {
							$sessionData[$key3] = $matchesS[3][$key2];
						}    // is timeToLive is greater then current time?
					}
				}
			}    // export the data in $_SESSION['data'] to many cookies

			if($name) {
				if(basic::$cfg['auth']['dataDictionary'][$name]['encrypt']==1 && basic::$cfg['auth']['allowEncryptHoleShubang']!=1) {
					if(isset($cookiesData[$name])) {
						$cookiesData[$name] = $encryption->decode($cookiesData[$name]);
					}
					if(isset($sessionData[$name])) {
						$sessionData[$name] = $encryption->decode($sessionData[$name]);
					}
				}
				if(basic::$cfg['auth']['dataDictionary'][$name]['method']=="cookie") {
					if(isset($cookiesData[$name])) {
						return $cookiesData[$name];
					}
					else return false;
				}
				if(isset($sessionData[$name]) && basic::$cfg['auth']['dataDictionary'][$name]['method']=="session") {
					if(isset($sessionData[$name])) {
						return $sessionData[$name];
					}
					else return false;
				}
			}
			else {
				return $cookiesData;
			}
		}
		else {

			$nameCS = $name."CS";
			if(isset($this->$nameCS) && $this->$nameCS) {
				return $this->$nameCS;
			}
			else {

			}

			if(basic::$cfg['auth']['dataDictionary'][$name]['method']=="cookie") {
				if(isset($_COOKIE[$name])) {
					$dataGet = $_COOKIE[$name];
				}
				//	else $dataGet = "";
			}
			if(basic::$cfg['auth']['dataDictionary'][$name]['method']=="session") {
				$sessions = basic::load("sessions","functions");
				$sessions::startSession();

				if(isset($_SESSION[$name])) {
					$sessionData = explode("|",$_SESSION[$name]);
				}
				else $sessionData = array(0 => "",2 => "");

				if($sessionData[0]>time()) {
					$dataGet = $sessionData[1];
				}
				else {
					unset($_SESSION[$name]);
					//		$dataGet = "";
				}
			}

			if(basic::$cfg['auth']['dataDictionary'][$name]['encrypt']==1) {
				$dataGet = $encryption->decode($dataGet);
			}

			return $dataGet;
		}
	}

	/**
	 * Check ip on cookie - checking if current ip is what we have on cookie
   * @return bool
   * @throws ReflectionException
	 * @example $this->checkIpOnCookie();
	 */
	public function checkIpOnCookie() {

		$ip = $_SERVER['REMOTE_ADDR'];

		if($this->getCookieNow($name = "connected")==$ip) {
			return true;
		}
		else return false;
	}

	/**
	 * Check self - checking if PHP_SELF is from the same site
	 * @return boolean
	 * @example $this->checkSelf();
	 */
	public function checkSelf() {

		$self = filter_var($_SERVER['PHP_SELF'],FILTER_SANITIZE_STRING);

		$posLastBreakerSelf = strrpos($self,"/");
		$afterBreakerSelf   = substr($self,$posLastBreakerSelf + 1);
		$posFirstQwSelf     = strrpos($afterBreakerSelf,"?");
		if($posFirstQwSelf) {
			$selfScriptName = substr($afterBreakerSelf,0,$posFirstQwSelf);
		}
		else $selfScriptName = $afterBreakerSelf;

		if($selfScriptName=="index.php") {
			return true;
		}
		else return false;
	}

	/**
	 * Check referer - checking if HTTP_REFERER is from the same site
	 * @return boolean
	 * @example $this->checkReferer();
	 */
	public function checkReferer() {

		$referer = myHelpers::getServerReferer();

		if($referer) {
			$posLastBreakerReferer = strrpos($referer,"/");
			$afterBreakerReferer   = substr($referer,$posLastBreakerReferer + 1);
			$posFirstQwReferer     = strrpos($afterBreakerReferer,"?");
			if($posFirstQwReferer) {
				$refererScriptName = substr($afterBreakerReferer,0,$posFirstQwReferer);
			}
			else $refererScriptName = $afterBreakerReferer;
		}
		else {
			$refererScriptName = "";
		}

		if($refererScriptName==basic::$cfg['auth']['authFileName']) {
			return true;
		}
		else return false;
	}

	/**
	 * Check token - check if token exist
	 * @return boolean
   * @throws ReflectionException
	 * @example $this->checkToken();
	 */
	public function checkToken() {

		if(basic::$cfg['auth']['tokenCheck']==1) {
			if($this->getCookieNow($name = "token") + basic::$cfg['auth']['timeout']>=time()) {
				return true;
			}
			else {
				return false;
			}
		}

		return true;
	}

	/**
	 * Check session - checking if we have valid session - if ip is the same
	 * @return boolean
   * @throws ReflectionException
	 * @example $this->checkSession();
	 */
	public function checkSession() {

		if(!$this->checkIpOnCookie() && $this->getCookieNow($name = "validation")) {    // if not valid erase cookies
			$this->revokeAll();

			if($this->checkSelf()) {
				//	Header("Location: ".basic::$cfg['auth']['authFileName']."?op=login&alert=7002");	die();
				return false;
			}
		}

		return true;
	}    // not used yet

	/**
	 * Captcha verify code - check the verification code and issue alert if wrong
	 * @return boolean/integer
   * @throws ReflectionException
	 * @example $this->captchaVerifyCode($at="ForgotMyPassword");
	 */
	public function captchaVerifyCode($at = "Login") {

		$verificationCode = "verification".$at."Code";

		if(empty($this->$verificationCode)) {
			return 7009;    // blank
		}

		$verificationCodeCS = $this->getCookieNow($name = "verification".$at."Code");

		if(!$verificationCodeCS) {
			return 7010;    // timeout
		}

		$verificationCode         = md5($this->$verificationCode);
		$verificationTriesCS      = $this->getCookieNow($name = "verification".$at."Tries");
		$timeLimitVerification    = explode("-",$verificationTriesCS);
		$timeLimitVerification[0] = (int)$timeLimitVerification[0];

		if(isset($timeLimitVerification[1]) && $timeLimitVerification[1]>=time()) {
			return 7011;    // too many tries
		}

		if($timeLimitVerification[0]>basic::$cfg['auth']['captcha']['verificationTriesLimit']) {
			Header("Location: ".basic::$cfg['auth']['authFileName']."?op=holding");
			die();
			$this->redirectInside($address = "holding",$notif = 7009);

			return 7011;    // too many tries
		}
		else {
			if($verificationCode!=$verificationCodeCS || empty($verificationCodeCS) || !isset($verificationCodeCS)) {
				return 7008;    // verification is wrong
			}
			else {
				$this->setCookieNow($name = "verification".$at."Code",$data = "",$till = -3600,$domain = "");
				$this->setCookieNow($name = "verification".$at."Tries",$data = "",$till = -3600,$domain = "");

				return false;    // ok pass
			}
		}
	}

  /**
   * Csrf Token - attach it to the view file, example: <?=$this->auth->csrfToken()?>
   * @return string
   * @throws ReflectionException
   */
  public function csrfToken() {

    if(!$this->csrfToken) {
      $this->csrfToken = $this->setCsrfToken();
    }
	  $inputRow = '<input type="hidden" id="csrfToken" name="csrfToken" class="csrfToken" value="'.$this->csrfToken.'">';

	  return $inputRow;
  }

  /**
   * Set csrf token
   * @return bool|mixed|string
   * @throws ReflectionException
   */
  private function setCsrfToken() {

    $csrfToken = $this->getCookieNow("csrfToken");

  //  p($csrfToken);

    if(!$csrfToken) {
      //Generate a secure token using openssl_random_pseudo_bytes.
      $csrfToken = bin2hex(openssl_random_pseudo_bytes(24));
      $this->setCookieNow($name = "csrfToken",$data = $csrfToken,$till = 3600,$domain = "");
    }

    return $csrfToken;
  }

  /**
   * Check csrf token
   * @return bool|string
   * @throws ReflectionException
   */
  public function checkCsrfToken() {

    $csrfCookie = $this->getCookieNow($name = "csrfToken");

    if(!empty($_POST['csrfToken']) && $csrfCookie===$_POST['csrfToken']) {
      $this->setCookieNow($name = "csrfToken",$data = $csrfCookie,$till = 3600,$domain = "");   // extending cookie time

      return true;
    }
    else {
      return false;
    }
  }
}

