<?php

/*************************************************************
 * Basic - method to load and initiate classes if they are not loaded yet
 * All classes/files will be load from labYard directory - so they must exist there
 * @example $this->load("aaa");                 // load from labYard (I) - from controller/model
 * @example $this->aaa = basic::load("aaa");    // load from labYard (II) - from any other + controller/model
 * classes/files that are located at "core" directory don't have to be included - because they already are at: "bootstrap"
 * @author    S M Kessner <limbercode@gmail.com>
 * @copyright 2010-2018 T2
 * @link      http://limbercode.com
 *************************************************************/
class basic {

  public static $defaultItinerary = "origin";
  public static $currentItinerary;
  public static $loaded = array();
  public static $cfg;

  /**
   * Constructor for Basic
   */
  public function __construct() {

    if(!file_exists(CONFIGPATH.basic::$defaultItinerary)) {
      echo "Error 82320: ".CONFIGPATH.basic::$defaultItinerary." directory is missing!";
      die();
    }

    basic::getItinerary();
    basic::loadCfg("config");        // load site config file
    basic::loadCfg("authConfig");    // load site auth file, have to load before session starts
  }

  /**
   * Itinerary - get the correct itinerary according to host ip
   */
  public static function getItinerary() {

    if(!isset(self::$currentItinerary)) {
      basic::loadCfg("itineraryConfig");
    }
    else {
      return self::$currentItinerary;
    }

    if(isset(basic::$cfg['itinerary']['domain'])) {
      foreach(basic::$cfg['itinerary']['domain'] as $key => $value) {
        if($_SERVER['SERVER_NAME']==$key) {
          basic::$currentItinerary = $value;
        }
      }
    }

    if(isset(basic::$cfg['itinerary']['ip'])) {
      foreach(basic::$cfg['itinerary']['ip'] as $key => $value) {
        if($_SERVER['SERVER_ADDR']==$key) {
          basic::$currentItinerary = $value;
        }
      }
    }

    if(!basic::$currentItinerary) {
      basic::$currentItinerary = basic::$defaultItinerary;
    }

    if(!is_dir(CONFIGPATH.basic::$currentItinerary)) {
      echo "Error 82320: ".CONFIGPATH.basic::$currentItinerary." directory is missing!";
      die();
    }
  }

  /**
   * Loads specific class
   * @param string $class - required - the class/file name
   * @param string $directory - optional - the directory that contain the class/file inside "labYard" directory
   * @param string|array $params - optional - parameters to pass to the class
   * @return mixed
   * @throws ReflectionException
   */

  public static function &load($class,$directory = "functions",$params = "") {

    static $storedClasses = array();

    if(isset($storedClasses[$class])) {        // if class already exist we don't need to continue
      self::logLoaded("class",$class,true);

      return $storedClasses[$class];
    }

    $name = self::requireClass($class,$directory);

    if($name===false) {
      $callers     = debug_backtrace();
      $callerClass = $callers[0]['class'];
      $lineClass   = $callers[0]['line'];

      $notif = basic::load("notif","core");
      $notif->genNotifByNum($file = "core",81200,array($class,$callerClass,$lineClass));

      if(function_exists("notifNum")) {
        $notif->genNotifByNum($file = "core",81200,array($class,$callerClass,$lineClass));
      }
      else {
        if(myHelpers::isEnv($env = "dev")) {
          // trigger_error("Error (code 81200) on '".$class."' called from: '".$callerClass."' class on line ".$lineClass,E_USER_ERROR);     // may force second render!!! no need to do it since notif log file can be saved
          //	die();
        }
      }

      return $storedClasses[$class];
    }
    else {
      self::logLoaded("class",$class);
      if($params) {   // will not work otherwise
        $storedClasses[$class] = call_user_func_array(array(new ReflectionClass($class),'newInstance'),$params);
      }
      else {
        $storedClasses[$class] = new $name();    // init the class we want
      }

      return $storedClasses[$class];
    }
  }

  /**
   * New instance - load new instance of a class
   * @param string $class - required - the class/file name
   * @param string $directory - optional - the directory that contain the class/file inside "labYard" directory
   * @param string $params - optional - parameters to pass to the class
   * @return string
   * @throws ReflectionException
   */
  public static function newInstance($class,$directory = "functions",$params = "") {

    $class = ucfirst(strtolower($class));
    $name  = "";     // just a reset

    if(!isset($storedClasses[$class])) {        // if class already exist we don't need to requireClass
      $name = self::requireClass($class,$directory);
    }

    $classContent = "";     // just a reset
    if($name!==false) {
      $classContent = new $class($params);
      self::logLoaded("class",$class);
    }
    else {
      $callers     = debug_backtrace();
      $callerClass = $callers[0]['class'];
      $lineClass   = $callers[0]['line'];

      $notif = basic::load("notif","core");
      $notif->genNotifByNum($file = "core",81205,array($class,$callerClass,$lineClass));

      if(function_exists("notifNum")) {
        $notif->genNotifByNum($file = "core",81205,array($class,$callerClass,$lineClass));
      }
      else {
        if(myHelpers::isEnv($env = "dev")) {
          // trigger_error("Error (code 81205) on '".$class."' called from: '".$callerClass."' class on line ".$lineClass,E_USER_ERROR);     // may force second render!!! no need to do it since notif log file can be saved
          //	die();

          // Note: We use die() rather then show_error() in order to avoid a
          // self-referencing loop with the Excptions class
        }
      }
    }

    return $classContent;
  }

  /**
   * Require class - add the file to the memory
   * @param string $class the class/file name
   * @param string $directory the directory that contain the class/file inside "labYard" directory
   * @return bool|string
   */
  private static function requireClass($class,$directory) {

    if(!$directory) {
      $libraryDirectory = LABYARDPATH;
    }
    else {
      $directory        = str_replace(array('/','\\'),DS,$directory);        // fix DS issues
      $libraryDirectory = LABYARDPATH.$directory;
    }

    $fileWithDirectory = "";
    if(file_exists($libraryDirectory.DS.$class.".php")) {                // catch the file name of the class
      $fileWithDirectory = $libraryDirectory.DS.$class.".php";
    }
    elseif(file_exists($libraryDirectory.DS.$class.".class.php")) {        // catch the file name of the class with ".class"
      $fileWithDirectory = $libraryDirectory.DS.$class.".class.php";
    }

    $name = false;

    if($fileWithDirectory) {
      if(class_exists($name)===false) {
        require_once($fileWithDirectory);
        $name = $class;
      }
    }

    return $name;
  }

  /**
   * Loads the specific config file
   * @param string $fileName the config file name
   * @param boolean $allowGlobal allow config to be on global basic::$cfg
   * @param string $type the type of config to load - can be file/xml/db (currently only file is working)
   * @return array of the config stuff
   * @throws ReflectionException
   */
  public static function loadCfg($fileName = "config",$allowGlobal = true,$type = "file") {

    if(strpos($fileName,"Config")===false && $fileName!="config") {
      $fileName = $fileName."Config";   // don't allow writing "nameConfig"
    }
    $varName         = str_replace("Config","",$fileName);
    $varName         = str_replace("config","",$varName);
    $originalVarName = $varName;

    self::logLoaded("config",$fileName);

    if($type=="file") {
      $fileToRequire        = CONFIGPATH.basic::$currentItinerary.DS.$fileName.".php";
      $fileToRequireDefault = CONFIGPATH.basic::$defaultItinerary.DS.$fileName.".php";
      $notLoaded            = "";

      if(file_exists($fileToRequire)) {
        require($fileToRequire);
      }
      elseif(file_exists($fileToRequireDefault)) {
        require($fileToRequireDefault);
      }
      else {
        basic::load("notif","core");
        notif::sendNotif($notifNum = 81190,$externalVar = "",$fileName,$fileToRequireDefault);
        $notLoaded = 1;
      }
    }

    if($fileName=="config") {
      self::$cfg = $cfg;      // will be loaded from included file

      return $cfg;
    }
    else {
      if(!$notLoaded) {
        if(!isset($cfg)) {
          basic::load("notif","core");
          notif::sendNotif($notifNum = 82300,$externalVar = $varName,$fileName,$fileToRequireDefault);
        }
        if(!isset($cfg[$varName])) {
          basic::load("notif","core");
          notif::sendNotif($notifNum = 82310,$externalVar = $varName,$fileName,$fileToRequireDefault);
        }

        if($allowGlobal!==false && !is_array(self::$cfg)) {
          return self::$cfg = $cfg;
        }
        elseif($allowGlobal!==false && !in_array($varName,self::$cfg)) {    // checks if we already included this config file
          return self::$cfg = array_merge(self::$cfg,$cfg);
        }
        else {
          return $cfg;
        }
      }
    }
  }

  /**
   * Log loaded config/class names
   * @param $type config/class
   * @param $name config/class name
   * @param bool $reloaded decides if we already loaded this before
   */
  private static function logLoaded($type,$name,$reloaded = false) {

    if($type=="class" || $type=="config") {
      $name   = lcfirst($name);
      $newKey = (int)0;
      $marker = "";

      if($type=="class") {
        $marker = "*";
      }
      if($type=="config") {
        $marker = "+";
      }
      if($reloaded!==false) {
        $marker = "+";
      }

      if(isset(self::$loaded[$type])) {
        $lastValue = end(self::$loaded[$type]);
        $lastKey   = (int)key(self::$loaded[$type]);
        $search    = in_array(lcfirst($name),self::$loaded[$type]);
        $newKey    = $lastKey + 1;
      }

      if(!empty($search)) {
        self::$loaded[$type][($newKey).$marker] = $name;
      }
      else {
        self::$loaded[$type][($newKey)] = $name;
      }
    }
  }

  /*
   * Get public directory of the server
   * @param bool $returnFull if true will return the full path with domain
   * @return string - will return the path of the public directory on server
   */
  public static function getPublicDirectory($returnFull = false) {

    $leadingDirectory = basic::$cfg['site']['directory']['main'];
    $leadingDirectory = $leadingDirectory ? $leadingDirectory : $_SERVER['DOCUMENT_ROOT'];

    $systemDrive = strtolower(GetEnv("SystemDrive"));   // if running on windows server and have drive letter on the directory path - we need to remove it

    if($systemDrive) {
      $leadingDirectory = str_replace($systemDrive,"",$leadingDirectory);
    }

    if($returnFull===true) {
      $directory = $_SERVER['SERVER_NAME'].$leadingDirectory.DS."public".DS;
    }
    else {
      $directory = $leadingDirectory.DS."public".DS;
      //  $directory = DS.$leadingDirectory.DS."public".DS;   // maybe that's how it should be
    }

    $directory = str_replace(array('/','\\'),DS,$directory);  // will replace wrong directory separator
    $directory = str_replace(DS.DS,DS,$directory);  // removes duplicated directory separator

    return $directory;
  }

  /*
   * File path of public directory of the server
   * @param string $fileName
   * @return string - will return the file name with full path
   */
  public static function filePathOfPublicDirectory($fileName) {

    $directory = basic::getPublicDirectory().$fileName;
    $directory = str_replace("public".DS."public".DS,"public".DS,$directory);  // removes duplicated "public" directory
    $directory = str_replace(array('/','\\'),DS,$directory);  // will replace wrong directory separator

    return $directory;
  }

  /**
   * Get config parameter
   * accepts string, example: basic::getConfig("db","main","activate")); => will get: $cfg['db']['main']['activate']
   * @return mixed|string
   * @throws ReflectionException
   */
  public static function getConfig() {

    $arrayOfParams   = [];
    $requestedConfig = "";

    foreach(func_get_args() as $param) {
      array_push($arrayOfParams,$param);
    }

    $arrayOfParamsLength = count($arrayOfParams);

    if($arrayOfParamsLength==1) {
      $requestedConfig = basic::$cfg[$arrayOfParams[0]];
    }
    if($arrayOfParamsLength==2) {
      $requestedConfig = basic::$cfg[$arrayOfParams[0]][$arrayOfParams[1]];
    }
    if($arrayOfParamsLength==3) {
      $requestedConfig = basic::$cfg[$arrayOfParams[0]][$arrayOfParams[1]][$arrayOfParams[2]];
    }
    if($arrayOfParamsLength==4) {
      $requestedConfig = basic::$cfg[$arrayOfParams[0]][$arrayOfParams[1]][$arrayOfParams[2]][$arrayOfParams[3]];
    }
    if($arrayOfParamsLength==5) {
      $requestedConfig = basic::$cfg[$arrayOfParams[0]][$arrayOfParams[1]][$arrayOfParams[2]][$arrayOfParams[3]][$arrayOfParams[4]];
    }

    if(isset($requestedConfig)) {
      return $requestedConfig;
    }
    else {
      $callers     = debug_backtrace();
      $callerClass = $callers[1]['class'];
      $lineClass   = $callers[0]['line'];

      $configName = implode("']['",$arrayOfParams);
      $configName = "\$cfg['".$configName."']";

      $notif = basic::load("notif","core");
      $notif->genNotifByNum($file = "core",81195,array($configName,self::$currentItinerary,$callerClass,$lineClass));
    }
  }
}
