<?php

/*************************************************************
 * Db Informant
 * - db ui information
 * - helps monitoring model with db live information on screen
 * @author    S M Kessner <limbercode@gmail.com>
 * @copyright 2010-2018 T2
 * @link      http://limbercode.com
 *************************************************************/
class dbInformant {

	public $dbInformantContainer;
	static public $keepForOutBurst;
	static public $dbUiContainerRun;
	static public $disableUi;
	static private $allowDbUi;

	/**
	 * Constructor for dbUi control
	 */
	public function __construct() {

		if(myHelpers::isEnv($env = "dev") && (basic::$cfg['errorControl']['dbUi']['allow']==1 || basic::$cfg['errorControl']['dbUi']['allow']==2)) {
			dbInformant::$allowDbUi = true;
		}
		elseif(basic::$cfg['errorControl']['dbUi']['allow']==1) {
			$auth = basic::load("auth","core");

			if($auth->isDevUser()) {
				dbInformant::$allowDbUi = true;
			}    // for dev user
			else {
			}    // for regular user
		}    // live environment
	}

	/**
	 * Enable disable show ui container
	 */
	public function showContainer() {

		if(dbInformant::$allowDbUi!=false) {
			$dbInformantContainer = dbInformant::showDbUiContainer();

			if(basic::$cfg['errorControl']['dbUi']['afterHtml']==0) {
				echo $dbInformantContainer;
				dbInformant::$dbUiContainerRun = true;
			}
			$this->dbInformantContainer = $dbInformantContainer;
		}
	}

	/**
	 * Show ui container
	 * @param bool $loadCssAndJs
	 * @return bool|string
	 */
	static public function showDbUiContainer($loadCssAndJs=false) {

		if(myHelpers::isAjax() || dbInformant::$dbUiContainerRun || dbInformant::$disableUi) {
			return false;      // don't run this twice
		}

		$dbUiHtml = "";

		if($loadCssAndJs!==false) {
			$dbUiHtml .= "<link rel='stylesheet' property='stylesheet' href='/public/css/du.css' type='text/css'>";
			$dbUiHtml .= "<script type='text/javascript' src='/public/js/du.js'></script>";
		}

		$dbUiHtml .= "<div id=\"dbUiButton\" style=\"display:none;\" onclick=\"showDbUi();\">db</div>
			<div id=\"dbUi\" class=\"dbUiNormal centerUi\" style=\"display:none;\">
				<div id=\"dbUiSideInfo\">
				<a href=\"javascript:hideDbUisOnPage()\"><span id=\"showHideDbUiButton\">Show</span> all queries on page</a>
				<br><a id=\"fixedOrAbsoluteDbUi\" href=\"javascript:fixedPositionDbUi();\">Make this window fixed</a>
				<br><a href=\"javascript:newWindowDbUi()\">Open in new window</a>
				<br><a href=\"javascript:pauseAjaxDbUi()\" id=\"pauseDbUisText\">Pause dbUi alerts</a>
				<br><a href=\"javascript:invertColorsDbUi()\">Invert Colors</a>
				<br><a href=\"javascript:clsQueries()\">Clear queries</a>
				<br>Text sizer:
				<a id=\"biggerTextDbUi\" href=\"javascript:biggerTextDbUi()\">T+</a>
				<a id=\"smallerTextDbUi\" href=\"javascript:smallerTextDbUi()\">T-</a>&nbsp;
				<br>Total of: <b><span id=\"totalDbUis\"></span></b> queries
				<br>Time wasted: <b><span id=\"totalTimeDbUis\"></span></b> to run
				<br><span>Ip: ".myHelpers::getUserIp()."&nbsp;</span>
				</div>
			</div>
		";

		return $dbUiHtml;
	}

	/**
	 * Show div dbUi
	 * @param string $query - the db query
	 * @param array $params - the params used
	 * @param string $timing - query running timing
	 * @param array $result - result of query
	 * @return bool|string
	 */
	public function showDbUi($query,$params,$timing,$result) {

		if(!dbInformant::$allowDbUi || !$query || dbInformant::$disableUi) {
			return false;
		}

		if(!$params) {
			$params = "No params defined";
		}
		else {
			if(!is_array($params)) {
				$notif = basic::load("notif","core");
				$notif->genNotifByNum("core",$num = 88100,'$params');

				return false;
			}
			if(count($params)!=count($params,COUNT_RECURSIVE)) {
				$notif = basic::load("notif","core");
				$notif->genNotifByNum("core",$num = 88110,'$params');

				return false;
			}
			else {
				$params = implode(', ',array_map(function ($v,$k) {

					return $k.' => '.$v;
				},$params,array_keys($params)));
			}
		}

		if(!$result) {
			$result = "Result is empty";
		}
		else {
			if(is_array($result)) {
				$result = array_slice($result,0,10);
			}
		}    // limit results to 10

		$function = $result['locationInfo']['calledFromFunction'];
		$class = $result['locationInfo']['calledFromClass'];
		$file = $result['locationInfo']['calledFromFile'];
		$line = $result['locationInfo']['calledFromLine'];
		unset($result['locationInfo']);

		$errorNote = "";
		if(isset($result['executedData']['error'])) {
			$errorNote = "<span class=\"flash\">MySql Error</span>";
		}

		$file = str_replace(ROOT,"",$file);

		$hideClassesOutput = array("router","notif","statistics");  //"pdoHelper","advancedController"

		$hideOutput = false;
		if(in_array($class,$hideClassesOutput)) {
			$hideOutput = "style=\"display:none; $class\"";
		}

		$createdAt     = date(basic::$cfg['log']['dateFormat']." ".basic::$cfg['log']['timeFormat'],$_SERVER['REQUEST_TIME']);
		$randomNumber = rand(10000,99999);

    $dbUiOutputHtml = "";

		if(basic::$cfg['errorControl']['dbUi']['allowShowOnPage']!=false) {
      $dbUiOutputHtml .= "<div id=\"dbUiOnPage-".$randomNumber."\" style=\"display:none;\" class=\"dbUiOnPage\">".$createdAt." - [".$timing."] - ".$query." - [".$params."]</div>";        // display:none must stay here cz css file is loading slower then this
			// was: echo "<div id=\"dbUiOnPag...
		}

		$backTraceIndividual  = $this->printDebugBacktrace();
		$dbUiOutputIndividual = "<br><div id=\"backTrace-$randomNumber\" class=\"backTrace normalPattern\">$backTraceIndividual</div><div id=\"dbUiResults-$randomNumber\" class=\"dbUiResults normalPattern\"><pre>".print_r($result,true)."</pre></div>";

		$dbUiOutputHtml .= "<div id=\"dbUiDiv-".$randomNumber."\" style=\"display:none;\" class=\"dbUiNum\" style='color:red;'>
			<div id=\"toggleDbUi-".$randomNumber."\" class=\"dbUiNumRest\" ".$hideOutput." style='color:red;'>";

		$dbUiOutputHtml .= "<table class=\"dbUiQueryTable\">";
		$dbUiOutputHtml .= "<tr><th class=\"dbUiQueryTableThNoBold\" colspan=\"3\">Called from: ".$file." on line:  ".$line.", function: ".$class.DS.$function."</th></tr>";
		$dbUiOutputHtml .= "<tr><th class=\"dbUiQueryTableTh\">Date+Time</th><th class=\"dbUiQueryTableTh\">Query</th><th class=\"dbUiQueryTableTh\">Params</th><th class=\"dbUiQueryTableTh\">Time (ms)</th></tr>";
		$dbUiOutputHtml .= "<tr><td class=\"dbUiQueryTableTd1\">".$createdAt."</td><td class=\"dbUiQueryTableTd2\">".$query."</td><td class=\"dbUiQueryTableTd3\">".$params."</td><td class=\"dbUiQueryTableTd4 dbUiTiming\">".$timing."</td></tr>\n";
		$dbUiOutputHtml .= "</table>";
		$dbUiOutputHtml .= "</div>
	<div class=\"dbUiNumRest\"><a href=\"javascript:showHideStuff(".$randomNumber.",'dbUiResults')\"><span id=\"showHideDbUiResultButton-".$randomNumber."\">Show</span> DB results</a>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<a href=\"javascript:showHideStuff(".$randomNumber.",'backTrace')\">Back Trace</a>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<a href=\"javascript:hideDbUisOnPage(".$randomNumber.")\"><span id=\"showHideDbUiButton-".$randomNumber."\">Show</span> db query on page</a> ".$errorNote." 
	".$dbUiOutputIndividual."
	</div>

	</div>
	<script>
		document.addEventListener('DOMContentLoaded', function(event) {
			var showDbUi = document.getElementsByClassName('dbUiNum');
			for(var i = 0; i < showDbUi.length; i++) {
				showDbUi[i].style.display='block';
			}
		});
	</script>
	";        // js here to show dbUi on page that won't load the "dbUiButton" from some reason

    dbInformant::$keepForOutBurst .= $dbUiOutputHtml;   // .= can be more then 1
	}

	/**
	 * Print debug back trace - output backtrace
	 * @return string
	 */
	public static function printDebugBacktrace() {

		$stacks       = debug_backtrace();
		$randomNumber = rand(10000,99999);

		$output = "<table id=\"mustPutId-$randomNumber\" class=\"backTraceTable\">";    // mustPutId for initMovingTables js function, although we dont use it here.. (have to fix this laer)
		$output .= "<tr><td><strong></strong></td><td><strong>File</strong></td><td><strong>Line</strong></td><td><strong>Class</strong></td><td><strong>Function</strong></td></tr>";

		foreach($stacks as $key => $value) {
			if(!isset($stacks[$key]['file'])) {
				$stacks[$key]['file'] = '[PHP Kernel]';
			}
			if(!isset($stacks[$key]['line'])) {
				$stacks[$key]['line'] = '';
			}
			if(!isset($stacks[$key]['class'])) {
				$stacks[$key]['class'] = '';
			}

			$output .= "<tr><td>".($key + 1).")</td><td><div class=\"backTraceDiv\">{$stacks[$key]["file"]}</div></td><td>{$stacks[$key]["line"]}</td><td>{$stacks[$key]["class"]}</td><td>{$stacks[$key]["function"]}</td></tr>";
		}
		$output .= "</table>";

		return $output;
	}
}

