<?php

/*************************************************************
 * Error control class
 * - all about error control
 * - error alert interface - reduce the need to use error.log by 90%
 * @author    S M Kessner <limbercode@gmail.com>
 * @copyright 2010-2018 T2
 * @link      http://limbercode.com
 *************************************************************/
class errorControl {

	public $errorControlContainer;
	private $allowErrorUi;
	private $alertContainerRun;
	private $db;

	/**
	 * Constructor for error control
	 */
	public function __construct() {

		if(myHelpers::isEnv($env = "dev") && (basic::$cfg['errorControl']['errorReportUi']['allow']==1 || basic::$cfg['errorControl']['errorReportUi']['allow']==2)) {
			$this->allowErrorUi = true;
			$this->enableErrorDisplay();
		}
		elseif(basic::$cfg['errorControl']['errorReportUi']['allow']==1) {
			$auth = basic::load("auth","core");

			if($auth->isDevUser()) {
				$this->allowErrorUi = true;
				$this->enableErrorDisplay();
			}    // for dev user
			else {
				$this->disableErrorDisplay();
			}    // for regular user
		}    // live environment
		else {
			$this->disableErrorDisplay();
		}    // for regular user
	}

	/**
	 * Enable disable show ui container
	 */
	public function showContainer() {

		if($this->allowErrorUi!==false) {
			$errorControlContainer = $this->showAlertContainer();
			if(basic::$cfg['errorControl']['errorReportUi']['afterHtml']==0) {
				echo $errorControlContainer;
			}
			$this->errorControlContainer = $errorControlContainer;
		}
	}

	/**
	 * Disable error display
	 */
	public function disableErrorDisplay() {

		$this->errorHandler();
		ini_set('display_errors',0);    // remove the error from screen			// it's here to catch errors on auth class
		register_shutdown_function(array(
			$this,
			'callRegisteredShutdownLive'
		));    // it's here to catch errors on auth class
	}

	/**
	 * Enable error display
	 */
	public function enableErrorDisplay() {

		$this->errorHandlerDivAlerts();
		error_reporting(0);                // fix that error won't be written to php_error_log
		ini_set('display_errors',0);       // remove the error from screen			// it's here to catch errors on auth class
		register_shutdown_function(array(
			$this,
			'callRegisteredShutdownDev'
		));    // it's here to catch errors on auth class
		//	error_reporting(E_ALL & ~E_NOTICE | E_STRICT);							// add the error from screen
	}

	/**
	 * Call registered shutdown dev - when we are on "dev" environment
	 */
	public function callRegisteredShutdownDev() {

		$error = error_get_last();

    if(!empty($error)) {
      if($error['type']===1 || $error['type']===4 || $error['type']===16 || $error['type']===64 || $error['type']===256) {    // to activate only on FATAL errors

        self::errorLogOnFile($errorOutputLog = $error,$fatal = true);
        echo self::showAlertContainer($fatal = true);
        $this->errorLoggerOnDbAndMail($error,$fatal = true);

        $this->showDivAlert($error['type'],$error['message'],$error['file'],$error['line'],$fatal = 1);
      }
    }
	}

	/**
	 * Call registered shutdown live - when we are on "live" environment this is the error the user gets
	 */
	public function callRegisteredShutdownLive() {

		$error = error_get_last();

		if($error['type']===1 || $error['type']===4 || $error['type']===16 || $error['type']===64 || $error['type']===256) {    // to activate only on FATAL errors

			self::errorLogOnFile($errorOutputLog = $error,$fatal = true);
			$this->errorLoggerOnDbAndMail($error,$fatal = true);

			echo "<html><head></head><body>";
			echo "Error on page - please contact the site manager...";
			echo "</body></html>";
		}
	}

	/**
	 * Fatal error logger - log it on db and sends mail to administrator
	 * @param array $error - error number + information like: file,line,type...
	 * @param boolean $fatal - is it a fatal error or not
	 * @example $this->errorLoggerOnDbAndMail($error);
   * @throws ReflectionException
	 */
	public function errorLoggerOnDbAndMail($error,$fatal = false) {

		if($fatal==true) {
			$errorType = "fatalErrors";
			$table     = "log_errors_fatal";
		}
		else {
			$errorType = "nonFatalErrors";
			$table     = "log_errors_not_fatal";
		}

		if(self::checkIfDbError($error['message'])) {
			$errorType = "dbErrors";
			$table     = "log_errors_db";
		}

		$foundRecentErrorOndb = "";    // just a reset
		$date                 = date(basic::$cfg['log']['dateFormat']." ".basic::$cfg['log']['timeFormat'],time());
		if(!isset(basic::$cfg['errorControl']['sendErrorByMailInterval'])) {
			basic::$cfg['errorControl']['sendErrorByMailInterval'] = 0;
		}
		$sqlDateCheck            = date(basic::$cfg['db']['dateFormat']." ".basic::$cfg['db']['timeFormat'],time() - 60 * basic::$cfg['errorControl']['sendErrorByMailInterval']);    // same error will be sent only after 5 minute interval
		$error['typeTranslated'] = self::translateErrorNumber($error['type'])." (".$error['type'].")";
		$error['createdAt']       = $date;

		if(isset(basic::$cfg['errorControl'][$errorType]['logMethod']) && in_array("db",basic::$cfg['errorControl'][$errorType]['logMethod'])) {
      if(basic::getConfig("db","main","activate")==true) {
        $this->db = basic::load(basic::getConfig("db","main","adapter"),'core/adapters');

        $dataCheck['type']     = $error['typeTranslated'];
        $dataCheck['file']     = $error['file'];
        $dataCheck['line']     = $error['line'];
        $dataCheck['createdAt'] = $sqlDateCheck;

        $this->db->pdo();   // to init the pdo

        if(!empty($this->db->pdo)) {
          $sel = $this->db->pdo->prepare("SELECT * FROM ".basic::getConfig("db","main","tablePrefix").$table." WHERE type=:type AND file=:file AND line=:line AND createdAt>:createdAt");
          $sel->execute($dataCheck);
          $sel->setFetchMode(PDO::FETCH_OBJ);
          $foundRecentErrorOndb = $sel->rowCount();    // check if the same error occurred recently
        }
      }
		}

		if(!$foundRecentErrorOndb) {
			$error['backtrace'] = $this->printDebugBacktrace();
			$this->sendMailNotificationError($error,$fatal);
		}    // check if the same error occurred recently - if not continue.. send the mail
	}

	/**
	 * Check error log for 20 last rows - checks if we have the same error logged within close time frame
	 * @param $error
	 * @return bool - true - send the mail, false - do not send the mail
   * @throws Exception
	 */
	static function checkErrorLogForRow($error) {

		$errorMatch = "";
		if(!isset(basic::$cfg['errorControl']['sendErrorByMailInterval'])) {
			basic::$cfg['errorControl']['sendErrorByMailInterval'] = 0;
		}
		$lastRowsInLog = myHelpers::tailFileToArray($filePath = IMPERMANENTPATH."logs".DS."php-error-db.log",$rows = 20);

		if(is_array($lastRowsInLog)) {
			foreach($lastRowsInLog as $row) {
				preg_match('/((.*?\]){3})(.+)$/',$row,$errorMatchPre);

				if(isset($errorMatchPre[3])) {
					preg_match('(\[.*?\])',$row,$createdAtMatch);
					$errorMatch    = substr($errorMatchPre[3],8);       // remove the word "error" from start
					$createdAtMatch = substr(substr($createdAtMatch[0],1),0,-12);
					$createdAtMatch = str_replace("-","",$createdAtMatch);
					$createdAtMatch = str_replace("/","-",$createdAtMatch);

					$date           = new DateTime($createdAtMatch,new DateTimeZone(basic::$cfg['site']['timeZone']));
					$timestampMatch = $date->getTimestamp();
				}

				if($errorMatch!=(is_array($error) ? $error['message'] : trim($error))) {   // send cz the error is not the same as we have on log
					return true;
				}
				else {
					if(time()>$timestampMatch + basic::$cfg['errorControl']['sendErrorByMailInterval']) {    // check if it's not too soon to send
						return true;   // send
					}
					else {
						return false;   // no need to send
					}
				}
			}
		}
		else {      // there are no errors yet - so send it
			return true;
		}
	}

	/**
	 * Send mail notification error
	 * @param array $error - error number + information like: file,line,type...
	 * @param boolean $fatal - is it a fatal error or not
	 * @param boolean $dontLogOnDb - allow log?
	 * @example $this->sendMailNotificationError($error);
   * @throws ReflectionException
   */
	public function sendMailNotificationError($error,$fatal = false,$dontLogOnDb = false) {

		$sent = "0";    // just a reset

		if($fatal==true) {
			$errorType = "fatalErrors";
			$table     = "log_errors_fatal";
		}
		else {
			$errorType = "nonFatalErrors";
			$table     = "log_errors_not_fatal";
		}

		$sendTheMail = self::checkErrorLogForRow($error);   // check if error occurs in last close range min - do not send mail

		if(!is_array($error)) {
			$tempMessage = $error;
			unset($error);
			$error['message']        = $tempMessage;
			$error['createdAt']       = date(basic::$cfg['log']['dateFormat']." ".basic::$cfg['log']['timeFormat'],time());
			$error['typeTranslated'] = "Mysql Conection";
			$error['file']           = "";
			$error['line']           = "";
			$error['backtrace']      = $this->printDebugBacktrace();

			$error['type'] = $error['typeTranslated'];                  // for db errors
			$this->errorLogOnFile($error,$fatal = true,$dbError = true);    // for db errors
		}    // if we have a PDO connection problem
		else {
			if(self::checkIfDbError($error['message'])) {
				$errorType = "dbErrors";
				$table     = "log_errors_db";
			}
		}

		if($sendTheMail && isset(basic::$cfg['errorControl'][$errorType]['logMethod']) && in_array("mail",basic::$cfg['errorControl'][$errorType]['logMethod'])) {
			$this->trans = basic::load("trans","core");
			$mail        = basic::load("mail","functions");

			$sent = $mail->sendMailToAdmin($messageId = 6500,$bodyFile = "siteError",$vars = $error,$subject = $this->trans->T("Site Error")." - ".$error['typeTranslated']." ".basic::$cfg['site']['name'].", env: ".ucfirst(basic::$cfg['site']['environment']));
		}

		if(isset(basic::$cfg['errorControl'][$errorType]['logMethod']) && in_array("db",basic::$cfg['errorControl'][$errorType]['logMethod'])) {
			$createData['type']     = $error['typeTranslated'];
			$createData['file']     = $error['file'];
			$createData['line']     = $error['line'];
			$createData['message']  = $error['message'];
			$createData['url']      = myHelpers::getServerUrl();
			$createData['referer']  = myHelpers::getServerReferer();
			$createData['sentMail'] = $sent;

			if(!$dontLogOnDb) {
				$this->db->insert($table = basic::getConfig("db","main","tablePrefix").$table,$createData,$advancedLead = true);    // log the error
			}
		}
	}

	/**
	 * Show alert container
	 * @param bool|false $fatal - if we call this function when fatal error occur
	 * @return bool|string
	 */
	public function showAlertContainer($fatal = false) {

		if($this->alertContainerRun) {
			return false;
		}      // don't run this twice

		$phpAlertHtml = "";
		$phpAlertHtml .= "
			<div id=\"phpAlertButton\" style=\"display:none;\" onclick=\"showPhpAlert();\">Alert</div>
			<div id=\"phpAlert\" class=\"phpAlertNormal centerUi\" style=\"display:none;\">
				<div id=\"alertSideInfo\">
				<a href=\"javascript:hideErrorsOnPage()\"><span id=\"showHideErrorButton\">Show</span> all errors on page</a>
				<br><a id=\"fixedOrAbsoluteErrorReport\" href=\"javascript:fixedPositionErrorReport();\">Make this window fixed</a>
				<br><a href=\"javascript:newWindowErrorReport()\">Open in new window</a>
				<br><a href=\"javascript:pauseAjaxErrorReport()\" id=\"pauseAlertsText\">Pause php alerts</a>
				<br><a href=\"javascript:invertColorsErrorReport()\">Invert Colors</a>
				<br><a href=\"javascript:clsAlerts()\">Clear alerts</a>
				<br>Text sizer:
				<a id=\"biggerTextErrorReport\" href=\"javascript:biggerTextErrorReport()\">T+</a>
				<a id=\"smallerTextErrorReport\" href=\"javascript:smallerTextErrorReport()\">T-</a>&nbsp;
				<br>Total of: <b><span id=\"totalErrors\"></span></b> errors
				<br><span>Ip: ".myHelpers::getUserIp()."&nbsp;</span>
				</div>
			</div>
		";

		if($fatal) {
			$phpAlertHtml .= "<link rel='stylesheet' property='stylesheet' href='/public/css/er.css' type='text/css'>";
			$phpAlertHtml .= "<script type='text/javascript' src='/public/js/er.js'></script>";
			$this->alertContainerRun = true;
		}

		return $phpAlertHtml;
	}

	/**
	 * Show div alert
	 * @param integer $eNumber - $error["type"]
	 * @param string $eString - $error["message"]
	 * @param string $eFile - $error["file"]
	 * @param integer $eLine - $error["line"]
	 * @param boolean $fatal - is it a fatal error or non-fatal error
	 * @example $this->showDivAlert($error["type"],$error["message"],$error["file"],$error["line"]);
   * @throws ReflectionException
   */
	public function showDivAlert($eNumber,$eString,$eFile,$eLine,$fatal = false) {    // set error handler

		if(basic::$cfg['errorControl']['dbUi']['allow']) {
			$auth = basic::load("auth","core");

			if($auth->isDevUser()) {
				echo dbInformant::showDbUiContainer($loadCssAndJs = true);
				dbInformant::$dbUiContainerRun = true;
				echo dbInformant::$keepForOutBurst;
				dbInformant::$keepForOutBurst = "";
			}
		}

		if(stringManipulation::startsWith($eString,"Uncaught exception 'PDOException' with message 'SQLSTATE")) {
			if(isset(basic::$cfg['errorControl']['dbQueries']['logMethod']) && (in_array("file",basic::$cfg['errorControl']['dbQueries']['logMethod']) || in_array("fileJson",basic::$cfg['errorControl']['dbQueries']['logMethod']))) {
				Log::logToFile($fileName = basic::$cfg['errorControl']['dbQueries']['logLocation'],$data = $eString."... file: ".$eFile.", line: ".$eLine);
			}
		}

		$preOpen  = "";
		$preClose = "";
		$uOpen    = "";
		$uClose   = "";
		if(strlen($eString)>50) {
			$preOpen  = "<pre>";
			$preClose = "</pre>";
		}    // could be array
		else {
			$uOpen  = "<span class=\"alertUnderline\"><b>";
			$uClose = "</b></span>";
		}    // regular error
		$createdAt     = date(basic::$cfg['log']['dateFormat']." ".basic::$cfg['log']['timeFormat'],$_SERVER['REQUEST_TIME']);
		$randomNumber = rand(10000,99999);
		$eMessage     = "";
		$exitNow      = false;
		$eType        = self::translateErrorNumber($eNumber);

		/*
		 * 	do not remove: <div class=\"\"></div> - it helps:
		 * 	if an error is activated on open tag, example: <a href='<?php echo URL; ?>login/register'>Register</a>
		 * 	the URL is invalid
		*/

		echo "<div class=\"\"></div><div id=\"errorOnPage-".$randomNumber."\" style=\"display:none;\" class=\"errorOnPage\">".$createdAt." - ".$eType." - ".$uOpen.$preOpen;        // display:none must stay here cz css file is loading slower then this

		//	if(strlen($eString)>50) {
		if(is_array($eString)) {
			printIt($eString,$eMessage,"");
			//	echo self::printRLevel($data=$eString,$level=7);
		}
		else {
			echo "$eString";
		}
		echo $preClose.$uClose.$preOpen." on line <b>".$eLine."</b> in file ".$eFile."</div>";

		if(!$eMessage) {
			$eMessage = $eType;
		}

		$backTraceIndividual = $this->printDebugBacktrace();
		$errorOutputIndividual = "<br><div id=\"backTrace-$randomNumber\" class=\"backTrace normalPattern\">$backTraceIndividual</div>";
		$errorOutputHtml = "<div id=\"phpAlertDiv-".$randomNumber."\" style=\"display:none;\" class=\"errorNum\">
			<div id=\"togglePhpError-".$randomNumber."\" class=\"errorNumRest\">
			".$createdAt." [".$eNumber." - ".$eMessage."] <b>Error:</b> ".$uOpen;    // display:none must stay here cz css file is loading slower then this
		//	if(strlen($eString)>50) {
		if(is_array($eString)) {
			$errorOutputHtml .= $preOpen.printIt($eString,$eMessage,"",false).$preClose."<br>";
			//	$errorOutputHtml .= self::printRLevel($data=$eString,$level=5);
		}
		else {
			$errorOutputHtml .= "<code>".$eString."</code><br>";
		}
		$errorOutputHtml .= $uClose." on line: <b>".$eLine."</b> in ".$eFile."<br>
	</div>
	<div class=\"errorNumRest\"><a href=\"javascript:showBackTrace(".$randomNumber.")\">Back Trace</a>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<a href=\"javascript:hideErrorsOnPage(".$randomNumber.")\"><span id=\"showHideErrorButton-".$randomNumber."\">Show</span> error on page</a>
	".$errorOutputIndividual."
	</div>

	</div>
	<script>
		document.addEventListener('DOMContentLoaded', function(event) {
			var showError = document.getElementsByClassName('errorNum');
			for(var i = 0; i < showError.length; i++) {
				showError[i].style.display='block';
			}
		});
	</script>
	";        // js here to show errors on page that won't load the "phpAlertButton" from some reason

		$errorOutputLog = "[".$eNumber." - ".$eMessage."] Error: ".self::printRLevel($data = $eString,$level = 5)." in ".$eFile." on line: ".$eLine;

		$error['type']    = $eNumber;
		$error['message'] = $eString;
		$error['file']    = $eFile;
		$error['line']    = $eLine;

		$dbError = "";
		if(self::checkIfDbError($error['message'])) {
			$dbError = true;
		}
		if($fatal!==1) {
			self::errorLogOnFile($errorOutputLog = $error,$fatal,$dbError);
		}
		if($fatal!==true) {
			$this->errorLoggerOnDbAndMail($error);
		}

		echo $errorOutputHtml;
	}

	/**
	 * Check if db error
	 * @param $errorMessage
	 * @return bool
	 */
	static function checkIfDbError($errorMessage) {

		if(strpos($errorMessage,'PDOException')!==false || strpos($errorMessage,'SQLSTATE')!==false) {
			return true;
		}

		return false;
	}

	/**
	 * Translate error number
	 * @param integer $eNumber - $error["type"] - error number
	 * @example self::translateErrorNumber($eNumber);
	 * @return string $eType - the error message
	 */
	static function translateErrorNumber($eNumber) {

		switch($eNumber) {
			case 1:
				$eType   = 'E_ERROR';
				$exitNow = true;
				break;
			case 2:
				$eType    = 'E_WARNING';
				$eMessage = "Warning";
				break;
			case 4:
				$eType = 'E_PARSE';
				break;
			case 8:
				$eType    = 'E_NOTICE';
				$eMessage = "Notice";
				break;
			case 16:
				$eType   = 'E_CORE_ERROR';
				$exitNow = true;
				break;
			case 32:
				$eType = 'E_CORE_WARNING';
				break;
			case 64:
				$eType   = 'E_COMPILE_ERROR';
				$exitNow = true;
				break;
			case 128:
				$eType = 'E_COMPILE_WARNING';
				break;
			case 256:
				$eType   = 'E_USER_ERROR';
				$exitNow = true;
				break;
			case 512:
				$eType = 'E_USER_WARNING';
				break;
			case 1024:
				$eType = 'E_USER_NOTICE';
				break;
			case 2048:
				$eType = 'E_STRICT';
				break;
			case 4096:
				$eType   = 'E_RECOVERABLE_ERROR';
				$exitNow = true;
				break;
			case 8192:
				$eType = 'E_DEPRECATED';
				break;
			case 16384:
				$eType = 'E_USER_DEPRECATED';
				break;
			case 30719:
				$eType   = 'E_ALL';
				$exitNow = true;
				break;
			default:
				$eType = 'E_UNKNOWN';
				break;
		}

		return $eType;
	}

	/**
	 * Error log on file
	 * @param array $error - error number + information like: file,line,type...
	 * @param boolean $fatal - is it a fatal error or not
	 * @param boolean $dbError - is it a db error or not
	 * @return string $eType - the error message
	 */
	static function errorLogOnFile($error,$fatal = false,$dbError = false) {

		$error['typeTranslated'] = self::translateErrorNumber($error['type']);
		$date                    = "";
		$time                    = "";
		$dateToShow              = "";
		$location                = "";
		$jumpLine                = PHP_EOL;
		$errorWriteType          = 0;
		if(basic::$cfg['log']['dateFormat']) {
			$date = basic::$cfg['log']['dateFormat'];
		}
		if(basic::$cfg['log']['timeFormat']) {
			$time = basic::$cfg['log']['timeFormat'];
		}
		if($date && $time) {
			$dateToShow     = "[".date($date." - ".$time)."] ";
			$errorWriteType = 3;
		}
		else {
			$jumpLine = "";
		}

		if($fatal==1) {
			$errorType = "fatalErrors";
		}
		else {
			$errorType = "nonFatalErrors";
		}

		if($dbError==true) {
			$errorType = "dbErrors";
		}

		if(isset(basic::$cfg['errorControl'][$errorType]['logLocation']) && (isset(basic::$cfg['errorControl'][$errorType]['logMethod']) && (in_array("file",basic::$cfg['errorControl'][$errorType]['logMethod']) || in_array("fileJson",basic::$cfg['errorControl'][$errorType]['logMethod'])))) {
			error_reporting(0);                // fix that error wont be written to php_error_log
			ini_set("error_log",basic::$cfg['errorControl'][$errorType]['logLocation']);

			$location   = basic::$cfg['errorControl'][$errorType]['logLocation'];
			$dirAndFile = myHelpers::fixDirAndFile($location,$ext = "log",$lastDir = IMPERMANENTPATH."logs");
		}
		else {
			$errorWriteType = 0;
			$dateToShow     = "";
			$jumpLine       = "";
			ini_restore("error_log");    // putting errors on: php_error_log
		}

		$referer = myHelpers::getServerReferer();
		if($referer) {
			$referer = ", Referer: ".$referer;
		}

		$urlPosition = myHelpers::getServerUrl();
		if($urlPosition) {
			$urlPosition = ", Url position: ".$urlPosition;
		}

		if(isset($error['type'])) {
			if($dbError==false) {
				$errorOutputLog = $dateToShow."[".myHelpers::getUserIp()."] [".$error['typeTranslated']." - ".$error["type"]."] Error: ".$error['message']." in ".$error["file"]." on line: ".$error["line"].$referer.$urlPosition;
			}
			else {
				$errorOutputLog = $dateToShow."[".myHelpers::getUserIp()."] [".$error['typeTranslated']." - ".$error["type"]."] Error: ".$error['message'].$referer.$urlPosition;
			}

			if((isset(basic::$cfg['errorControl'][$errorType]['logMethod']) && (in_array("file",basic::$cfg['errorControl'][$errorType]['logMethod']) || in_array("fileJson",basic::$cfg['errorControl'][$errorType]['logMethod'])))) {

				if(in_array("file",basic::$cfg['errorControl'][$errorType]['logMethod'])) {
					error_log($errorOutputLog.$jumpLine,$errorWriteType,$dirAndFile);
				}

				if(in_array("fileJson",basic::$cfg['errorControl'][$errorType]['logMethod'])) {
					$jsonAddExt = ".json";

					$dirAndFile               = $dirAndFile.$jsonAddExt;
					$errorJsonAdd['createdAt'] = $dateToShow;
					$errorJsonAdd['ip']       = myHelpers::getUserIp();
					$error                    = $errorJsonAdd + $error;

					$jsonErrorLog = json_encode($error);

					error_log($jsonErrorLog.$jumpLine,$errorWriteType,$dirAndFile);
				}
			}
			else {
				error_log($errorOutputLog.$jumpLine,$errorWriteType,$dirAndFile);
			}
		}
	}

	/**
	 * @param $eNumber
	 * @param $eString
	 * @param $eFile
	 * @param $eLine
	 * @param bool $fatal
   * @throws ReflectionException
   */
	public function dontShowDivAlert($eNumber,$eString,$eFile,$eLine,$fatal = false) {    // set error handler

		$error['typeTranslated'] = self::translateErrorNumber($eNumber);
		$error['type']           = $eNumber;
		$error['message']        = $eString;
		$error['file']           = $eFile;
		$error['line']           = $eLine;

		$dbError = "";
		if(self::checkIfDbError($error['message'])) {
			$dbError = true;
		}

		self::errorLogOnFile($errorOutputLog = $error,$fatal = false,$dbError);
		$this->errorLoggerOnDbAndMail($error);
	}

	/**
	 * Error handler with no div alerts
	 */
	public function errorHandler() {

		set_error_handler(array($this,'dontShowDivAlert'));
	}

	/**
	 * Error handler for div alerts
	 */
	public function errorHandlerDivAlerts() {

		set_error_handler(array($this,'showDivAlert'));
	}

	/**
	 * Print R level - prints out the error message
	 * @param string $data - the error message
	 * @param integer $level - the error level
	 * @return string output
	 * @example self::printRLevel($data=$eString,$level=5)
	 */
	public function printRLevel($data,$level = 5) {

		static $innerLevel = 1;
		static $tabLevel = 1;
		$self          = __FUNCTION__;
		$type          = gettype($data);
		$tabs          = str_repeat('    ',$tabLevel);
		$quoteTabs    = str_repeat('    ',$tabLevel - 1);
		$output        = '';
		$elements      = array();
		$recursiveType = array('object','array');
		// Recursive
		if(in_array($type,$recursiveType)) {
			// If type is object, try to get properties by Reflection.
			if($type=='object') {
				$output     = get_class($data).' '.ucfirst($type);
				$ref        = new \ReflectionObject($data);
				$properties = $ref->getProperties();
				foreach($properties as $property) {
					$property->setAccessible(true);
					$pType = $property->getName();
					if($property->isProtected()) {
						$pType .= ":protected";
					}
					elseif($property->isPrivate()) {
						$pType .= ":".$property->class.":private";
					}
					if($property->isStatic()) {
						$pType .= ":static";
					}
					$elements[$pType] = $property->getValue($data);
				}
			}
			// If type is array, just retun it's value.
			elseif($type=='array') {
				$output   = ucfirst($type);
				$elements = $data;
			}
			// Start dumping data
			if($level==0 || $innerLevel<$level) {
				// Start recursive print
				$output .= "\n{$quoteTabs}(";
				foreach($elements as $key => $element) {
					$output .= "<br>\n{$tabs}[{$key}] => ";
					// Increment level
					$tabLevel = $tabLevel + 2;
					$innerLevel++;
					$output .= in_array(gettype($element),$recursiveType) ? $self($element,$level) : $element;
					// Decrement level
					$tabLevel = $tabLevel - 2;
					$innerLevel--;
				}
				$output .= "<br>\n{$quoteTabs})\n";
			}
			else {
				$output .= "<br>\n{$quoteTabs}*MAX LEVEL*\n";
			}
		}
		else {
			$output = $data;
		}

		return $output;
	}

	/**
	 * Debug bt - output backtrace
	 */
	public function debugBt() {

		if(!function_exists('debug_backtrace')) {
			echo 'function debug_backtrace does not exists'."\r\n";

			return;
		}
		echo '<pre>';
		echo "\r\n".'-------------------------'."\r\n";
		echo 'Debug backtrace:'."\r\n";
		echo '-------------------------'."\r\n";
		foreach(debug_backtrace() as $t) {
			echo "\t".'@ ';
			if(isset($t['file'])) {
				echo basename($t['file']).':'.$t['line'];
			}
			else {
				// if file was not set, I assumed the functioncall
				// was from PHP compiled source (ie XML-callbacks).
				echo '<PHP inner-code>';
			}
			echo ' -- ';
			if(isset($t['class'])) {
				echo $t['class'].$t['type'];
			}
			echo $t['function'];
			if(isset($t['args']) && sizeof($t['args'])>0) {
				echo '(...)';
			}
			else echo '()';
			echo "\r\n";
		}
		echo '</pre>';
		//	exit;
	}

	/**
	 * Log trace - output backtrace
	 * @param string $message - error message
	 */
	public function logTrace($message = "") {

		$trace = debug_backtrace();
		if($message) {
			error_log($message);
		}
		$caller        = array_shift($trace);
		$function_name = $caller['function'];
		error_log(sprintf('%s: Called from %s:%s',$function_name,$caller['file'],$caller['line']));
		foreach($trace as $entry_id => $entry) {
			$entry['file'] = $entry['file'] ?: '-';
			$entry['line'] = $entry['line'] ?: '-';
			if(empty($entry['class'])) {
				error_log(sprintf('%s %3s. %s() %s:%s',$function_name,$entry_id + 1,$entry['function'],$entry['file'],$entry['line']));
			}
			else {
				error_log(sprintf('%s %3s. %s->%s() %s:%s',$function_name,$entry_id + 1,$entry['class'],$entry['function'],$entry['file'],$entry['line']));
			}
		}
	}

	/**
	 * Get back trace - output backtrace
	 * @param integer $ignore - error message
	 * @return string
	 */
	protected function getBacktrace($ignore = 2) {

		$trace = '';
		foreach(debug_backtrace() as $k => $v) {
			if($k<$ignore) {
				continue;
			}
			array_walk($v['args'],function (&$item,$key) {

				$item = var_export($item,true);
			});
			$trace .= '<br>#'.($k - $ignore).' '.$v['file'].'('.$v['line'].'): '.(isset($v['class']) ? $v['class'].'->' : '').$v['function'].'('.implode(', ',$v['args']).')'."\n";
		}

		return $trace;
	}

	/**
	 * Generate call trace - output backtrace
	 * @return string
	 */
	public function generateCallTrace() {

		$e     = new Exception();
		$trace = explode("\n",$e->getTraceAsString());
		// reverse array to make steps line up chronologically
		$trace = array_reverse($trace);
		array_shift($trace); // remove {main}
		array_pop($trace); // remove call to this method
		$length = count($trace);
		$result = array();
		for($i = 0; $i<$length; $i++) {
			$result[] = '('.($i + 1).')'.substr($trace[$i],strpos($trace[$i],' ')); // replace '#someNum' with '$i)', set the right ordering
		}

		return stripslashes(implode("<br>",$result));
	}

	/**
	 * Print debug back trace - output backtrace
	 * @return string
	 */
	public static function printDebugBacktrace() {

		$stacks       = debug_backtrace();
		$randomNumber = rand(10000,99999);

		$output = "<table id=\"mustPutId-$randomNumber\" class=\"backTraceTable\">";    // mustPutId for initMovingTables js function, although we dont use it here.. (have to fix this laer)
		$output .= "<tr><td><strong></strong></td><td><strong>File</strong></td><td><strong>Line</strong></td><td><strong>Class</strong></td><td><strong>Function</strong></td></tr>";

		foreach($stacks as $key => $value) {
			if(!isset($stacks[$key]['file'])) {
				$stacks[$key]['file'] = '[PHP Kernel]';
			}
			if(!isset($stacks[$key]['line'])) {
				$stacks[$key]['line'] = '';
			}
			if(!isset($stacks[$key]['class'])) {
				$stacks[$key]['class'] = '';
			}

			$output .= "<tr><td>".($key + 1).")</td><td><div class=\"backTraceDiv\">{$stacks[$key]["file"]}</div></td><td>{$stacks[$key]["line"]}</td><td>{$stacks[$key]["class"]}</td><td>{$stacks[$key]["function"]}</td></tr>";
		}
		$output .= "</table>";

		return $output;
	}
}


