<?php

/*************************************************************
 * Log class
 * - all functions about logs
 * @author    S M Kessner <limbercode@gmail.com>
 * @copyright 2010-2018 T2
 * @link      http://limbercode.com
 *************************************************************/
class log {

	private $infoUiContainerRun;
	private $allowInfoUi;
	public $infoUiContainer;

	/**
	 * Constructor for log
	 */
	public function __construct() {

		if(myHelpers::isEnv($env = "dev") && (basic::$cfg['log']['infoUi']['allow']==1 || basic::$cfg['log']['infoUi']['allow']==2)) {
			$this->allowInfoUi = true;
		}
		elseif(basic::$cfg['log']['infoUi']['allow']==1) {
			$auth = basic::load("auth","core");

			if($auth->isDevUser()) {
				$this->allowInfoUi = true;
			}    // for dev user
			else {
			}    // for regular user
		}    // live environment
	}

	/**
	 * Log some text
	 * @param string (required) $fileName - the chosen file name or db name (if it's db \"_log\" will be added), will be located at: \\logs
	 * @param string (required) $data - the data row
	 * @param string (optional) $title - the title row
	 * @example Log::log($fileName="captain",$data="Scotty can\'t take it no more",$title="Directive");
	 */
	public static function log($fileName,$data,$title = "") {

		if(basic::$cfg['log']['default']=="file") {
			self::logToFile($fileName,$data,$title);
		}
		elseif(basic::$cfg['log']['default']=="db") {
			// not yet done
		}
		elseif(basic::$cfg['log']['default']=="mail") {
			// not yet done - but we have similar on mail class
		}
	}

	/**
	 * Log to file - create's a row in a log file that can be sniffed later (don't take it to your lung). the file will be written at "logs" directory according to config parameters. if full path is given, it will be used
	 * @param string (required) $fileName - the chosen file name, will be located at: \\logs
	 * @param string|array|object (required) $data - the data row
	 * @param string (optional) $title - the title row
	 * @example Log::logToFile($fileName="mysql",$data="SELECT * FROM table",$title="SELECT");
	 */
	public static function logToFile($fileName,$data,$title = "someLog") {

		$dirAndFile = myHelpers::fixDirAndFile($fileName,$ext = "log",$lastDir = "impermanent".DS."logs");

		$dir    = explode(DS,$dirAndFile);
		$sliced = array_slice($dir,0,-1);
		$dir    = implode(DS,$sliced);        // take the dir without the file name

		if(!is_dir($dir)) {
			mkdir($dir);
		}

		if($title)
			$title = " - ".$title;
		if(is_array($data) || is_object($data))
			$data = print_r($data,true);

		$text = "[".date(basic::$cfg['site']['dateFormat']." ".basic::$cfg['site']['timeFormat'],time())."] [".myHelpers::getUserIp()."]".$title." - ".$data."\r\n";

    if (!file_exists($dirAndFile)) {
      $createFile = fopen($dirAndFile,"wb");
      fclose($createFile);
    }

		file_put_contents($dirAndFile,$text,FILE_APPEND);
	}

	/**
	 * Enable disable show ui container
	 */
	public function showContainer() {

		if($this->allowInfoUi!=false) {
			$infoUiContainer = $this->showInfoUiContainer();

			if(!isset($_SERVER['SCRIPT_URL']) || $_SERVER['SCRIPT_URL']!="/sil/check" && $_SERVER['SCRIPT_URL']!="/login") {      // so it wont be stacked on: headers already sent...
				if(basic::$cfg['log']['infoUi']['afterHtml']==0) {
					echo $infoUiContainer;
				}
			}
			$this->infoUiContainer = $infoUiContainer;
		}
	}

	/**
	 * Show alert container
	 * @return bool|string
	 */
	public function showInfoUiContainer() {

		if($_POST && basic::$cfg['log']['infoUi']['infoUiOnPost']!=1) {       // if we doing redirect we don't need to show this dbUi - it will stuck if so
			$this->infoUiContainerRun = true;
		}

		if(myHelpers::isAjax() || $this->infoUiContainerRun) {
			return false;
		}      // don't run this twice

		$infoHtml = "<div id=\"infoUiButton\" style=\"display:none;\" onclick=\"showInfoUi();\">I</div>
			<div id=\"infoUi\" class=\"infoUiNormal centerUi\" style=\"display:none;\">
				<span class=\"infoUiHead\">Info on:</span>
				<ul id=\"infoUiTabs\">
				</ul>
				<div class=\"clear\"></div>

				<div id=\"infoUiSideInfo\">
<!--				<a href=\"javascript:hideErrorsOnPage()\"><span id=\"showHideErrorButton\">Show</span> all errors on page</a>-->
				<br><a id=\"fixedOrAbsoluteInfoUi\" href=\"javascript:fixedPositionInfoUi();\">Make this window fixed</a>
				<br><a href=\"javascript:newWindowInfoUi()\">Open in new window</a>
				<br><a href=\"javascript:pauseAjaxInfoUi()\" id=\"pausePrintItText\">Pause printIt catching</a>
				<br><a href=\"javascript:invertColorsInfoUi()\">Invert Colors</a>
				<br><a href=\"javascript:clsAlerts()\">Clear alerts</a>
				<br>Text sizer:
				<a id=\"biggerTextInfoUi\" href=\"javascript:biggerTextInfoUi()\">T+</a>
				<a id=\"smallerTextInfoUi\" href=\"javascript:smallerTextInfoUi()\">T-</a>&nbsp;
				<br>Total of: <b><span id=\"totalInfoUis\"></span></b> printIt
				<br><span>Ip: ".myHelpers::getUserIp()."&nbsp;</span>
				</div>
				<div id=\"infoUiContent\">
				</div>
			</div>
		";

		$this->infoUiContainerRun = true;

		return $infoHtml;
	}

	/**
	 * Show pre defined prints
	 */
	public function showPreDefinedPrints() {

		if($this->allowInfoUi!=false) {
			echo "<div style=\"display:none;\">";
			foreach(basic::$cfg['log']['infoUi']['functions'] as $key => $value) {
				printIt($value['output'],$value['name'],false,true,$value['logToFile']);
			}
			echo "</div>";
		}
	}
}

