<?php

/*************************************************************
 * My helpers class
 * @author    S M Kessner <limbercode@gmail.com>
 * @copyright 2010-2018 T2
 * @link      http://limbercode.com
 *************************************************************/
class myHelpers {

  /**
   * Gets server url path
   * @return string
   */
  public static function getServerUrl() {

    $pageURL = 'http';
    if(isset($_SERVER["HTTPS"]) && $_SERVER["HTTPS"]=="on") {
      $pageURL .= "s";
    }
    $pageURL .= "://";

    if(isset($_SERVER["SCRIPT_URL"])) {
      $uri = $_SERVER["SCRIPT_URL"];
    }
    else {
      $uri = $_SERVER["REQUEST_URI"];
    }

    if($_SERVER["SERVER_PORT"]!="80") {
      $pageURL .= $_SERVER["SERVER_NAME"].":".$_SERVER["SERVER_PORT"].$uri.$_SERVER["QUERY_STRING"];
    }
    else {
      $pageURL .= $_SERVER["SERVER_NAME"].$uri.$_SERVER["QUERY_STRING"];
    }

    return $pageURL;
  }

  /**
   * Gets server referer url path
   * @return string
   */
  public static function getServerReferer() {

    if(!isset($_SERVER['HTTP_REFERER'])) {
      return false;
    }

    return $_SERVER['HTTP_REFERER'];
  }

  /**
   * Unregister globals - check register globals and remove them
   */
  public static function unregisterGlobals() {

    if(ini_get('register_globals')) {
      $array = array('_SESSION','_POST','_GET','_COOKIE','_REQUEST','_SERVER','_ENV','_FILES');
      foreach($array as $value) {
        foreach($GLOBALS[$value] as $key => $var) {
          if($var===$GLOBALS[$key]) {
            unset($GLOBALS[$key]);
          }
        }
      }
    }
  }

  /**
   * Remove magic quotes - check for Magic Quotes and remove them
   */
  public static function removeMagicQuotes() {

    if(isset($_GET)) {
      $_GET = stringManipulation::stripSlashesDeep($_GET);
    }

    if(isset($_POST)) {
      $_POST = stringManipulation::stripSlashesDeep($_POST);
    }

    if(isset($_COOKIE)) {
      $_COOKIE = stringManipulation::stripSlashesDeep($_COOKIE);
    }
  }

  /**
   * Get client IP address
   * @return string
   */
  public static function getUserIp() {

    if(getenv("HTTP_CLIENT_IP")) {
      $ip = getenv("HTTP_CLIENT_IP");
    }
    elseif(getenv("HTTP_X_FORWARDED_FOR")) {
      $ip = getenv("HTTP_X_FORWARDED_FOR");
    }
    elseif(getenv("REMOTE_ADDR")) {
      $ip = getenv("REMOTE_ADDR");
    }
    else $ip = "UNKNOWN";

    if(isset($_SERVER)) {
      if(isset($_SERVER["HTTP_X_FORWARDED_FOR"])) {
        $ip = $_SERVER["HTTP_X_FORWARDED_FOR"];
      }
      elseif(isset($_SERVER["HTTP_CLIENT_IP"])) {
        $ip = $_SERVER["HTTP_CLIENT_IP"];
      }
      else $ip = $_SERVER["REMOTE_ADDR"];
    }

    if($ip=="::1") {
      $ip = "127.0.0.1";
    }

    return $ip.":".$_SERVER['SERVER_PORT'];
  }

  /**
   * Is ajax - check if the action is AJAX request
   * return boolean
   */
  public static function isAjax() {

    return (!empty($_SERVER['HTTP_X_REQUESTED_WITH']) && (strtolower($_SERVER['HTTP_X_REQUESTED_WITH'])=='xmlhttprequest'));
  }

  /**
   * Minify content - output html content
   * @return string
   */
  public static function minifyContent($content) {

    $path     = self::UrlContent('~/labYard/min/lib/','');
    $filename = $path.'Minify/HTML.php';
    $html     = $js = $css = false;

    if(file_exists($filename)) {
      require_once $filename;
      $html = true;
    }

    $filename = $path.'Minify/CSS.php';
    if(file_exists($filename)) {
      require_once $filename;
      $css = true;
    }

    $filename = $path.'JSMin.php';
    if(file_exists($filename)) {
      require_once $filename;
      $js = true;
    }

    // if we pass above files
    if(($html==true) && ($js==true) && ($css==true)) {
      $content = Minify_HTML::minify($content,array(
        'xhtml'       => array('Minify','minify'),
        'cssMinifier' => array('Minify_CSS','minify'),
        'jsMinifier'  => array('JSMin','minify')
      ));
    }

    //	}

    return $content;
  }

  /**
   * Minify output js content
   * @return string
   */
  public static function minifyContentJs($content) {

    $filename = self::UrlContent('~/labYard/min/lib/JSMin.php','');
    if(!myHelpers::isLocalhost() && file_exists($filename)) {
      require_once $filename;
      $content = JSMin::minify($content);
    }

    return $content;
  }

  /**
   * Minify output css content
   * @return string
   */
  public static function minifyContentCss($content) {

    $filename = self::UrlContent('~/labYard/min/lib/Minify/CSS.php','');
    if(!myHelpers::isLocalhost() && file_exists($filename)) {
      require_once $filename;
      $content = Minify_CSS::minify($content);
    }

    return $content;
  }

  /**
   * Url content - resolves a virtual path into an absolute path
   * @param string $path
   * @param string $sub
   * @return string
   */
  public static function UrlContent($path,$sub = "architecture".DS."application") {

    if(stringManipulation::startsWith($path,"~")) {
      $path    = str_replace("/",DS,$path);
      $appPath = ROOT.DS.$sub.(stringManipulation::startsWith($path,"~/") ? "" : DS);
      $result  = str_replace("~",$appPath,$path);
      $result  = str_replace(DS.DS,DS,$result);

      return $result;
    }
    else {
      return $path;
    }
  }

  /**
   * Remote file exists - check if file exists
   * @param string $url - the url to check
   * @return boolean
   */
  public static function remoteFileExists($url) {

    $ch = curl_init($url);
    curl_setopt($ch,CURLOPT_NOBODY,true);
    curl_exec($ch);
    $retcode = curl_getinfo($ch,CURLINFO_HTTP_CODE);
    curl_close($ch);

    if($retcode==200) {
      return true;
    }
    else return false;
  }

  /**
   * Tail file last rows
   * @param $filePath
   * @param int $rows - number of last rows
   * @return string
   */
  static function tailFile($filePath,$rows = 1) {

    if(file_exists($filePath)) {
      return trim(implode("",array_slice(file($filePath),-$rows)));
    }
    else return false;
  }

  /**
   * Tail file last rows to array
   * @param $filePath
   * @param int $rows - number of last rows
   * @return array
   */
  static function tailFileToArray($filePath,$rows = 1) {

    if(file_exists($filePath)) {
      return array_map('trim',array_slice(file($filePath),-$rows));
    }
  }

  /**
   * Check environment type - and return true or fals
   * @param $env
   * @return bool
   */
  static function isEnv($env) {   // environmentControl

    $env     = strtolower($env);
    $envDev  = array("dev","develop","development","tech","test");
    $envLive = array("live","prod","production");

    if(in_array($env,$envDev)) {
      $check = "dev";
    }
    if(in_array($env,$envLive)) {
      $check = "live";
    }
    if(in_array(basic::$cfg['site']['environment'],$envDev)) {
      $current = "dev";
    }
    if(in_array(basic::$cfg['site']['environment'],$envLive)) {
      $current = "live";
    }

    if($check===$current) {
      return true;
    }
  }

  static function getEnv() {

    return basic::$cfg['site']['environment'];
  }

  /**
   * Fix directory plus file - fixes problems that may occur with dir/file input
   * @param $location - the location of the file name - can be with directory
   * @param $ext - default extension of the file
   * @param $lastDir - default directory to be inserted before $location starts
   * @return mixed|string
   */
  public static function fixDirAndFile($location,$ext,$lastDir) {

    $location = preg_replace('#//+#','/',$location);                    // fix double slashes problem
    $location = preg_replace('#\\\\+#','\\',$location);                 // fix double slashes problem
    if(DS=='/') {
      $location = str_replace('\\',DS,$location);
    }              // fix linux/windows slashes problem
    if(DS=='\\') {
      $location = str_replace('/',DS,$location);
    }            // fix linux/windows slashes problem

    $dirAndFile = $location;
    if(strpos($location,'.')===false) {
      $dirAndFile = $location.".".$ext;
    }      // fix extension if any

    if($lastDir) {
      $lastDir = $lastDir.DS;
    }
    if((strpos($dirAndFile,$lastDir)===false)) {
      $dirAndFile = $lastDir.$dirAndFile;
    }
    if((strpos($dirAndFile,ROOT)===false)) {
      $dirAndFile = ROOT.DS.$dirAndFile;
    }

    return $dirAndFile;
  }

  /**
   * Check if file exist case insensitive
   * @param $file
   * @param $lastDir
   * @return mixed
   */
  public static function checkIfFileExistCaseInsensitive($file,$lastDir) {

    $filesAndFoldersOnDir = @scandir($lastDir);

    if(array_search($file.".json",$filesAndFoldersOnDir)) {
      return true;
    }

    return false;
  }

  /**
   * Get files in directory - will show all files inside directory of a directory
   * @param $directory
   * @return array
   */
  public static function getFilesInDirectory($directory) {

    $return = array();

    if(is_dir($directory)) {
      $files = scandir($directory);

      if(false!==$files) {
        foreach($files as $file) {
          if($file!="." && $file!="..") {
            if(is_dir($directory.DS.$file)) {
              $return[$file] = self::getFilesInDirectory($directory.DS.$file);
            }
            else {
              $pathParts             = pathinfo($directory.DS.$file);
              $pathParts['size']     = filesize($directory.DS.$file);
              $pathParts['modified'] = filemtime($directory.DS.$file);
              $return[]              = $pathParts;
            }
          }
        }
      }

      return $return;
    }
  }

  /**
   * Is Json + decode json
   * @param $string
   * @return bool
   */
  public static function isJson($string) {

    if(is_string($string)) {
      $jsonDecoded = json_decode($string);

      if(json_last_error()===JSON_ERROR_NONE) {    // JSON is valid
        return $jsonDecoded;
      }
    }

    return false;
  }

  /**
   * Load json file for advanced system
   * @param $file
   * @param $lastDir
   * @return mixed
   */
  public static function loadJsonFile($file,$lastDir) {

    $dirAndFile  = myHelpers::fixDirAndFile($file,$ext = "json",$lastDir);
    $decodedJson = "";

    if(file_exists($dirAndFile)) {
      $fileContents = file_get_contents($dirAndFile);
      $decodedJson  = json_decode($fileContents,true);
    }

    if(!empty($decodedJson[0])) {
      return $decodedJson[0];
    }

    return false;
  }

  /**
   * Store json file for advanced system
   * @param $file - directory and file name
   * @param $lastDir - directory name
   * @param $data - data to store - can be array or object
   * @return string
   */
  public static function storeJsonFile($file,$lastDir,$data) {

    $encodedData = json_encode($data,JSON_PRETTY_PRINT);
    $dirAndFile  = myHelpers::fixDirAndFile($file,$ext = "json",$lastDir);

    $dir    = explode(DS,$dirAndFile);
    $sliced = array_slice($dir,0,-1);
    $dir    = implode(DS,$sliced);        // take the dir without the file name

    if(!is_dir($dir)) {
      mkdir($dir);
    }

    if(!file_put_contents($dirAndFile,$encodedData)) {
      return false;
    }

    return true;
  }

  /**
   * Ensure page refresh on pressing: back button
   */
  public static function ensurePageRefresh() {

    header("Cache-Control: private, must-revalidate, max-age=0");
    header("Pragma: no-cache");
    header("Expires: Sat, 26 Jul 1997 05:00:00 GMT");
  }

/////////////////////// no need the next 2 functions cz we have getFilesInDirectory ------- but for testing

  public static function recursiveDirectoryIterator($path) {

    foreach(new RecursiveIteratorIterator(new RecursiveDirectoryIterator($path)) as $fileInfo) {
      if(!$fileInfo->isDir()) {
        yield $fileInfo->getFilename()."*".$fileInfo->getExtension()."*".$fileInfo->getSize()."*".$fileInfo->getBasename();
      }
    }
  }

  public static function getDirFiles($directory) {

    $instance    = self::recursiveDirectoryIterator($directory);
    $total_files = 0;
    $www         = [];
    $n           = 0;

    foreach($instance as $value) {
      $n++;
      $qqq                  = explode("*",$value);
      $www[$n]['dirname']   = $directory;
      $www[$n]['basename']  = $qqq[0];
      $www[$n]['filename']  = str_replace(".".$qqq[1],"",$qqq[3]);
      $www[$n]['extension'] = $qqq[1];
      $www[$n]['size']      = $qqq[2];
    }

    return $www;
  }

  public static function isDirEmpty($dir) {

    $handle = opendir($dir);
    while(false!==($entry = readdir($handle))) {
      if($entry!="." && $entry!="..") {
        closedir($handle);

        p($entry);

        return false;
      }
    }
    closedir($handle);

    return true;
  }
}
