<?php

/*************************************************************
 * Notif class
 * - all functions about notifications
 * @author    S M Kessner <limbercode@gmail.com>
 * @copyright 2010-2018 T2
 * @link      http://limbercode.com
 *************************************************************/
class notif {

	public $textLoadFile;
	public $textLoad;
	public $notifContainer;
	public static $notifsContent;
	public static $lastNotif;
	private $userLoggedAuthLevel;
	private $allowNotifUi;
	private $notifUiContainerRun;
	private static $cfg;
	private static $notifsToShow = [];

	/**
	 * Constructor for Notif
	 */
	public function __construct() {

		self::$cfg = basic::loadCfg("notifConfig",false);       // notification config is set to load privately

		if(self::$cfg['notif']['allow']) {
			$this->allowNotifUi = true;
		}
	}

	/**
	 * Load notifications - from array/xml
	 * @param string $fileName - file to load from, located at: \text\notifs\language code\ (ends with php/xml)
	 * @return array - the notifications from file
	 * @example $this->loadNotifs($fileName="core");
	 */
	public function loadNotifs($fileName) {

		if(self::$cfg['notif']['allow']==2 && $fileName=="core") {
			return false;
		}

		if(!self::$cfg['notif']['loadMethod'] || self::$cfg['notif']['loadMethod']=="array") {
			return $this->loadNotifsViaArray($fileName);
		}
		elseif(self::$cfg['notif']['loadMethod']=="xml") {
			return $this->loadNotifsViaXml($fileName);
		}
	}

	/**
	 * Load notifications via php array
	 * @param string $fileName - file to load from, located at: "\text\notifs\language code\" (ends with php)
	 * @return array - the notifications from file
	 * @example $this->loadNotifsViaArray($fileName);
	 */
	public function loadNotifsViaArray($fileName) {

		$loadFile = SOURCEPATH."text".DS."notifs".DS.lang::$currentLanguage.DS.$fileName.".php";

		if(!isset(self::$notifsContent[$fileName])) {
			if(file_exists($loadFile)) {
				require($loadFile);

				if(isset($notifs)) {
					return self::$notifsContent[$fileName] = $notifs;    // returns only the var $notifs
				}
			}
			else {
				$loadFileDefaultLanguage = SOURCEPATH."text".DS."notifs".DS.lang::getDefaultLanguage().DS.$fileName.".php";

				if(file_exists($loadFileDefaultLanguage)) {
					require($loadFileDefaultLanguage);  // load en notifs
				}
				else {
					$loadFileSecondaryLanguage = SOURCEPATH."text".DS."notifs".DS."en".DS.$fileName.".php";

					if(file_exists($loadFileSecondaryLanguage)) {
						require($loadFileSecondaryLanguage);  // load en notifs
					}
				}   // try to load in english (fallback)

				if(isset($notifs)) {
					return self::$notifsContent[$fileName] = $notifs;    // returns only the var $notifs
				}
			}

			$this->genNotifByNum("core",81210,lang::$currentLanguage."/".$fileName.".php");    // sometimes crash the system
		}
	}

	/**
	 * Load notifications via xml
	 * @param string $fileName - file to load from, located at: "\text\notifs\language code\" (ends with xml)
	 * @return array - the notifications from file
	 * @example $this->loadNotifsViaXml($fileName);
	 */
	public function loadNotifsViaXml($fileName) {

		lang::getCurrentLanguage();
		$xml = basic::load('xml','functions');    // get the text codes from xml

		$loadFile = SOURCEPATH."text".DS."notifs".DS.lang::$currentLanguage.DS.$fileName.".xml";

		if(!isset(self::$notifsContent[$fileName])) {
			if(file_exists($loadFile)) {
				$loadVariables = $xml::xmlToArray(file_get_contents($loadFile));
				$loadVariables = $this->parseNotifsVariablesFromXml($loadVariables);

				return self::$notifsContent[$fileName] = $loadVariables;
			}
			else {
				$loadFileDefaultLanguage = SOURCEPATH."text".DS."notifs".DS.lang::getDefaultLanguage().DS.$fileName.".xml";

				if(file_exists($loadFileDefaultLanguage)) {        // must load the default "en" notif file
					$loadVariablesDefaultLanguage = $xml::xmlToArray(file_get_contents($loadFileDefaultLanguage));
					$loadVariablesDefaultLanguage = $this->parseNotifsVariablesFromXml($loadVariablesDefaultLanguage);

					return self::$notifsContent[$fileName] = $loadVariablesDefaultLanguage;
				}
				else {
					$this->genNotifByNum("core",81220,lang::$currentLanguage."/".$fileName.".xml");    // sometimes crash the system
				}
			}
		}
	}

	/**
	 * Parse notifications variables from xml
	 * @param string $loadVariables
	 * @return array - the notifications from file
	 * @example $this->parseNotifsVariablesFromXml($loadVariables);
	 */
	private function parseNotifsVariablesFromXml($loadVariables) {

		$newArray = array();

		foreach($loadVariables as $key => $value) {
			$newKey            = str_replace("notif-","",$key);
			$newArray[$newKey] = array();
			$notifType         = "";
			$authLevel         = "";
			$text              = "";

			if(is_array($loadVariables[$key])) {
				foreach($loadVariables[$key] as $key2 => $value2) {
					if(isset($loadVariables[$key]['type'])) {
						$notifType = $loadVariables[$key]['type'];
					}
					if(isset($loadVariables[$key]['authLevel'])) {
						$authLevel = $loadVariables[$key]['authLevel'];
						if(!$authLevel) {
							$authLevel = "0";
						}
					}
					if(isset($loadVariables[$key]['text'])) {
						$text = $loadVariables[$key]['text'];
					}
					$newArray[$newKey][$notifType][$authLevel] = $text;
				}
			}
		}    // parse the xml to be like an array

		return $newArray;
	}

	/**
	 * Parse notification num - get the notification by it's number
	 * @param string $fileName - the file name, located at: "\text\notifs\language code\"
	 * @param string $notifNum - the notification number
	 * @param string|array|null $externalVar - the external variant to use in this notification
	 * @return array - the specific notification including: $notifType,$authLevel,$notifTextRand
	 * @example $this->parseNotifNum($fileName,$notifNum,$externalVar);
	 */
	private function parseNotifNum($fileName,$notifNum,$externalVar = "") {

		$notifType = "";
		$authLevel = "";

		if(isset(self::$notifsContent[$fileName][$notifNum])) {
			$notifType = key(self::$notifsContent[$fileName][$notifNum]);
			$authLevel = key(self::$notifsContent[$fileName][$notifNum][$notifType]);

			$notifTextArray = self::$notifsContent[$fileName][$notifNum][$notifType][$authLevel];

			if(is_array($notifTextArray)) {
				$notifText     = array_rand($notifTextArray);
				$notifTextRand = $notifTextArray[$notifText]." (Code: ".$notifNum.")";
			}
			else {
				$notifTextRand = $notifTextArray." (Code: ".$notifNum.")";
			}

			if(is_array($externalVar)) {
				foreach($externalVar as $key => $value) {
					$notifTextRand = str_replace("$"."externalVar[".$key."]",$externalVar[$key],$notifTextRand);
				}
			}
			else {
				$notifTextRand = str_replace("$"."externalVar[]",$externalVar,$notifTextRand);
				$notifTextRand = str_replace("$"."externalVar",$externalVar,$notifTextRand);
			}

		//	$notifTextRand = str_replace("'","\\'",$notifTextRand);    // making the sign ' be valid - add slash
		}
		else {
			$notifTextRand = false;
		}

		return array($notifType,$authLevel,$notifTextRand);
	}

	/**
	 * Notif generate number - generate the notification by it's number
	 * @param string $fileName - the file name, located at: "\text\notifs\language code\"
	 * @param string|array $notifNum - the notification number
	 * @param string|array|null $externalVar - the external variant to use in this notification
	 * @return array - the specific notification including: $notifType,$authLevel,$notifTextRand
	 * @example $this->genNotifByNum($fileName="auth",7024,$externalVar="yoyo");        // from controller
	 */
	public function genNotifByNum($fileName,$notifNum,$externalVar = "") {    // $externalVar - can be an array too - it's text replacer

    if(self::$cfg['notif']['allow']==true) {
			if(!$fileName) {    // the default notification file
				$this->loadNotifs($fileName = "core");
			}

			if(!isset(self::$notifsContent[$fileName])) {
				$this->loadNotifs($fileName);
			}

			if(!$this->userLoggedAuthLevel) {
				$auth                      = basic::load("auth","core");
				$this->userLoggedAuthLevel = $auth->getUserLoggedAuthLevel();
			}

			$fileDetails = "";

			if($fileName=="core" && self::$cfg['notif']['calledInfo']==1 && (basic::$cfg['site']['environment']=="dev" || 4<=$this->userLoggedAuthLevel)) {    // or user level have authLevel 4 or above
				$fileDetails = debug_backtrace()[0]['file'];
				$fileDetails = str_replace(ROOT.DS,"",$fileDetails);
			//	$fileDetails = addslashes($fileDetails);
				$fileDetails = ", File: ".$fileDetails;
			}    // get the file that called the notification

			if(is_array($notifNum)) {    // if we have array of notifications
				foreach($notifNum as $key => $value) {
					$this->notifNum($fileName,$value,$externalVar,$fileDetails);
				}
			}
			else {
				$this->notifNum($fileName,$notifNum,$externalVar,$fileDetails);
			}
		}
	}

  private function setNotifNum($notifNum,$externalVar) {

    if(empty(self::$notifsToShow[$notifNum]) || !is_array(self::$notifsToShow[$notifNum])) {
      self::$notifsToShow[$notifNum] = array();
    }

    if(!in_array($externalVar,self::$notifsToShow[$notifNum])) {
      self::$notifsToShow[$notifNum] = array_merge(self::$notifsToShow[$notifNum],array($externalVar));
    }
  }

	/**
	 * Notif number - issue the notification
	 * @param string $fileName - the file name, located at: "\text\notifs\language code\"
	 * @param string $notifNum - the notification number
	 * @param string|array|null $externalVar - (optional) the external variant to use in this notification
	 * @param string $fileDetails - details about the location that the notification is called from
	 * @return string - the specific notification that's coming out
	 * @example $this->notifNum($fileName,$value,$externalVar,$fileDetails);
	 */
	private function notifNum($fileName,$notifNum,$externalVar = "",$fileDetails) {

    if(!empty(self::$notifsToShow)) {   // in case we make more then 1 instance of controller we don't need to show the notif more then once
      foreach(self::$notifsToShow as $key => $value) {
        if(!is_array($notifNum)) {
          if($key==$notifNum) {
            if(in_array($externalVar,$value)) {
              return;
            }
            else {
              $this->setNotifNum($notifNum,$externalVar);
            }
          }
          else {
            if(empty(self::$notifsToShow[$notifNum]) || !in_array($externalVar,self::$notifsToShow[$notifNum])) {
              $this->setNotifNum($notifNum,$externalVar);
            }
          }
        }
      }
    }
    else {
      $this->setNotifNum($notifNum,$externalVar);
    }

  	list($notifTypePre,$authLevel,$notifMessage) = $this->parseNotifNum($fileName,$notifNum,$externalVar);

		if(!$notifMessage && $notifNum && self::$cfg['notif']['allow']!=2) {
			//	list($notifTypePre,$authLevel,$notifMessage) = $this->parseNotifNum(81260,$externalVar=$notifNum);
			$notifType    = "red";
			$notifTypePre = 3;
			$authLevel    = 4;
			$notifMessage = "No such notif code or maybe notifs ".lang::$currentLanguage." file is missing (Code: ".$notifNum.") - (Code: 81260)";
		}

		if(!isset($notifType)) {
			$notifType = "";
		}
		if($notifTypePre) {
			$notifType = self::$cfg['notif']['type'][$notifTypePre];
		}    // get the right color from the self::$cfg['notif']

		if(basic::$cfg['site']['environment']=="tech" || basic::$cfg['site']['environment']=="dev" || $authLevel<=$this->userLoggedAuthLevel) {    // or $authLevel is user level
			$theme = basic::$cfg['template']['theme']."Theme";

			if(!method_exists($theme,"notif")) {
				$notifTemplateClass = APPLICATIONPATH."themes".DS.basic::$cfg['template']['theme'].DS.basic::$cfg['template']['theme'].".theme.class.php";
				if(file_exists($notifTemplateClass)) {
					require_once($notifTemplateClass);
				}
			}

			self::$lastNotif = $notifMessage.$fileDetails;

			if(isset(self::$cfg['notif']['logNotifs']['logMethod'][$notifTypePre])) {
				if(is_array(self::$cfg['notif']['logNotifs']['logMethod'][$notifTypePre])) {
					if(in_array("file",self::$cfg['notif']['logNotifs']['logMethod'][$notifTypePre]) || in_array("fileJson",self::$cfg['notif']['logNotifs']['logMethod'][$notifTypePre])) {
						$auth                = basic::load("auth","core");
						$getUserLoggedUserId = $auth->getUserLoggedUserId() ? "User Id: ".$auth->getUserLoggedUserId() : "";

						if(isset(self::$cfg['notif']['logNotifs']['logLocation'][$notifTypePre])) {
							if(self::$cfg['notif']['logNotifs']['logLocation'][$notifTypePre]) {
								$logPath = self::$cfg['notif']['logNotifs']['logLocation'][$notifTypePre];
							}
							else {
								$logPath = "notifs-type-".$notifTypePre;
							}
						}
						$dirAndFile = myHelpers::fixDirAndFile($logPath,$ext = "log",$lastDir = IMPERMANENTPATH."logs");
						log::logToFile($dirAndFile,$data = self::$lastNotif." ,From: ".MyHelpers::getServerUrl(),$getUserLoggedUserId);
					}
				}

				$canPass = array(83200,83210,83220,83230);

				if(!in_array($notifNum,$canPass)) {    // those are mail errors that we do not need here - will cause a loop
					if(in_array("mail",self::$cfg['notif']['logNotifs']['logMethod'][$notifTypePre])) {

						if(!isset(basic::$cfg['notif']['sendErrorByMailInterval']))
							basic::$cfg['notif']['sendErrorByMailInterval'] = 0;        // sendErrorByMailInterval not done yet

						$mail   = basic::load("mail","functions");
						$auth   = basic::load("auth","core");
						$trans  = basic::load("trans","core");
						$userId = $auth->getUserLoggedUserId();
						if(!$userId) {
							$userId = "none";
						}
						$mail->sendMailToAdmin($messageId = "",$bodyFile = "notifs",$vars = array(
							"notifMessage" => $notifMessage,
							"fileName"     => $fileName,
							"userId"       => $userId
						),$subject = basic::$cfg['site']['name'].", env: ".ucfirst(basic::$cfg['site']['environment']).", ".$trans->T("notif number").": ".$notifNum);
					}

					if(in_array("db",self::$cfg['notif']['logNotifs']['logMethod'][$notifTypePre])) {
            if(basic::getConfig("db","main","activate")==true) {
              $db                      = basic::load(basic::getConfig("db","main","adapter"),'core/adapters');
              $table                   = "log_notifs";
              $createData['notifNum']  = strtolower($notifNum);
              $createData['notifType'] = strtolower($notifTypePre);
              $createData['authLevel'] = strtolower($authLevel);
              $createData['message']   = strtolower($notifMessage);
              $createData['fileName']  = strtolower($fileName);
              $createData['url']       = MyHelpers::getServerUrl();
              $createData['referer']   = MyHelpers::getServerReferer();

              if(!empty($db->pdo)) {
                $db->insert($table = basic::getConfig("db","main","tablePrefix").$table,$createData,$advancedLead = true);
              }
            }
					}
				}
			}

			if(array_key_exists($notifTypePre,self::$cfg['notif']['interval'])) {
				$lifeSpan = self::$cfg['notif']['interval'][$notifTypePre];

				if(method_exists($theme,"notif")) {
					$notif = $theme::notif($text = "** ".$notifMessage.$fileDetails." **",$notifType,$link = "",$linkText = "",$more = array("lifeSpan" => $lifeSpan),$breakOut = "");

					return $notif;
				}
				else {
					echo "<p><span style=\"color:orange;\">**</span> ".$notifMessage.$fileDetails." <span style=\"color:red;\">**</span></p>";
				}
			}
		}
	}

	/**
	 * Send notif - load notifs and sends them
	 * @param $notifNum
	 * @param $externalVar
	 * @param $fileName
	 * @param $fileToRequireDefault
   * @throws ReflectionException
   */
	public static function sendNotif($notifNum,$externalVar="",$fileName,$fileToRequireDefault) {

		$callers    = debug_backtrace();
		$callerFile = $callers[0]['file'];

		if(class_exists("Template")) {
			$notif                = basic::load("notif","core");
			$fileToRequireDefault = str_replace(ROOT.DS,"",$fileToRequireDefault);
			$fileToRequireDefault = addslashes($fileToRequireDefault);
			$notif->genNotifByNum($file = "core",$notifNum,"'".$fileToRequireDefault."' called from: '".$callerFile."'",$externalVar);
		}
		if(function_exists("notifNum")) {
			$notif->genNotifByNum($file = "core",$notifNum,"'".$fileName."' called from: '".$callerFile."'",$externalVar);
		}
		else {
			if(!class_exists('myHelpers') || myHelpers::isEnv($env = "dev")) {
				echo "Error (code ".$notifNum.") on '".$fileName."' called from: '".$callerFile."'";
				die();

				// Note: We use die() rather then show_error() in order to avoid a
				// self-referencing loop with the Exceptions class
			}
		}
	}

	/**
	 * Enable disable show ui container
	 */
	public function showNotifContainer() {

		if($this->allowNotifUi!==false) {
			$notifContainer = $this->showNotifUiContainer();

			if(basic::$cfg['notif']['afterHtml']==0) {
				echo $notifContainer;
			}
			$this->notifContainer = $notifContainer;
		}
	}

	/**
	 * Show ui container
	 */
	public function showNotifUiContainer() {

		if($this->notifUiContainerRun) {
			return false;
		}      // don't run this twice

		$dbUiHtml = "";
		$dbUiHtml .= "
			<div id=\"notifUiButton\" style=\"display:none;\">N</div>
			<div id=\"notifUi\" class=\"top-right notifUiNormal centerOverlay showAll\">
				<div class=\"bozoNotification bozoEmpty\" style=\"display:none;\">TT</div>
			</div>
			<div id=\"bozoCloseAll\" onclick=\"showNotifUi(99999);\" class=\"notifUiNum bozoNotification bozoCloser showed\"  style=\"display:none;\">[ close all ]</div>
		";

		$theme = basic::$cfg['template']['theme']."Theme";

		if(!method_exists($theme,"jsNotif")) {
			$notifTemplateClass = APPLICATIONPATH."themes".DS.basic::$cfg['template']['theme'].DS.basic::$cfg['template']['theme'].".theme.class.php";
			if(file_exists($notifTemplateClass)) {
				require_once(LABYARDPATH.DS.'mvc'.DS.'template.class.php');
				require_once($notifTemplateClass);
			}
		}

		if(class_exists($theme) && method_exists($theme,"jsNotif")) {
			$dbUiHtml .= $theme::jsNotif();
		}
		else {
			if(class_exists("rimTheme","jsNotif") && method_exists("rimTheme","jsNotif")) {
				$dbUiHtml .= rimTheme::jsNotif();
			}
		}

		$this->notifUiContainerRun = true;

		return $dbUiHtml;
	}
}

