<?php

/*************************************************************
 * Translation class
 * - all functions about translations
 * @author    S M Kessner <limbercode@gmail.com>
 * @copyright 2010-2018 T2
 * @link      http://limbercode.com
 *************************************************************/
class trans {

	public $textLoadFile;
	public $textLoad;
	private static $cfg;

	/**
	 * Constructor for Trans
	 */
	public function __construct() {

		self::$cfg = basic::loadCfg("langConfig",false);
	}

	/**
	 * Load translate - load the translation text
	 * @param string /array $fileName - the file name to load, must be located at: \text\translations\langCode
	 * @return array - the translation data - contain all the translated text
	 * @example $this->loadTrans($textFile);
	 */
	public function loadTrans($fileName) {

		if(!self::$cfg['lang']['loadMethod'] || self::$cfg['lang']['loadMethod']=="xml") {
			if(is_array($fileName)) {
				foreach($fileName as $key => $value) {
					$this->loadTransViaXml($fileName[$key]);
				}
			}
			else {
				return $this->loadTransViaXml($fileName);
			}
		}
		elseif(self::$cfg['lang']['loadMethod']=="array") {
			if(is_array($fileName)) {
				foreach($fileName as $key => $value) {
					$this->loadTransViaArray($fileName[$key]);
				}
			}
			else {
				return $this->loadTransViaArray($fileName);
			}
		}
	}

	/**
	 * Load translation via php array - load the translation text
	 * @param string $fileName - the file name to load, must be located at: \text\translations\langCode
	 * @return array - the translation array - contain all the translated text
	 */
	public function loadTransViaArray($fileName) {

		$loadFile = SOURCEPATH."text".DS."translations".DS.lang::getCurrentLanguage().DS.$fileName.".php";

		if(file_exists($loadFile)) {
			require_once($loadFile);

			if(isset($translations)) {
				$this->textLoadFile = $fileName;

				if($this->textLoad) {
					$merged = array_merge($this->textLoad,$translations);

					return $this->textLoad = $merged;
				}
				else {
					return $this->textLoad = $translations;    // returns only the var $translations
				}
			}
		}
		else {
			if(lang::$currentLanguage!=lang::getDefaultLanguage()) {        // "en" translation files can be missing - since it's based on english
			  // was: lang::$currentLanguage==lang::getDefaultLanguage()

				$notif = basic::load("notif","core");
				$notif->genNotifByNum("core",81230,lang::getCurrentLanguage()."/".$fileName.".php");
			}
		}
	}

	/**
	 * Load translation via php xml - load the translation text
	 * @param string $fileName - the file name to load, must be located at: \text\translations\langCode
	 * @return array - the translation array - contain all the translated text
	 */
	public function loadTransViaXml($fileName) {

		$xml      = basic::load('xml','functions');        // get the text codes from xml
		$loadFile = SOURCEPATH."text".DS."translations".DS.lang::$currentLanguage.DS.$fileName.".xml";

		if(file_exists($loadFile)) {
			$this->textLoadFile = $fileName;
			$xmlFileLoaded      = $xml::xmlToArray(file_get_contents($loadFile));

			if($this->textLoad) {
				$merged['translation'] = array_merge($this->textLoad['translation'],$xmlFileLoaded['translation']);

				return $this->textLoad = $merged;
			}
			else {
				return $this->textLoad = $xmlFileLoaded;    // returns only the var $notifs
			}
		}
		else {
			if(lang::$currentLanguage==lang::getDefaultLanguage()) {        // "en" translation files can be missing - since it's based on english
				$notif = basic::load("notif","core");
				$notif->genNotifByNum("core",81240,lang::$currentLanguage."/".$fileName.".xml");
			}
		}
	}

	/**
	 * Translate text - return the translation
	 * @param string $text - the text to be translated - written in the default language (en)
	 * @param string /array $externalVar - containing the values that will effect the translation
	 * @return string - the translated text
	 * @example $this->T($text="text");
	 */
	public function T($text,$externalVar = "") {

		return $this->extractTrans($text,$externalVar);
	}

	/**
	 * Extract translate - return the translate of the text including the use of $externalVar
	 * @param string $text - the text
	 * @param string /array/null $externalVar - the external variant to use in this translate
	 * @return string - the translation
	 * @example $this->extractTrans($text,$externalVar);
	 */
	public function extractTrans($text,$externalVar = "") {

		if(self::$cfg['lang']['loadMethod']=="array") {
			return $this->extractTransFromArray($text,$externalVar);
		}
		elseif(self::$cfg['lang']['loadMethod']=="xml") {
			return $this->extractTransFromXml($text,$externalVar);
		}
	}

	/**
	 * Extract translate from array - return the translate of the text including the use of $externalVar
	 * @param string $text - the text
	 * @param string /array/null $externalVar - the external variant to use in this translate
	 * @return string - the translation
	 * @example $this->extractTrans($text,$externalVar);
	 */
	private function extractTransFromArray($text,$externalVar = "") {

		$arrayTextFound = "";
    $textFound = "";

		if(isset($this->textLoad)) {        // if we found trans
			if(is_array($this->textLoad)) {    // if we have array of translation on file
				foreach($this->textLoad as $key => $value) {        // if we have the text on the file
					if(isset($this->textLoad[$key]) && $key==$text) {
						$arrayTextFound = $this->textLoad[$key];
					}
					elseif($arrayTextFound=="") {      // if we don't have the text on the file
						$arrayTextFound = $text;
					}
				}
			}
		}
		else {        // if we don't have any translation on the file
			$text = str_replace('$externalVar',"$externalVar",$text);

			return $text;
		}

		if(is_array($arrayTextFound)) {
			foreach($arrayTextFound as $key => $value) {    // leading number is the "key"
				if($externalVar==$key && !is_int($key)) {
					$textFound = $arrayTextFound[$key];
					break;
				}
				if($externalVar>=$key) {
					$textFound = $arrayTextFound[$key];
					continue;
				}
			}    // get the text according to the $externalVar number
		}
		else {
			$textFound = $arrayTextFound;
		}

		if($externalVar && isset($textFound)) {
		  if(is_array($externalVar)) {
		    foreach($externalVar as $key => $value) {
          $textFound = str_replace('$externalVar['.$key.']',$externalVar[$key],$textFound);    // replace what have to be replaced
        }
      }
		  else {
        $textFound = str_replace('$externalVar',$externalVar,$textFound);    // replace what have to be replaced
      }
		}

		if(!$textFound) {
			$textFound = $text;
		}

		return $textFound;
	}

	/**
	 * Extract translate from xml - return the translate of the text including the use of $externalVar
	 * @param string $text - the text
	 * @param string /array/null $externalVar - the external variant to use in this translate
	 * @return string - the translation
	 * @example $this->extractTrans($text,$externalVar);
	 */
	private function extractTransFromXml($text,$externalVar = "") {

		$arrayTextFound = "";

		if(isset($this->textLoad['translation'])) {        // if we found trans
			if(is_array($this->textLoad['translation'])) {    // if we have array of translation on file
				foreach($this->textLoad['translation'] as $key => $value) {        // if we have the text on the file
					if(isset($this->textLoad['translation'][$key]['@attributes']['text']) && $this->textLoad['translation'][$key]['@attributes']['text']==$text) {
						$arrayTextFound = $this->textLoad['translation'][$key]['@attributes'];
					}
					elseif($arrayTextFound=="") {      // if we don't have the text on the file
						$arrayTextFound = $text;
					}
				}    // extract the array of the text we need (exctract the array)
			}
			else {    // if we have only 1 translation on file
				if(isset($this->textLoad['translation']['@attributes']['text']) && $this->textLoad['translation']['@attributes']['text']==$text) {
					$arrayTextFound = $this->textLoad['translation']['@attributes'];
				}
				else {        // if we don't have the text on the file
					$arrayTextFound = $text;
				}
			}
		}
		else {        // if we don't have any tanslation on the file
			$text = str_replace('$externalVar',"$externalVar",$text);

			return $text;
		}

		// extract the trans line according to $externalVar
		if(isset($arrayTextFound['trans'])) {       // if we have only 1 translation available
			$textFound = $arrayTextFound['trans'];
		}
		else {
			$textFound = $arrayTextFound;
		}

		if(count($arrayTextFound)>2 && $externalVar>1) {    // if we have more then 1 translation and $externalVar>1
			foreach($arrayTextFound as $key => $value) {
				if($key!="text") {
					$transExplode = explode(".",$key);        // find the leading number

					if(isset($transExplode[1])) {
						if($externalVar>=$transExplode[1]) {
							$textFound = $arrayTextFound[$key];
						}
					}    // get the text according to the $externalVar number
				}
			}
		}

		if($externalVar) {
			$textFound = str_replace('$externalVar',"$externalVar",$textFound);    // replace what have to be replaced
		}

		return $textFound;
	}

	/**
	 * Seconds 2 human - converts seconds to human readable time left
	 * @param integer $s - the seconds to compute
	 * @return string - the human readable time
	 * @example $this->trans->seconds2Human($s=22554);    // from controller
	 */
	public function seconds2Human($s) {

		$this->loadTrans("seconds");     // loads translations

		$ss = $s % 60;
		$m  = (floor(($s % 3600) / 60)>0) ? floor(($s % 3600) / 60)." ".$this->T("minutes",floor(($s % 3600) / 60)) : "";
		$h  = (floor(($s % 86400) / 3600)>0) ? floor(($s % 86400) / 3600)." ".$this->T("hours",floor(($s % 86400) / 3600)) : "";
		$d  = (floor(($s % 2592000) / 86400)>0) ? floor(($s % 2592000) / 86400)." ".$this->T("days",floor(($s % 2592000) / 86400)) : "";
		$M  = (floor($s / 2592000)>0) ? floor($s / 2592000)." ".$this->T("months",floor($s / 2592000)) : "";

		$sec = "";
		if($ss) {
			$sec = $ss." ".$this->T("seconds",$ss);
		}

		$return = trim("$M $d $h $m $sec");
		if(!$return) {
			$return = "0";
		}

		return $return;
	}
}

