<?php

/*************************************************************
 * Array manipulation
 * @author    S M Kessner <limbercode@gmail.com>
 * @copyright 2010-2018 T2
 * @link      http://limbercode.com
 *************************************************************/
class arrayManipulation {

  /**
   * Build tree - generate an array of parent/child
   * @param array $data - the array of data
   * @param string $leadKey - the id - "leadId"
   * @param string $parent - the parent id - "parentId"
   * @param string $itemExist - if we want list of categories that have items - the name of the var, example: "saleSiteExist"
   * @param int $seek
   * @return mixed
   */
  static public function buildTree($data,$leadKey,$parent,$itemExist = "",$seek = 0) {

    $tree     = array();
    $children = "";
    $n        = "";

    foreach($data as $key => $value) {

      if($value[$parent]==$seek) {
        $i = $value[$leadKey];
        unset($data[$key],$value[$leadKey],$value[$parent]);

        if($parent) {
          $children = self::buildTree($data,$leadKey,$parent,$itemExist,$i);
        }

        if($children) {
          $tree[$i] = array(
            'attributes' => $value,
            'children'   => $children
          );
        }
        else {
          if(empty($itemExist) || (!empty($itemExist) && !empty($value[$itemExist]))) {
            $tree[$i] = array(
              'attributes' => $value
            );
          }
        }
      }
    }

    return $tree;
  }

  /**
   * @param $array
   * @param $checkEmptyKey
   * @return mixed
   */
  static public function removeEmptyArray($array,$checkEmptyKey) {

    foreach($array as $key => $value) {
      if(is_array($value[$checkEmptyKey])) {
        $array = self::removeEmptyArray($array,$checkEmptyKey);
      }
      else {
        if(empty($value[$checkEmptyKey])) {
          unset($array[$key]);
        }
      }
    }

    return $array;
  }

  /**
   * Array multidimensional key exists
   * @param $key
   * @param array $haystack
   * @return bool
   */
  static function arrayMultidimensionalKeyExists($key,array $haystack) {

    if(array_key_exists($key,$haystack)) {       // is in base array?
      return true;
    }

    foreach($haystack as $element) {      // check arrays contained in this array
      if(is_array($element)) {
        if(self::arrayMultidimensionalKeyExists($key,$element)) {
          return true;
        }
      }
    }

    return false;
  }

  /**
   * Array change key - changing a key of array
   * @param array $arr - the subject array
   * @param string $from - what to replace
   * @param string $to - replace to what
   * @return bool|array - the array after the replace
   */
  static public function arrayChangeKey(&$arr,$from,$to) {

    if(!isset($arr[$from]) || isset($arr[$to])) {
      return false;
    }

    $arr[$to] = $arr[$from];
    unset($arr[$from]);

    return true;
  }

  /**
   * Array sort by column - will make the array start from 0 key
   * @param $array
   * @param $column
   * @param string $direction
   */
  static public function arraySortByColumn(&$array,$column,$direction = "asc") {

    $dir = $direction=="asc" ? SORT_ASC : SORT_DESC;

    $sortColumn = array();
    foreach($array as $key => $row) {
      $sortColumn[$key] = $row[$column];
    }

    array_multisort($sortColumn,$dir,$array);
  }

  /**
   * Array depth - will return the array depth
   * @param array $array
   * @return int
   */
  static function arrayDepthSize(array $array) {

    $maxDepth = 1;

    foreach($array as $value) {
      if(is_array($value)) {
        $depth = self::arrayDepthSize($value) + 1;

        if($depth>$maxDepth) {
          $maxDepth = $depth;
        }
      }
    }

    return $maxDepth;
  }

  /**
   * Array depth - will return the array depth
   * @param $array
   * @param bool $limit
   * @return int
   */
  static function arrayDepth($array,$limit = false) {

    if(!is_array($array)) {
      return 0;
    }
    $array = json_encode($array);

    $limit  = !empty($limit) ? $limit : strlen($array);
    $varSum = 0;
    $depth  = 0;
    for($i = 0; $i<$limit; $i++) {
      $varSum += intval($array[$i]=='[') - intval($array[$i]==']');
      if($varSum>$depth) {
        $depth = $varSum;
      }
    }

    return $depth;
  }

  /**
   * Check if array Multidimensional
   * @param $array
   * @return bool
   */
  static function checkIfArrayMultidimensional($array) {

    if(count($array)!=count($array,COUNT_RECURSIVE)) {
      return true;
    }

    return false;
  }

  /**
   * Array type - Check if array is numeric or sequential associative
   * @param $array
   * @return string
   */
  static function arrayType($array) {

    $type = "associative";

    if(array()===$array) {
      $type = "numeric";
    }

    if(array_keys($array)!==range(0,count($array) - 1)) {
      $type = "sequential";
    }

    return $type;
  }

  /**
   * Array type - Check if all array values is numeric or sequential associative
   * @param $array
   * @return string
   */
  static function arrayValuesType($array) {

    $type = "associative";

    if(ctype_digit(implode('',$array))) {
      $type = "numeric";
    }

    return $type;
  }

  /**
   * Is array equal?
   * @param $a
   * @param $b
   * @return bool
   */
  /*  static function isArrayEqual($a,$b) {
      return (
        is_array($a)
        && is_array($b)
        && count($a) == count($b)
        && array_diff($a,$b) === array_diff($b,$a)
      );
    }*/

  /**
   * Is array equal?
   * @param $array1
   * @param $array2
   * @param $checkPositioning - if to check also positioning
   * @return bool
   */
  static function isArrayEqual($array1,$array2,$checkPositioning = false) {

    if(count($array1)!=count($array2)) {    // if number of keys not the same, no need to check the rest
      return false;
    }

    if($checkPositioning==true) {
      if(serialize($array1)==serialize($array2)) {
        return true;
      }
    }
    else {
      array_multisort($array1);
      array_multisort($array2);

      if(serialize($array1)===serialize($array2)) {
        return true;
      }
    }
  }

  /**
   * Push into position
   * @param $array
   * @param $key
   * @param $value
   * @return array
   */
  static function pushIntoPosition($array,$key,$value) {

    $array = array_slice($array,0,$key - 1,true) + array($key => $value) + array_slice($array,$key - 1,count($array),true);

    return $array;
  }

  /**
   * Search Key in multidimensional array
   * @param $array
   * @param $keySearch
   * @param bool|string $returnValueType - can be: false,key (parent key),value (parent array with values),path (path - backwards path of the array)
   * @param false $path
   * @return bool|mixed
   */
  static function searchKey($keySearch,$array,$returnValueType = false,$path = false) {

    foreach($array as $key => $item) {
      if($key==$keySearch) {
        if($returnValueType==true) {
          return $item;
        }
        else {
          return true;
        }
      }
      elseif(is_array($item)) {
        $path = self::searchKey($keySearch,$item,$returnValueType,$path);

        if($path) {
          if($returnValueType=="key") {
            return $key;
          }
          elseif($returnValueType=="value") {
            return $item;
          }
          elseif($returnValueType=="path") {
            $path[] = $key;
            return $path;
          }
          else {
            return true;
          }
        }
      }
    }

    return false;
  }

  /**
   * Recursive array search
   * @param $needle
   * @param $haystack
   * @return false|int|string
   */
  static function recursiveArraySearch($needle,$haystack) {

    foreach($haystack as $key => $value) {
      $currentKey = $key;
      if($needle===$value or (is_array($value) && self::recursiveArraySearch($needle,$value)!==false)) {
        return $currentKey;
      }
    }

    return false;
  }

  /**
   * Insert into array
   * @param $array
   * @param $searchKey
   * @param $insertKey
   * @param $insertValue
   * @param bool $insertAfterFoundedKey
   * @param bool $appendIfNotFound
   * @return array
   */
  static function insertIntoArray($array,$searchKey,$insertKey,$insertValue,$insertAfterFoundedKey = true,$appendIfNotFound = false) {

    $newArray = array();

    foreach($array as $key => $value) {
      // insert before the current key?
      // only if current key is the key we are searching for, and we want to insert before that founded key
      if($key===$searchKey && !$insertAfterFoundedKey) {
        $newArray[$insertKey] = $insertValue;
      }

      $newArray[$key] = $value;   // copy the current key/value from old array to a new array

      // insert after the current key?
      // only if current key is the key we are searching for, and we want to insert after that founded key
      if($key===$searchKey && $insertAfterFoundedKey) {
        $newArray[$insertKey] = $insertValue;
      }
    }

    if($appendIfNotFound && count($array)==count($newArray)) {    // append if key isn't founded
      $newArray[$insertKey] = $insertValue;
    }

    return $newArray;
  }

  static function arrayRecursiveDiff($aArray1,$aArray2) {   // but not checking positioning
    $aReturn = array();

    foreach($aArray1 as $mKey => $mValue) {
      if(array_key_exists($mKey,$aArray2)) {
        if(is_array($mValue)) {
          $aRecursiveDiff = self::arrayRecursiveDiff($mValue,$aArray2[$mKey]);
          if(count($aRecursiveDiff)) {
            $aReturn[$mKey] = $aRecursiveDiff;
          }
        }
        else {
          if($mValue!=$aArray2[$mKey]) {
            $aReturn[$mKey] = $mValue;
          }
        }
      }
      else {
        $aReturn[$mKey] = $mValue;
      }
    }

    return $aReturn;
  }

  static function compareArrayValues($arrayOne,$arrayTwo,&$diff = [],$reversed = false) {   // not checking positioning
    foreach($arrayOne as $key => $val) {
      if(!isset($arrayTwo[$key])) {
        $diff[$key] = 'MISSING IN '.($reversed ? 'FIRST' : 'SECOND');
      }
      else if(is_array($val) && (json_encode($arrayOne[$key])!==json_encode($arrayTwo[$key]))) {
        self::compareArrayValues($arrayOne[$key],$arrayTwo[$key],$diff[$key],$reversed);
      }
      else if($arrayOne[$key]!==$arrayTwo[$key]) {
        $diff[$key] = 'DIFFERENT';
      }
    }

    return $diff;
  }

  /**
   * Compare two arrays and return a list of items only in array1 (deletions) and only in array2 (insertions)
   *
   * @param array $array1 The 'original' array, for comparison. Items that exist here only are considered to be deleted (deletions).
   * @param array $array2 The 'new' array. Items that exist here only are considered to be new items (insertions).
   * @param array|null $keysToCompare A list of array key names that should be used for comparison of arrays (ignore all other keys)
   * @return array[] array with keys 'insertions' and 'deletions'
   */
  public static function arrayDifference(array $array1,array $array2,array $keysToCompare = null) {   // not so good.......
    $serialize = function (&$item,$idx,$keysToCompare) {

      if(is_array($item) && $keysToCompare) {
        $a = array();
        foreach($keysToCompare as $k) {
          if(array_key_exists($k,$item)) {
            $a[$k] = $item[$k];
          }
        }
        $item = $a;
      }
      $item = serialize($item);
    };

    $deserialize = function (&$item) {

      $item = unserialize($item);
    };

    array_walk($array1,$serialize,$keysToCompare);
    array_walk($array2,$serialize,$keysToCompare);

    // Items that are in the original array but not the new one
    $deletions  = array_diff($array1,$array2);
    $insertions = array_diff($array2,$array1);

    array_walk($insertions,$deserialize);
    array_walk($deletions,$deserialize);

    return array('insertions' => $insertions,'deletions' => $deletions);
  }

  static function arrayRecursiveDiff2($aArray1,$aArray2) {   // not checking positioning
    $aReturn = array();

    foreach($aArray1 as $mKey => $mValue) {
      if(array_key_exists($mKey,$aArray2)) {
        if(is_array($mValue)) {
          $aRecursiveDiff = self::arrayRecursiveDiff2($mValue,$aArray2[$mKey]);
          if(count($aRecursiveDiff)) {
            $aReturn[$mKey] = $aRecursiveDiff;
          }
        }
        else {
          if($mValue!=$aArray2[$mKey]) {
            $aReturn[$mKey] = $mValue;
          }
        }
      }
      else {
        $aReturn[$mKey] = $mValue;
      }
    }

    return $aReturn;
  }

  static function array_diff_assoc_recursive($array1,$array2) {    // not checking positioning

    $difference = null;
    foreach($array1 as $key => $value) {
      if(is_array($value)) {
        if(!array_key_exists($key,$array2)) {
          $difference[$key] = $value;
        }
        elseif(!is_array($array2[$key])) {
          $difference[$key] = $value;
        }
        else {
          $new_diff = self::array_diff_assoc_recursive($value,$array2[$key]);
          if($new_diff!=false) {
            $difference[$key] = $new_diff;
          }
        }
      }
      elseif(!array_key_exists($key,$array2) || $array2[$key]!=$value) {
        $difference[$key] = $value;
      }
    }

    return !isset($difference) ? 0 : $difference;
  }

  static function arrayDiffRecursive($firstArray,$secondArray,$reverseKey = false) {

    $oldKey = 'old';
    $newKey = 'new';
    if($reverseKey) {
      $oldKey = 'new';
      $newKey = 'old';
    }
    $difference = [];
    foreach($firstArray as $firstKey => $firstValue) {
      if(is_array($firstValue)) {
        if(!array_key_exists($firstKey,$secondArray) || !is_array($secondArray[$firstKey])) {
          $difference[$oldKey][$firstKey] = $firstValue;
          $difference[$newKey][$firstKey] = '';
        }
        else {
          $newDiff = self::arrayDiffRecursive($firstValue,$secondArray[$firstKey],$reverseKey);
          if(!empty($newDiff)) {
            $difference[$oldKey][$firstKey] = $newDiff[$oldKey];
            $difference[$newKey][$firstKey] = $newDiff[$newKey];
          }
        }
      }
      else {
        if(!array_key_exists($firstKey,$secondArray) || $secondArray[$firstKey]!=$firstValue) {
          $difference[$oldKey][$firstKey] = $firstValue;
          $difference[$newKey][$firstKey] = $secondArray[$firstKey];
        }
      }
    }

    return $difference;
  }
}

