<?php

/*************************************************************
 * Captcha class
 * @author    S M Kessner <limbercode@gmail.com>
 * @copyright 2010-2018 T2
 * @link      http://limbercode.com
 *************************************************************/
class captcha {

	private static $cfg;
	private $codeToEncode;
	private $verificationTriesCSN;
	private $verificationCodeCSN;
	private $image;

	/**
	 * Constructor for captcha
	 * @param string $auth - auth class object
   * @param $verificationTriesCSN
   * @param $verificationCodeCSN
   * @throws ReflectionException
   */
	function __construct($auth,$verificationTriesCSN,$verificationCodeCSN) {

		$this->verificationTriesCSN = $verificationTriesCSN;
		$this->verificationCodeCSN  = $verificationCodeCSN;
		self::$cfg                  = basic::loadCfg("captchaConfig",false);
		$this->captchaRenderImage();
		$this->captchaShowImage();
	}

	/**
	 * Captcha show image
	 */
	function captchaShowImage() {

		global $auth;

		$verificationTriesCS = $auth->getCookieNow($name = $this->verificationTriesCSN);

		if(basic::$cfg['auth']['captcha']['verificationKeySensitive']!=1) {
			$this->codeToEncode = strtolower($this->codeToEncode);
		}

		if(strpos($verificationTriesCS,"T")===false && basic::$cfg['auth']['captcha']['verificationTriesLimit']>$verificationTriesCS) {
			$verificationTries = (int) $verificationTriesCS + 1;
		}    // if timeLimit is not yet set and verificationTries is NOT over verificationTriesLimit
		else {
			$createdAt          = time() + basic::$cfg['auth']['captcha']['verificationWaitDelay'];
			$verificationTries = "T-".$createdAt;    // "T" will define timeLimit
		}    // if verificationTries is over verificationTriesLimit create timeLimit

		$auth->setCookieNow($name = $this->verificationCodeCSN,$data = md5($this->codeToEncode),$till = "",$domain = "");
		$auth->setCookieNow($name = $this->verificationTriesCSN,$data = $verificationTries,$till = "",$domain = "");
		$auth->setCookieNow($name = "token",$data = time(),$till = "",$domain = "");

		// Output the image to browser
		header("Content-type: image/png");
		header("Expires: Sun,1 Jan 2000 12:00:00 GMT");
		header("Last-Modified: ".gmdate("D,d M Y H:i:s")."GMT");
		header("Cache-Control: no-store,no-cache,must-revalidate");
		header("Cache-Control: post-check=0,pre-check=0",false);
		header("Pragma: no-cache");
		imagepng($this->image);
		imagedestroy($this->image);
	}

	/**
	 * Captcha render image
	 */
	function captchaRenderImage() {

		// Create an image
		list($widthOfCredit,$heightOfCredit) = getimagesize(basic::getPublicDirectory().self::$cfg['captcha']['credits']);
		self::$cfg['captcha']['credits'] ? $this->image = imagecreate(self::$cfg['captcha']['width'],($heightOfCredit + self::$cfg['captcha']['height'])) : $this->image = imagecreate(self::$cfg['captcha']['width'],self::$cfg['captcha']['height']);

		// Random background and color scheme. Can be red,green or blue
		// $backgrounds = array("FF0000","00FF00","0000FF");

		if(self::$cfg['captcha']['colorType']=="dark") {
			$backgrounds = array("FF0000","0000FF","");
		}
		elseif(self::$cfg['captcha']['colorType']=="bright") {
			$backgrounds = array("00FFFF","00FF00","FFFF00");
		}

		$colorScheme = $backgrounds[mt_rand(0,2)];

		if($colorScheme) {
			$colors = array(
				hexdec("0x".$colorScheme[0].$colorScheme[1]),
				hexdec("0x".$colorScheme[2].$colorScheme[3]),
				hexdec("0x".$colorScheme[4].$colorScheme[5])
			);
		}
		else {
			$colors = array(0 => "0",1 => "0",2 => "0");
		}

		$bgcolor = imagecolorallocate($this->image,$colors[0],$colors[1],$colors[2]);

		// Let's make some lighter and darker colors
		if($colorScheme=="FF0000" || $colorScheme=="00FFFF") {
			$lightColor[] = imagecolorallocate($this->image,$colors[0],$colors[1] + mt_rand(230,240),$colors[2] + mt_rand(230,240));
			$lightColor[] = imagecolorallocate($this->image,$colors[0],$colors[1] + mt_rand(230,240),$colors[2] + mt_rand(230,240));
			$lightColor[] = imagecolorallocate($this->image,$colors[0],$colors[1] + mt_rand(160,220),$colors[2] + mt_rand(160,220));
			$darkColor[]  = imagecolorallocate($this->image,$colors[0] - mt_rand(50,100),$colors[1] + mt_rand(0,50),$colors[2] + mt_rand(0,50));
			$darkColor[]  = imagecolorallocate($this->image,$colors[0] - mt_rand(50,100),$colors[1] + mt_rand(0,50),$colors[2] + mt_rand(0,50));
			$darkColor[]  = imagecolorallocate($this->image,$colors[0] - mt_rand(50,100),$colors[1] + mt_rand(0,50),$colors[2] + mt_rand(0,50));
		}
		elseif($colorScheme=="00FF00" || $colorScheme=="FFFF00") {
			$lightColor[] = imagecolorallocate($this->image,$colors[0] + mt_rand(230,240),$colors[1],$colors[2] + mt_rand(230,240));
			$lightColor[] = imagecolorallocate($this->image,$colors[0] + mt_rand(230,240),$colors[1],$colors[2] + mt_rand(230,240));
			$lightColor[] = imagecolorallocate($this->image,$colors[0] + mt_rand(150,190),$colors[1],$colors[2] + mt_rand(150,190));
			$darkColor[]  = imagecolorallocate($this->image,$colors[0] + mt_rand(0,130),$colors[1] - mt_rand(50,100),$colors[2] + mt_rand(0,130));
			$darkColor[]  = imagecolorallocate($this->image,$colors[0] + mt_rand(0,130),$colors[1] - mt_rand(50,100),$colors[2] + mt_rand(0,130));
			$darkColor[]  = imagecolorallocate($this->image,$colors[0] + mt_rand(0,130),$colors[1] - mt_rand(50,100),$colors[2] + mt_rand(0,130));
		}
		else {
			$lightColor[] = imagecolorallocate($this->image,$colors[0] + mt_rand(210,230),$colors[1] + mt_rand(210,230),$colors[2]);
			$lightColor[] = imagecolorallocate($this->image,$colors[0] + mt_rand(210,230),$colors[1] + mt_rand(210,230),$colors[2]);
			$lightColor[] = imagecolorallocate($this->image,$colors[0] + mt_rand(180,200),$colors[1] + mt_rand(180,200),$colors[2]);
			$darkColor[]  = imagecolorallocate($this->image,$colors[0] + mt_rand(0,100),$colors[1] + mt_rand(0,100),$colors[2] - mt_rand(70,150));
			$darkColor[]  = imagecolorallocate($this->image,$colors[0] + mt_rand(0,100),$colors[1] + mt_rand(0,100),$colors[2] - mt_rand(70,150));
			$darkColor[]  = imagecolorallocate($this->image,$colors[0] + mt_rand(0,100),$colors[1] + mt_rand(0,100),$colors[2] - mt_rand(70,150));
		}

		for($i = 1; $i<=self::$cfg['captcha']['rectangles']["mass"]; $i++) {        // draw rectangles
			imagefilledrectangle($this->image,$i * self::$cfg['captcha']['width'] / 10 + mt_rand(4,self::$cfg['captcha']['rectangles']["maxWidth"]),mt_rand(4,self::$cfg['captcha']['rectangles']["maxHeight"]),$i * self::$cfg['captcha']['width'] / 10 - mt_rand(4,self::$cfg['captcha']['rectangles']["maxWidth"]),mt_rand(4,self::$cfg['captcha']['rectangles']["maxHeight"]),$darkColor[mt_rand(0,2)]);
		}

		if(self::$cfg['captcha']['distortion']["color"]) {        // draw the lines and dots
			/* set the colours */
			$background_color = imagecolorallocate($this->image,255,255,255);
			$text_color       = imagecolorallocate($this->image,20,40,100);
			$noise_color      = imagecolorallocate($this->image,100,120,180);

			for($i = 0; $i<(self::$cfg['captcha']['width'] * self::$cfg['captcha']['height']) / 3; $i++) {
				imagefilledellipse($this->image,mt_rand(0,self::$cfg['captcha']['width']),mt_rand(0,self::$cfg['captcha']['height']),1,1,$noise_color);
			}
			for($i = 0; $i<(self::$cfg['captcha']['width'] * self::$cfg['captcha']['height']) / 150; $i++) {
				imageline($this->image,mt_rand(0,self::$cfg['captcha']['width']),mt_rand(0,self::$cfg['captcha']['height']),mt_rand(0,self::$cfg['captcha']['width']),mt_rand(0,self::$cfg['captcha']['height']),$noise_color);
			}
		}

		// Renormalize contrast
		$contrast   = 60;
		$contrast   = 255 * ($contrast / 100.0);
		$oCcontrast = 1.3 * $contrast;

		for($i = 1; $i<self::$cfg['captcha']['circles']["mass"]; $i++) {        // draw random circles
			$color = imagecolorallocatealpha($this->image,mt_rand(0,$oCcontrast),mt_rand(0,$oCcontrast),mt_rand(0,$oCcontrast),self::$cfg['captcha']['circles']["alfa"]);
			//	$color = imagecolorallocate($this->image, mt_rand(0,$oCcontrast), mt_rand(0,$oCcontrast), mt_rand(0,$oCcontrast));
			imagefilledellipse($this->image,mt_rand(0,self::$cfg['captcha']['width']),mt_rand(0,self::$cfg['captcha']['height']),mt_rand(self::$cfg['captcha']['circles']["minRadius"],self::$cfg['captcha']['circles']["maxRadius"]),mt_rand(self::$cfg['captcha']['circles']["minRadius"],self::$cfg['captcha']['circles']["maxRadius"]),$color);
		}

		for($i = 1; $i<self::$cfg['captcha']['polygons']["mass"]; $i++) {        // draw polygons
			$vertices = array(
				mt_rand(-0.25 * self::$cfg['captcha']['width'],self::$cfg['captcha']['width'] * 1.25),
				mt_rand(-0.25 * self::$cfg['captcha']['width'],self::$cfg['captcha']['width'] * 1.25),
				mt_rand(-0.25 * self::$cfg['captcha']['width'],self::$cfg['captcha']['width'] * 1.25),
				mt_rand(-0.25 * self::$cfg['captcha']['width'],self::$cfg['captcha']['width'] * 1.25),
				mt_rand(-0.25 * self::$cfg['captcha']['width'],self::$cfg['captcha']['width'] * 1.25),
				mt_rand(-0.25 * self::$cfg['captcha']['width'],self::$cfg['captcha']['width'],self::$cfg['captcha']['width'] * 1.25)
			);
			$color    = imagecolorallocatealpha($this->image,mt_rand(0,$oCcontrast),mt_rand(0,$oCcontrast),mt_rand(0,$oCcontrast),self::$cfg['captcha']['polygons']["alfa"]);
			imagefilledpolygon($this->image,$vertices,3,$color);
		}

		for($i = 1; $i<self::$cfg['captcha']['lines']["mass"]; $i++) {        // draw random lines
			$x1    = mt_rand(-self::$cfg['captcha']['width'] * 0.25,self::$cfg['captcha']['width'] * self::$cfg['captcha']['lines']["maxWidthTimes"]);
			$y1    = mt_rand(-self::$cfg['captcha']['height'] * 0.25,self::$cfg['captcha']['height'] * self::$cfg['captcha']['lines']["maxHeightTimes"]);
			$x2    = mt_rand(-self::$cfg['captcha']['width'] * 0.25,self::$cfg['captcha']['width'] * self::$cfg['captcha']['lines']["maxWidthTimes"]);
			$y2    = mt_rand(-self::$cfg['captcha']['height'] * 0.25,self::$cfg['captcha']['height'] * self::$cfg['captcha']['lines']["maxHeightTimes"]);
			$color = imagecolorallocatealpha($this->image,mt_rand(0,$oCcontrast),mt_rand(0,$oCcontrast),mt_rand(0,$oCcontrast),self::$cfg['captcha']['lines']["alfa"]);
			imagesetthickness($this->image,mt_rand(self::$cfg['captcha']['lines']["minThickness"],self::$cfg['captcha']['lines']["maxThickness"]));
			imageline($this->image,$x1,$y1,$x2,$y2,$color);
		}

		for($i = 1; $i<=self::$cfg['captcha']['width'] / self::$cfg['captcha']['gridLines']["spacing"]; $i++) {        // grid lines
			imageline($this->image,$i * self::$cfg['captcha']['gridLines']["spacing"] + mt_rand(4,26),0,$i * self::$cfg['captcha']['gridLines']["spacing"] - mt_rand(4,26),self::$cfg['captcha']['height'] - 1,$lightColor[mt_rand(0,2)]);
			imageline($this->image,$i * self::$cfg['captcha']['gridLines']["spacing"] + mt_rand(4,26),self::$cfg['captcha']['height'] - 1,$i * self::$cfg['captcha']['gridLines']["spacing"] - mt_rand(4,26),0,$lightColor[mt_rand(0,2)]);
		}

		if(self::$cfg['captcha']['questionType']=="text") {        // This creates the captcha text
			$symbols  = array(
				"2",
				"3",
				"4",
				"5",
				"6",
				"7",
				"8",
				"9",
				"A",
				"C",
				"E",
				"G",
				"H",
				"K",
				"M",
				"N",
				"P",
				"R",
				"S",
				"U",
				"V",
				"W",
				"Z",
				"Y",
				"Z"
			);    // allowed characters
			$stringer = "";        // reset variant
			for($i = 1; $i<=self::$cfg['captcha']['numOfChars']; $i++) {
				$stringer .= $symbols[mt_rand(0,24)];
			}

			self::$cfg['captcha']['numOfChars'] = strlen($stringer);
			$this->codeToEncode                 = $stringer;
		}
		elseif(self::$cfg['captcha']['questionType']=="math") {
			$operators = array("+","-","*");
			// first number random value; keep it lower than $secondNum
			$firstNum = rand(1,5);
			// second number random value
			$secondNum = rand(6,11);
			$thirdNum  = rand(6,11);

			shuffle($operators);

			for($counter = 0; $counter<1; $counter++) {    // loop untill no negative $mathResult
				$expression = $secondNum.$operators[0].$firstNum.$operators[1].$thirdNum;

				if(preg_match('~^(?:["0-9()*/%+-<>="]+)$~',$expression)>0) {     // safe to eval()
					eval("\$mathResult=$expression;");

					if(preg_match('/^["1-9"]/',$mathResult)!=1) {    // if negative number
						$counter = $counter - 1;
					}
				}
			}

			$stringer                           = $expression;
			$part                               = str_split($stringer);
			self::$cfg['captcha']['numOfChars'] = self::$cfg['captcha']['numOfChars'] + 1;    // because the last number have to show - like 11 that is 2 chars actually

			$this->codeToEncode = $mathResult;        // the result
		}
		elseif(file_exists(dirname(__FILE__)."/".self::$cfg['captcha']['questionType'])) {        // This creates the captcha word
			//		$words = array("walla","wawawi","nooo");	// random words
			$wordsFile = dirname(__FILE__)."/".self::$cfg['captcha']['questionType'];    // or random file words

			$fp     = fopen($wordsFile,"r");
			$length = strlen(fgets($fp));
			$line   = rand(1,(filesize($wordsFile) / $length) - 2);
			if(fseek($fp,$length * $line)==-1) {
				return false;
			}
			$wordsFromFile = trim(fgets($fp));
			fclose($fp);

			// change ramdom volcals
			/*          if($extended) {
			$wordsFromFile   = preg_split("//", $wordsFromFile, -1, PREG_SPLIT_NO_EMPTY);
			$vocals = array("a", "e", "i", "o", "u");
			foreach ($wordsFromFile as $i => $char) {
			if(mt_rand(0, 1) && in_array($char, $vocals)) {
			$wordsFromFile[$i] = $vocals[mt_rand(0, 4)];
			}
			}
			$wordsFromFile = implode("", $wordsFromFile);
			}  */

			$words = array();

			if($wordsFile) {
				$stringer = $wordsFromFile;
			}
			else {
				$stringer = $words[array_rand($words)];
			}

			$countChars         = strlen($stringer);
			$this->codeToEncode = $stringer;
		}
		else {
			$this->codeToEncode = "";
		}

		if($this->codeToEncode) {
			if(self::$cfg['captcha']['textColor']=="black") {
				$lightColor   = "";
				$lightColor[] = imagecolorallocate($this->image,0,0,0);
				$lightColor[] = imagecolorallocate($this->image,0,0,0);
				$lightColor[] = imagecolorallocate($this->image,0,0,0);
			}
			elseif(self::$cfg['captcha']['textColor']=="white") {
				$lightColor   = "";
				$lightColor[] = imagecolorallocate($this->image,255,255,255);
				$lightColor[] = imagecolorallocate($this->image,255,255,255);
				$lightColor[] = imagecolorallocate($this->image,255,255,255);
			}

			if(function_exists("imagettftext")) {        // inserting the text
				for($i = 0; $i<=self::$cfg['captcha']['numOfChars']; $i++) {
					if(isset($stringer[$i])) {
						imagettftext($this->image,mt_rand(self::$cfg['captcha']['fontSize'] - self::$cfg['captcha']['fontSize'] / 6,self::$cfg['captcha']['fontSize']),mt_rand(-20,20),$i * mt_rand(self::$cfg['captcha']['fontSize'] / 2 + self::$cfg['captcha']['fontSize'] / 2 - self::$cfg['captcha']['spaceMinimizer'],self::$cfg['captcha']['fontSize'] / 2 + 6 + self::$cfg['captcha']['fontSize'] / 2 - self::$cfg['captcha']['spaceMinimizer']) + self::$cfg['captcha']['spaceFromLeft'] + mt_rand(2,4),mt_rand(self::$cfg['captcha']['fontSize'],self::$cfg['captcha']['fontSize'] * 1.20),$lightColor[mt_rand(0,1)],ROOT.DS."public".DS."fonts".DS."captcha".DS."font".mt_rand(1,4).".ttf",$stringer[$i]);
					}
				}
			}
			else {
				for($i = 0; $i<=self::$cfg['captcha']['numOfChars']; $i++) {
          $fontFile = ROOT.DS.'public'.DS.'fonts'.DS.'captcha'.DS.'font'.mt_rand(1,3).'.gdf';

					imagestring($this->image,imageloadfont($fontFile),$i * mt_rand(30 + self::$cfg['captcha']['fontSize'] / 2,36 + self::$cfg['captcha']['fontSize'] / 2),0 + mt_rand(2,4),$stringer[$i],$lightColor[mt_rand(0,1)]);
				}
			}

			if(self::$cfg['captcha']['allowWave']==1) {
				/** Wave configuracion in X and Y axes */
				$Yperiod    = 12;
				$Yamplitude = 14;
				$Xperiod    = 11;
				$Xamplitude = 5;

				$scale = 1;        // 1: low, 2: medium, 3: high (size factor)

				// X-axis wave generation
				$xp = $scale * $Xperiod * rand(1,3);
				$k  = rand(0,100);
				for($i = 0; $i<(self::$cfg['captcha']['width'] * $scale); $i++) {
					imagecopy($this->image,$this->image,$i - 1,sin($k + $i / $xp) * ($scale * $Xamplitude),$i,0,1,self::$cfg['captcha']['height'] * $scale);
				}

				// Y-axis wave generation
				$k  = rand(0,100);
				$yp = $scale * $Yperiod * rand(1,2);
				for($i = 0; $i<(self::$cfg['captcha']['height'] * $scale); $i++) {
					imagecopy($this->image,$this->image,sin($k + $i / $yp) * ($scale * $Yamplitude),$i - 1,0,$i,self::$cfg['captcha']['width'] * $scale,1);
				}
			}

			if(self::$cfg['captcha']['allowNoise']==1) {        // Noise over the stringer
				imagesetstyle($this->image,array(
					$darkColor[mt_rand(0,2)],
					$darkColor[mt_rand(0,2)],
					$darkColor[mt_rand(0,2)],
					$darkColor[mt_rand(0,2)],
					$darkColor[mt_rand(0,2)],
					$darkColor[mt_rand(0,2)],
					$darkColor[mt_rand(0,2)]
				));
				for($i = 1; $i<=self::$cfg['captcha']['numOfChars']; $i++) {
					imageline($this->image,0,mt_rand(0,self::$cfg['captcha']['height'] - 1),self::$cfg['captcha']['width'],mt_rand(0,self::$cfg['captcha']['height'] - 1),IMG_COLOR_STYLED);
				}
				$lineys = array(
					mt_rand(0,self::$cfg['captcha']['height'] - 1),
					mt_rand(0,self::$cfg['captcha']['height'] - 1),
					mt_rand(0,self::$cfg['captcha']['height'] - 1),
					mt_rand(0,self::$cfg['captcha']['height'] - 1),
					mt_rand(0,self::$cfg['captcha']['height'] - 1),
					mt_rand(0,self::$cfg['captcha']['height'] - 1),
					mt_rand(0,self::$cfg['captcha']['height'] - 1),
					mt_rand(0,self::$cfg['captcha']['height'] - 1),
					mt_rand(0,self::$cfg['captcha']['height'] - 1),
					mt_rand(0,self::$cfg['captcha']['height'] - 1)
				);
				$lineye = array(
					mt_rand(0,self::$cfg['captcha']['height'] - 1),
					mt_rand(0,self::$cfg['captcha']['height'] - 1),
					mt_rand(0,self::$cfg['captcha']['height'] - 1),
					mt_rand(0,self::$cfg['captcha']['height'] - 1),
					mt_rand(0,self::$cfg['captcha']['height'] - 1),
					mt_rand(0,self::$cfg['captcha']['height'] - 1),
					mt_rand(0,self::$cfg['captcha']['height'] - 1),
					mt_rand(0,self::$cfg['captcha']['height'] - 1),
					mt_rand(0,self::$cfg['captcha']['height'] - 1),
					mt_rand(0,self::$cfg['captcha']['height'] - 1)
				);
				for($i = 1; $i<=self::$cfg['captcha']['numOfChars']; $i++) {
					imageline($this->image,$i * self::$cfg['captcha']['width'] / 10 + mt_rand(1,6),$lineys[$i],$i * 16 + mt_rand(1,6),$lineye[$i],$lightColor[mt_rand(0,1)]);
					imageline($this->image,$i * self::$cfg['captcha']['width'] / 10 + mt_rand(1,6),$lineys[$i],$i * 16 + mt_rand(1,6),$lineye[$i],$lightColor[mt_rand(0,1)]);
				}
			}

			if(self::$cfg['captcha']['frame']) {        // sets the frame
				$rgbColor = $this->rgb2array(self::$cfg['captcha']['frame']);

				$bgR = 240;        // background color in RGB: Red (0 -> 255)
				$bgG = 240;        // background color in RGB format: Green (0 -> 255)
				$bgB = 240;        // background color in RGB format: Blue (0 -> 255)

				$charR = 0;        // Color of characters in format RGB: Red (0->255)
				$charG = 0;        // Color of characters in format RGB: Green (0->255)
				$charB = 0;        // Color of characters in format RGB: Blue (0->255)

				//	$framecol = imagecolorallocate($this->image,($bgR*3+$charR)/4,($bgG*3+$charG)/4,($bgB*3+$charB)/4);
				$framecol = imagecolorallocate($this->image,$rgbColor[0],$rgbColor[1],$rgbColor[2]);

				imagerectangle($this->image,0,0,self::$cfg['captcha']['width'] - 1,self::$cfg['captcha']['height'] - 1,$framecol);
			}

			if(self::$cfg['captcha']['credits']) {        // credits image
				$creditsImage = imagecreatefrompng(basic::getPublicDirectory().self::$cfg['captcha']['credits']);
				imagecopy($this->image,$creditsImage,0,self::$cfg['captcha']['height'],0,0,self::$cfg['captcha']['width'],$heightOfCredit);
			}
		}
	}

	/**
	 * Color 2 rgb - convert color from imageColorAllocate() to hex in XXXXXX (eg. FFFFFF, 000000, FF0000)
	 * @param string $color
	 * @return string
	 */
	private function color2rgb($color) {

		return str_pad(base_convert($color,10,16),6,0,STR_PAD_LEFT);
	}

	/**
	 * Rgb 2 array - convert color from hex in XXXXXX (eg. FFFFFF, 000000, FF0000) to array(R, G, B) of integers (0-255)
	 * @param string $rgb
	 * @return string
	 */
	private function rgb2array($rgb) {

		if(preg_match('/["0-9a-fA-F"]{6}/',$rgb)) {    // validate input
			return array(
				base_convert(substr($rgb,0,2),16,10),
				base_convert(substr($rgb,2,2),16,10),
				base_convert(substr($rgb,4,2),16,10),
			);
		}
	}
}

