<?php

/*************************************************************
 * Encryption class
 * - all about encryption
 * @author    S M Kessner <limbercode@gmail.com>
 * @copyright 2010-2018 T2
 * @link      http://limbercode.com
 *************************************************************/
class encryption {

	private $cipher = "AES-256-CBC";    // or "AES-128-CBC" or "AES-256-CTR"
  private $separator = "::";
	private $hashKey;

	/**
	 * Constructor for encryption
	 */
	public function __construct() {

		$this->hashKey = basic::$cfg['auth']['hashKey'];

		if(strlen($this->hashKey)<=16) {
			$this->hashKey = str_pad($this->hashKey,16,"\0",STR_PAD_RIGHT);
		}
		elseif(strlen($this->hashKey)<=24) {
			$this->hashKey = str_pad($this->hashKey,24,"\0",STR_PAD_RIGHT);
		}
		elseif(strlen($this->hashKey)<=32) {
			$this->hashKey = str_pad($this->hashKey,32,"\0",STR_PAD_RIGHT);
		}    // fix for: Error: mcrypt_encrypt(): Key of size 17 not supported by this algorithm. Only keys of sizes 16, 24 or 32 supported
	}

	/**
	 * Safe b64 encode
	 * @param $string
	 * @return string - encoded
	 */
	public function safeB64encode($string) {

		$data = base64_encode($string);
		$data = str_replace(array('+','/','='),array('-','_',''),$data);

		return $data;
	}

	/**
	 * Safe b64 decode
	 * @param $string
	 * @return string - decoded
	 */
	public function safeB64decode($string) {

		$data = str_replace(array('-','_'),array('+','/'),$string);
		$mod4 = strlen($data) % 4;
		if($mod4) {
			$data .= substr('====',$mod4);
		}

		return base64_decode($data);
	}

	/**
	 * Encode
	 * @param string $value - text to encode
	 * @return string - encoded
	 */
	public function encode($value) {

		if(!$value) {
			return false;
		}

		$encryptionKey = $this->safeB64encode($this->hashKey);
		// generate an initialization vector
		$iv = openssl_random_pseudo_bytes(openssl_cipher_iv_length($this->cipher));
		// encrypt the data using AES 256 encryption in CBC mode using our encryption key and initialization vector.
		$encrypted = openssl_encrypt($value, $this->cipher, $encryptionKey, OPENSSL_RAW_DATA, $iv);
		// the $iv is just as important as the key for decrypting, so save it with our encrypted data using a unique separator (::)
		$encryptedData =  $this->safeB64encode($encrypted . $this->separator . $iv);

		return $encryptedData;
	}

	/**
	 * Decode
	 * @param string $value - text to decode
	 * @return string - decoded
	 */
	public function decode($value) {

		if(!$value) {
			return false;
		}

		$encryptionKey = $this->safeB64encode($this->hashKey);
		// to decrypt, split the encrypted data from our IV - our unique separator used was "::"

		$valueDecoded = $this->safeB64decode($value);
		$checkPaamaim = stripos($valueDecoded,$this->separator);

		if($checkPaamaim!==false) {
      if (strpos($valueDecoded, ":::") !== false) {   // to fix a problem with 3 paamaim
        list($encryptedData,$iv) = explode(":::",$valueDecoded,2);
        $encryptedData = $encryptedData.":";
      }
      else {
        list($encryptedData,$iv) = explode("::",$valueDecoded,2);
      }

			$decryptedData = openssl_decrypt($encryptedData,$this->cipher,$encryptionKey,OPENSSL_RAW_DATA,$iv);
		}
		else {
			log::logToFile("encryptionError","Encryption value decoded wrong! (error code: 894274)");

			echo"Encryption value decoded wrong! (error code: 894274)";  // that means that we don't have "::" on the $value, so have to make new value
			die();
		}

		return $decryptedData;
	}

	/**
	 * Encrypt for url
	 * @param string $value - text to encode
	 * @return string - encoded
	 */
	function encryptForUrl($value) {

		$dirty = array("+","/","=");
		$clean = array("_P_","_S_","_E_");

		$cipherText = $this->encode($value);

		return str_replace($dirty,$clean,$cipherText);
	}

	/**
	 * Decrypt for url
	 * @param string $value - text to decode
	 * @return string - decoded
	 */
	function decryptForUrl($value) {

		$dirty = array("+","/","=");
		$clean = array("_P_","_S_","_E_");

		$encryptedString = str_replace($clean,$dirty,$value);
		$decryptedString = $this->decode($encryptedString);

		return $decryptedString;
	}

	/**
	 * Get optimal bcrypt cost parameter
	 * @Param int $minMs Minimum amount of time in milliseconds that it should take
	 * to calculate the hashes
	 */
	public function getOptimalBcryptCostParameter($minMs = 250) {    // prints 12 in my case

		for($i = 4; $i<31; $i++) {
			$options    = ['cost' => $i,'salt' => 'usesomesillystringforsalt'];
			$time_start = microtime(true);
			password_hash("rasmuslerdorf",PASSWORD_BCRYPT,$options);
			$time_end = microtime(true);
			if(($time_end - $time_start) * 1000>$minMs) {
				return $i;
			}
		}
	}
}

