<?php

/*************************************************************
 * Mail class
 * - use of mail
 * @author    S M Kessner <limbercode@gmail.com>
 * @copyright 2010-2018 T2
 * @link      http://limbercode.com
 *************************************************************/
class mail {

  public $messageId;
  public $bodyFile;
  public $vars = array();
  public $fromName;
  public $fromAddress;
  public $replayAddress;
  public $toName;
  public $toAddress;
  public $subject;
  public $ccName;
  public $ccAddress;
  public $attachments;
  public $debug;

  /**
   * Load body file
   * @return array
   */
  private function loadBodyFile() {

    $errorAlert    = "";
    $bodyFileFound = "";
    $body          = "";

    if(file_exists(SOURCEPATH."mail".DS.lang::getCurrentLanguage().DS.$this->bodyFile.".php")) {
      $this->bodyFile = SOURCEPATH."mail".DS.lang::getCurrentLanguage().DS.$this->bodyFile.".php";
      $bodyFileFound  = 1;
    }
    elseif(file_exists(SOURCEPATH."mail".DS.lang::getDefaultLanguage().DS.$this->bodyFile.".php")) {
      $this->bodyFile = SOURCEPATH."mail".DS.lang::getDefaultLanguage().DS.$this->bodyFile.".php";
      $bodyFileFound  = 1;
    }

    if(!$bodyFileFound) {
      if(file_exists(SOURCEPATH."mail".DS."en".DS.$this->bodyFile.".php")) {
        $this->bodyFile = SOURCEPATH."mail".DS."en".DS.$this->bodyFile.".php";
        $bodyFileFound  = 1;
      }
    }

    if(!$bodyFileFound) {
      $notif = basic::load("notif","core");
      $notif->genNotifByNum($file = "core",83220,$this->bodyFile);
      if(notif::$lastNotif) {
        $errorAlert = notif::$lastNotif;
      }    // get the last notif
    }
    else {
      ob_start();
      include $this->bodyFile;
      $body = ob_get_contents();
      ob_end_clean();
    }

    return array($body,$errorAlert);
  }

  /**
   * Log mail sent
   * @param integer $sent
   * @return boolean - if mail was logged
   */
  private function logMailSent($sent = 0) {

    $createData['messageId'] = $this->messageId;
    if(is_array($this->fromName)) {
      $createData['fromName'] = implode(", ",$this->fromName);
    }
    else {
      $createData['fromName'] = $this->fromName;
    }

    if(is_array($this->fromAddress)) {
      $createData['fromAddress'] = implode(", ",$this->fromAddress);
    }
    else {
      $createData['fromAddress'] = $this->fromAddress;
    }

    if(is_array($this->replayAddress)) {
      $createData['replayAddress'] = implode(", ",$this->replayAddress);
    }
    else {
      $createData['replayAddress'] = $this->replayAddress;
    }

    if(is_array($this->toName)) {
      $createData['toName'] = implode(", ",$this->toName);
    }
    else {
      $createData['toName'] = $this->toName;
    }

    if(is_array($this->toAddress)) {
      $createData['toAddress'] = implode(", ",$this->toAddress);
    }
    else {
      $createData['toAddress'] = $this->toAddress;
    }

    if(is_array($this->ccName)) {
      $createData['ccName'] = implode(", ",$this->ccName);
    }
    else {
      $createData['ccName'] = $this->ccName;
    }

    if(is_array($this->ccAddress)) {
      $createData['ccAddress'] = implode(", ",$this->ccAddress);
    }
    else {
      $createData['ccAddress'] = $this->ccAddress;
    }

    $createData['subject']      = $this->subjectWithoutEncode;
    $createData['body']         = "file: ".$this->bodyFile." \nvars: ".print_r($this->vars,true);
    $createData['actionTypeId'] = 110000;
    $createData['sent']         = $sent;

    if(basic::getConfig("db","main","activate")==true) {
      $db = basic::load(basic::getConfig("db","main","adapter"),'core/adapters');
      $db->insert($table = basic::getConfig("db","main","tablePrefix")."log_mail",$createData,$advancedLead = true);
    }

    return true;
  }

  private function antiSpam($vars) {     // have to put it somewhere - to prevent spam on mails

    $referer = !empty($_SERVER['HTTP_REFERER']) ? $_SERVER['HTTP_REFERER'] : "";

    if(!$referer || strpos($referer,basic::$cfg['site']['domain'])===false) {
      $vars['fromIp'] = $_SERVER['REMOTE_ADDR'];
      Log::logToFile($fileName = "mailAntiSpam-noReferer.log",$data = $vars);

      return false;
    }

    return true;
  }

  /**
   * Send mail
   * @param string|null $messageId - the message id
   * @param string $bodyFile - the file to load from: "mail/lang/fileName"
   * @param array|null $vars - the variants to use in the $bodyFile
   * @param string $subject - subject of the mail
   * @param string $fromName - "from" title when sending mail
   * @param string|null $fromAddress - "fromAddress" email when sending mail
   * @param string|null $replayAddress - "replayAddress" email when sending mail
   * @param string|array $toName - "to" title when sending mail
   * @param string|array $toAddress - "toAddress" title when sending mail
   * @param string|array|null $ccName
   * @param string|array|null $ccAddress
   * @param string|array|null $attachments
   * @param boolean|integer $debug - 0/1/2
   * @param boolean $dontLogMailSent
   * @return boolean - if mail was sent
   * @example $sent = $mail->sendMail($messageId=6530
   * ,$bodyFile = "contactUs"
   * ,$vars = array("who","is")
   * ,$subject = $this->trans->T("Contact us - message from").": ".$message['email']
   * ,$fromName = basic::$cfg['site']['name']
   * ,$fromAddress = basic::$cfg['mail']['defaultAddress']
   * ,$replayAddress = basic::$cfg['mail']['defaultAddress']
   * ,$toName = $mailRecipient[0]
   * ,$toAddress = $mailRecipient[1]
   * ,$ccName="",$ccAddress="",$attachments="",$debug="",$dontLogMailSent=""
   * );
   */
  public function sendMail($messageId = "",$bodyFile = "",$vars = null,$subject = "",$fromName = "",$fromAddress = "",$replayAddress = "",$toName = "",$toAddress = "",$ccName = "",$ccAddress = "",$attachments = "",$debug = false,$dontLogMailSent = false) {

    if($this->antiSpam($vars)!==true) {
      return false;
    }

    $this->messageId = $messageId;          // for logging purposes
    $this->bodyFile  = $bodyFile;
    $this->vars      = $vars;
    $this->fromName  = $fromName;
    if(!$fromAddress) {
      $fromAddress = basic::$cfg['mail']['defaultAddress'];
    }
    $this->fromAddress   = $fromAddress;
    $this->replayAddress = $replayAddress;
    $this->toName        = $toName;           // can be array
    $this->toAddress     = $toAddress;        // can be array
    $this->subject       = $subject;
    $this->ccName        = $ccName;           // can be array
    $this->ccAddress     = $ccAddress;        // can be array
    $this->attachments   = $attachments;      // can be array
    $this->debug         = $debug;            // 0/1/2

    if(basic::$cfg['mail']['method']=="builtInMailer") {
      $return = $this->sendBuiltInMailer($dontLogMailSent);
      if(!$return) {
        $return = false;
      }   // fix for a logging prob on errorControl - please do not remove!

      return $return;
    }
    if(basic::$cfg['mail']['method']=="PHPMailer") {
      $return = $this->sendMailPhpMailer($dontLogMailSent);

      if(!$return) {
        $return = false;
      }   // fix for a logging prob on errorControl - please do not remove!

      return $return;
    }
  }

  /**
   * Send mail with built-in basic php
   * @param boolean $dontLogMailSent
   * @return boolean - if mail was sent
   */
  public function sendBuiltInMailer($dontLogMailSent = false) {

    $preferences                = ["input-charset" => "UTF-8","output-charset" => "UTF-8"];
    $this->subjectWithoutEncode = $this->subject;

    $headers   = array();
    $headers[] = "MIME-Version: 1.0";
    $headers[] = "Content-type: text/html; charset=utf-8";
    $headers[] = "From: ".$this->fromName." <".$this->fromAddress.">";
    //	$headers[] = "Bcc: JJ huha <bcc@domain8.com>";
    //	$headers[] = "Reply-To: ".$this->fromName." <".$this->fromAddress.">";

    if($this->replayAddress) {
      $headers[] = "Reply-To: ".$this->replayAddress;
    }
    else {
      $headers[] = "Reply-To: ".$this->fromAddress;
    }
    $headers[]     = "Return-Path: ".$this->fromAddress;
    $this->subject = "Subject: ".iconv_mime_encode("Subject",$this->subject,$preferences);
    $this->subject = str_replace("Subject: ","",$this->subject);
    //	$headers[]     = "X-Mailer: PHP/".phpversion();

    $notif = basic::load("notif","core");
    list($body,$errorAlert) = $this->loadBodyFile();

    if(is_array($this->toName) && is_array($this->toAddress)) {
      foreach($this->toAddress as $key => $value) {
        $send = mail($this->toAddress[$key],$this->subject,$body,implode("\r\n",$headers));
      }
    }
    else {
      $send = mail($this->toAddress,$this->subject,$body,implode("\r\n",$headers));
    }

    return $this->ifLogMailSent($send,$dontLogMailSent,$errorAlert);
  }

  /**
   * Send mail php mailer
   * @param boolean $dontLogMailSent
   * @return boolean - if mail was sent
   */
  public function sendMailPhpMailer($dontLogMailSent = false) {

    require_once(OUTSOURCEPATH.DS."mail".DS."PhpMailer".DS."class.phpmailer.php");
    require_once(OUTSOURCEPATH.DS."mail".DS."PhpMailer".DS."class.smtp.php");

    $mail = new PHPMailer();

    if($this->debug) {
      echo "<pre>";
    }

    $mail->IsSMTP();                       // telling the class to use SMTP
    $mail->SMTPDebug = $this->debug;       // enables SMTP debug information, 1 = errors and messages, 2 = messages only
    $mail->SMTPAuth  = true;                                       // enable SMTP authentication
    if(isset(basic::$cfg['mail']['smtp']['prefix'])) {
      $mail->SMTPSecure = basic::$cfg['mail']['smtp']['prefix'];
    }      // sets the prefix to the server
    if(isset(basic::$cfg['mail']['smtp']['server'])) {
      $mail->Host = basic::$cfg['mail']['smtp']['server'];
    }      // sets GMAIL as the SMTP server
    if(isset(basic::$cfg['mail']['smtp']['port'])) {
      $mail->Port = basic::$cfg['mail']['smtp']['port'];
    }      // set the SMTP port for the GMAIL server
    if(isset(basic::$cfg['mail']['smtp']['userName'])) {
      $mail->Username = basic::$cfg['mail']['smtp']['userName'];
    }      // GMAIL username
    if(isset(basic::$cfg['mail']['smtp']['password'])) {
      $mail->Password = basic::$cfg['mail']['smtp']['password'];
    }      // GMAIL password
    if($this->debug) {
      $mail->SMTPDebug = 1;
    }

    $mail->smtpConnect(     // fixes gmail issues
      array(
        "ssl" => array(
          "verify_peer"       => false,
          "verify_peer_name"  => false,
          "allow_self_signed" => true
        )
      ));

    $preferences    = ["input-charset" => "UTF-8","output-charset" => "UTF-8"];
    $this->fromName = iconv_mime_encode("fromName",$this->fromName,$preferences);
    $this->fromName = str_replace("fromName: ","",$this->fromName);
    $mail->SetFrom($this->fromAddress,$this->fromName);       // not really changes the "from" address
    $mail->AddReplyTo($this->replayAddress,$this->fromName);    // so we have to add replay info

    $this->subjectWithoutEncode = $this->subject;
    $this->subject              = iconv_mime_encode("Subject",$this->subject,$preferences);
    $this->subject              = str_replace("Subject: ","",$this->subject);
    $mail->Subject              = $this->subject;

    $mail->AltBody = "To view the message, please use an HTML compatible email viewer!"; // optional, comment out and test

    if(is_array($this->toName) && is_array($this->toAddress)) {
      foreach($this->toAddress as $key => $value) {
        $mail->AddAddress($this->toAddress[$key],$this->toName[$key]);
      }
    }
    else {
      $mail->AddAddress($this->toAddress,$this->toName);
    }

    if(is_array($this->ccName) && is_array($this->ccAddress)) {
      foreach($this->ccAddress as $key => $value) {
        $mail->AddCC($this->ccAddress[$key],$this->ccName[$key]);
      }
    }
    elseif($this->ccAddress) {
      $mail->AddCC($this->ccAddress,$this->ccName);
    }

    if(is_array($this->attachments)) {
      foreach($this->attachments as $key => $value) {
        $mail->AddAttachment($this->attachments[$key]);
      }
    }
    else {
      $mail->AddAttachment($this->attachments);
    }

    $notif = basic::load("notif","core");
    list($body,$errorAlert) = $this->loadBodyFile();

    if($body) {
      $mail->MsgHTML($body);
    }

    if($this->debug) {
      echo "</pre>";
    }

    $this->ifLogMailSent($send = $mail->Send(),$dontLogMailSent,$errorAlert,$mail);

    return $send;
  }

  /**
   * Send mail to admin
   * @param string $messageId
   * @param $bodyFile
   * @param $vars
   * @param $subject
   * @return bool
   */
  public function sendMailToAdmin($messageId = "",$bodyFile = "",$vars = "",$subject = "") {

    $dontLogOnDb = 0;

    foreach(basic::$cfg['site']['contacts'] as $mailRecipient) {
      $mailNames[]     = $mailRecipient['name'];
      $mailAddresses[] = $mailRecipient['address'];
    }

    $sent = $this->sendMail($messageId,$bodyFile,$vars,$subject,$fromName = basic::$cfg['site']['name'],$fromAddress = basic::$cfg['mail']['defaultAddress'],$replayAddress = basic::$cfg['mail']['defaultAddress'],$toName = $mailNames,$toAddress = $mailAddresses);

    return $sent;
  }

  /**
   * @param bool $send
   * @param $dontLogMailSent
   * @param $errorAlert
   * @param bool|object $mail
   * @return bool
   */
  private function ifLogMailSent($send = false,$dontLogMailSent = "",$errorAlert = "",$mail = false) {

    $logged = "";
    $notif  = basic::load("notif","core");

    if(!$mail) {
      $mail            = new stdClass();
      $mail->ErrorInfo = "no info available..";
    }

    if(!$send) {
      $this->bodyFile = $this->bodyFile." - ".$mail->ErrorInfo." - ".$errorAlert;
      if(!$dontLogMailSent) {
        $logged = $this->logMailSent($sent = 0);
      }

      $notif->genNotifByNum($file = "core",83200,$mail->ErrorInfo);
    }
    else {
      if(!$dontLogMailSent) {
        $logged = $this->logMailSent($sent = 1);
      }

      if($logged==true) {
        $notif->genNotifByNum($file = "core",83230);

        return true;
      }
      else {
        $notif->genNotifByNum($file = "core",83210);

        return false;
      }
    }
  }

  /**
   * Correct encoding - to utf-8
   * @param string $string
   * @return string - the corrected string
   */
  static function correctEncoding($string) {

    $current_encoding = mb_detect_encoding($string,'auto');
    $string           = iconv($current_encoding,'UTF-8',$string);

    return $string;
  }
}

