<?php
/*************************************************************
 * Resizer
 * @author    S M Kessner <limbercode@gmail.com>
 * @copyright 2010-2018 T2
 * @link      http://limbercode.com
 *************************************************************/

class resizer {

	private $file;
	private $number;
	private $filePath;
	private $fileBasename;
	private $fileName;
	private $fileExtension;
	private $image;
	private $width;
	private $height;
	private $mimeType;
	private $imageResized;
	private $newWidth;
	private $newHeight;
	private $quality;
	private $pattern;
	private $watermarkFile;
	private $opacity;
	private $pos;
	private $padding;
	private $target;
	private $targetDirectory;
	private $targetFileName;
	private $targetFileAddOnText;   // if we have "*" sign - we will replace it with $this->number

	/**
	 * @param $newWidth - in px
	 */
	public function setNewWidth($newWidth) {

		$this->newWidth = $newWidth;
	}

	/**
	 * @param $newHeight - in px
	 */
	public function setNewHeight($newHeight) {

		$this->newHeight = $newHeight;
	}

	/**
	 * @param $quality - in percents (without percent sign)
	 */
	public function setQuality($quality) {

		$this->quality = $quality;
	}

	public function setPattern($pattern) {

		$this->pattern = $pattern;
	}

	public function setWatermarkFile($watermarkFile) {

		$this->watermarkFile = $watermarkFile;
	}

	public function setOpacity($opacity) {

		$this->opacity = $opacity;
	}

	public function setPos($pos) {

		$this->pos = $pos;
	}

	public function setPadding($padding) {

		$this->padding = $padding;
	}

	public function setTarget($target) {

		$this->target = $target;
	}

	public function setTargetFileName($targetFileName) {

		$this->targetFileName = $targetFileName;
	}

	public function setTargetDirectory($directory) {

		$this->targetDirectory = $directory;
	}

	public function setTargetFileAddOnText($text) {

		$this->targetFileAddOnText = $text;
	}

	public function resize($files) {

		if(is_array($files)) {
			foreach($files as $key => $value) {
				$this->file = $files[$key];
				$this->number = $key+1;
				$this->resizeIt();
			}
		}
		else {
			$this->file = $files;
			$this->resizeIt();
		}
	}

	public function resizeIt() {

		if(!@file_exists($this->file)) {
			echo "<br>The requested image does not exist: $this->file<br>";       // replace with alert
		}

		$info           = GetImageSize($this->file);
		$this->width    = $info[0];
		$this->height   = $info[1];
		$this->mimeType = $info['mime'];
		//	$this->fileSize = filesize($this->file);

		// *** Get extension
	//	$this->extension = strrchr($this->file,'.');
	//	$this->extension = strtolower($this->extension);

		$pathParts = pathinfo($this->file);
		$this->filePath = $pathParts['dirname'];
		$this->fileBasename = $pathParts['basename'];
		$this->fileExtension = $pathParts['extension'];
		$this->fileName = $pathParts['filename'];


		$this->image = $this->openImage($this->file);






		//	if(!$source && $sourceFromUrl) {
		//	$source        = $sourceFromUrl;
		$newWidth      = $this->newWidth;
		$newHeight     = $this->newHeight;
		$quality       = $this->quality;
		$pattern       = "";
		$watermarkFile = "";
		$opacity       = "";
		$pos           = "";
		$padding       = "";
		$target        = $this->target;
		$outcome       = "noShow";
		//	}

		// *** 1) Initialise / load image
		//	$resizeObj = new resize($source);

		// *** 2) Resize image (options: exact,portrait,landscape,auto,crop,noaspect)
		$this->resizeImage($newWidth,$newHeight,$pattern,$watermarkFile,$opacity,$pos,$padding);

		// *** 3) Save image, image quality
		$this->saveImage($target,$quality,$outcome);
	}

	private function openImage($file) {

		$extension = substr(strrchr($this->mimeType,'/'),1);

		// *** Get extension
		switch($extension) {
			case 'jpeg':
				$img = @imageCreateFromJpeg($file);
				break;

			case 'png':
				$img = @imageCreateFromPng($file);
				break;

			case 'bmp':
				$img = @imageCreateFromBmpDiverse($file);
				break;

			case 'gif':
				$img = @imageCreateFromGif($file);
				break;

			case 'vnd.wap.wbmp':
				$img = @imageCreateFromBmpDiverse($file);
				break;

			case 'xbm':
				$img = @imageCreateFromXbm($file);
				break;

			default:
				$img = false;
				break;
		}

		return $img;
	}

	## --------------------------------------------------------

	public function resizeImage($newWidth,$newHeight,$pattern,$watermarkFile,$opacity,$pos,$padding) {

		if($newWidth<2) {
			$ratio_orig = $this->height / $this->width;
			$newWidth   = $newHeight * $ratio_orig;
		}

		if($newHeight<2) {
			$ratio_orig = $this->height / $this->width;
			$newHeight  = $newWidth * $ratio_orig;
		}

		if($newWidth<2 && $newHeight<2) {
			$newWidth  = $this->width;
			$newHeight = $this->height;
		}

		if(!$pattern || ($pattern!="exact" && $pattern!="portrait" && $pattern!="landscape" && $pattern!="auto" && $pattern!="crop" && $pattern!="noaspect")) {
			$pattern = $pattern = "auto";
		}
		// *** Get optimal width and height - based on $pattern
		$patternArray = $this->getDimensions($newWidth,$newHeight,$pattern);

		$optimalWidth  = $patternArray['optimalWidth'];
		$optimalHeight = $patternArray['optimalHeight'];

		// *** Resample - create image canvas of x,y size
		$this->imageResized = imagecreatetruecolor($optimalWidth,$optimalHeight);
		imagecopyresampled($this->imageResized,$this->image,0,0,0,0,$optimalWidth,$optimalHeight,$this->width,$this->height);

		// *** if option is 'crop',then crop too
		if($pattern=='crop') {
			$this->crop($optimalWidth,$optimalHeight,$newWidth,$newHeight);
		}

		if($watermarkFile) {
			$this->imageResized = $this->waterMark($this->file,$watermarkFile,$opacity,$pos,$padding,$this->imageResized,$optimalWidth,$optimalHeight);
		}
	}

	## --------------------------------------------------------

	private function getDimensions($newWidth,$newHeight,$pattern) {

		switch($pattern) {
			case 'exact':
				$optimalWidth  = $newWidth;
				$optimalHeight = $newHeight;
				break;

			case 'portrait':
				$optimalWidth  = $this->getSizeByFixedHeight($newHeight);
				$optimalHeight = $newHeight;
				break;

			case 'landscape':
				$optimalWidth  = $newWidth;
				$optimalHeight = $this->getSizeByFixedWidth($newWidth);
				break;

			case 'auto':
				$patternArray  = $this->getSizeByAuto($newWidth,$newHeight);
				$optimalWidth  = $patternArray['optimalWidth'];
				$optimalHeight = $patternArray['optimalHeight'];
				break;

			case 'crop':
				$patternArray  = $this->getOptimalCrop($newWidth,$newHeight);
				$optimalWidth  = $patternArray['optimalWidth'];
				$optimalHeight = $patternArray['optimalHeight'];
				break;

			case 'noaspect':
				$optimalWidth  = $newWidth;
				$optimalHeight = $newHeight;
				break;
		}

		return array('optimalWidth' => $optimalWidth,'optimalHeight' => $optimalHeight);
	}

	## --------------------------------------------------------

	private function getSizeByFixedHeight($newHeight) {

		$ratio    = $this->width / $this->height;
		$newWidth = $newHeight * $ratio;

		return $newWidth;
	}

	private function getSizeByFixedWidth($newWidth) {

		$ratio     = $this->height / $this->width;
		$newHeight = $newWidth * $ratio;

		return $newHeight;
	}

	private function getSizeByAuto($newWidth,$newHeight) {

		if($this->height<$this->width) {        // *** Image to be resized is wider (landscape)
			$optimalWidth  = $newWidth;
			$optimalHeight = $this->getSizeByFixedWidth($newWidth);
		}
		elseif($this->height>$this->width) {        // *** Image to be resized is taller (portrait)
			$optimalWidth  = $this->getSizeByFixedHeight($newHeight);
			$optimalHeight = $newHeight;
		}
		else {        // *** Image to be resizerd is a square
			if($newHeight<$newWidth) {
				$optimalWidth  = $newWidth;
				$optimalHeight = $this->getSizeByFixedWidth($newWidth);
			}
			else if($newHeight>$newWidth) {
				$optimalWidth  = $this->getSizeByFixedHeight($newHeight);
				$optimalHeight = $newHeight;
			}
			else {
				// *** Sqaure being resized to a square
				$optimalWidth  = $newWidth;
				$optimalHeight = $newHeight;
			}
		}

		return array('optimalWidth' => $optimalWidth,'optimalHeight' => $optimalHeight);
	}

	## --------------------------------------------------------

	private function getOptimalCrop($newWidth,$newHeight) {

		$heightRatio = $this->height / $newHeight;
		$widthRatio  = $this->width / $newWidth;

		if($heightRatio<$widthRatio) {
			$optimalRatio = $heightRatio;
		}
		else {
			$optimalRatio = $widthRatio;
		}

		$optimalHeight = $this->height / $optimalRatio;
		$optimalWidth  = $this->width / $optimalRatio;

		return array('optimalWidth' => $optimalWidth,'optimalHeight' => $optimalHeight);
	}

	## --------------------------------------------------------

	private function crop($optimalWidth,$optimalHeight,$newWidth,$newHeight) {

		// *** Find center - this will be used for the crop
		$cropStartX = ($optimalWidth / 2) - ($newWidth / 2);
		$cropStartY = ($optimalHeight / 2) - ($newHeight / 2);

		$crop = $this->imageResized;
		//imagedestroy($this->imageResized);

		// *** Now crop from center to exact requested size
		$this->imageResized = imagecreatetruecolor($newWidth,$newHeight);
		imagecopyresampled($this->imageResized,$crop,0,0,$cropStartX,$cropStartY,$newWidth,$newHeight,$newWidth,$newHeight);
	}

	## --------------------------------------------------------

	private function waterMark($source,$watermarkFile = "options",$opacity="",$pos="",$padding="",$resampled_id="",$source_width="",$source_height="") {

		global $WatermarkSizeA,$WatermarkSizeB,$WatermarkSizeC,$watermarkPosition,$watermarkOpacity;

		//$source =			Filename of the picture into that $watermarkFile will be inserted.
		//$watermarkFile =	Filename of the picture that is to be inserted into $source.
		//$target =			Filename of the modified picture.
		//$opacity =		Intensity of the transition (in percent)
		//$pos =			Position where $watermarkFile will be inserted in $source
		//                0 = middle
		//                1 = top left
		//                2 = top right
		//                3 = bottom right
		//                4 = bottom left
		//                5 = top middle
		//                6 = middle right
		//                7 = bottom middle
		//                8 = middle left

		if(!$opacity)
			$opacity = $watermarkOpacity;
		if(!$pos)
			$pos = $watermarkPosition;

		if(!$opacity)
			$opacity = 80;
		if(!$pos)
			$pos = 3;

		$source_id = $resampled_id;        // get the image after resized it

		if($watermarkFile=="options") {
			if($source_width<200 && $source_height<200) {
				$watermarkFile = $WatermarkSizeB;
			}
			else {
				$watermarkFile = $WatermarkSizeC;
			}
			if($source_width<100 && $source_height<100) {
				$watermarkFile = $WatermarkSizeA;
			}
		}

		//////////////////////////////////////////////////////////////////////// watermark selector by image size

		$info               = GetImageSize($watermarkFile);
		$typeOfWatermarkImg = $info['mime'];

		$typeOfWatermarkImg = substr(strrchr($typeOfWatermarkImg,'/'),1);

		//Get the resource id's of watermark
		switch($typeOfWatermarkImg) {
			case 'jpeg':
				$watermark_id = imageCreateFromJPEG($watermarkFile);
				break;

			case 'gif':
				$watermark_id = imageCreateFromGIF($watermarkFile);
				break;

			case 'png':
				$watermark_id = imageCreateFromPNG($watermarkFile);
				break;

			case 'bmp':
				$watermark_id = imageCreateFromBmpDiverse($watermarkFile);
				break;

			default:
				// *** No extension - No save.
				$watermark_id = "";
				break;
		}

		if($watermark_id) {
			//Get the sizes of watermark
			$watermark_width  = imageSX($watermark_id);
			$watermark_height = imageSY($watermark_id);

			//middle
			if($pos==0) {
				$dest_x = $padding + ($source_width / 2) - ($watermark_width / 2);
				$dest_y = $padding + ($source_height / 2) - ($watermark_height / 2);
			}

			//top left
			if($pos==1) {
				$dest_x = $padding + 0;
				$dest_y = $padding + 0;
			}

			//top right
			if($pos==2) {
				$dest_x = -$padding + $source_width - $watermark_width;
				$dest_y = $padding + 0;
			}

			//bottom right
			if($pos==3) {
				$dest_x = -$padding + $source_width - $watermark_width;
				$dest_y = -$padding + $source_height - $watermark_height;
			}

			//bottom left
			if($pos==4) {
				$dest_x = $padding + 0;
				$dest_y = -$padding + $source_height - $watermark_height;
			}

			//top middle
			if($pos==5) {
				$dest_x = $padding + (($source_width - $watermark_width) / 2);
				$dest_y = $padding + 0;
			}

			//middle right
			if($pos==6) {
				$dest_x = $source_width - $watermark_width;
				$dest_y = $padding + ($source_height / 2) - ($watermark_height / 2);
			}

			//bottom middle
			if($pos==7) {
				$dest_x = (($source_width - $watermark_width) / 2);
				$dest_y = -$padding + $source_height - $watermark_height;
			}

			//middle left
			if($pos==8) {
				$dest_x = $padding + 0;
				$dest_y = ($source_height / 2) - ($watermark_height / 2);
			}
			//The main thing: merge the two pix
			imageCopyMerge($source_id,$watermark_id,$dest_x,$dest_y,0,0,$watermark_width,$watermark_height,$opacity);
		}

		return $source_id;
	}

	public function saveImage() {

		if(!$this->targetFileName) {
			$this->targetFileName = $this->fileName;
		}

		$targetFileAddOnText = str_replace("*",$this->number,$this->targetFileAddOnText);

		if(!is_dir($this->targetDirectory)) {
			mkdir($this->targetDirectory,0755,true);
		}

		$targetFile = $this->targetDirectory.DS.$this->targetFileName.$targetFileAddOnText.".".$this->fileExtension;

		if(!$this->quality) {
			$this->quality = 90;
		}

			$outcome = "resize";



/*		p($this->targetDirectory);
		p($this->number);
		p($targetFile);
		p($this->fileExtension);
		p($this->mimeType);*/

		switch($this->fileExtension) {
			case 'jpg':
			case 'jpeg':
				if(imagetypes() & IMG_JPG) {
					if($outcome=="show") {
						header('Content-Type: image/jpeg');
						imagejpeg($this->imageResized);
					}
					else {
						imagejpeg($this->imageResized,$targetFile,$this->quality);
					}
				}
				break;

			case 'gif':
				if(imagetypes() & IMG_GIF) {
					if($outcome=="show") {
						header('Content-Type: image/gif');
						imagegif($this->imageResized);
					}
					else {
						imagegif($this->imageResized,$targetFile);
					}
				}
				break;

			case 'png':
				// *** Scale quality from 0-100 to 0-9
				$scaleQuality = round(($this->quality / 100) * 9);

				// *** Invert quality setting as 0 is best,not 9
				$invertScaleQuality = 9 - $scaleQuality;

				if(imagetypes() & IMG_PNG) {
					if($outcome=="show") {
						header('Content-Type: image/png');
						imagepng($this->imageResized);
					}
					else {
						imagepng($this->imageResized,$targetFile,$invertScaleQuality);
					}
				}
				break;

			// ... etc

			default:
				// *** No extension - No save.
				header('Content-Type: image/jpeg');
				imagejpeg($this->imageResized);
				break;
		}

		imagedestroy($this->imageResized);

		$this->targetFileName = "";     // a must reset if we have array of files
	}
}




function resizeToFileOld($sourcefile,$dest_x,$dest_y,$targetfile,$jpegqual,$typeofimgr) {    // $typeofimgr can be removed, it's not needed

	/* Get the dimensions of the source images */
	$picsize  = getimagesize($sourcefile);
	$source_x = $picsize[0];
	$source_y = $picsize[1];
	//	$imagetype = $picsize['mime'];

	switch($picsize['mime']) {
		case "image/gif":
			$typeofimgr = "gif";
			break;
		case "image/jpeg":
			$typeofimgr = "jpg";
			break;
		case "image/png":
			$typeofimgr = "png";
			break;
		case "image/bmp":
			$typeofimgr = "bmp";
			break;
	}

	if($typeofimgr=="jpg" || $typeofimgr=="jpeg") {
		$source_id = imageCreateFromJPEG("$sourcefile");
	}
	if($typeofimgr=="gif") {
		$source_id = imageCreateFromGIF("$sourcefile");
	}
	if($typeofimgr=="png") {
		$source_id = imageCreateFromPNG("$sourcefile");
	}
	if($typeofimgr=="bmp") {
		$source_id = imageCreateFromBmpDiverse("$sourcefile");
	}

	$target_id = imagecreatetruecolor($dest_x,$dest_y);
	/* Resize the original images and copy it into the just created image
	object. Because of the lack of space I had to wrap the parameters to
	several lines. I recommend putting them in one line in order keep your
	code clean and readable */

	$target_pic = imagecopyresampled($target_id,$source_id,0,0,0,0,$dest_x,$dest_y,$source_x,$source_y);
	/* Create a jpeg with the quality of "$jpegqual" out of the
	image object "$target_pic".
	This will be saved as $targetfile */

	if($typeofimgr=="jpg" || $typeofimgr=="jpeg") {
		imagejpeg($target_id,"$targetfile",$jpegqual);
	}
	if($typeofimgr=="gif") {
		imagejpeg($target_id,"$targetfile",$jpegqual);
	}
	if($typeofimgr=="png") {
		imagejpeg($target_id,"$targetfile",$jpegqual);
	}
	//if: imagepng -> must do: $jpegqual=9
	if($typeofimgr=="bmp") {
		imagejpeg($target_id,"$targetfile",$jpegqual);
	}

	imagedestroy($target_id);

	return true;
}

function waterMarkOld($sourcefile,$watermark = "options",$targetfile="",$opacity="",$pos="",$padding="") {

	global $WatermarkSizeA,$WatermarkSizeB,$WatermarkSizeC,$watermarkPosition,$watermarkOpacity;

	//$sourcefile =		Filename of the picture into that $watermark will be inserted.
	//$watermark =		Filename of the picture that is to be inserted into $sourcefile.
	//$targetfile =		Filename of the modified picture.
	//$opacity =		Intensity of the transition (in percent)
	//$pos =			Position where $watermark will be inserted in $sourcefile
	//                0 = middle
	//                1 = top left
	//                2 = top right
	//                3 = bottom right
	//                4 = bottom left
	//                5 = top middle
	//                6 = middle right
	//                7 = bottom middle
	//                8 = middle left

	if(!$targetfile)
		$targetfile = $sourcefile;
	if(!$opacity)
		$opacity = $watermarkOpacity;
	if(!$pos)
		$pos = $watermarkPosition;

	if(!$targetfile)
		$targetfile = $sourcefile;
	if(!$opacity)
		$opacity = 80;
	if(!$pos)
		$pos = 3;

	$typei           = explode(".",$sourcefile);
	$indextp         = count($typei) - 1;
	$typeofsourceimg = strtolower($typei[$indextp]);

	$typei           = explode(".",$targetfile);
	$indextp         = count($typei) - 1;
	$typeoftargetimg = strtolower($typei[$indextp]);

	//	$mimetype = mime_content_type($sourcefile);

	//	echo"<br>xxxxxxxxx $mimetype";

	if($typeofsourceimg=="jpg" || $typeofsourceimg=="jpeg") {
		//Get the resource id's of the source
		$sourcefile_id = imageCreateFromJPEG($sourcefile);
	}

	if($typeofsourceimg=="gif") {
		//Get the resource id's of the source
		$sourcefile_id = imageCreateFromGIF($sourcefile);
	}

	//Get the sizes of both source
	$sourcefile_width  = imageSX($sourcefile_id);
	$sourcefile_height = imageSY($sourcefile_id);

	if($watermark=="" || $watermark=="options") {

		if($sourcefile_width<200 && $sourcefile_height<200) {
			$watermark = $WatermarkSizeA;
		}
		else {
			$watermark = $WatermarkSizeB;
		}
		if($sourcefile_width<75 && $sourcefile_height<75) {
			$watermark = $WatermarkSizeC;
		}
	}

	//Get the resource id's of watermark
	$watermark_id = imageCreateFromGIF($watermark);

	//Get the sizes of watermark
	$watermark_width  = imageSX($watermark_id);
	$watermark_height = imageSY($watermark_id);

	//middle
	if($pos==0) {
		$dest_x = $padding + ($sourcefile_width / 2) - ($watermark_width / 2);
		$dest_y = $padding + ($sourcefile_height / 2) - ($watermark_height / 2);
	}

	//top left
	if($pos==1) {
		$dest_x = $padding + 0;
		$dest_y = $padding + 0;
	}

	//top right
	if($pos==2) {
		$dest_x = -$padding + $sourcefile_width - $watermark_width;
		$dest_y = $padding + 0;
	}

	//bottom right
	if($pos==3) {
		$dest_x = -$padding + $sourcefile_width - $watermark_width;
		$dest_y = -$padding + $sourcefile_height - $watermark_height;
	}

	//bottom left
	if($pos==4) {
		$dest_x = $padding + 0;
		$dest_y = -$padding + $sourcefile_height - $watermark_height;
	}

	//top middle
	if($pos==5) {
		$dest_x = $padding + (($sourcefile_width - $watermark_width) / 2);
		$dest_y = $padding + 0;
	}

	//middle right
	if($pos==6) {
		$dest_x = $sourcefile_width - $watermark_width;
		$dest_y = $padding + ($sourcefile_height / 2) - ($watermark_height / 2);
	}

	//bottom middle
	if($pos==7) {
		$dest_x = (($sourcefile_width - $watermark_width) / 2);
		$dest_y = -$padding + $sourcefile_height - $watermark_height;
	}

	//middle left
	if($pos==8) {
		$dest_x = $padding + 0;
		$dest_y = ($sourcefile_height / 2) - ($watermark_height / 2);
	}

	//The main thing: merge the two pix
	imageCopyMerge($sourcefile_id,$watermark_id,$dest_x,$dest_y,0,0,$watermark_width,$watermark_height,$opacity);

	if($typeoftargetimg=="jpg" || $typeoftargetimg=="jpeg") {
		//Create a jpeg out of the modified picture
		imagejpeg($sourcefile_id,$targetfile);
	}

	if($typeoftargetimg=="gif") {
		//Create a jpeg out of the modified picture
		imagegif($sourcefile_id,$targetfile);
	}

	imagedestroy($sourcefile_id);
	imagedestroy($watermark_id);
}

/*// *** 1) Initialise / load image
$resizeObj = new resize('sample.jpg');

// *** 2) Resize image (options: exact,portrait,landscape,auto,crop,noaspect)
$resizeObj -> resizeImage(300,300,'crop',$watermarkFile="1.jpg",$opacity,$pos,$padding=30);

// *** 3) Save image, image quality
$resizeObj -> saveImage('sample-resized.jpg',100,'save');*/

/*
echo"aaaaaaaaaaaaa <img src=\"resize-class.php?image=sample.jpg&w=200&h=200&p=crop&wa=1.png&pa=20\">";

//.htaccess - http://mrphp.com.au/code/image-cache-using-phpthumb-and-modrewrite

<IfModule mod_rewrite.c>
RewriteEngine on
RewriteCond %{REQUEST_FILENAME} !-f
RewriteCond %{REQUEST_FILENAME} !-d
RewriteRule ^(.*)$ index.php?thumb=$1 [L,QSA]
</IfModule>
*/

//	resizeImage($source="sample.jpg",$newWidth,$newHeight=200,$quality,$pattern="crop",$outcome,$target,$watermarkFile="1.png",$opacity,$pos=1,$padding);

//////////////////////////////////////////////////////////////////////////////////////////////// testing

//$imageFromUrl = $_GET['image'];
/*if(isset($imageFromUrl)) {
	if($imageFromUrl) resizeImage($source,$newWidth,$newHeight,$quality,$pattern,$outcome,$target,$watermarkFile,$opacity,$pos,$padding);
}*/

function resizeImage($source,$newWidth,$newHeight,$quality,$pattern,$outcome,$target,$watermarkFile,$opacity,$pos,$padding) {

	if(isset($_GET['image']))
		$sourceFromUrl = $_GET['image'];

	if($source || $sourceFromUrl) {
		if(!$source && $sourceFromUrl) {
			$check = $sourceFromUrl;
		}
		else {
			$check = $source;
		}
		if(!@file_exists($check)) {
			//	exit('The requested image does not exist.');
			echo "<br>The requested image does not exist: $source<br>";
		}

		if(!$source && $sourceFromUrl) {
			$source        = $sourceFromUrl;
			$newWidth      = $_GET['w'];
			$newHeight     = $_GET['h'];
			$quality       = $_GET['q'];
			$pattern       = $_GET['p'];
			$watermarkFile = $_GET['wa'];
			$opacity       = $_GET['oo'];
			$pos           = $_GET['po'];
			$padding       = $_GET['pa'];
			$target        = "";
			$outcome       = "show";
		}

		// *** 1) Initialise / load image
		$resizeObj = new resize($source);

		// *** 2) Resize image (options: exact,portrait,landscape,auto,crop,noaspect)
		$resizeObj->resizeImage($newWidth,$newHeight,$pattern,$watermarkFile,$opacity,$pos,$padding);

		// *** 3) Save image, image quality
		$resizeObj->saveImage($target,$quality,$outcome);
	}
}

Class resize {

	// *** Class variables
	private $image;
	private $width;
	private $height;
	private $imageResized;
	private $fileName;

	function __construct($fileName) {

		// *** Open up the file
		// *** Get width and height
		$info           = GetImageSize($fileName);
		$this->width    = $info[0];
		$this->height   = $info[1];
		$this->mimeType = $info['mime'];
		//	$this->fileSize = filesize($fileName);
		$this->originalFileName = $fileName;

		$this->image = $this->openImage($fileName);
	}

	## --------------------------------------------------------

	private function openImage($fileName) {

		$extension = substr(strrchr($this->mimeType,'/'),1);

		// *** Get extension
		switch($extension) {
			case 'jpeg':
				$img = @imageCreateFromJpeg($fileName);
				break;

			case 'png':
				$img = @imageCreateFromPng($fileName);
				break;

			case 'bmp':
				$img = @imageCreateFromBmpDiverse($fileName);
				break;

			case 'gif':
				$img = @imageCreateFromGif($fileName);
				break;

			case 'vnd.wap.wbmp':
				$img = @imageCreateFromBmpDiverse($fileName);
				break;

			case 'xbm':
				$img = @imageCreateFromXbm($fileName);
				break;

			default:
				$img = false;
				break;
		}

		return $img;
	}

	## --------------------------------------------------------

	public function resizeImage($newWidth,$newHeight,$pattern,$watermarkFile,$opacity,$pos,$padding) {

		if($newWidth<2) {
			$ratio_orig = $this->height / $this->width;
			$newWidth   = $newHeight * $ratio_orig;
		}

		if($newHeight<2) {
			$ratio_orig = $this->height / $this->width;
			$newHeight  = $newWidth * $ratio_orig;
		}

		if($newWidth<2 && $newHeight<2) {
			$newWidth  = $this->width;
			$newHeight = $this->height;
		}

		if(!$pattern || ($pattern!="exact" && $pattern!="portrait" && $pattern!="landscape" && $pattern!="auto" && $pattern!="crop" && $pattern!="noaspect")) {
			$pattern = $pattern = "auto";
		}
		// *** Get optimal width and height - based on $pattern
		$patternArray = $this->getDimensions($newWidth,$newHeight,$pattern);

		$optimalWidth  = $patternArray['optimalWidth'];
		$optimalHeight = $patternArray['optimalHeight'];

		// *** Resample - create image canvas of x,y size
		$this->imageResized = imagecreatetruecolor($optimalWidth,$optimalHeight);
		imagecopyresampled($this->imageResized,$this->image,0,0,0,0,$optimalWidth,$optimalHeight,$this->width,$this->height);

		// *** if option is 'crop',then crop too
		if($pattern=='crop') {
			$this->crop($optimalWidth,$optimalHeight,$newWidth,$newHeight);
		}

		if($watermarkFile) {
			$this->imageResized = $this->waterMark($this->originalFileName,$watermarkFile,$opacity,$pos,$padding,$this->imageResized,$optimalWidth,$optimalHeight);
		}
	}

	## --------------------------------------------------------

	private function getDimensions($newWidth,$newHeight,$pattern) {

		switch($pattern) {
			case 'exact':
				$optimalWidth  = $newWidth;
				$optimalHeight = $newHeight;
				break;

			case 'portrait':
				$optimalWidth  = $this->getSizeByFixedHeight($newHeight);
				$optimalHeight = $newHeight;
				break;

			case 'landscape':
				$optimalWidth  = $newWidth;
				$optimalHeight = $this->getSizeByFixedWidth($newWidth);
				break;

			case 'auto':
				$patternArray  = $this->getSizeByAuto($newWidth,$newHeight);
				$optimalWidth  = $patternArray['optimalWidth'];
				$optimalHeight = $patternArray['optimalHeight'];
				break;

			case 'crop':
				$patternArray  = $this->getOptimalCrop($newWidth,$newHeight);
				$optimalWidth  = $patternArray['optimalWidth'];
				$optimalHeight = $patternArray['optimalHeight'];
				break;

			case 'noaspect':
				$optimalWidth  = $newWidth;
				$optimalHeight = $newHeight;
				break;
		}

		return array('optimalWidth' => $optimalWidth,'optimalHeight' => $optimalHeight);
	}

	## --------------------------------------------------------

	private function getSizeByFixedHeight($newHeight) {

		$ratio    = $this->width / $this->height;
		$newWidth = $newHeight * $ratio;

		return $newWidth;
	}

	private function getSizeByFixedWidth($newWidth) {

		$ratio     = $this->height / $this->width;
		$newHeight = $newWidth * $ratio;

		return $newHeight;
	}

	private function getSizeByAuto($newWidth,$newHeight) {

		if($this->height<$this->width) {        // *** Image to be resized is wider (landscape)
			$optimalWidth  = $newWidth;
			$optimalHeight = $this->getSizeByFixedWidth($newWidth);
		}
		elseif($this->height>$this->width) {        // *** Image to be resized is taller (portrait)
			$optimalWidth  = $this->getSizeByFixedHeight($newHeight);
			$optimalHeight = $newHeight;
		}
		else {        // *** Image to be resizerd is a square
			if($newHeight<$newWidth) {
				$optimalWidth  = $newWidth;
				$optimalHeight = $this->getSizeByFixedWidth($newWidth);
			}
			else if($newHeight>$newWidth) {
				$optimalWidth  = $this->getSizeByFixedHeight($newHeight);
				$optimalHeight = $newHeight;
			}
			else {
				// *** Sqaure being resized to a square
				$optimalWidth  = $newWidth;
				$optimalHeight = $newHeight;
			}
		}

		return array('optimalWidth' => $optimalWidth,'optimalHeight' => $optimalHeight);
	}

	## --------------------------------------------------------

	private function getOptimalCrop($newWidth,$newHeight) {

		$heightRatio = $this->height / $newHeight;
		$widthRatio  = $this->width / $newWidth;

		if($heightRatio<$widthRatio) {
			$optimalRatio = $heightRatio;
		}
		else {
			$optimalRatio = $widthRatio;
		}

		$optimalHeight = $this->height / $optimalRatio;
		$optimalWidth  = $this->width / $optimalRatio;

		return array('optimalWidth' => $optimalWidth,'optimalHeight' => $optimalHeight);
	}

	## --------------------------------------------------------

	private function crop($optimalWidth,$optimalHeight,$newWidth,$newHeight) {

		// *** Find center - this will be used for the crop
		$cropStartX = ($optimalWidth / 2) - ($newWidth / 2);
		$cropStartY = ($optimalHeight / 2) - ($newHeight / 2);

		$crop = $this->imageResized;
		//imagedestroy($this->imageResized);

		// *** Now crop from center to exact requested size
		$this->imageResized = imagecreatetruecolor($newWidth,$newHeight);
		imagecopyresampled($this->imageResized,$crop,0,0,$cropStartX,$cropStartY,$newWidth,$newHeight,$newWidth,$newHeight);
	}

	## --------------------------------------------------------

	private function waterMark($source,$watermarkFile = "options",$opacity="",$pos="",$padding="",$resampled_id="",$source_width="",$source_height="") {

		global $WatermarkSizeA,$WatermarkSizeB,$WatermarkSizeC,$watermarkPosition,$watermarkOpacity;

		//$source =			Filename of the picture into that $watermarkFile will be inserted.
		//$watermarkFile =	Filename of the picture that is to be inserted into $source.
		//$target =			Filename of the modified picture.
		//$opacity =		Intensity of the transition (in percent)
		//$pos =			Position where $watermarkFile will be inserted in $source
		//                0 = middle
		//                1 = top left
		//                2 = top right
		//                3 = bottom right
		//                4 = bottom left
		//                5 = top middle
		//                6 = middle right
		//                7 = bottom middle
		//                8 = middle left

		if(!$opacity)
			$opacity = $watermarkOpacity;
		if(!$pos)
			$pos = $watermarkPosition;

		if(!$opacity)
			$opacity = 80;
		if(!$pos)
			$pos = 3;

		$source_id = $resampled_id;        // get the image after resized it

		if($watermarkFile=="options") {
			if($source_width<200 && $source_height<200) {
				$watermarkFile = $WatermarkSizeB;
			}
			else {
				$watermarkFile = $WatermarkSizeC;
			}
			if($source_width<100 && $source_height<100) {
				$watermarkFile = $WatermarkSizeA;
			}
		}

		//////////////////////////////////////////////////////////////////////// watermark selector by image size

		$info               = GetImageSize($watermarkFile);
		$typeOfWatermarkImg = $info['mime'];

		$typeOfWatermarkImg = substr(strrchr($typeOfWatermarkImg,'/'),1);

		//Get the resource id's of watermark
		switch($typeOfWatermarkImg) {
			case 'jpeg':
				$watermark_id = imageCreateFromJPEG($watermarkFile);
				break;

			case 'gif':
				$watermark_id = imageCreateFromGIF($watermarkFile);
				break;

			case 'png':
				$watermark_id = imageCreateFromPNG($watermarkFile);
				break;

			case 'bmp':
				$watermark_id = imageCreateFromBmpDiverse($watermarkFile);
				break;

			default:
				// *** No extension - No save.
				$watermark_id = "";
				break;
		}

		if($watermark_id) {
			//Get the sizes of watermark
			$watermark_width  = imageSX($watermark_id);
			$watermark_height = imageSY($watermark_id);

			//middle
			if($pos==0) {
				$dest_x = $padding + ($source_width / 2) - ($watermark_width / 2);
				$dest_y = $padding + ($source_height / 2) - ($watermark_height / 2);
			}

			//top left
			if($pos==1) {
				$dest_x = $padding + 0;
				$dest_y = $padding + 0;
			}

			//top right
			if($pos==2) {
				$dest_x = -$padding + $source_width - $watermark_width;
				$dest_y = $padding + 0;
			}

			//bottom right
			if($pos==3) {
				$dest_x = -$padding + $source_width - $watermark_width;
				$dest_y = -$padding + $source_height - $watermark_height;
			}

			//bottom left
			if($pos==4) {
				$dest_x = $padding + 0;
				$dest_y = -$padding + $source_height - $watermark_height;
			}

			//top middle
			if($pos==5) {
				$dest_x = $padding + (($source_width - $watermark_width) / 2);
				$dest_y = $padding + 0;
			}

			//middle right
			if($pos==6) {
				$dest_x = $source_width - $watermark_width;
				$dest_y = $padding + ($source_height / 2) - ($watermark_height / 2);
			}

			//bottom middle
			if($pos==7) {
				$dest_x = (($source_width - $watermark_width) / 2);
				$dest_y = -$padding + $source_height - $watermark_height;
			}

			//middle left
			if($pos==8) {
				$dest_x = $padding + 0;
				$dest_y = ($source_height / 2) - ($watermark_height / 2);
			}
			//The main thing: merge the two pix
			imageCopyMerge($source_id,$watermark_id,$dest_x,$dest_y,0,0,$watermark_width,$watermark_height,$opacity);
		}

		return $source_id;
	}

	public function saveImage($targetName,$quality,$outcome) {

		if(!$targetName)
			$targetName = $this->originalFileName;
		if(!$quality)
			$quality = 90;
		if(!$outcome)
			$outcome = "show";

		// *** Get extension
		$extension = strrchr($targetName,'.');
		$extension = strtolower($extension);

		switch($extension) {
			case '.jpg':
			case '.jpeg':
				if(imagetypes() & IMG_JPG) {
					if($outcome=="show") {
						header('Content-Type: image/jpeg');
						imagejpeg($this->imageResized);
					}
					else {
						imagejpeg($this->imageResized,$targetName,$quality);
					}
				}
				break;

			case '.gif':
				if(imagetypes() & IMG_GIF) {
					if($outcome=="show") {
						header('Content-Type: image/gif');
						imagegif($this->imageResized);
					}
					else {
						imagegif($this->imageResized,$targetName);
					}
				}
				break;

			case '.png':
				// *** Scale quality from 0-100 to 0-9
				$scaleQuality = round(($quality / 100) * 9);

				// *** Invert quality setting as 0 is best,not 9
				$invertScaleQuality = 9 - $scaleQuality;

				if(imagetypes() & IMG_PNG) {
					if($outcome=="show") {
						header('Content-Type: image/png');
						imagepng($this->imageResized);
					}
					else {
						imagepng($this->imageResized,$targetName,$invertScaleQuality);
					}
				}
				break;

			// ... etc

			default:
				// *** No extension - No save.
				header('Content-Type: image/jpeg');
				imagejpeg($this->imageResized);
				break;
		}

		imagedestroy($this->imageResized);
	}

	## --------------------------------------------------------
}

/**
 * @create a BMP image
 * @param string $fileName
 * @return bin string on success
 * @return bool false on failure
 */
function imageCreateFromBmpDiverse($fileName) {

	/*** create a temp file ***/
	$tmp_name = tempnam("temp","GD");    //////////////////////////////////////// T2: tmp directory must be valid!
	/*** convert to gd ***/
	if(bmp2gd($fileName,$tmp_name)) {
		/*** create new image ***/
		$img = imagecreatefromgd($tmp_name);
		/*** remove temp file ***/
		unlink($tmp_name);

		/*** return the image ***/
		return $img;
	}

	return false;
}

function bmp2gd($src,$dest = false) {

	/*** try to open the file for reading ***/
	if(!($src_f = fopen($src,"rb"))) {
		return false;
	}

	/*** try to open the destination file for writing ***/
	if(!($dest_f = fopen($dest,"wb"))) {
		return false;
	}

	/*** grab the header ***/
	$header = unpack("vtype/Vsize/v2reserved/Voffset",fread($src_f,14));

	/*** grab the rest of the image ***/
	$info = unpack("Vsize/Vwidth/Vheight/vplanes/vbits/Vcompression/Vimagesize/Vxres/Vyres/Vncolor/Vimportant",fread($src_f,40));

	/*** extract the header and info into varibles ***/
	extract($info);
	extract($header);

	/*** check for BMP signature ***/
	if($type!=0x4D42) {
		return false;
	}

	/*** set the pallete ***/
	$palette_size = $offset - 54;
	$ncolor       = $palette_size / 4;
	$gd_header    = "";

	/*** true-color vs. palette ***/
	$gd_header .= ($palette_size==0) ? "\xFF\xFE" : "\xFF\xFF";
	$gd_header .= pack("n2",$width,$height);
	$gd_header .= ($palette_size==0) ? "\x01" : "\x00";
	if($palette_size) {
		$gd_header .= pack("n",$ncolor);
	}
	/*** we do not allow transparency ***/
	$gd_header .= "\xFF\xFF\xFF\xFF";

	/*** write the destination headers ***/
	fwrite($dest_f,$gd_header);

	/*** if we have a valid palette ***/
	if($palette_size) {
		/*** read the palette ***/
		$palette = fread($src_f,$palette_size);
		/*** begin the gd palette ***/
		$gd_palette = "";
		$j          = 0;
		/*** loop of the palette ***/
		while($j<$palette_size) {
			$b = $palette[$j++];
			$g = $palette[$j++];
			$r = $palette[$j++];
			$a = $palette[$j++];
			/*** assemble the gd palette ***/
			$gd_palette .= "$r$g$b$a";
		}
		/*** finish the palette ***/
		$gd_palette .= str_repeat("\x00\x00\x00\x00",256 - $ncolor);
		/*** write the gd palette ***/
		fwrite($dest_f,$gd_palette);
	}

	/*** scan line size and alignment ***/
	$scan_line_size  = (($bits * $width) + 7) >> 3;
	$scan_line_align = ($scan_line_size & 0x03) ? 4 - ($scan_line_size & 0x03) : 0;

	/*** this is where the work is done ***/
	for($i = 0,$l = $height - 1; $i<$height; $i++,$l--) {
		/*** create scan lines starting from bottom ***/
		fseek($src_f,$offset + (($scan_line_size + $scan_line_align) * $l));
		$scan_line = fread($src_f,$scan_line_size);
		if($bits==24) {
			$gd_scan_line = "";
			$j            = 0;
			while($j<$scan_line_size) {
				$b = $scan_line[$j++];
				$g = $scan_line[$j++];
				$r = $scan_line[$j++];
				$gd_scan_line .= "\x00$r$g$b";
			}
		}
		elseif($bits==8) {
			$gd_scan_line = $scan_line;
		}
		elseif($bits==4) {
			$gd_scan_line = "";
			$j            = 0;
			while($j<$scan_line_size) {
				$byte = ord($scan_line[$j++]);
				$p1   = chr($byte >> 4);
				$p2   = chr($byte & 0x0F);
				$gd_scan_line .= "$p1$p2";
			}
			$gd_scan_line = substr($gd_scan_line,0,$width);
		}
		elseif($bits==1) {
			$gd_scan_line = "";
			$j            = 0;
			while($j<$scan_line_size) {
				$byte = ord($scan_line[$j++]);
				$p1   = chr((int)(($byte & 0x80)!=0));
				$p2   = chr((int)(($byte & 0x40)!=0));
				$p3   = chr((int)(($byte & 0x20)!=0));
				$p4   = chr((int)(($byte & 0x10)!=0));
				$p5   = chr((int)(($byte & 0x08)!=0));
				$p6   = chr((int)(($byte & 0x04)!=0));
				$p7   = chr((int)(($byte & 0x02)!=0));
				$p8   = chr((int)(($byte & 0x01)!=0));
				$gd_scan_line .= "$p1$p2$p3$p4$p5$p6$p7$p8";
			}
			/*** put the gd scan lines together ***/
			$gd_scan_line = substr($gd_scan_line,0,$width);
		}
		/*** write the gd scan lines ***/
		fwrite($dest_f,$gd_scan_line);
	}
	/*** close the source file ***/
	fclose($src_f);
	/*** close the destination file ***/
	fclose($dest_f);

	return true;
}