<?php

/*************************************************************
 * Sessions class
 * - use of server sessions
 * @author    S M Kessner <limbercode@gmail.com>
 * @copyright 2010-2018 T2
 * @link      http://limbercode.com
 *************************************************************/
class sessions {

	/**
	 * All sessions - return
	 * @param string /null $nameToPull - the session name to pull    // if left blank it will fetch for all sessions
	 * @return array of sessions
	 * @example sessions::allSessions("userId");
	 */
	static function allSessions($nameToPull = "") {

		$sessions = array();

		session_write_close();
		$path  = realpath(session_save_path());
		$files = array_diff(scandir($path),array('.','..'));

		foreach($files as $file) {
			$sessions[$file] = self::unserializeSession(file_get_contents($path.'/'.$file),$nameToPull);

			if(!$sessions[$file]) {
				unset($sessions[$file]);
			}    // do not display empty sessions
			else {
				foreach($sessions[$file] as $ids => $val) {
					$sessions[$file][$ids] = explode('|',$sessions[$file][$ids]);
				}
			}    // explode to array the ids
		}

		session_start();

		return $sessions;
	}

	/**
	 * Unserialize session - recursive
	 * @param string /null $sessionData - the session file to pull from
	 * @param string /null $nameToPull - the session name to pull    // if left blank it will fetch for all sessions
	 * @param integer /null $start_index -
	 * @param string /null $dict -
	 * @return array of sessions
	 * @example sessions::unserializeSession(file_get_contents($path.'/'.$file),$nameToPull);
	 */
	static function unserializeSession($sessionData,$nameToPull = "",$startIndex = 0,&$dict = null) {

		isset($dict) or $dict = array();
		$nameEnd = strpos($sessionData,"|",$startIndex);

		if($nameEnd!==false) {
			$name  = substr($sessionData,$startIndex,$nameEnd - $startIndex);
			$rest  = substr($sessionData,$nameEnd + 1);
			$value = unserialize($rest);        // PHP will unserialize up to "|" delimiter.

			if($nameToPull) {
				if($name==$nameToPull) {
					$dict[$name] = $value;
				}
			}
			else {
				$dict[$name] = $value;
			}

			return self::unserializeSession($sessionData,$nameToPull,$nameEnd + 1 + strlen(serialize($value)),$dict);
		}

		return $dict;
	}

	/**
	 * Unlink session session id
	 * @param string $id - the session id to unlink
	 * @example sessions::unlinkSessionId($key);
	 */
	static function unlinkSessionId($id) {

		session_write_close();
		$path = realpath(session_save_path());

		unlink($path.'/'.$id);
		session_start();
	}

	/**
	 * Regenerate session
	 * Prevent network problems when regenerate session (80%) - happans because error in "unlink()" function
	 */
	static function regenerateSession() {

		if(session_status()==PHP_SESSION_ACTIVE) {
			//	@session_regenerate_id();    // will replace the current session id with a new one, and keep the current session information

			$oldSessionId = session_id();
			session_regenerate_id(false); // Create new session without destroying the old one
			$newSession = session_id(); // Grab current session ID and close both sessions to allow other scripts to use them
			session_write_close();
			session_id($newSession); // Set session ID to the new one, and start it back up again
			session_start();

			if(!empty($cfg['auth']['directoryForSessions'])) {
        $sessionDirectory = ROOT.DS.basic::$cfg['auth']['directoryForSessions'];
        $oldSessionFile   = $sessionDirectory.DS."sess_".$oldSessionId;

        if(file_exists($oldSessionFile)) {
          unlink($oldSessionFile);
        }

        self::removeOldSessionFiles($sessionDirectory);
      }
		}
	}

  /**
   * Remove old session files
   * @param $sessionDirectory
   */
  static function removeOldSessionFiles($sessionDirectory) {

    $files = scandir($sessionDirectory);

    $numOfFiles = count($files)-2;

    if($numOfFiles > 2000) {     // will remove unnecessary files from "directoryForSessions"
      foreach($files as $file) {
        if(!in_array($file,array(".",".."))) {
          $fileSize = filesize($sessionDirectory.DS.$file);
          $fileDate = filemtime($sessionDirectory.DS.$file);

//            echo $fileSize . ' bytes';
//            echo $fileDate . ' date';
//            echo date_format($fileDate,"Y/m/d H:i:s") . ' date';

          if(($fileDate < strtotime('-3 days') && $fileSize==false) || $fileDate < strtotime('-14 days')) {
            unlink($sessionDirectory.DS.$file);
          }
        }
      }
    }
  }

	/**
	 * Start session
	 */
	static function startSession() {

		if(!isset($_SESSION)) {
			session_start();
		}
	}
}