<?php

/*************************************************************
 * Class String manipulation - deals with all string/array convert/search
 * @author    S M Kessner <limbercode@gmail.com>
 * @copyright 2010-2018 T2
 * @link      http://limbercode.com
 *************************************************************/
class stringManipulation {

	/**
	 * Strip slashes deep - check for Magic Quotes and remove them
	 * @param string|array $value
	 * @return string
	 */
	public static function stripSlashesDeep($value) {

		$value = is_array($value) ? array_map('self::stripSlashesDeep',$value) : stripslashes($value);

		return $value;
	}

	/**
	 * String before - find the string before string given
	 * @param $string - the complete string
	 * @param $substring - what before that?
	 * @return string
	 */
	public static function stringBefore($string,$substring) {

		$pos = strpos($string,$substring);
		if($pos===false) {
			return $string;
		}
		else {
			return (substr($string,0,$pos));
		}
	}

	/**
	 * String after - find the string after string given
	 * @param $string
	 * @param $substring
	 * @return string
	 */
	function stringAfter($string,$substring) {

		$pos = strpos($string,$substring);
		if($pos===false) {
			return $string;
		}
		else {
			return (substr($string,$pos + strlen($substring)));
		}
	}

	/**
	 * First replace - replaces the first occurrence of a string inside a string
	 * @param string $search - what to search
	 * @param string $replace - what to replace
	 * @param string $subject - the subject string
	 * @return string - the string after the replace
	 * @example stringManipulation::firstReplace($search="/",$replace="/".lang::getCurrentLanguageForUrls(),$path);
	 */
	static function firstReplace($search,$replace,$subject) {

		if(strpos($subject,$search)!==false) {
			$occurrence = strpos($subject,$search);

			return substr_replace($subject,$replace,$occurrence,strlen($search));
		}

		return $subject;
	}

	/**
	 * Last replace - replaces the last occurrence of a string inside a string
	 * @param string $search - what to search
	 * @param string $replace - what to replace
	 * @param string $subject - the subject string
	 * @return string - the string after the replace
	 * @example stringManipulation::lastlReplace("/","/".$this->currentLanguage."/",$this->layout);
	 */
	static function lastlReplace($search,$replace,$subject) {

		$pos = strrpos($subject,$search);
		if($pos!==false) {
			$subject = substr_replace($subject,$replace,$pos,strlen($search));
		}

		return $subject;
	}

	/**
	 * String contains - find out if string contain specific string
	 * @param string $haystack - the subject string
	 * @param string $needle - what to search
	 * @return integer - the position of the found string    // returns true if $needle is a substring of $haystack
	 * @example stringManipulation::stringContains($_SERVER['REQUEST_URI'],"lala");
	 */
	static function stringContains($haystack,$needle) {

		return strpos($haystack,$needle)!==false;
	}

	/**
	 * String contains array - find out if string contain specific strings (array)
	 * @param string $haystack - the subject string
	 * @param array $needles - what to search
	 * @return boolean - returns true if $needle is a substring of $haystack
	 * @example stringManipulation::stringContainsArray($_SERVER['REQUEST_URI'],$dontAuthUrl);
	 */
	static function stringContainsArray($haystack,array $needles) {

		foreach($needles as $needles) {
			if(strpos($haystack,$needles)!==false) {
				return true;
			}
		}

		return false;
	}

	/**
	 * String starts with - find out if string starts with specific strings
	 * @param string $haystack - the subject string
	 * @param string $needle - what to search
	 * @return boolean - returns true if $haystack is starting with $needle
	 */
	static function stringStartsWith($haystack,$needle) {

		return strpos($haystack,$needle)===0;
		//	return substr($haystack,0,strlen($needle))===$needle;
	}

	/**
	 * String ends with - find out if string ends with specific strings
	 * @param string $haystack - the subject string
	 * @param array $needle - what to search
	 * @return boolean - returns true if $haystack is ending with $needle
	 * @example stringManipulation::stringEndsWith($haystack=$this->viewPath,$needle="views".DS."shared".DS."_404.php")
	 */
	static function stringEndsWith($haystack,$needle) {        // returns true if $needle is ending with $haystack

		return $needle==="" || strpos($haystack,$needle,strlen($haystack) - strlen($needle))!==false;
	}

  /**
   * Starts with - if string starts with other string
   * @param mixed $haystack the source content
   * @param mixed $needle the string to search
   * @return string
   */
  public static function startsWith($haystack,$needle) {

    $length = strlen($needle);

    return (substr($haystack,0,$length)===$needle);
  }

  /**
   * Ends with - if string ends with other string
   * @param mixed $haystack the source content
   * @param mixed $needle the string to search
   * @return string
   */
  public static function endsWith($haystack,$needle) {

    $length = strlen($needle);
    if($length==0) {
      return true;
    }

    $start = $length * -1;      // negative
    return (substr($haystack,$start)===$needle);
  }

	/**
	 * Trim start character
	 */
	public static function trimStart($prefix,$string) {

		if(substr($string,0,strlen($prefix))==$prefix) {
			$string = substr($string,strlen($prefix),strlen($string));
		}

		return $string;
	}

	/**
	 * trim end character
	 */
	public static function trimEnd($suffix,$string) {

		if(substr($string,(strlen($suffix) * -1))==$suffix) {
			$string = substr($string,0,strlen($string) - strlen($suffix));
		}

		return $string;
	}

	/**
	 * Capitalize string/array and multidimensional array
	 * @param string|array $data - the data to convert
	 * @param bool|false $everyWord - if true, convert every word
	 * @param bool|false $changeKeys - if true, convert also array keys to capitalize
	 * @return string|array - the outcome text
	 */
	static function capitalize($data,$everyWord = false,$changeKeys = false) {

		if(is_array($data)) {
			foreach($data as $key => $value) {
				if($changeKeys==true) {
					$convertedKey                                        = self::capitalize($key,$everyWord);
					$data[self::capitalize($key,$everyWord,$changeKeys)] = self::capitalize($data[$key],$everyWord,$changeKeys);
					if($convertedKey!=$key) {
						unset($data[$key]);
					}
				}
				else {
					$data[$key] = self::capitalize($data[$key],$everyWord,$changeKeys);
				}
			}
		}
		else {
			if($everyWord!==false) {
				$data = ucwords($data);
			}
			else {
				$data = ucfirst($data);
				//	$data = strtolower($data);        // make everything lower case

				$data = preg_replace_callback('/([\.!\?-]\s+|\A)(\w)/',function ($m) {

					return strtoupper($m[0]);
				},$data);                             // make first letter in a sentence upper case
			}
		}

		return $data;
	}

	/**
	 * Grammatically correct - putting dot in end of sentences, string/array and multidimensional array
	 * @param string|array $data - the data to convert
	 * @return string|array - the outcome text
	 */
	static function grammaticallyCorrect($data) {

		if(is_array($data)) {
			foreach($data as $key => $value) {
				$data[$key] = self::grammaticallyCorrect($data[$key]);
			}
		}
		else {
			if($data) {
				if(substr($data,-1)!="." && substr($data,-1)!="?" && substr($data,-1)!="!") {   // end sentence with dot
					$data = $data.".";
				}
			}
			else {
				$data = "&nbsp;";       // will create a line jump in the text
			}
		}

		return $data;
	}

	/**
	 * Tag morphing - replaces a tag envelope and keep the content intact
	 * @param string|array $data - the subject data
	 * @param string $tagName - name of the tag to search
	 * @param string $replacer - the string to put instead - regex code to use is: $2
	 * @return string|array - the outcome text
	 * @example $docData->$value = stringManipulation::tagMorphing($docData->$value,$tagName = "docTag",$replacer = "<a href='".lang::url("$2")."' class='ajaxLink'>$2</a>");
	 */
	static function tagMorphing($data,$tagName,$replacer) {

		if(is_array($data)) {
			foreach($data as $key => $value) {
				$data[$key] = stringManipulation::tagMorphing($data[$key],$tagName,$replacer);
			}
		}

		$replacedContent = preg_replace('/(<'.$tagName.'[^>]*>)(.*?)(<\/'.$tagName.'>)/i',$replacer,$data);

		//	$replacedContent   = preg_replace('/((<'.$tagName.' '.$typeInsertFormat1.'[^<])|(<'.$tagName.' '.$typeInsertFormat2.')[^>]*>)(.*?)(<\/'.$tagName.'>)/i',$replacer,$data);

		//	$replacedContent = preg_replace('/(<'.$tagName.' '.$typeInsert.'[^>]*>)(.*?)(<\/'.$tagName.'>)/i',$replacer,$data);
		//	$replacedContent = preg_replace('/(<'.$tagName.'[^>]?(<'.$tagName.'[^>]*>)(.*?)(<\/'.$tagName.'>)|(type="(.*)".*)*>)(.*?)(<\/'.$tagName.'>)/i',$replacer,$data);
		//	$replacedContent = preg_replace('/((<'.$tagName.'[^>]type="(.*)".*)*>)(.*?)(<\/'.$tagName.'>)|((<'.$tagName.'[^>]*>)(.*?)(<\/'.$tagName.'>))/i',$replacer,$data);
		//	$replacedContent = preg_replace('/(<'.$tagName.'[^>](type="(.*)".*)?.*>)(.*?)(<\/'.$tagName.'>)/i',$replacer,$data);
		//	$replacedContent = preg_replace('/(<docTag[^>](type="(.*)".*)?.*>)(.*?)(<\/docTag>)/i', '<a href="">$4</a>', '<docTag type="class">parseBots</docTag> <docTag type="class">parseBots2</docTag>');

		return $replacedContent;
	}

	/**
	 * String to file name - convert string to file name
	 * @param $string (required)
	 * @return mixed|string
	 */
	static function stringToFileName($string) {

		$string = ucwords($string);
		$string = str_replace(" ","",$string);
		$string = lcfirst($string);

		return $string;
	}

	/**
	 * Shortens string words - will not break words
	 * @param string|array $string (required)
	 * @param integer $length (required) - count chars allowed
	 * @return mixed|string
	 */
	static function shortensStringWords($string,$length) {

		if(is_array($string)) {
			foreach($string as $key => $value) {
				$string[$key] = stringManipulation::shortensStringWords($string[$key],$length);
			}
		}

		if(strlen($string)>$length) {
			$string = substr($string,0,strpos($string," ",$length));
		}

		$string = trim($string,' ');
		$string = trim($string,'.');
		$string = trim($string,',');

		return trim($string);
	}

	/**
	 * Remove links from string
	 * @param string|array $string (required)
	 * @param bool $keepText (optional) - keep the text inside the link
	 * @return string
	 */

	static function removeLinks($string,$keepText = true) {

		if(is_array($string)) {
			foreach($string as $key => $value) {
				$string[$key] = stringManipulation::removeLinks($string[$key],$keepText);
			}
		}

		if($keepText) {
			$string = preg_replace('#<a.*?>(.*?)</a>#i','\1',$string);
		}
		else {
			$string = preg_replace('#<a.*?>.*?</a>#i','',$string);
		}

		return $string;
	}
}

