<?php

/*************************************************************
 * Validation class
 * @author    S M Kessner <limbercode@gmail.com>
 * @copyright 2010-2018 T2
 * @link      http://limbercode.com
 *************************************************************/
class validation {

	private $string;
	private $type;
	private $minLength;
	private $maxLength;
	private $constraint;
	private $varName;
	private $errorOnValidate;

	/**
	 * @param $string
	 * @param string $type
	 * @param int|string $length - if we want to set max chars we will set an integer. if we want to set also a minimum length we will set it at range like: xx-xx - of min and max
	 * @param string $constraint - here we choose if this string or array is required or optional
	 * @param bool $varName - the string or array name - so the function can return it with an error - if occur
	 * @return array|bool|mixed|string
	 */
	public function validate(&$string,$type = "varchar",$length = 0,$constraint = "optional",$varName = false) {

		$this->string     = $string;
		$this->type       = $type;
		$this->constraint = $constraint;
		$this->varName    = $varName;

		if($this->constraint!="required" && $this->constraint!="optional") {
			$stacks = debug_backtrace();
			$notif  = basic::load("notif","core");

			$notif->genNotifByNum($file = "validation",90040,array(
				$this->constraint,
				$stacks[0]['file'],
				$stacks[0]['line']
			));

			$this->errorOnValidate = true;
		}

		if($this->constraint=="required" && empty($this->string)) {
			$stacks = debug_backtrace();
			$notif  = basic::load("notif","core");
			$notif->genNotifByNum($file = "validation",90050,array(
				$this->varName,
				$stacks[0]['file'],
				$stacks[0]['line']
			));

			$this->errorOnValidate = true;
		}

		$this->getCorrectLength($length);

		if(is_array($this->string)) {
			array_walk_recursive($this->string,"self::validateArray");
		}
		elseif(is_object($this->string)) {
      $this->string = json_decode(json_encode($this->string),true);
		  array_walk_recursive($this->string,"self::validateArray");
    }
		else {
			$this->string = $this->validateLength($this->string);
			$this->string = $this->validator($this->string,$this->type);
		}

		return $this->string;
	}

  public function validateRegex() {

  }

	/**
	 * Validate array - used by array_walk_recursive
	 * @param $value
	 */
	private function validateArray(&$value) {

		$value = $this->validateLength($value);
		$value = $this->validator($value,$this->type);
	}

	/**
	 * Get correct length - get min and max length
	 * @param $length
	 */
	private function getCorrectLength($length) {

		$explodedLength = explode("-",$length);

		$minLength = "";
		$maxLength = "";

		if(count($explodedLength)==2) {
			$minLength = $explodedLength[0];
			$maxLength = $explodedLength[1];
		}
		elseif(count($explodedLength)==1) {
			$minLength = 0;
			$maxLength = $explodedLength[0];
		}

		$this->minLength = $minLength;
		$this->maxLength = $maxLength;

		foreach($explodedLength as $key => $value) {
			if((!is_numeric($value) || $value<1) && $value) {
				$stacks = debug_backtrace();
				$notif  = basic::load("notif","core");
				$notif->genNotifByNum($file = "validation",90010,array($stacks[0]['file'],$stacks[0]['line']));

				$this->string          = "";
				$this->errorOnValidate = true;
			}
		}
	}

	/**
	 * Validate length
	 * @param $string
	 * @param bool $minLength
	 * @param bool $maxLength
	 * @return string
	 */
	public function validateLength($string,$minLength = false,$maxLength = false) {

		if($minLength) {
			$this->minLength = $minLength;
		}

		if($maxLength) {
			$this->maxLength = $maxLength;
		}

		if($this->minLength) {       // min length is in effect
			if(strlen($string)<$this->minLength) {
				$stacks = debug_backtrace();
				$notif  = basic::load("notif","core");
				$notif->genNotifByNum($file = "validation",90020,array(
					$this->varName,
					$this->minLength,
					$stacks[1]['file'],
					$stacks[1]['line']
				));

				$string                = "";
				$this->errorOnValidate = true;
			}
		}

		if($this->maxLength) {
			if(strlen($string)>$this->maxLength) {
				$stacks = debug_backtrace();
        $file = !empty($stacks[1]['line']) ? $stacks[1]['line'] : $stacks[0]['line'];
        $line = !empty($stacks[1]['line']) ? $stacks[1]['line'] : $stacks[0]['line'];

				foreach($stacks as $key => $value) {
				  if(!empty($stacks[$key]['file']) && $stacks[$key]['function']=="validate" && $stacks[$key]['class']=="validation") {
            $file = $stacks[$key]['file'];
            $line = $stacks[$key]['line'];

            break;
          }
        }

				$notif  = basic::load("notif","core");
				$notif->genNotifByNum("validation",90030,array(
					$this->varName,
					$this->maxLength,
          $file,
          $line
				));

				$string                = "";
				$this->errorOnValidate = true;
			}
		}

		return $string;
	}

	/**
	 * Is validated - checks if all validations passed good or not
	 * @return bool
	 */
	public function isValidated() {

		if($this->errorOnValidate) {
			$stacks = debug_backtrace();
			$notif  = basic::load("notif","core");
			$notif->genNotifByNum($file = "validation",90100,array(
				$stacks[0]['file'],
				$stacks[0]['line']
			));

			return false;
		}
		else {
			return true;
		}
	}

	/**
	 * Validator - validates the string by type
	 * @param $string
	 * @param string $type
	 * @return bool|mixed|string
	 */
	public function validator($string,$type = "varchar") {

		$validationTypes = array(
			"xss",
			"varchar",
			"text",
			"tinyint",
			"smallint",
			"mediumint",
			"int",
			"integer",
			"bigint",
			"decimal",
			"float",
			"double",
			"characters",
			"filename",
			"url",
			"email",
			"ip",
			"date"
		);

		if(!in_array($type,$validationTypes)) {
			$stacks = debug_backtrace();
			$notif  = basic::load("notif","core");
			$notif->genNotifByNum($file = "validation",91000,array(
				$this->varName,
				$type,
				$stacks[1]['file'],
				$stacks[1]['line']
			));

			$string                = 0;
			$this->errorOnValidate = true;
		}

		if($type=="xss") {              // for extra security

			$string = trim($string);    // remove whitespaces
			$string = stripslashes($string);    // apply stripslashes if magic_quotes_gpc is enabled
			$string = preg_replace("/[^a-zA-Z0-9א-ת,.~ :%]/i"," ",$string);    // remove everything except the 0-9 and a-z and blanks spaces and ,.~  characters
		}

		if($type=="varchar" || $type=="text" || $type=="xss") {
			$string = filter_var($string,FILTER_SANITIZE_STRING,FILTER_FLAG_NO_ENCODE_QUOTES );
			$string = trim($string);   // check white spaces at the end
		}

		if($type=="tinyint" || $type=="smallint" || $type=="mediumint" || $type=="int" || $type=="integer" || $type=="bigint") {
			if($string) {
				$string = filter_var($string,FILTER_SANITIZE_NUMBER_INT);
        $string = $string=="-" ? "" : $string;

				//		if(!filter_var($int, FILTER_VALIDATE_INT)) {$string="";}
				if($string=="") {
					$stacks = debug_backtrace();
					$notif  = basic::load("notif","core");
					$notif->genNotifByNum($file = "validation",91060,array(
						$this->varName,
						$stacks[1]['file'],
						$stacks[1]['line']
					));

					$string                = 0;
					$this->errorOnValidate = true;
				}
			}
		}

		if($type=="decimal" || $type=="float" || $type=="double") {
			if($string) {
				$string = filter_var($string,FILTER_SANITIZE_NUMBER_FLOAT,
					FILTER_FLAG_ALLOW_FRACTION);

				if($string=="") {
					$stacks = debug_backtrace();
					$notif  = basic::load("notif","core");
					$notif->genNotifByNum($file = "validation",91070,array(
						$this->varName,
						$stacks[1]['file'],
						$stacks[1]['line']
					));

					$string                = 0;
					$this->errorOnValidate = true;
				}
			}
		}

		if($type=="characters") {
			if(empty($string)) {
				return false;
			}
			if(eregi('(Content-Type)|(MIME-Version)|(Content-Disposition)|(\\\\n)|(%0A)|(0x0A)|(\\\\r)|(0x0D)|(%0D)|(to:)|(cc:)|(bcc:)',$string)) {
				return false;
			}

			return $string;
		}

		if($type=="filename" && $string) {    // also filter
      $string = urldecode($string);
      $string = str_replace(' ','-',$string); // Replaces all spaces with hyphens.
      $string = preg_replace('/[^A-Za-z0-9.\-]/','',$string); // Removes special chars.
      $string = preg_replace('/-+/','-',$string); // Replaces multiple hyphens with single one.

			if(!$string) {
				$stacks = debug_backtrace();
				$notif  = basic::load("notif","core");
				$notif->genNotifByNum($file = "validation",91050,array(
					$this->varName,
					$stacks[0]['file'],
					$stacks[0]['line']
				));

				$this->errorOnValidate = true;
			}
		}

		if($type=="url") {
			if(!preg_match("/^(http|https|ftp):\/\/([A-Z0-9][A-Z0-9_-]*(?:\.[A-Z0-9][A-Z0-9_-]*)+):?(\d+)?\/?/i",$string)) {
				$stacks = debug_backtrace();
				$notif  = basic::load("notif","core");
				$notif->genNotifByNum($file = "validation",91010,array(
					$this->varName,
					$stacks[0]['file'],
					$stacks[0]['line']
				));

				$string                = "";
				$this->errorOnValidate = true;
			}
		}

		if($type=="email") {
			$string = filter_var($string,FILTER_SANITIZE_EMAIL);    // this linw can be deleted
			$string = preg_replace('((?:\n|\r|\t|%0A|%0D|%08|%09)+)i','',$string);
			$string = substr($string,0,100);    // limit the characters lenght
			//		if(!filter_var($string, FILTER_VALIDATE_EMAIL)) {$string="";}
			$string = $this->validateEmailAddress($string);
		}

		if($type=="ip") {
			if(!filter_var($string,FILTER_VALIDATE_IP)) {
				$stacks = debug_backtrace();
				$notif  = basic::load("notif","core");
				$notif->genNotifByNum($file = "validation",91030,array(
					$this->varName,
					$stacks[0]['file'],
					$stacks[0]['line']
				));

				$string                = "";
				$this->errorOnValidate = true;
			}
		}

		if($type=="date") {
			if(preg_match("/^([0-9]{4})-([0-9]{2})-([0-9]{2})$/",$string,$parts)) {    //check weather the date is valid of not
				if(!checkdate($parts[2],$parts[3],$parts[1])) {
					$string = "";
				}
			}
			else {
				$string = "";
			}

			if(!$string) {
				$stacks = debug_backtrace();
				$notif  = basic::load("notif","core");
				$notif->genNotifByNum($file = "validation",91040,array(
					$this->varName,
					$stacks[0]['file'],
					$stacks[0]['line']
				));

				$this->errorOnValidate = true;
			}
		}

		return $string;
	}

	public function validateDateOpt2($date) {

		$stamp = strtotime($date);

		if(!is_numeric($stamp)) {
			return false;
		}
		$month = date('m',$stamp);
		$day   = date('d',$stamp);
		$year  = date('Y',$stamp);

		if(checkdate($month,$day,$year)) {
			return true;
		}

		return false;
	}

	/**
	 * Validate an email address.
	 * Provide email address (raw input)
	 * Returns true if the email address has the email
	 * address format and the domain exists.
	 */
	public function validateEmailAddress($email) {

		$isValid = true;
		$atIndex = strrpos($email,"@");
		if(is_bool($atIndex) && !$atIndex) {
			$isValid = false;
		}
		else {
			$domain    = substr($email,$atIndex + 1);
			$local     = substr($email,0,$atIndex);
			$localLen  = strlen($local);
			$domainLen = strlen($domain);
			if($localLen<1 || $localLen>64) {
				// local part length exceeded
				$isValid = false;
			}
			else if($domainLen<1 || $domainLen>255) {
				// domain part length exceeded
				$isValid = false;
			}
			else if($local[0]=='.' || $local[$localLen - 1]=='.') {
				// local part starts or ends with '.'
				$isValid = false;
			}
			else if(preg_match('/\\.\\./',$local)) {
				// local part has two consecutive dots
				$isValid = false;
			}
			else if(!preg_match('/^[A-Za-z0-9\\-\\.]+$/',$domain)) {
				// character not valid in domain part
				$isValid = false;
			}
			else if(preg_match('/\\.\\./',$domain)) {
				// domain part has two consecutive dots
				$isValid = false;
			}
			else if(!preg_match('/^(\\\\.|[A-Za-z0-9!#%&`_=\\/$\'*+?^{}|~.-])+$/',str_replace("\\\\","",$local))) {
				// character not valid in local part unless
				// local part is quoted
				if(!preg_match('/^"(\\\\"|[^"])+"$/',str_replace("\\\\","",$local))) {
					$isValid = false;
				}
			}
			if($isValid && !(checkdnsrr($domain,"MX") || checkdnsrr($domain,"A"))) {
				// domain not found in DNS
				$isValid = false;
			}
		}

		if(!$isValid) {
			$stacks = debug_backtrace();
			$notif  = basic::load("notif","core");
			$notif->genNotifByNum($file = "validation",91020,array(
				$this->varName,
				$stacks[0]['file'],
				$stacks[0]['line']
			));

			$email                 = "";
			$this->errorOnValidate = true;
		}

		return $email;
	}

	public function validateEmail($email) {

		if(preg_match("/^([a-zA-Z0-9])+([a-zA-Z0-9\._-])*@([a-zA-Z0-9_-])+([a-zA-Z0-9\._-]+)+$/",$email)) {
			list($username,$domain) = explode('@',$email);

			if(!getmxrr($domain,$mxhosts)) {
				$emailError = true;
			}
			else {
				$emailError = false;
			}
		}
		else {
			$emailError = true;
		}

		if($emailError) {
			$stacks = debug_backtrace();
			$notif  = basic::load("notif","core");
			$notif->genNotifByNum($file = "validation",91020,array(
				$this->varName,
				$stacks[0]['file'],
				$stacks[0]['line']
			));

			$email                 = "";
			$this->errorOnValidate = true;
		}

		return $email;
	}

	public function validateDomain($email,$record = 'MX') {

		list($user,$domain) = explode('@',$email);

		if(checkdnsrr($domain,$record)) {
			/*	echo('This MX records exists; I will accept this email as valid.');*/
			return true;
		}
		else {
			/*	echo('No MX record exists;  Invalid email.');*/
			return false;
		}
	}

	public function setReporting() {

		if(DEVELOPMENT_ENVIRONMENT==true) {
			error_reporting(E_ALL);
			ini_set('display_errors','On');
		}
		else {
			error_reporting(E_ALL);
			ini_set('display_errors','Off');
			ini_set('log_errors','On');
			ini_set('error_log',ROOT.DS.'tmp'.DS.'logs'.DS.'error.log');
		}
	}

	/** Check for Magic Quotes and remove them **/

	public function stripSlashesDeep($value) {

		$value = is_array($value) ? array_map('stripSlashesDeep',$value) : stripslashes($value);

		return $value;
	}

	public function removeMagicQuotes() {

		$_GET    = $this->stripSlashesDeep($_GET);
		$_POST   = $this->stripSlashesDeep($_POST);
		$_COOKIE = $this->stripSlashesDeep($_COOKIE);
	}
}
