<?php

/*************************************************************
 * Xml class
 * - use of xml
 * @author    S M Kessner <limbercode@gmail.com>
 * @copyright 2010-2018 T2
 * @link      http://limbercode.com
 *************************************************************/
class xml {

	/**
	 * Xml creator
	 */
	public function xmlCreator() {

		// Set the content type to be XML, so that the browser will   recognise it as XML.
		header("content-type: application/xml; charset=ISO-8859-15");

		// "Create" the document.
		$xml = new DOMDocument("1.0","ISO-8859-15");

		// Create some elements.
		$xml_album = $xml->createElement("Album");
		$xml_track = $xml->createElement("Track","The ninth symphony");

		// Set the attributes.
		$xml_track->setAttribute("length","0:01:15");
		$xml_track->setAttribute("bitrate","64kb/s");
		$xml_track->setAttribute("channels","2");

		// Create another element, just to show you can add any (realistic to computer) number of sublevels.
		$xml_note = $xml->createElement("Note","The last symphony composed by Ludwig van Beethoven.");

		// Append the whole bunch.
		$xml_track->appendChild($xml_note);
		$xml_album->appendChild($xml_track);

		// Repeat the above with some different values..
		$xml_track = $xml->createElement("Track","Highway Blues");

		$xml_track->setAttribute("length","0:01:33");
		$xml_track->setAttribute("bitrate","64kb/s");
		$xml_track->setAttribute("channels","2");
		$xml_album->appendChild($xml_track);

		$xml->appendChild($xml_album);

		print $xml->saveXML();        // Parse the XML.

		///////////////////////// method 2: not saving encoding (problem with windows-1255)

		/*$sXML = '<root><element><key>a</key><value>b</value></element></root>';
		$doc = new DOMDocument();
		$doc->preserveWhiteSpace = false;
		$doc->formatOutput   = true;
		$doc->loadXML($sXML);
		echo $doc->saveXML();*/
	}

	/**
	 * Array to xml - recursive
	 * @param array $array - the array to convert
	 * @param string $head - name of the title array
	 * @param string /null $file
	 * @param object $root
	 * @return xml
	 */
	public function arrayToXml($array,$head = "dataFromArray",$file = "",SimpleXMLElement $root = null) {

		if(!isset($root) || null==$root)
			$root = new SimpleXMLElement("<?xml version='1.0' encoding='utf-8'?><".$head."/>");

		if(is_array($array)) {
			foreach($array as $k => $v) {
				if(is_string($k) && $k[0]=="@") {    // special: attributes
					if(is_string($v)) {
						$root->addAttribute(substr($k,1),$v);
					}
				}    // normal: append
				else {
					$this->arrayToXml($v,$head,$file,$root->addChild(is_numeric($k) ? "element-".$k : $k));
				}    // fix 'invalid xml name' by prefixing numeric keys
			}
		}
		else {
			$root[0] = $array;
		}

		$root = $this->getFormattedXml($root,$file);

		return $root;
	}

	/**
	 * Get formatted xml
	 * @param object $xml
	 * @param string $file
	 * @param string /null $domVer
	 * @param string /null $preserveWhitespace
	 * @param string /null $formatOutput
	 * @return xml
	 * @example $this->getFormattedXml($root,$file);
	 */
	public function getFormattedXml(SimpleXMLElement $xml,$file,$domVer = null,$preserveWhitespace = true,$formatOutput = true) {

		// create new wrapper, so we can get formatting options
		$dom                     = new DOMDocument($domVer,"UTF-8");
		$dom->preserveWhiteSpace = $preserveWhitespace;
		$dom->formatOutput       = $formatOutput;

		// now import the xml (converted to dom format)
		/*
		$ix = dom_import_simplexml($xml);
		$ix = $dom->importNode($ix, true);
		$dom->appendChild($ix); */
		$dom->loadXML($xml->asXML());

		if($file) {
			$dom->save($file.".xml");
		}

		return $dom->saveXML();
	}

	/**
	 * Convert xml string to php array - useful to get a serializable value
	 * @param string $xmlstr
	 * @return array
	 */
	static function xmlToArray($xmlstr) {

		$doc = new DOMDocument();
		$doc->loadXML($xmlstr);

		return self::domNodeToArray($doc->documentElement);
	}

	/**
	 * Convert xml string to php array - useful to get a serializable value
	 * @param string $node
	 * @return array
	 */
	static function domNodeToArray($node) {

		$output = array();
		switch($node->nodeType) {
			case XML_CDATA_SECTION_NODE:
			case XML_TEXT_NODE:
				$output = trim($node->textContent);
				break;
			case XML_ELEMENT_NODE:
				for($i = 0,$m = $node->childNodes->length; $i<$m; $i++) {
					$child = $node->childNodes->item($i);
					$v     = self::domNodeToArray($child);

					if(isset($child->tagName)) {
						$t = $child->tagName;

						if(!isset($output[$t])) {
							$output[$t] = array();
						}

						$output[$t][] = $v;
					}
					elseif($v) {
						$output = (string)$v;
					}
				}
				if(is_array($output)) {
					if($node->attributes->length) {
						$a = array();
						foreach($node->attributes as $attrName => $attrNode) {
							$a[$attrName] = (string)$attrNode->value;
						}
						$output['@attributes'] = $a;
					}
					foreach($output as $t => $v) {
						if(is_array($v) && count($v)==1 && $t!='@attributes') {
							$output[$t] = $v[0];
						}
					}
					foreach($output as $t => $v) {
						if(substr($t,0,8)==="element-") {
							$tNew = substr($t,8);

							$output[$tNew] = $output[$t];
							unset($output[$t]);
						}
					} // fix issue with row-0...
				}
				break;
		}

		return $output;
	}
}

