<?php

/*************************************************************
 * Advanced system class for files
 * - configure the db handling system
 * @author    S M Kessner <limbercode@gmail.com>
 * @copyright 2010-2018 T2
 * @link      http://limbercode.com
 *************************************************************/
trait advancedTrait {


  private $advancedForMysql;
  private $advancedForFiles;
  private $advancedEntries;
  private $advancedBlueprints;
  private $blueprintName;
  private $entrySearchTerm;
  private $activeSearchTerm;
  private $archiveSearchTerm;
  private $totalEntriesOnTable;
  private $totalActiveBlueprints;
  private $totalArchivedBlueprints;
  private $pageNum;
  private $activeRowsLimit = 10;
  private $archiveRowsLimit = 10;
  private $entryRowsLimit = 10;
  private $pastEntryRowsLimit = 20;
  private $pageColumn;
  private $pageOrdering;
  private $leadId;
  private $createdAt;
  private $isActive;
  private $isDeleted;
  private $actionId;
  private $userId;
  private $auth;
  private $validation;
  private $varcharType = array(
    'text',
    'textArea',
    'color',
    'email',
    'password',
    'url',
    'phone',
    'dataFetcherVar',
    'range',
  );
  private $repeaterBlueprintDbEnding = "_id";
  private $repeaterType = array('fieldsRepeater');
  private $selectorType = array('selectBox','checkBoxes','radioButtons');
  private $intType = array('number','rating','dataFetcherInt');
  private $dateType = array('date','dataFetcherDate');
  private $timeType = array('time','dataFetcherTime');
  private $dateNtimeType = array('dateNtime','dataFetcherDateNTime');
  private $fileType = array('image','file');
//  private $arrayOfFieldsThatHaveNoLimit = array("select","radio","checkbox");

  private $imagesTempStoreFolder = ROOT.DS."impermanent".DS."uploads".DS."images".DS."advanced".DS;
//	private $imagesStoreFolder = ROOT.DS."source".DS."itemImages".DS;
//  private $imagesStoreFolder = ROOT.DS."public".DS."images".DS."advanced".DS;
  private $imagesPath = "/"."public"."/"."images"."/"."advanced"."/";
  private $imagesStoreFolder = "images".DS."advanced".DS;
  private $allowView = false;
  private $allowNew = false;
  private $allowEdit = false;
  private $allowDelete = false;



  /**
   * Constructor for advanced
   */
  function __construct() {

  }

  /**
   * @return string
   */
  private function blockNotAjax() {

    $isAjax = myHelpers::isAjax();

    if(!$isAjax) {
      echo $this->loadView("advanced/failed");
      die();
    }
  }

  /**
   * Pre validate
   * @param $param
   * @param $type - GET/POST
   * @return array
   */
  private function preValidate($param,$type) {

//    if($type!="GET")
    //  p($param);

    if(empty($param[$type]) || $param[$type]=="-") {
      $param        = [];
      $param[$type] = "";
      //    $param['GET'] = (!empty($param['GET']) && $param['GET']!="-") ? $param['GET'] : "";
    }

    return $param;
  }

  private function setRecentlyCookie($cookieName,$value) {

    if(!empty($_COOKIE[$cookieName])) {
      $recentlyUsedBlueprints = json_decode($_COOKIE[$cookieName],true);
    }

    if(!empty($recentlyUsedBlueprints)) {
      if(!in_array($value,$recentlyUsedBlueprints)) {
        array_push($recentlyUsedBlueprints,$value);
      }
    }
    else {
      $recentlyUsedBlueprints = array($value);
    }

    if(!headers_sent()) {
      setcookie($cookieName,json_encode($recentlyUsedBlueprints),time() + (86400 * 30),"/");
    }
  }


  private function paginationCalculation($type,$rowsLimit,$totalRows) {

//    $this->pageNum = !empty($this->pageNum) ? $this->pageNum : 1;
//    $this->view->pageNum = $this->pageNum;

    if($type=="active") {
      $this->view->path = "/advanced/showActive";
    }
    if($type=="archive") {
      $this->view->path = "/advanced/showArchive";
    }
    if($type=="entry") {
      $this->view->path = "/advanced/showEntries/".$this->blueprintName;
    }

    $this->view->type                = $type;
    $this->view->{$type}             = new stdClass();
    $this->view->{$type}->rowsOnPage = "";
    if(!empty($this->blueprintData)) {
      $this->view->{$type}->rowsOnPage = count($this->blueprintData);
    }

    $this->view->{$type}->totalPages = ceil($totalRows / $rowsLimit);
    $this->view->{$type}->offset     = ($this->pageNum - 1) * $rowsLimit;

    $this->view->{$type}->range             = 2;
    $this->view->{$type}->firstPage         = 1;
    $this->view->{$type}->previousPage      = $this->pageNum - 1;
    $this->view->{$type}->nextPage          = $this->pageNum + 1;
    $this->view->{$type}->lastPage          = !empty($this->view->{$type}->totalPages) ? $this->view->{$type}->totalPages : 1;
    $this->view->{$type}->firstPageClass    = "";
    $this->view->{$type}->previousPageClass = "";
    $this->view->{$type}->nextPageClass     = "";
    $this->view->{$type}->lastPageClass     = "";

    //      if(!empty($this->pageNum) && is_int($this->pageNum)) {
    if(!empty($this->pageNum)) {
      $this->view->{$type}->firstPageClass    = ($this->pageNum==1) ? "disabledArrow" : "";
      $this->view->{$type}->previousPageClass = ($this->pageNum<=1) ? "disabledTriangle" : "";
      $this->view->{$type}->nextPageClass     = ($this->pageNum>=$this->view->{$type}->totalPages) ? "disabledTriangle" : "";
      $this->view->{$type}->lastPageClass     = ($this->pageNum==$this->view->{$type}->totalPages || $this->view->{$type}->lastPage==1) ? "disabledArrow" : "";
    }

    if(($this->pageNum - $this->view->{$type}->range)<=1) {
      $this->view->{$type}->startFrom = 1;
      $this->view->{$type}->endAt     = 5;
    }
    else if($this->pageNum>=($this->view->{$type}->totalPages - $this->view->{$type}->range)) {
      $this->view->{$type}->startFrom = $this->view->{$type}->totalPages - 4;
      $this->view->{$type}->endAt     = $this->view->{$type}->totalPages;
    }
    else {
      $this->view->{$type}->startFrom = $this->pageNum - $this->view->{$type}->range;
      $this->view->{$type}->endAt     = $this->pageNum + $this->view->{$type}->range;
    }
  }

  private function getLoggedUserId() {

    $auth         = basic::load("auth","core");
    $this->userId = $auth->getUserLoggedUserId();

    return $this->userId;
  }

  private function setAuthActions($advancedData = "") {

    $this->view->allowView = false;
    $this->view->allowNew = false;
    $this->view->allowEdit = false;
    $this->view->allowDelete = false;

    if(empty($advancedData)) {
      if(!empty($this->blueprintName)) {
        $advancedData = $this->advancedForFiles->loadBlueprint($this->blueprintName);
      }
    }

    if(!empty($advancedData)) {
      $auth                = basic::load("auth","core");
      $userLoggedAuthLevel = $auth->getUserLoggedAuthLevel();

      if(!empty($advancedData['security']['authLevelView'])) {
        if($advancedData['security']['authLevelView']<=$userLoggedAuthLevel) {
          $this->allowView       = true;
          $this->view->allowView = true;
        }
      }

      if(!empty($advancedData['security']['authLevelNew'])) {
        if($advancedData['security']['authLevelNew']<=$userLoggedAuthLevel) {
          $this->allowNew       = true;
          $this->view->allowNew = true;
        }
      }

      if(!empty($advancedData['security']['authLevelEdit'])) {
        if($advancedData['security']['authLevelEdit']<=$userLoggedAuthLevel) {
          $this->allowEdit       = true;
          $this->view->allowEdit = true;
        }
      }

      if(!empty($advancedData['security']['authLevelDelete'])) {
        if($advancedData['security']['authLevelDelete']<=$userLoggedAuthLevel) {
          $this->allowDelete       = true;
          $this->view->allowDelete = true;
        }
      }
    }
    else {
      $this->notif->genNotifByNum($file = "advanced",9415,$this->blueprintName);
    }
  }

  private function creatingNewDatabase($databaseName) {

    $createdNewDatabase = $this->model->createDatabase($databaseName);
    $this->model->useDatabase($databaseName);

    return $createdNewDatabase;
  }

  private function creatingNewBlueprint($advancedData) {    // we keep this function here cz we need it on installSystemTables()

    if(empty($this->blueprintName)) {
      echo"No blueprint name!";
      die();
    }

    if(empty($advancedData)) {
      $advancedData = $this->advancedForFiles->loadBlueprint($this->blueprintName);
    }

    $checkIfTableExist = $this->model->checkIfTableExist($this->blueprintName);
    $advancedModel     = "";

    if(!$checkIfTableExist) {
      $createData = [];

      $fields = !empty($advancedData['fields']) ? $advancedData['fields'] : "";;
      $dbQueryProblem = false;

      if(!empty($fields) && is_array($fields)) {
        foreach($fields as $key => $field) {
          $standardFields = !empty($field['standard']) ? $field['standard'] : "";;
          $addOnFields = !empty($field['addOn']) ? $field['addOn'] : "";;

          if(!empty($standardFields)) {
            $fieldType  = !empty($standardFields['fieldType']) ? $standardFields['fieldType'] : "";
            $allowFloat = !empty($addOnFields['allowFloat']) ? $addOnFields['allowFloat'] : "";

            if(!empty($fieldType)) {
              if(!empty($standardFields['name']) || $fieldType=="fieldsRepeater") {
                if($fieldType=="fieldsRepeater") {
                  $fieldName = $addOnFields['repeaterBlueprint'].$this->repeaterBlueprintDbEnding;
                }
                else {
                  $fieldName = $standardFields['name'];
                }

                $createData['columns'][$fieldName] = [];
              }
              if(!empty($fieldName)) {
                $maxLength          = !empty($standardFields['maxLength']) ? $standardFields['maxLength'] : "";
                $forceInt           = false;
                $choiceKey          = !empty($addOnFields['choice-key']) ? $addOnFields['choice-key'] : "";
                $choiceBlueprint    = !empty($addOnFields['choice-blueprint']) ? $addOnFields['choice-blueprint'] : "";
                $choiceBlueprintKey = !empty($addOnFields['choice-blueprint-key']) ? $addOnFields['choice-blueprint-key'] : "";

                if(!empty($choiceKey) || (!empty($choiceBlueprint) && !empty($choiceBlueprintKey))) {
                  list($forceInt,$maxLength) = $this->determentIfForceInt($choiceKey,$choiceBlueprint,$choiceBlueprintKey,$maxLength);
                  $advancedData['fields'][$key]['standard']['maxLength'] = $maxLength;
                }

                list($fieldNameDataType,$standardFieldsMaxLength,$maxLengthIsGivenFromMysqlAuto) = $this->creatingColumns($field,$fieldType,$maxLength,$allowFloat,$forceInt);

                if(!empty($fieldNameDataType)) {
                  $createData['columns'][$fieldName]['dataType'] = $fieldNameDataType;
                }

                if(!empty($standardFieldsMaxLength)) {
                  $standardFields['maxLength'] = $standardFieldsMaxLength;
                }

                if(!empty($standardFields['maxLength']) || $maxLengthIsGivenFromMysqlAuto===true) {
                  $fieldMaxLength                                 = $maxLengthIsGivenFromMysqlAuto===true ? "" : $standardFields['maxLength'];
                  $createData['columns'][$fieldName]['lengthSet'] = $fieldMaxLength;
                  $createData['columns'][$fieldName]['addOn']     = "not null";
                }
                else {
                  $dbQueryProblem = true;
                  $this->notif->genNotifByNum($file = "advanced",9163,array($this->blueprintName,$key." (maxLength)"));
                }
                /*
                                if(!empty($standardFields['defaultValue'])) {
                                  $fieldDefaultValue = $standardFields['defaultValue'];
                                }
                                if(!empty($standardFields['necessity'])) {
                                  $fieldNecessity = $standardFields['necessity'];
                                }
                                if(!empty($standardFields['placeHolder'])) {
                                  $fieldPlaceHolder = $standardFields['placeHolder'];
                                }
                                if(!empty($standardFields['returnValue'])) {
                                  $fieldReturnValue = $standardFields['returnValue'];
                                }
                                if(!empty($standardFields['instructions'])) {
                                  $fieldInstructions = $standardFields['instructions'];
                                }*/

                if(empty($createData['columns'][$fieldName]['dataType'])) {
                  $dbQueryProblem = true;
                  $this->notif->genNotifByNum($file = "advanced",9163,array($this->blueprintName,$key." (dataType)"));
                }
              }
              else {
                $dbQueryProblem = true;
                $this->notif->genNotifByNum($file = "advanced",9163,array($this->blueprintName,$key." (name)"));
              }
            }
            else {
              $dbQueryProblem = true;
              $this->notif->genNotifByNum($file = "advanced",9163,array($this->blueprintName,$key." (fieldType)"));
            }
            /*            switch($fieldType) {
                          case "text":
                       //     echo "Your favorite color is red!";

                            $createData['columns'][$fieldName];
                            break;
                          case "textArea":
                         //   echo "Your favorite color is blue!";
                            break;
                          default:
                            echo "FieldType invalid!";
                        }*/
          }
        }
      }

      /*      p(2222222);
            p($createData);
            p($fieldName);
            die();*/

      //    $fileContents = file_get_contents('json_array.txt');
      //    $decoded = json_decode($fileContents, true);

      /*      $createData = array(
              "columns" => array(
            //    "id" => array("int","11",""),
                "name" => array(
                  "dataType" => "varchar",
                  "charLimit" => "90",
                  "addOn" => "not null",
                ),
              ),
         //   "primaryKey" => "id",
              "characterSet" => "utf8 COLLATE utf8_general_ci",
              "engine" => "InnoDB",
            );*/

      if(!empty($createData['columns']) && $dbQueryProblem!==true) {
        $advancedModel = $this->model->createNewTable($this->blueprintName,$createData);
      }
      else {
        $this->notif->genNotifByNum($file = "advanced",9015,$this->blueprintName);
      }
    }
    else {
      $this->notif->genNotifByNum($file = "advanced",9014,$this->blueprintName);
    }

    return $advancedModel;
  }

  /**
   * Determent if forceInt
   * @param $addOnFieldChoiceKey
   * @param $maxLength
   * @return array
   * @throws ReflectionException
   */
  private function determentIfForceInt($addOnFieldChoiceKey,$addOnFieldChoiceBlueprint,$addOnFieldChoiceBlueprintKey,$maxLength) {

    $forceInt = false;

    if(!empty($addOnFieldChoiceBlueprint) && !empty($addOnFieldChoiceBlueprintKey)) {
      if($addOnFieldChoiceBlueprintKey=="leadId") {
        $forceInt  = true;
        $maxLength = 11;
      }
      else {
        $advancedData = $this->advancedForFiles->loadBlueprint($addOnFieldChoiceBlueprint);

        if(!empty($advancedData['fields'])) {
          foreach($advancedData['fields'] as $key => $value) {
            if($addOnFieldChoiceBlueprintKey==$value['name']) {
              if(!empty($value['maxLength'])) {
                $maxLength = $value['maxLength'];
              }
            }
          }
        }
      }
    }

    if(!empty($addOnFieldChoiceKey)) {
      basic::load("arrayManipulation","functions");
      $arrayValuesType = arrayManipulation::arrayValuesType($addOnFieldChoiceKey);
      $forceInt        = $arrayValuesType!="associative";

      if($forceInt) {
        $maxLength = 4;
      }
      else {
        $lengths   = array_map('strlen',$addOnFieldChoiceKey);
        $maxLength = max($lengths);
      }
    }

    return array($forceInt,$maxLength);
  }

  /**
   * Creating columns
   * @param $advancedDataField
   * @param $fieldType
   * @param $lengthSet
   * @param $allowFloat
   * @param false $forceInt
   * @return array
   */
  private function creatingColumns($advancedDataField,$fieldType,$lengthSet,$allowFloat,$forceInt = false) {    // we keep this function here cz we need it on installSystemTables()

    $dataType                      = false;
    $maxLengthIsGivenFromMysqlAuto = false;

    if(in_array($fieldType,$this->selectorType)) {
      $dataType = "varchar";

      if($forceInt) {
        $dataType = "int";

        if($lengthSet<=4) {
          $dataType = "tinyInt";
        }
        if($lengthSet>11) {
          $dataType = "bigInt";
        }
      }

      if(!empty($advancedDataField['addOn']['selectMultiple']) && $advancedDataField['addOn']['selectMultiple']==true) {    // don't forceInt too
        $dataType  = "varchar";
        $lengthSet = 150;
      }
    }
    if(in_array($fieldType,$this->varcharType)) {
      $dataType = "varchar";
    }
    if(in_array($fieldType,$this->fileType)) {
      $dataType  = "varchar";
      $lengthSet = 255;
    }
    if(in_array($fieldType,$this->repeaterType)) {
      $dataType  = "varchar";
      $lengthSet = 205;
    }

    if(in_array($fieldType,$this->intType)) {
      if(!$allowFloat) {
        $dataType = "int";

        if($lengthSet<=4) {
          $dataType = "tinyInt";
        }
        if($lengthSet>11) {
          $dataType = "bigInt";
        }
      }
      else {
        $lengthSet = !empty($lengthSet) ? $lengthSet : "10";
        $dataType  = "float";
        $lengthSet = $lengthSet.",2"; // or 10,4
      }
    }

    if(in_array($fieldType,$this->dateType)) {
      $dataType                      = "date";
      $maxLengthIsGivenFromMysqlAuto = true;
    }
    if(in_array($fieldType,$this->timeType)) {
      $dataType                      = "time";
      $maxLengthIsGivenFromMysqlAuto = true;
    }
    if(in_array($fieldType,$this->dateNtimeType)) {
      $dataType                      = "datetime";
      $maxLengthIsGivenFromMysqlAuto = true;
    }

    /*
        if(!empty($dataType)) {
          if(!empty($fieldName)) {
            $data['columns'][$fieldName]['dataType'] = $dataType;
          }
          else {
            $data = $dataType;
          }
        }*/

    return array($dataType,$lengthSet,$maxLengthIsGivenFromMysqlAuto);
  }
}

