<?php

/*************************************************************
 * Api controller class
 * - controller engine for the mvc
 * - extend this class on: \application\controllers\"className"Controller.php
 * @author    S M Kessner <limbercode@gmail.com>
 * @copyright 2010-2018 T2
 * @link      http://limbercode.com
 *************************************************************/
class apiController extends controllerWarehouse {

	private static $cfg;

	/**
	 * Constructor for controller
	 * @param string $model
	 * @param string $controller
	 * @param string $action
	 */
	public function __construct($model = "apiModel",$controller = "apiController",$action = "index") {

		header("Expires: Mon, 26 Jul 1990 05:00:00 GMT");
		header("Last-Modified: ".gmdate("D, d M Y H:i:s")." GMT");
		header("Cache-Control: no-store, no-cache, must-revalidate, max-age=-1");
		header("Cache-Control: post-check=0, pre-check=0",false);
		header("Pragma: no-cache");

		self::$cfg   = basic::loadCfg("apiConfig");    // set config
		$this->trans = basic::load("trans","core");
		$this->notif = basic::load("notif","core");

		// construct MVC
		$this->controller = $controller;
		$this->action     = $action;
		$this->init();

		// start construct models
		$this->model             = new $model($this->db);
		$this->model->controller = $this;
		$this->table             = $controller;

		$this->showAuthAlert();        // if we have alerts to show
	}

	/**
	 * Initialize the required classes and variables
	 */
	protected function init() {

		parent::init();
	}

	/**
	 * Show auth (sil) alert
	 */
	public function showAuthAlert() {

		parent::showAuthAlert();
	}    // calls this function on "template.class" - happens only if we use loadActionFromView

	/**
	 * Load classes from controller
	 * @param string $class - name
	 * @param string $directory - path under the labYard
	 * @example $this->load($class="lang",$directory="core");
	 */
	public function load($class,$directory = "") {

		parent::load($class,$directory);
	}

	/**
	 * Load translations from controller - load the text translations file
	 * @param string $textFile - file name to load under the path: text/translations
	 * @example $this->translations($textFile="sil");
	 */
	public function translations($textFile) {

		return parent::translations($textFile);
	}

	/**
	 * Translate text from controller - return the translation text in the controller files
	 * @param string $text - the text to be translated - written in the default language (en)
	 * @param string /array $externalVar - containing the values that will effect the translation
	 * @return string - the translation
	 * @example $this->T($text="text");
	 */
	public function T($text,$externalVar = "") {

		return parent::T($text,$externalVar);
	}

	/**
	 * Redirect to action
	 * @param string $action - action name
	 * @param boolean /string $controller - controller name
	 * @param array $params - parameters to be used
	 * @return array
	 */
	public function redirectToAction($action,$controller = false,$params = array()) {

		return parent::redirectToAction($action,$controller,$params);
	}

	/**
	 * Process default action view -  make the default action path
	 * @param array $params - parameters to be used
	 * @return string $this->view() - the page itself
	 */
	public function defaultAction($params = null) {

		return parent::defaultAction($params);       // dispatch the result
	}

	/**
	 * UnknownAction - feed 404 header to the client
	 */
	public function unknownAction() {

    if(!headers_sent()) {
      header("HTTP/1.0 404 Not Found");
    }

	  echo"Api - No route here..";
	  die();
	}

	/**
	 * View - returns the template result
	 * @return string - view output
	 */
	public function view() {

		return $this->view;        // dispatch the result of the template class
	}

	/**
	 * Load view from other location (from controller)
	 * @param string $path - path where the view file exists (under view directory)
   * @param string $sub - path where the view file exists (under which architecture)
	 * @return string - view output
	 * @example $this->loadView($path="sil/userPasswordResetInvalid");
	 */
  public function loadView($path,$sub = "architecture".DS."api") {

		return parent::loadView($path,$sub);            // dispatch the result
	}

	/**
	 * Set a variables into view from controller
	 * @param string $name - name of the variable
	 * @param string $value - value of the variable
	 * @example $this->set($name="timed",$value);        // will be $timed on view side
	 */
	public function set($name,$value) {        // set the parameters to the template class

		//    $this->view->set($name,$value);
	}

	/**
	 * Auth level needed to view specific page
	 * @param integer $num - the minimum auth level needed 0-5
	 * @example $this->authLevelNeeded(4);
	 */
	public function authLevelNeeded($num) {

		parent::authLevelNeeded($num);
	}

	/**
	 * Before action - the function that comes before all the functions in the controller class
	 */
	public function beforeAction() {

		parent::beforeAction();
	}

	/**
	 * After action - the function that comes before all the functions in the controller class
	 */
	public function afterAction() {

		parent::afterAction();
	}

	/**
	 * Return api data - returns the data with $cfgApi['defaultReturnType'] type
	 * @param string /array $data - the data to convert
	 * @example returnApi($data);
	 */
	public function returnApiData($data) {

		if(self::$cfg['api']['defaultReturnType']=="array") {
			$this->returnApiDataAsArray($data);
		}
		elseif(self::$cfg['api']['defaultReturnType']=="json") {
			$this->returnApiDataAsJson($data);
		}
		elseif(self::$cfg['api']['defaultReturnType']=="xml") {
			$this->returnApiDataAsXml($data);
		}
	}

	/**
	 * Return api data as array
	 * @param string /array $data - the data to convert
	 * @example returnApi($data);
	 */
	public function returnApiDataAsArray($data) {

		print_r($data);
	}

	/**
	 * Return api data as json
	 * @param string /array $data - the data to convert
	 * @example returnApi($data);
	 */
	public function returnApiDataAsJson($data) {

		header('content-type: application/json; charset=utf-8');

		//	$data = array_map("utf8_encode",$data);
		//	echo json_encode($data);
		echo json_encode($data,JSON_UNESCAPED_UNICODE);
	}

	/**
	 * Return api data as xml
	 * @param string /array $data - the data to convert
	 * @example returnApi($data);
	 */
	public function returnApiDataAsXml($data) {

		header('Content-type: text/xml; charset=utf-8');
		$this->xml = basic::load("xml","functions");
		echo $this->xml->arrayToXml($data,$head = "ApiData");
	}
}

