<?php

/*************************************************************
 * Controller warehouse class
 * - controller engine for the mvc
 * - extend this class on: \labYard\mvc\controllerWarehouse
 * @author    S M Kessner <limbercode@gmail.com>
 * @copyright 2010-2018 T2
 * @link      http://limbercode.com
 *************************************************************/
abstract class controllerWarehouse {

	// was private
	public $variables;
	public $cancelViewExecution;
	protected $canonical;
	protected $jsonLd;
	protected $db;
	protected $model;
	protected $advanced;
	protected $table;
	protected $trans;
	protected $notif;
	protected $authLevelNeeded;
	protected $doNotRenderPartOfTemplate;
	protected $currentTemplate;
	protected $callingClass;
	protected $callingFunction;

	// was protected
	protected $operatorsToActivate = array();
	protected $operatorsToUnset = array();
	protected $controller;
	protected $action;
	protected $view;
	protected $addJsIncludes;

	// operatorsController
	protected $addCssIncludes;
	protected $typeOfHeaderFooter;
	protected $operatorsModel;
	protected $architectureType = "application";

	/**
	 * Constructor for controller
	 * @param string $model
	 * @param string $controller
	 * @param string $action
   * @throws ReflectionException
	 */
	public function __construct($model = "model",$controller = "controller",$action = "index") {

		if($model=="faviconModel") {            // fix: if it's favicon we don't have to continue
			die();
		}

		$this->trans = basic::load("trans","core");
		$this->notif = basic::load("notif","core");

		// construct MVC
		$this->controller = $controller;
		$this->action     = $action;

		$this->init();
		$this->loadTemplate();

		// call the function for derived class
		$this->view->doNotRenderPartOfTemplate = $this->doNotRenderPartOfTemplate;    // if you wish to disable part of template
		$this->view->operatorsToActivate       = $this->operatorsToActivate;    // operators that must be activated
		$this->view->operatorsToUnset          = $this->operatorsToUnset;       // operators that must be removed
		//	$this->view->authLevel = $this->authLevel;		// we might not need this anymore

		$this->view->addJsIncludes  = $this->addJsIncludes;     // add js files from the controller init
		$this->view->addCssIncludes = $this->addCssIncludes;    // add css files from the controller init

		// start construct models
		$this->model             = new $model($this->db);
		$this->model->controller = $this;
		$this->table             = $controller;

	//	$this->advanced = basic::load("advanced","core");
		$this->advanced = new advanced();

    if(basic::getConfig("db","main","activate")==true) {
      $this->advanced->pdoHelper = basic::load("pdoHelper",'core/adapters');
      $this->advanced->installSystemTables();
    }

    $this->showAuthAlert();        // if we have alerts to show
	}

	/**
	 * Initialize the required classes and variables
	 */
	protected function init() {
	}

	/**
	 * Load template theme files and class
	 */
	public function loadTemplate() {

		if($this->currentTemplate) {
			$themeName = $this->currentTemplate;
		}
		else {
			$themeName = basic::$cfg['template']['theme'];
			$this->currentTemplate = $themeName;
		}

		$themeFile      = APPLICATIONPATH."themes".DS.$themeName.DS.$themeName.".theme.class.php";
		$themeClassName = $themeName."Theme";

		if(file_exists($themeFile)) {    // initialise the template class through the $themeClassName extension
			require_once($themeFile);
			$this->oldView = $this->view;
			$this->view = new $themeClassName($this->controller,$this->action,$this->architectureType);
			$this->view->currentTemplate = $themeName;

			if($this->oldView) {
				foreach($this->oldView as $key => $value) {
					if(!isset($this->view->$key)) {
						$this->view->$key = $value;
					}
				}
			}       // extracting the old theme vars into the new template - if any
		}
		else {
			$notif = $this->notif->genNotifByNum($file = "core",81130,$themeName);
			echo $notif;
			die();
		}
	}

	/**
	 * Set template theme - will be used in current session
	 * @param string $name - name of the template theme
	 * @example $this->setTemplate($name="most");
	 */
	public function setTemplate($name) {

		if($this->currentTemplate!=$name) {
			$this->currentTemplate = $name;
			$this->loadTemplate();
		}
	}

	/**
	 * Show auth (sil) alert
	 */
	public function showAuthAlert() {

		if(basic::$cfg['auth']['allowNotifs']==1) {
			$notifNum = Auth::catchAuthAlert($CatchableAlerts = "",$unCatchableAlerts = array(
				7004,
				7008,
				7011,
				7012,
				7024,
				7025
			));                        // catch auth alerts		// alerts that should be catch only by login controller
			if(is_array($notifNum)) {
				$externalVar = "";
				if(in_array(7014,$notifNum)) {
					$timeSignature = "?&t=".time();
					$externalVar   = array(".",$timeSignature);
				}

        $this->notif->genNotifByNum($file = "auth",$notifNum,$externalVar);    // issue alerts
			}
		}    // it's here to show "redirecting to page..."
	}

	/**
	 * Load classes from controller
	 * @param string $class - name
	 * @param string $directory - path under the labYard
	 * @example $this->load($class="lang",$directory="core");
   * @throws ReflectionException
	 */
	public function load($class,$directory = "") {

		$this->$class =& basic::load($class,$directory);
	}

	/**
	 * Load translations from controller - load the text translations file
	 * @param string $textFile - file name to load under the path: text/translations
	 * @example $this->translations($textFile="sil");
   * @return mixed
	 */
	public function translations($textFile) {

		return $this->trans->loadTrans($textFile);
	}

	/**
	 * Translate text from controller - return the translation text in the controller files
	 * @param string $text - the text to be translated - written in the default language (en)
	 * @param string /array $externalVar - containing the values that will effect the translation
	 * @return string - the translation
	 * @example $this->T($text="text");
	 */
	public function T($text,$externalVar = "") {

		return $this->trans->extractTrans($text,$externalVar);
	}

	/**
	 * Add Js includes from controller init/action - add the Js files to the final Html page
	 * @param string|array $file - file name - can be inside/outsource url
	 * @example $this->addJsIncludes = array($file="js/auth/login-register.js");
	 */
	public function addJsIncludes($file) {

		if(!is_array($file)) {
			$file = array($file);
		}    // can be string also

		if(!empty($this->addJsIncludes)) {
			$this->addJsIncludes = array_merge($this->addJsIncludes,$file);
		}
		else {
			$this->addJsIncludes = $file;
		}

		if(isset($this->view)) {
			$this->view->addJsIncludes = $this->addJsIncludes;
		}
	}

	/**
	 * Add Css includes from controller init/action - add the Css files to the final Html page
	 * @param string|array $file - file name - can be inside/outsource url
	 * @example $this->addCssIncludes($file="themes/".$this->currentTemplate."/css/auth.css");
	 */
	public function addCssIncludes($file) {

		if(!is_array($file)) {
			$file = array($file);
		}    // can be string also

		if(!empty($this->addCssIncludes)) {
			$this->addCssIncludes = array_merge($this->addCssIncludes,$file);
		}
		else {
			$this->addCssIncludes = $file;
		}

		if(isset($this->view)) {
			$this->view->addCssIncludes = $this->addCssIncludes;
		}
	}

	/**
	 * Redirect to action or to controller's private method
	 * @param string $action - action name
	 * @param boolean/string $controller - controller name
	 * @param array $params - parameters to be used
	 * @return array
	 * @example: $this->redirectToAction($action="checkIfFileCanBeDownloaded",$controller = "download",$params = array("222"));   // from other controller class
	 */
	public function redirectToAction($action,$controller = false,$params = array()) {

		$controllerNamer = $controller."Controller";
		if($controller===false) {
			$controller = get_called_class();
		}
		else if(is_string($controller) && class_exists($controller.'Controller')) {
			$modelNamer = $controller."Model";

			if(class_exists($modelNamer)) {
				$controller = new $controllerNamer($modelNamer,$controller,$action);
			}
			else {
				$controller = new $controllerNamer();
			}
		}

		$hasActionFunctionAccess = (int)is_callable(array(
			$controllerNamer,
			$action
		));     // check if class is private

		if($hasActionFunctionAccess) {
			return call_user_func_array(array($controller,$action),$params);
		}
		else {
			if(method_exists($controllerNamer,$action)) {
				return $this->callPrivateMethod($controller,$action,$params);
			}
			else {
        $this->notif = basic::load("notif","core");
				$this->notif->genNotifByNum($file = "core",81710,array($action,$controllerNamer));
			}
		}
	}

	/**
	 * Call private method
	 * @param $controller
	 * @param $methodName
	 * @param $params
	 * @return mixed
   * @throws ReflectionException
	 */
	private function callPrivateMethod($controller,$methodName,$params) {

		$reflectionClass = new \ReflectionClass($controller);
		$reflectionMethod = $reflectionClass->getMethod($methodName);
		$reflectionMethod->setAccessible(true);

		return $reflectionMethod->invokeArgs($controller,$params);
	}

	/**
	 * Process default action view - make the default action path
	 * @param array $params - parameters to be used
	 * @return string $this->view() - the page itself
	 */
	public function defaultAction($params = null) {

	  $defaultLang = lang::getDefaultLanguage();
		$path = myHelpers::UrlContent("~/views/{$this->controller}/{$this->action}.php");
    $pathWithDefaultLang = myHelpers::UrlContent("~/views/{$this->controller}/{$defaultLang}/{$this->action}.php");

    if(file_exists($pathWithDefaultLang) && $defaultLang!="en") {                      // if we have action name and it's valid
      $this->view->viewPath = $pathWithDefaultLang;
    }
		elseif(file_exists($path)) {                      // if we have action name and it's valid
			$this->view->viewPath = $path;
		}
		else {                                        // if action is not valid
			$this->unknownAction();
		}

		if(!empty($params) && is_array($params)) {    // if we have parameters
			foreach($params as $key => $value) {      // assign local variables
				$this->view->set($key,$value);
			}
		}

		return $this->view();                         // dispatch the result
	}

	/**
	 * UnknownAction - feed 404 header to the client
	 */
	public function unknownAction() {

		if(!headers_sent()) {
			header("HTTP/1.0 404 Not Found");
		}

		$path = myHelpers::UrlContent("~/views/shared/_404.php");    // find custom 404 page

		if(file_exists($path)) {                                    // if we have custom 404 page, then use it
			$this->view->viewPath = $path;

			return $this->view();
		}
		else {
			exit;                                                    // do not do any more work in this script.
		}
	}

	/**
	 * View - returns the template result
	 * @return string - view output
	 */
	public function view() {

		//	$this->view->authLevel = $this->authLevel;
		$this->view->doNotRenderPartOfTemplate = $this->doNotRenderPartOfTemplate;        // to deploy it on actions

		if($this->canonical) {
			$this->view->canonical = $this->canonical;
		}
		else {
			$this->view->canonical = $this->view->canonical();
		}

    if($this->jsonLd) {
      $this->view->jsonLd = json_encode($this->jsonLd,JSON_UNESCAPED_UNICODE|JSON_UNESCAPED_SLASHES);

      if(!$this->view->jsonLd) {
        $this->notif->genNotifByNum($file = "core",82150,array($this->callingClass,$this->callingFunction));
      }
    }
    else {
      $this->view->jsonLd = json_encode($this->view->jsonLd(),JSON_UNESCAPED_UNICODE|JSON_UNESCAPED_SLASHES);
    }

		return $this->view;        // dispatch the result of the template class
	}

	/**
	 * Load view from other location (from controller)
	 * @param string $path - path where the view file exists (under view directory)
   * @param string $sub - path where the view file exists (under which architecture) - if we want to use advanced we put: "architecture".DS."advanced"
	 * @return string - view output
	 * @example $this->loadView($path="sil/userPasswordResetInvalid");
	 */
	public function loadView($path,$sub = "") {

    $sub = !empty($sub) ? $sub : basic::$cfg['site']['directory']['application'];

		$originalPath = $languagePath = $path;
		if(lang::getCurrentLanguageForUrls()) {
			$languagePath = stringManipulation::firstReplace($search = "/",$replace = "/".lang::getCurrentLanguageForUrls(),$languagePath);
			$languagePath = stringManipulation::firstReplace($search = DS,$replace = DS.lang::getCurrentLanguageForUrls(),$languagePath);
		}

		$path         = myHelpers::UrlContent("~/views/{$path}.php",$sub);
		$languagePath = myHelpers::UrlContent("~/views/{$languagePath}.php",$sub);

		if(file_exists($languagePath)) {            // if view is valid in other language
			$this->view->viewPath = $languagePath;
		}
		elseif(file_exists($path)) {                // if view is valid
			$this->view->viewPath = $path;
		}
		else {                                      // if view is not valid
			$this->unknownAction();
			$this->notif->genNotifByNum($file = "core",82140,addslashes($path.".php"));
		}

		return $this->view();
	}

	/**
	 * Load action from view actually happens - including the view file
	 * @param string $controller - the controller name
	 * @param string $action - the action name
	 * @example $this->loadActionFromView("sil","index");    // from view/template  // like in "logout" that adds "login" view
	 */
	public function loadActionFromView($controller,$action) {

		$path            = myHelpers::UrlContent("~/views/");
		$controllerNamer = $controller."Controller";

		$hasActionFunction       = (int)method_exists($controllerNamer,$action);    // check if class exists
		$hasActionFunctionAccess = (int)is_callable(array(
			$controllerNamer,
			$action
		));    // check if Action is not private

		if($hasActionFunction && $hasActionFunctionAccess) {
			$attachController = new $controllerNamer;
			$attachAction     = $attachController->$action();                       // loading parameters into memory

			foreach($attachController->view as $key => $value) {
				if(!isset($this->$key)) {
					$this->$key = $attachController->view->$key;
				}    // only if current action does not have the object parameter - bring it from the attached action
			}

			if(file_exists($path.$controller.DS.$action.".php")) {
				include($path.$controller.DS.$action.".php");
			}
			else {
				$this->notif->genNotifByNum($file = "core",81120,$action,$controller);
			}
		}
		else {
			$this->notif->genNotifByNum($file = "core",81140,$action,$controller);
		}
	}    // calls this function on "template.class" - happens only if we use loadActionFromView

	/**
	 * Include fragment file from fragments directory
	 * @param string $path - path to the fragment
	 * @param string $dir - path to the fragment (under which architecture)
	 * @param string $architectureType - under which architecture - application/advanced
   * @param bool $renderAtOnce - if true will be rendered immediately
   * @return mixed
	 * @example $this->includeFragment("themes/rim/fragments/navBar");        // from view
	 */
	public function includeFragment($path,$dir="",$architectureType="application",$renderAtOnce=false) {

    $dir = !empty($dir) ? $dir : APPLICATIONPATH;
    $this->architectureType = $architectureType;

		$themeFile      = APPLICATIONPATH."themes".DS.$this->currentTemplate.DS.$this->currentTemplate.".theme.class.php";
		$themeClassName = $this->currentTemplate."Theme";

		if(file_exists($themeFile)) {    // initialise the template class
			$include = new $themeClassName($this->controller,$this->action,$this->architectureType);

			foreach(get_object_vars($this) as $key => $value) {
				if($key!='controller' && $key!='action' && $key!='architectureType') {
					$include->$key = $value;
				}
			}    // convert the parameters to $include

      if($this->view) {
        foreach($this->view as $key => $value) {
          if(!isset($this->$key)) {
            $include->$key = $value;
          }
        }
      }    // extracting the view vars into the template - if any

			return $include->includeFragment($path,$dir,$renderAtOnce);
		}
		else {
			$notif = $this->notif->genNotifByNum($file = "core",81130,$this->currentTemplate);
			echo $notif;
			die();
		}
	}

	/**
	 * Auth level needed to view specific page
	 * @param integer $num - the minimum auth level needed 0-5
	 * @example $this->authLevelNeeded(4);
   * @throws ReflectionException
	 */
	public function authLevelNeeded($num) {

		global $auth;

		if($num>=1) {
			if($auth) {        // if site is basic::$cfg['auth']['restrictedSite'] we already have $auth
				if(basic::$cfg['auth']['restrictedSite']<$num) {
					$auth->authLevelNeeded = $num;
					$auth->checkAuthLevel();
				}
			}
			else {            // if site is not basic::$cfg['auth']['restrictedSite']
				//	if(basic::$cfg['auth']['restrictedSite']==false) {		// if site is not basic::$cfg['auth']['restrictedSite']
				$auth = basic::load("auth","core");
			}
		}
	}

	/**
	 * Before action - the function that comes before all the functions in the controller class
	 */
	public function beforeAction() {
	}

	/**
	 * After action - the function that comes before all the functions in the controller class
	 */
	public function afterAction() {

		session_write_close();
	}

	/**
	 * Set canonical
	 * @param string $address - set the canonical address for the page/controller
	 * @example $this->setCanonical("ksjbfsjkdfjgsbvjsfsd.html"); from controller
	 */
	public function setCanonical($address) {

		$this->canonical = $address;
	}

	/**
   * Set JSON for Linking Data
   * @param array $pack - set the JSON-LD for the page/controller
   * @example $this->setJsonLd(array); from controller
   */
	public function setJsonLd($pack) {

		$this->jsonLd = $pack;
	}

	/**
	 * Set do not render part of template
	 * @param string /array $set - set the parts (themePlans) that we want to remove from page
	 * @example $this->setDoNotRenderPartOfTemplate(array("header","footer")); from controller
	 */
	public function setDoNotRenderPartOfTemplate($set) {

		$this->doNotRenderPartOfTemplate = $set;
	}

	/**
	 * Set a variables into view from controller
	 * @param string $name - name of the variable
	 * @param string $value - value of the variable
	 * @example $this->set($name="timed",$value);        // will be $timed on view side
	 */
	public function set($name,$value) {        // set the parameters to the template class

		$this->view->set($name,$value);
	}

	/**
	 * Unset operator from controller
	 * @param string $toUnset - the operator file to unset, the operator will not show on that page/controller
	 * @example $this->unsetOperator();                    // * empty value will unset all operators
	 * @example $this->unsetOperator("serverLoad");        // unset by file name
	 */
	public function unsetOperator($toUnset = "") {

		if(!$toUnset) {
			$this->operatorsToUnset[] = "all";
		}    // remove all operators from page
		else {
			$this->operatorsToUnset[] = $toUnset;
		}    // remove operators from page - by file name
	}

	/**
	 * Set operators to activate
	 * @param array $set - set the operators to activate on page/controller
	 * @example $this->setOperatorsToActivate(array('default','login','login888'));
	 * @example $this->setOperatorsToActivate(array(
	 *                    'position'=>'right',
	 *                    'file'=>'unique',
	 *                    'title'=>'unique999',
	 *                    'allowMoreThenOnce'=>2,
	 *                    'mustShow'=>1,
	 *                    ));
	 */
	public function setOperatorsToActivate(array $set) {

		$this->operatorsToActivate['operators'][] = $set;
	}
}

