<?php

/*************************************************************
 * Model warehouse class
 * - controller engine for the mvc
 * - extend this class on: \labYard\mvc\modelWarehouse
 * @author    S M Kessner <limbercode@gmail.com>
 * @copyright 2010-2018 T2
 * @link      http://limbercode.com
 *************************************************************/
class modelWarehouse {

	public $controller;
	protected $model;
	protected $table;
	protected $db;
	protected $notif;

	/**
	 * Constructor for model
	 */
	public function __construct() {

	  if(basic::getConfig("db","main","activate")==true) {
      $this->db     = basic::load(basic::getConfig("db","main","adapter"),'core/adapters');
      $this->model  = get_class($this);
      $defaultModel = ($this->model=='Model');

      if(!$defaultModel) {
        $this->table = preg_replace('/Model$/','',$this->model); // remove ending Model
      }

      $this->init();
    }
	  else {
      $notif = basic::load("notif","core");
      $notif->genNotifByNum($file = "core",$notifNum = 84200);

	    return false;
    }
	}

	/**
	 * Initialize the required classes and variables
	 */
	protected function init() {
	}

	/**
	 * Load data from file - the file must contain $data as array, can load in other languages too
	 * @param $name - the directory and file name (without extension)
	 * @return mixed
	 * @example - $this->loadData($name="documentationData/texter/url");
	 */
	public function loadData($name) {

		$nameInOtherLanguage = "";
		if(lang::getCurrentLanguage()!=lang::getDefaultLanguage()) {
			$nameStartPos        = strrpos($name,DS);
			$nameStart           = substr($name,0,$nameStartPos);
			$nameEnd             = substr($name,$nameStartPos + 1);
			$nameInOtherLanguage = $nameStart.DS.lang::getCurrentLanguage().DS.$nameEnd;
		}

		$pathToData = SOURCEPATH.DS."data".DS;

		if($nameInOtherLanguage && file_exists($pathToData.$nameInOtherLanguage.".php")) {
			include($pathToData.$nameInOtherLanguage.".php");
		}
		elseif(file_exists($pathToData.$name.".php")) {
			include($pathToData.$name.".php");
		}

		if(isset($data)) {
			return $data;
		}           // from the file we include
		else return false;
	}

  /**
   * Redirect to method or to model's private method
   * @param string $method - method name
   * @param boolean/string $model - model name
   * @param array $params - parameters to be used
   * @return false|mixed
   * @throws ReflectionException
   * @example: $this->redirectToModelMethod($method="items",$model = "download",$params = array("222")); // from other model class
   */
  public function redirectToModelMethod($method,$model = false,$params = array()) {

    $modelNamer = $model."Model";
    if($model===false) {
      $model = get_called_class();
    }
    else if(is_string($model) && class_exists($model.'Model')) {
      $modelNamer = $model."Model";

      if(class_exists($modelNamer)) {
        $model = new $modelNamer($modelNamer,$model,$method);
      }
      else {
        $model = new $modelNamer();
      }
    }

    $hasActionFunctionAccess = (int)is_callable(array(
      $modelNamer,
      $method
    ));     // check if class is private

    if($hasActionFunctionAccess) {
      return call_user_func_array(array($model,$method),$params);
    }
    else {
      if(method_exists($modelNamer,$method)) {
        return $this->callPrivateMethod($model,$method,$params);
      }
      else {
      //  echo"Method \"$method\" does not exists on \"$modelNamer\" model";
        $this->notif = basic::load("notif","core");
        $this->notif->genNotifByNum($file = "core",81710,array($method,$modelNamer));
      }
    }
  }

  /**
   * Call private method
   * @param $controller
   * @param $methodName
   * @param $params
   * @return mixed
   * @throws ReflectionException
   */
  private function callPrivateMethod($controller,$methodName,$params) {

    $reflectionClass = new \ReflectionClass($controller);
    $reflectionMethod = $reflectionClass->getMethod($methodName);
    $reflectionMethod->setAccessible(true);

    return $reflectionMethod->invokeArgs($controller,$params);
  }
}

