<?php

/*************************************************************
 * Operators controller class
 * - controller engine for the operators mvc
 * - extend this class on: \operators\controllers\"className"Controller.php
 * @author    S M Kessner <limbercode@gmail.com>
 * @copyright 2010-2018 T2
 * @link      http://limbercode.com
 *************************************************************/
class operatorsController extends controllerWarehouse {

	public $addCssIncludes;
	public $addJsIncludes;
	public $variables;
	public $view;

	/**
	 * Constructor for operators controller
	 * @param object $operatorsModel - the operator model
	 */
	public function __construct($operatorsModel) {

		$this->view           = new stdClass();
		$this->operatorsModel = $operatorsModel;
		$this->translations("operators");

		$this->init();
	}

	/**
	 * Load translations from controller - load the text translations file
	 * @param string $textFile - file name to load under the path: text/translations
	 * @example $this->translations($textFile="sil");
	 */
	public function translations($textFile,$directory = "") {        // load the text translations file

		$this->load("trans","core");
		parent::translations($textFile);
	}

	/**
	 * Load classes from controller
	 * @param string $class - name
	 * @param string $directory - path under the labYard
	 * @example $this->load($class="lang",$directory="core");
	 */
	public function load($class,$directory = "") {

		parent::load($class,$directory);
	}

	/**
	 * Initialize the required classes and variables
	 */
	protected function init() {

		parent::init();
	}

	/**
	 * Add Js includes from controller init/action - add the Js files to the final Html page
	 * @param string /array $file - file name - can be inside/outsource url
	 * @example $this->addJsIncludes = array($file="js/auth/login-register.js");
	 */
	public function addJsIncludes($file) {    // add js files to load - from controller init/action

		parent::addJsIncludes($file);
	}

	/**
	 * Add Css includes from controller init/action - add the Css files to the final Html page
	 * @param string /array $file - file name - can be inside/outsource url
	 * @example $this->addCssIncludes($file="themes/".$this->currentTemplate."/css/auth.css");
	 */
	public function addCssIncludes($file) {    // add css files to load - from controller init/action

		parent::addCssIncludes($file);
	}

	/**
	 * set the variables for operators
	 * @param $name
	 * @param $value
	 */
	public function set($name,$value) {        // set the parameters to the template class

		$this->variables[$name] = $value;
	}

	/**
	 * Cancel view execution
	 */
	function cancelViewExecution() {

		$this->cancelViewExecution = true;
	}

	/**
	 * Unset operator from controller
	 * @param string $toUnset - the operator file to unset, the operator will not show on that page/controller
	 * @example $this->unsetOperator();                    // * empty value will unset all operators
	 * @example $this->unsetOperator("serverLoad");        // unset by file name
	 */
	public function unsetOperator($toUnset = "") {

		parent::unsetOperator($toUnset);
	}

	/**
	 * Set operators to activate
	 * @param array $set - set the operators to activate on page/controller
	 * @example $this->setOperatorsToActivate(array('default','login','login888'));
	 * @example $this->setOperatorsToActivate(array(
	 *                    'position'=>'right',
	 *                    'file'=>'unique',
	 *                    'title'=>'unique999',
	 *                    'allowMoreThenOnce'=>2,
	 *                    'mustShow'=>1,
	 *                    ));
	 */
	public function setOperatorsToActivate(array $set) {

		parent::setOperatorsToActivate($set);
	}
}

