<?php

/*************************************************************
 * View template operators class
 * - template engine for the operators mvc
 * - loading the view files from: \application\views\"viewName".php
 * @author    S M Kessner <limbercode@gmail.com>
 * @copyright 2010-2018 T2
 * @link      http://limbercode.com
 *************************************************************/
class operatorsView {

	/**
	 * Constructor for template
	 * @param string $operatorName - the operator name
	 * @param object operatorsController $operatorsController
	 */
	public function __construct($operatorName,operatorsController $operatorsController/*, operatorsModel $operatorsModel*/) {

		$this->trans = basic::load('trans','core');
		$this->notif = basic::load("notif","core");

		if($operatorsController->cancelViewExecution==false) {    // cancelViewExecution if true
			$this->broadcast($operatorName,$operatorsController);
		}
	}

	/**
	 * Broadcast the operator
	 * @param string $operatorName - the operator name
	 * @param object operatorsController $operatorsController
	 */
	public function broadcast($operatorName,operatorsController $operatorsController) {

		foreach($operatorsController->view as $key => $value) {
			if(!isset($this->$key)) {
				$this->$key = $operatorsController->view->$key;
			}    // only if current action does not have the object parameter - bring it from the attached action
		}

		if(isset($operatorsController->variables)) {
			extract($operatorsController->variables);        // set the parameters to the template class
		}

		$operatorViewFile = ROOT.DS.basic::$cfg['site']['directory']['operators'].DS."views".DS.$operatorName.".php";
		require($operatorViewFile);        // do not "require_once" since we can have operators that loaded twice
	}

	/**
	 * Translate text from operator controller - return the translation text in the operator controller files
	 * @param string $text - the text to be translated - written in the default language (en)
	 * @param string /array $externalVar - containing the values that will effect the translation
	 * @return string - the translation
	 * @example $this->T($text="text");
	 */
	public function T($text,$externalVar = "") {        // load the translation text in the controller files

		return $this->trans->extractTrans($text,$externalVar);
	}

	/**
	 * Include fragment file from fragments directory
	 * @param string $path - path to the fragment
	 * @example $this->includeFragment("themes/rim/fragments/navBar");        // from view
	 */
	public function includeFragment($path) {

		if(strpos($path,'/')===false) {
			$pathWithLanguage = $this->controller.DS."fragments".DS.lang::getCurrentLanguage()."/".$path;
			$path             = $this->controller."/"."fragments"."/".$path;
		}    // if we did not contain the controller name in $path
		else {
			$pathWithLanguage = stringManipulation::lastlReplace("/","/".lang::getCurrentLanguage()."/",$path);
		}

		$finalPathWithLanguage = APPLICATIONPATH.$pathWithLanguage.".php";
		$finalPath             = APPLICATIONPATH.$path.".php";

		if(lang::getCurrentLanguage() && lang::getCurrentLanguage()!=lang::getDefaultLanguage()) {
			if(file_exists($finalPathWithLanguage)) {
				include($finalPathWithLanguage);
			}
			else {
				if(file_exists($finalPath)) {
					include($finalPath);
				}
				else {
					$this->notif->genNotifByNum($file = "core",$num = 81290,$externalVar = array(
						$path,
						$this->controller
					));
				}
			}
		}
		else {
			if(file_exists($finalPath)) {
				include($finalPath);
			}
			else {
				$this->notif->genNotifByNum($file = "core",$num = 81290,$externalVar = array($path,$this->controller));
			}
		}
	}
}

