<?php

/*************************************************************
 * Router class
 * - create route for the mvc site
 * - using routing config
 * - full multi languages (support google language uri)
 * - removes duplicated url's
 * @author    S M Kessner <limbercode@gmail.com>
 * @copyright 2010-2018 T2
 * @link      http://limbercode.com
 *************************************************************/
class router {

	public static $protocol;
	public static $domain;
	public static $subDomain;
	public static $path;
	public static $breaker;
	public static $localHost;              // in case we are in localHost server
	public static $localHostMain;          // in case we are in localHost server
	public static $localHostNextDir;       // in case we are in localHost server
	public static $allParams;
	public static $isPost = null;
	private static $cfg;
	protected $controller;
	protected $action;
	protected $view;
	protected $architectureType;
	protected $params;
	protected $route;
	protected $redirects;
	protected $language;
	protected $inPath;

	/**
	 * Constructor for router
	 * @param string $route - the url + query string
	 */
	public function __construct($route) {

		$this->redirects  = array();
		$this->route      = $route;
		$this->controller = "Controller";
		$this->action     = "index";
		$this->params     = array();
		//	$this->params     = new stdClass();
		$this->view = false;        // the initial view

		self::$cfg = basic::loadCfg("langConfig",false) + basic::loadCfg("routerConfig",false);

		if(self::$cfg['router']['preferredProtocol']) {
			$this->preferredProtocol();
		}
		if(self::$cfg['router']['removeUnnecessary']['www']) {
			$this->removeWww();
		}
		if(self::$cfg['router']['removeUnnecessary']['slashes']) {
			$this->removeSlashes();
		}
		if(self::$cfg['router']['removeUnnecessary']['public']) {
			$this->removePublic();
		}

		$this->whatIsLocalhostPath();
		$this->parseRoute();                                                    // call to parse routes
	}

	/**
	 * Is localhost server?
	 * @return bool
	 */
	public static function isLocalhost() {

		$checkList = array('127.0.0.1','::1');
		if(in_array($_SERVER['REMOTE_ADDR'],$checkList) && $_SERVER['SERVER_NAME']=="localhost") {
			return true;
		}
	}

	/**
	 * What is localhost path - finds out what is the localhost path
	 */
	private function whatIsLocalhostPath() {

		if(router::isLocalhost()) {
			router::$localHostMain    = $_SERVER['SERVER_NAME'];
			router::$localHostNextDir = stringManipulation::stringBefore($_SERVER['SCRIPT_FILENAME'],"public");
			router::$localHostNextDir = trim(router::$localHostNextDir,"/");
			router::$localHostNextDir = substr(strrchr(router::$localHostNextDir,'/'),0);

			if(router::$localHostNextDir) {
				router::$localHost = router::$localHostMain.router::$localHostNextDir;
			}
		}
	}

	/**
	 * Is secure - checks if we have "https" on url
	 * @return bool
	 */
	private function isSecure() {

		if((!empty($_SERVER['HTTPS']) && $_SERVER['HTTPS']!=='off') || (!empty($_SERVER['HTTP_X_FORWARDED_PROTO']) && $_SERVER['HTTP_X_FORWARDED_PROTO']=='https') || (!empty($_SERVER['HTTP_X_FORWARDED_SSL']) && $_SERVER['HTTP_X_FORWARDED_SSL']=='on') || (isset($_SERVER['SERVER_PORT']) && $_SERVER['SERVER_PORT']==443) || (isset($_SERVER['HTTP_X_FORWARDED_PORT']) && $_SERVER['HTTP_X_FORWARDED_PORT']==443) || (isset($_SERVER['REQUEST_SCHEME']) && $_SERVER['REQUEST_SCHEME']=='https')) {
			return true;
		}
		else {
			return false;
		}
	}

	/**
	 * Set preferred protocol - will redirect to the preferred protocol
	 */
	private function preferredProtocol() {

		if(self::$cfg['router']['preferredProtocol']=="http" && $this->isSecure()) {
			header("HTTP/1.1 301 Moved Permanently");
			Header("Location: http://".$_SERVER['HTTP_HOST'].$_SERVER['REQUEST_URI']);
			die();
		}
		elseif(self::$cfg['router']['preferredProtocol']=="https" && !$this->isSecure()) {
			header("HTTP/1.1 301 Moved Permanently");
			Header("Location: https://".$_SERVER['HTTP_HOST'].$_SERVER['REQUEST_URI']);
			die();
		}
	}

	/**
	 * Remove www - of uri - remove "www" from the start of the url, like: http://www.example.com/public/walla => http://example.com/walla
	 */
	private function removeWww() {

		if(strstr($_SERVER['HTTP_HOST'],"www.")) {
			$removeWww = substr($_SERVER['HTTP_HOST'],4);
			$protocol  = isset($_SERVER["HTTPS"]) ? "https" : "http";

			header("HTTP/1.1 301 Moved Permanently");
			Header("Location: ".$protocol."://".$removeWww.$_SERVER['REQUEST_URI']);
			die();
		}
	}

	/**
	 * Remove slashes - of uri - remove double slashes from url, like: http://example.com///walla => http://example.com/walla
	 */
	private function removeSlashes() {

		if(!router::isLocalhost() && substr_count($_SERVER['REQUEST_URI'],"/")>1 && substr($_SERVER['REQUEST_URI'],-1)=="/") {
			$removeSlash = trim($_SERVER['REQUEST_URI'],"/");

			header("HTTP/1.1 301 Moved Permanently");
			Header("Location: /".$removeSlash);
			die();
		}   // if we have "/" after url without it being only the domain

		if(strpos($_SERVER['REQUEST_URI'],"//")!==false) {
			$removeSlashes = preg_replace('#/+#','/',$_SERVER['REQUEST_URI']);

			header("HTTP/1.1 301 Moved Permanently");
			Header("Location: ".$removeSlashes);
			die();
		}   // if we have 2 "/"
	}

	/**
	 * Remove public - of uri - remove "public" from the url, like: http://example.com/public/walla => http://example.com/walla
	 */
	private function removePublic() {

		if(strpos($_SERVER['REQUEST_URI'],"/public/")!==false) {
			$removePublicDir = str_replace("/public/","/",$_SERVER['REQUEST_URI']);

			header("HTTP/1.1 301 Moved Permanently");
			Header("Location: ".$removePublicDir);
			die();
		}
	}

	/**
	 * Parse route - redirect to the appropriate action
	 */
	private function parseRoute() {

		if(isset($this->route)) {                   // parse path info
			if(substr($this->route,-2,1)=="/") {    // making route to be consistent in case that url ends with "/"
				$this->route = substr($this->route,0,-2)."?";
			}

			$this->inPath = $this->route;             // the request path
			$this->inPath = strtok($this->inPath,'?');  // remove query string

			if(self::$cfg['lang']['setBy']=="url") {
				$this->getLanguageFromUrl();

				if(self::$cfg['router']['removeUnnecessary']['defaultLang']) {
					if($this->language==lang::getDefaultLanguage()) {
						if(self::$cfg['lang']['route']=="inPath") {
							$removeDefaultLanguageDir = str_replace("/".$this->language."/","/",$_SERVER['REQUEST_URI']);
							if("/".$this->language==$_SERVER['REQUEST_URI']) {
								$removeDefaultLanguageDir = "/";
							}    // prevent crash
						}
						if(self::$cfg['lang']['route']=="inDomain") {
							$removeDefaultLanguageDir = str_replace($this->language.".","",$_SERVER['SERVER_NAME']);
							$removeDefaultLanguageDir = self::$protocol.$removeDefaultLanguageDir."/".$_SERVER['REQUEST_URI'];
						}    // prevent crash

						header("HTTP/1.1 301 Moved Permanently");
						Header("Location: ".$removeDefaultLanguageDir);
						die();
					}    // removes "en" from the url (the default language)
				}
			}

			if(stringManipulation::startsWith($haystack = $this->inPath,$needle = "advanced")) {
        $this->architectureType = "advanced";
				$this->inPath  = stringManipulation::firstReplace($search = "advanced/",$replace = "",$subject = $this->inPath);
			}
      if(stringManipulation::startsWith($haystack = $this->inPath,$needle = "advancedBlueprints")) {
        $this->architectureType = "advancedBlueprints";
        $this->inPath  = stringManipulation::firstReplace($search = "advancedBlueprints/",$replace = "",$subject = $this->inPath);
      }
      if(stringManipulation::startsWith($haystack = $this->inPath,$needle = "advancedEntries")) {
        $this->architectureType = "advancedEntries";
        $this->inPath  = stringManipulation::firstReplace($search = "advancedEntries/",$replace = "",$subject = $this->inPath);
      }
			elseif(stringManipulation::startsWith($haystack = $this->inPath,$needle = "api")) {
        $this->architectureType = "api";
				$this->inPath = stringManipulation::firstReplace($search = "api/",$replace = "",$subject = $this->inPath);
			}

			$paths = explode("/",$this->inPath);

			$params = array();
			if(empty($this->inPath)) {      // if this is home page route
				$this->controller = "index";
				$this->action     = "index";
			}
			else {
				if(isset($paths[0])) {
					$this->controller = $paths[0];
				}
				if(isset($paths[1])) {
					$this->action = $paths[1];
				}
				foreach($paths as $key => $value) {
				  $position = 1;
          if($this->architectureType=="advanced" || $this->architectureType=="advancedBlueprints" || $this->architectureType=="advancedEntries") {
            $position = 0;
          }

					if($key>$position) {
						$params[$key - ($position+1)] = $paths[$key];
					}
				}
			}

			$query = array();                                   // get query string from url
			$parse = parse_url($this->route);

			if(!empty($parse['query'])) {                       // if we have query string
				parse_str($parse['query'],$query);              // parse query string
				if(!empty($query)) {                            // if query parameter is parsed
					$_GET     = array_merge($_GET,$query);      // merge the query paramaters to $_GET variables
					$_REQUEST = array_merge($_REQUEST,$query);  // merge the query parameter to $_REQUEST variables
				}
			}
		}

		$method = $_SERVER['REQUEST_METHOD'];    // gets the request method

		if(!empty($params)) {                    // set param id to the id we have
			foreach($params as $key => $value) {
				$this->params['GET']["param".$key] = $value;
				//	$this->params->GET->$key = $value;
			}
		}

		$queryString = $_SERVER['QUERY_STRING'];
		parse_str($queryString,$queryParams);

		if(!empty($queryParams)) {      // get the $_GET from the query string // it will get it no matter what is the send $method
			foreach($queryParams as $key => $value) {
				//	$this->params[$key] = $value;
				$this->params['GET'][$key] = $value;
			}
		}

		switch($method) {       // assign params by methods
			case "GET":         // view
				unset($_GET['route']);            // we need to remove _route in the $_GET params

				foreach($_GET as $key => $value) {
					$this->params['GET'][$key] = $value;
				}
				break;
			case "POST":        // create
			case "PUT":         // update
			case "DELETE": {    // delete
				if(!array_key_exists("HTTP_X_FILE_NAME",$_SERVER)) {        // ignore the file upload
					if($method=="POST") {
						foreach($_POST as $key => $value) {
							$this->params['POST'][$key] = $value;
						}
					}
					else {
						$p       = array();                                 // temp params
						$content = file_get_contents("php://input");        // the request payload
						parse_str($content,$p);            // parse the content string to check we have [data] field or not
						if(!array($p)) {
							$p = json_decode($content,true);         // if we have data field
						}

						foreach($p as $key => $value) {
							//	$this->params->$method->$key = $value;
							$this->params[$method][$key] = $value;
						}
					}
				}
			}
				break;
			default: {
				$p       = array();                                 // temp params
				$content = file_get_contents("php://input");        // the request payload
				parse_str($content,$p);            // parse the content string to check we have [data] field or not
				if(!array($p)) {
					$p = json_decode($content,true);         // if we have data field
				}

				foreach($p as $key => $value) {
					$this->params[$method][$key] = $value;
				}
			}
				break;
		}

		if(!empty($_FILES)) {
      foreach($_FILES as $key => $value) {
        $this->params['FILES'][$key] = $value;
      }
    }

		if(isset(self::$cfg['router']['logRequests']['logLocation']) && (isset(self::$cfg['router']['logRequests']['logMethod']) && (in_array("file",self::$cfg['router']['logRequests']['logMethod']) || in_array("fileJson",self::$cfg['router']['logRequests']['logMethod'])))) {
			$logData['method']  = strtolower($method);
			$logData['data']    = $this->params;
			$logData['url']     = myHelpers::getServerUrl();
			$logData['referer'] = myHelpers::getServerReferer();
			$auth               = basic::load("auth","core");
			//	$userId             = $auth->getUserLoggedUserId();

			$logData['userId'] = $auth->getUserLoggedUserId();

			$dirAndFile = myHelpers::fixDirAndFile(self::$cfg['router']['logRequests']['logLocation'],$ext = "log",$lastDir = IMPERMANENTPATH."logs");

			if(in_array("file",self::$cfg['router']['logRequests']['logMethod'])) {
				log::logToFile($dirAndFile,print_r($logData,true));
			}

			if(in_array("fileJson",self::$cfg['router']['logRequests']['logMethod'])) {
				log::logToFile($dirAndFile,json_encode($logData));
			}
		}

		if((isset(self::$cfg['router']['logRequests']['logMethod']) && (in_array("db",self::$cfg['router']['logRequests']['logMethod'])))) {
      if(basic::getConfig("db","main","activate")==true) {
        $db                    = basic::load(basic::getConfig("db","main","adapter"),'core/adapters');
        $table                 = "log_requests";
        $createData['method']  = strtolower($method);
        $createData['data']    = json_encode($this->params);
        $createData['url']     = myHelpers::getServerUrl();
        $createData['referer'] = myHelpers::getServerReferer();
        $db->insert($table = basic::getConfig("db","main","tablePrefix").$table,$createData,$advancedLead = true);
      }
		}

		if($this->controller=="index") {
			$this->params = array($this->params);
		}

		$this->redirect();  // redirect the page to specific location - via controller and action

		if($this->controller=="login") {
			/*put something here*/
		}
	}

	/**
	 * Get language from url - of uri    // very similar to lang::getLanguageFromUrl() but not the same
	 */
	public function getLanguageFromUrl() {

		if(self::$cfg['lang']['route']=="inPath") {
			if(!$this->inPath) {
				$this->inPath = "/";
			}
			list($this->language,$this->inPath) = lang::getLanguageFromUrl($path = $this->inPath);
		}
		elseif(self::$cfg['lang']['route']=="inDomain") {
			list($this->language) = lang::getLanguageFromUrl();
		}
		elseif(self::$cfg['lang']['route']=="" || self::$cfg['lang']['route']=="none") {
			$this->language = self::disassembleUrl();
		}
	}

	/**
	 * disassembleUrl - to parts
	 */
	public static function disassembleUrl() {

		if(!empty($_SERVER['REQUEST_SCHEME'])) {
			$serverProtocol = $_SERVER['REQUEST_SCHEME'];
		}
		else {
			$serverProtocol = isset($_SERVER["HTTPS"]) ? "https" : "http";
		}

		$fullAddress = $serverProtocol."://".$_SERVER['SERVER_NAME'].$_SERVER['REQUEST_URI'];
		if(strlen($_SERVER['REQUEST_URI'])>2 && strlen($_SERVER['REQUEST_URI'])<5) {
			$fullAddress = $fullAddress."/";
		}

		$matches   = array();                                     // initialize the matches
		$regexLang = '/((?:http[s]*\:\/\/)*)([^\/]*)?\/?()+([\/\?])([^\?]*)/';

		if(preg_match($regexLang,$fullAddress,$matches)) {        // if language is requested
			router::$protocol = $matches[1];
			router::$domain   = $matches[2];
			router::$breaker  = $matches[4];
			router::$path     = $matches[5];
		}
	}

	/**
	 * Redirect - redirect to the appropriate action
	 */
	private function redirect() {

		$this->loadRedirects("routes");

		foreach($this->redirects['routes'] as $key => $value) {
			if((isset($this->redirects['routes'][$key]['direct']) && $this->controller==$this->redirects['routes'][$key]['redirect']['controller'] && $this->action==$this->redirects['routes'][$key]['redirect']['action'])
			) {
				$removeDuplicateAddress = str_replace($this->controller."/","/",$this->inPath);
				//	$removeDuplicateAddress = preg_replace('/\/+/',"/",$removeDuplicateAddress);	// remove double "/" when we have language on url...

				$preLang = "";
				if($this->language) {
					$preLang = "/";
				}
				header("HTTP/1.1 301 Moved Permanently");
				Header("Location: ".$preLang.basic::$cfg['site']['directory']['main'].$this->language.$removeDuplicateAddress);
			}

			if(isset($this->redirects['routes'][$key]['subDomain']) && $this->controller==$this->redirects['routes'][$key]['redirect']['controller']) {

				$preLang = "";
				if($this->language) {
					$preLang = "/";
				}
				header("HTTP/1.1 301 Moved Permanently");
				Header("Location: ".router::$protocol.$this->redirects['routes'][$key]['subDomain'].".".router::$domain.$preLang.$this->language);
			}
		}    // to prevent double addresses - must come b4 the other

		foreach($this->redirects['routes'] as $key => $value) {
			if(isset($this->redirects['routes'][$key]['direct']) && $this->controller==$this->redirects['routes'][$key]['direct']) {
				$this->controller = $this->redirects['routes'][$key]['redirect']['controller'];
				$this->action     = $this->redirects['routes'][$key]['redirect']['action'];
			}

			if(isset($this->redirects['routes'][$key]['subDomain']) && router::$subDomain==$this->redirects['routes'][$key]['subDomain']) {
				$this->controller = $this->redirects['routes'][$key]['redirect']['controller'];
			//	$this->action     = $this->redirects['routes'][$key]['redirect']['action'];
			}
		}    // redirect to action
  }

	static function extractSubDomain() {

		$domain            = explode('.',router::$domain);

		router::$subDomain = array_slice($domain,0,count($domain) - 2);
		router::$subDomain = implode(".",router::$subDomain);
	}

	/**
	 * Load redirects - if we have special redirects
	 * @param string $file - name - usually "routes"
	 */
	private function loadRedirects($file) {

		if(!self::$cfg['router']['redirect']['loadMethod'] || self::$cfg['router']['redirect']['loadMethod']=="xml") {
			$this->loadRedirectsViaXml($file);
		}
		if(self::$cfg['router']['redirect']['loadMethod']=="array") {
			$this->loadRedirectsViaPhpArray($file);
		}
	}

	/**
	 * Load redirects via php xml
	 * @param string $file - name - usually "routes"
	 */
	private function loadRedirectsViaXml($file) {

		// or get the text codes from xml
		// require_once(LABYARDPATH."functions".DS."xml.php");
		$xml             = basic::load("xml","functions");
		$this->redirects = $xml::xmlToArray(file_get_contents(ROOT.DS."config".DS.basic::$currentItinerary.DS.$file.".xml"));
	}

	/**
	 * Load redirects via php array
	 * @param string $file - name - usually "routes"
	 */
	private function loadRedirectsViaPhpArray($file) {

		$loadFile        = ROOT.DS."config".DS.basic::$currentItinerary.DS.$file.".php";
		$loadFileDefault = ROOT.DS."config".DS.basic::$defaultItinerary.DS.$file.".php";
		$routes          = array();

		if(file_exists($loadFile)) {
			require_once($loadFile);
		}
		elseif(file_exists($loadFileDefault)) {
			require_once($loadFileDefault);
		}

		$this->redirects['routes'] = $routes;        // $routes - comes from the file we required
	}

	/**
	 * Dispatch
	 */
	public function dispatch() {

		global $auth;

		$controllerName = $this->controller;                // set controller name

    if(!empty($_POST)) {   // checking csrfToken
      if($this->architectureType!="api") {
        $auth       = basic::load("auth","core");
        $csrfResult = $auth->checkCsrfToken();

        $referer = $_SERVER['HTTP_REFERER'] ? $_SERVER['HTTP_REFERER'] : "/";

        if($csrfResult!==true) {
          Header("Location: ".$referer."&authAlert=8001");
          die();
        }
      }
    }

    if($this->architectureType=="advanced") {   // system reserved words - part of urls (also on template.class)
      /*      $advancedSystemControllers = array("new","editingBlueprint","addingNewBlueprint","edit","export","import","entryList","addEntry","addingEntry","editEntry","editingEntry","entryActivation","entryDeletion","exportEntries","archiveBlueprint","exportBlueprint","importingBlueprint","resetBlueprint","permanentDeleteBlueprint","archiveBlueprints","exportBlueprints","showActive","showArchive","showEntries","pastEntry","retrievePastEntry","entrySearch","blueprintSearch","showActiveList","setEntryOrderingPosition","experimentalLudicrousDisplay","checkIfBlueprintNameAvailable","checkIfFieldNameAvailable","imagesUpload","deleteUploadedFile","getAvailableBlueprints","getAvailableBlueprintFields");*/

      //     $advancedJsonFile = SOURCEPATH."data".DS."advanced".DS.$this->controller.".json";

      //  p($this->controller);
      //   p($this->action);

      /*      if(in_array($this->controller,$advancedSystemControllers)) {
              $this->action = $this->controller;
              $this->controller = "advanced";
            }*/

      /*      if(file_exists($advancedJsonFile)) {
              $this->action = "addNewEntry";
              $this->controller = "advanced";

      //        if($this->isPost()) {
      //          $this->action = "addingNewEntry";
      //        }
            }*/
    }

    if($this->architectureType=="advancedBlueprints") {
      $advancedBlueprintsSystemControllers = array("new","editingBlueprint","addingNewBlueprint","edit","export","import","archiveBlueprint","exportBlueprint","importingBlueprint","resetBlueprint","permanentDeleteBlueprint","archiveBlueprints","exportBlueprints","showActive","showArchive","blueprintSearch","showActiveList","checkIfBlueprintNameAvailable","checkIfFieldNameAvailable","getAvailableBlueprints","getAvailableBlueprintFields");

      if(in_array($this->controller,$advancedBlueprintsSystemControllers)) {
        $this->action = $this->controller;
        $this->controller = "advancedBlueprints";
      }
    }
    if($this->architectureType=="advancedEntries") {
      $advancedEntriesSystemControllers = array("entryList","addEntry","addingEntry","editEntry","editingEntry","entryActivation","entryDeletion","exportEntries","showEntries","pastEntry","retrievePastEntry","entrySearch","setEntryOrderingPosition","experimentalLudicrousDisplay","imagesUpload","deleteUploadedFile");

      if(in_array($this->controller,$advancedEntriesSystemControllers)) {
        $this->action = $this->controller;
        $this->controller = "advancedEntries";
      }
    }

		$originalModel = $model = $this->controller."Model";                    // set model name
		//	$model = class_exists($model) ? $model : $controllerName."Model";   // if we have extended model

		$model = class_exists($model) ? $model : "model";                       // if we have extended model

		$modelControllerErrorArray = array();

		if($model=="model" && $originalModel!="indexModel") {                   // but not "indexModel" that is homePage
			$modelControllerErrorArray[] = "model";
		}

		$this->controller .= "Controller";    // assign controller full name
		$this->controller = class_exists($this->controller) ? $this->controller : "controller";    // if we have extended controller

		if($this->controller=="controller") {
			$modelControllerErrorArray[] = "controller";
		}

		if($modelControllerErrorArray) {
			$modelControllerError = implode(" + ",$modelControllerErrorArray);

			$notif = basic::load("notif","core");
			$notif->genNotifByNum("core",$num = 81400,$modelControllerError);
		}

		$dispatch = new $this->controller($model,$controllerName,$this->action);        // construct the controller class
		// if we have action function in controller
		$hasActionFunction = (int)method_exists($this->controller,$this->action);        // check if class exists

    if(function_exists("is_callable")) {   // only b4 php 8
//      $callableFunction  = (int)is_callable(array(
//        $this->controller,
//        $this->action
//      ));  // check if function is not private
//    }
//    else {   // only beyond php 8
      $callableFunction  = (int)is_callable(array(
        $dispatch,
        $this->action
      ));  // check if function is not private
    }

		$hasActionFunctionAccess = "";
		$restrictedActions       = array("init","beforeAction","afterAction");

		if((!in_array($this->action,$restrictedActions) && $callableFunction && !stringManipulation::startsWith($this->action,"internal")) || $this->action=="internal") {
			$hasActionFunctionAccess = $callableFunction;    // check if action is not private and not "internal".Action
		}
		else {
			if($hasActionFunction || (stringManipulation::startsWith($this->action,"internal"))) {
				$notif = basic::load("notif","core");
				$notif->genNotifByNum("core",$num = 82200,$this->action);
			}
		}

		// we need to reference the parameters to a correct order in order to match the arguments order
		// of the calling function
		$action           = $hasActionFunction && $hasActionFunctionAccess ? $this->action : "defaultAction";
		$reflection       = new ReflectionMethod($this->controller,$action);
		$reflectionParams = $reflection->getParameters();
		$paramsToPut      = array();
		$numberOfParams   = $reflection->getNumberOfParameters();   // not needed

		$reflectionParamsDegraded = array();
		foreach($reflectionParams as $key => $value) {
			try {
        if(function_exists("getClass")) {   // only b4 php 8
          $classExist = $reflectionParams[$key]->getClass();
        }
        else {   // only beyond php 8
          $classExist = $reflectionParams[$key]->getType() && !$reflectionParams[$key]->getType()->isBuiltin() ? new ReflectionClass($reflectionParams[$key]->getType()->getName()) : null;
        }

				if(is_object($classExist)===false) {        // if not dependency
					$reflectionParamsDegraded[] = $value->name;
				}
			}
			catch(Exception $e) {
				$notif = basic::load("notif","core");
				$notif->genNotifByNum("core",81700,$e->getMessage());
				$dependencyFileMissing = 1;
			}
		}

//p($reflectionParamsDegraded);
//p($this->params);

		foreach($reflectionParamsDegraded as $key => $value) {
			foreach($this->params as $key2 => $value2) {
				if(array_key_exists($value,$this->params[$key2])!==false) {
					$paramsToPut[$value][$key2] = $this->params[$key2][$value];
				}
				else {
					$paramsToPut[$value] = array();
				}
			}
		}                                                   // getting values from all methods

		$paramsToPut = array_values($paramsToPut);          // fixing a problem may occur later with requests ($_REQUEST) on controllers parameters

    foreach($reflectionParams as $key => $value) {      // decide if to make injection
			if(($reflectionParams[$key]->isDefaultValueAvailable() || $reflectionParams[$key]->isOptional()) && (!isset($paramsToPut[$key]) || !$paramsToPut[$key])) {            // if default value
				$paramsToPut[$key] = $reflectionParams[$key]->getDefaultValue();
			}

      if(isset($reflectionParams[$key]) && $reflectionParams[$key] && !empty($this->params['GET']['param'.$key])) {
        $paramsToPut[$key]['GET'] = $this->params['GET']['param'.$key];
      }                                                   // putting values from url slashes (new ver - add GET even if we have POST)

			if(isset($this->params[$key]) && $this->params[$key]) {
				$paramsToPut[$key] = $this->params[$key];         // putting values from url slashes
			}

			if(!isset($paramsToPut[$key])) {                    // if no default value and no value
				$paramsToPut[$key] = "";
			}

			if(!isset($dependencyFileMissing)) {
        if(function_exists("getClass")) {   // only b4 php 8
          $classExist = $reflectionParams[$key]->getClass();
        }
        else {   // only beyond php 8
          $classExist = $reflectionParams[$key]->getType() && !$reflectionParams[$key]->getType()->isBuiltin() ? new ReflectionClass($reflectionParams[$key]->getType()->getName()) : null;
        }

				if($classExist) {       // if heading class
					$inject    = basic::load("inject","core");
					$theSolver = $inject->create($reflectionParams[$key]->getClass()->name);

					$paramsToPut[$key] = $theSolver;
				}
			}
		}

		router::$allParams = $this->params;

		if((int)method_exists($dispatch,"beforeAction")) {
			$this->view .= call_user_func_array(array($dispatch,"beforeAction"),$paramsToPut);
		}

		$this->view .= call_user_func_array(array($dispatch,$action),$paramsToPut);

		if((int)method_exists($dispatch,"afterAction")) {
			$this->view .= call_user_func_array(array($dispatch,"afterAction"),$paramsToPut);
		}

		if($this->controller!="silController" || $this->action=="logged") {    // if it's not some kind of "sil" stuff
			$auth->setTempAddress();    // if site is not restricted
		}
		/*		if($this->view) {				// finally, we print it out
					echo $this->view;
				}*/
	}

	/**
	 * Is POST - checks if it's post request - to fix problems on dbUi where $_POST is gone cz someone deleted it
	 * @return bool
	 */
	static function isPost() {

		if(self::$isPost==null) {
			if(empty($_POST)) {
				self::$isPost = false;
			}
			else {
				self::$isPost = true;
			}
		}

		if(self::$isPost) {
			return true;
		}

		return false;
	}

	/**
	 * Get all params - from GET/POST/PUT and ect...
	 * @return mixed
	 */
	static function getAllParams() {

		return router::$allParams;
	}
}

