<?php

/*************************************************************
 * Template class
 * - template engine for the mvc
 * - loading the view files from: \application\views\"viewName".php
 * @author    S M Kessner <limbercode@gmail.com>
 * @copyright 2010-2018 T2
 * @link      http://limbercode.com
 *************************************************************/
class template {

	protected $controller;
	protected $action;
  protected $architectureType = "application";
	protected $bodyContent;
	protected $themePlan;
	private $cssIncludesToMinify = array();
	private $jsIncludesToMinify = array();
	public $variables = array();
	public $path;
	public $viewPath;
	public $operatorsPath;
	public $loadedOperators;                    // do we need it?
	public $themesPath;
	public $viewsPath;
	public $themePath;
	public $controllerPath;                     //	$authLevel
	public $currentTemplate;
	public $doNotRenderPartOfTemplate;
	public $operatorsToActivate = array();
	public $operatorsToUnset = array();
	public $filePlanLoaded;
	public $filePlanContentLoaded;
	public $mainEvantLoaded;
	public $mainEvantLoadedVars = array();
	public $cssIncludes = array();
	public $jsIncludes = array();
	public $js;
	public $css;
	public $templatePlan;
	public $section = array();
	public $layout;
	public $trans;
	public $notif;
	private static $cfg;

	/**
	 * Constructor for template
	 * @param string $controller - the controller name
	 * @param string $action - the action name
	 * @param string $architectureType - the action name
   * @throws ReflectionException
   */
	public function __construct($controller,$action,$architectureType) {

    $this->controller       = $controller;
    $this->action           = $action;
    $this->architectureType = $architectureType;
    $this->trans            = basic::load("trans","core");
    $this->notif            = basic::load("notif","core");
    $this->currentTemplate  = basic::$cfg['template']['theme'];

		if(basic::$cfg['errorControl']['errorReportUi']['allow']) {
			$this->cssIncludes[] = "css".DS."er.css";       // shortcut for errorReport
			$this->jsIncludes[]  = "js".DS."er.js";         // shortcut for errorReport
		}
		if(basic::$cfg['errorControl']['dbUi']['allow']) {
			$this->cssIncludes[] = "css".DS."du.css";       // shortcut for dbUi
			$this->jsIncludes[]  = "js".DS."du.js";         // shortcut for dbUi
		}
		if(basic::$cfg['notif']['allow']) {
			$this->cssIncludes[] = "css".DS."nu.css";       // shortcut for notifUi
			$this->jsIncludes[]  = "js".DS."nu.js";         // shortcut for notifUi
		}
		if(basic::$cfg['log']['infoUi']['allow']) {
			$this->cssIncludes[] = "css".DS."in.css";           // shortcut for notifUi
			$this->jsIncludes[]  = "js".DS."in.js";             // shortcut for notifUi
		}

    self::$cfg = basic::loadCfg("operatorConfig",false) + basic::loadCfg("minifierConfig",false);       // minifier config is set to load privately
		//	$this->set("cfg",basic::$cfg);        // we set the configuration variables to local variables for rendering
  }

	/**
	 * Destructor
	 * in case we don't have "path" param on backtrace - it means that there is: die(); somewhere in the code - so we will have to put those css + js for the informant/error functions to work (half way)
	 */
	public function __destruct() {

		$stacks = debug_backtrace();

//		if(!$stacks[0]['object']->path) {   // i don't know why it was here
			$phpAlertHtml = "";
			/*			$phpAlertHtml .= "<link rel='stylesheet' property='stylesheet' href='/public/css/du.css' type='text/css'>";
						$phpAlertHtml .= "<script type='text/javascript' src='/public/js/du.js'></script>";
						$phpAlertHtml .= "<link rel='stylesheet' property='stylesheet' href='/public/css/er.css' type='text/css'>";
						$phpAlertHtml .= "<script type='text/javascript' src='/public/js/er.js'></script>";*/

			echo $phpAlertHtml;

      if(myHelpers::isAjax()) {
				if(basic::$cfg['errorControl']['dbUi']['allow'] && dbInformant::$disableUi==false) {
					$auth = basic::load("auth","core");

          if($auth->isDevUser()) {
						$dbUiAjaxShow                 = dbInformant::$keepForOutBurst;
						dbInformant::$keepForOutBurst = "";

						if($dbUiAjaxShow) {
							echo $dbUiAjaxShow;
						}
					}
				}
			}
//		}
	}

	/**
	 * Set template theme - will be used in current session
	 * @param string $name - name of the template theme
	 * @example $this->setTemplate($name="most");
	 */
	public function setTemplate($name) {

		$this->currentTemplate = $name;
	}

	/**
	 * Translate text from all over the site (view side)
	 * @param string $text - the text to be translated - written in the default language (en)
	 * @param string /array $externalVar - containing the values that will effect the translation
	 * @example $this->T($text="text");
	 * @return the translated text
	 */
	public function T($text,$externalVar = "") {        // load the translation text in the template files

		return $this->trans->extractTrans($text,$externalVar);
	}

	/**
	 * Set a variables into view from all over the site (view side)
	 * @param string $name - name of the variable
	 * @param string $value - value of the variable
	 * @example $this->set($name="timed",$value);        // will be $timed on view side
	 */
	public function set($name,$value) {

		$this->variables[$name] = $value;
	}

	/**
	 * Set action - set the current action
	 * @param string $action - the action name
	 */
	public function setAction($action) {

		$this->action = $action;
	}

	/**
	 * Render body
	 */
	public function renderBody() {

		// if we have content, then deliver it
		if(!empty($this->bodyContent)) {
			echo $this->bodyContent;
		}
	}

	/**
	 * Render section
	 * @param string $section - the source content
	 * @example $this->renderSection('head');    // on view side
	 */
	public function renderSection($section) {

		if(!empty($this->section) && array_key_exists($section,$this->section)) {
			echo $this->section[$section];
		}
	}

	/**
	 * Get paths - assign path to parameters
	 */
	public function getPaths() {

		extract($this->variables);

		//	$this->doNotRenderPartOfTemplate = $doNotRenderPartOfTemplate;
		$this->operatorsPath  = OPERATORSPATH;
		$this->themesPath     = "themes".DS;
		$this->viewsPath      = "views".DS;

    if($this->architectureType=="advanced") {   // system reserved words - part of urls (also on router.class)
      $advancedSystemControllers = array("new","edit","export","import");

      if(in_array($this->controller,$advancedSystemControllers)) {
        $this->controller = "advanced";
      }
    }

		$this->controllerPath = $this->viewsPath.$this->controller.DS;

		if(isset($this->currentTemplate)) {
			if($this->currentTemplate) {
				$this->themePath = APPLICATIONPATH.$this->themesPath.$this->currentTemplate.DS;
			}
			else {
				$this->themePath = APPLICATIONPATH.$this->themesPath.'default'.DS;
			}
		}
		else {
			$this->themePath = APPLICATIONPATH.$this->themesPath.'default'.DS;
		}
	}

	/**
	 * Render - display the template
	 */
	public function render() {

		global $notif;

		$this->path = myHelpers::UrlContent('~/views/',$sub = "architecture".DS.$this->architectureType);    // the view path

		$this->getPaths();
		$this->operatorsBegins();    // we have to put it here so we can load the css/js for operators

		if(method_exists($this,'templatePlan')) {    // from theme class
			$this->themePlan = $this->templatePlan();
		}
		else {
			if(basic::$cfg['site']['environment']=="tech" || basic::$cfg['site']['environment']=="dev") {
				//	$this->notif->genNotifByNum($file="core",$num=82110,array($this->currentTemplate));			// not working
				echo "<br>** \"templatePlan\" function is missing on \"".$this->currentTemplate."\" theme (code 82110)";
			}
			else {
				echo "<br>** Error - Please contact site manager (code 82110)";
			}
			die();
		}

		if(method_exists($this,'jsIncludes')) {        // from theme class
			if($this->jsIncludes()) {
				$this->jsIncludes = array_merge($this->jsIncludes,$this->jsIncludes());
			}
		}
		else {
			if(basic::$cfg['site']['environment']=="tech" || basic::$cfg['site']['environment']=="dev") {
				//	$this->notif->genNotifByNum($file="core",$num=82120,array($this->currentTemplate));			// not working
				echo "<br>** \"jsIncludes\" function is missing on \"".$this->currentTemplate."\" theme (code 82120)";
			}
		}

		if(method_exists($this,'cssIncludes')) {    // from theme class
			if($this->cssIncludes()) {
				$this->cssIncludes = array_merge($this->cssIncludes,$this->cssIncludes());
			}
		}
		else {
			if(basic::$cfg['site']['environment']=="tech" || basic::$cfg['site']['environment']=="dev") {
				//	$this->notif->genNotifByNum($file="core",$num=82130,array($this->currentTemplate));			// not working
				echo "<br>** \"cssIncludes\" function is missing on \"".$this->currentTemplate."\" theme (code 82130)";
			}
		}

		//	if(!$doNotRenderPartOfTemplate) $doNotRenderPartOfTemplate = array();
		foreach($this->themePlan as $key => $value) {    // from theme class
			if(!is_array($this->doNotRenderPartOfTemplate)) {
				if($value==$this->doNotRenderPartOfTemplate) {            // if we have string
					unset($this->themePlan[$key]);
					continue;
				}
			}
			else {
				if(in_array($value,$this->doNotRenderPartOfTemplate)) {    // if we have array
					unset($this->themePlan[$key]);
					continue;
				}
				if(in_array("clean",$this->doNotRenderPartOfTemplate)) {
					if(method_exists($this,'templatePlanCleaned')) {
						$this->themePlan = $this->templatePlanCleaned();
					}
					else {
						$this->notif->genNotifByNum($file = "core",$num = 81160,$this->currentTemplate);
						break;
					}
					continue;
				}
			}
		}    // unsettling the not needed themePlans

		foreach($this->themePlan as $key => $value) {    // from theme class
			$hasActionFunction = (int)method_exists($this->currentTemplate."theme",$value);

			if($hasActionFunction) {
				//	echo "<br>".$this->currentTemplate."   ".$value."   $hasActionFunction";
				$themePlanFunctionReturn[$value] = $this->$value();
			}
		}    // loading theme functions like: header/footer.. ect.., function inside theme class if available
		// if we have a return of some kind it will be echo after the css/js includes - at the right point

		$this->jsIncludesToMinify  = $this->jsIncludes;
		$this->cssIncludesToMinify = $this->cssIncludes;

    $this->mergeScriptNStyleFiles("js");
    $this->mergeScriptNStyleFiles("css");

		foreach($this->cssIncludes as $key => $value) {
			$value             = str_replace(array('/','\\'),DS,$value);
			$file              = pathinfo($value);
			$fileName          = $file['basename'];
			$fileDirectory     = $file['dirname'];
			$filePathToInclude = $filePathToCheck = $fileDirectory.DS.lang::getCurrentLanguageForDir().$fileName;

			if(!lang::getCurrentLanguageForDir()) {     // fix for "he"
				$filePathToInclude = $filePathToCheck = $fileDirectory.DS.lang::getDefaultLanguage().DS.$fileName;
			}

			if(basic::$cfg['site']['directory']['main']) {
				$filePathToCheck = str_replace(basic::$cfg['site']['directory']['main'],"",$filePathToInclude);
			}    // fix

			if(file_exists(basic::filePathOfPublicDirectory($filePathToCheck)) && !in_array($filePathToInclude,$this->cssIncludes) && !in_array(substr($filePathToInclude,1),$this->cssIncludesToMinify)) {
				$this->cssIncludes[]         = $filePathToInclude;
				$this->cssIncludesToMinify[] = $filePathToInclude;
			}
		}    // add a language css - if css file is on the server and in it's directory we have language directory (like "he") and in this directory we have the same file name - we will load that file too

    $this->cssIncludes = array_unique($this->cssIncludes);    // remove duplicates
    $this->cssIncludesToMinify = array_unique($this->cssIncludesToMinify);    // remove duplicates

		$this->js    = $this->js();
		$this->css   = $this->css();
		$attachJsCss = $this->cssJsInjections();

		if(basic::$cfg['template']['loadMainEventB4']==true) {
			$this->loadMainEvent();
		}

		foreach($this->themePlan as $filePlanSector) {
			$this->attachFile($filePlanSector,$attachJsCss);

			if($filePlanSector=="mainEvent" || $filePlanSector=="mainEventStart") {
				//	self::putSubjectData();
			}

			if($filePlanSector=="operatorRight" || $filePlanSector=="operators") {
				//	self::putOperators();
			}
		}       // preparing $this->filePlanContentLoaded

		if(!empty(basic::$cfg['template']['headers'])) {
			foreach(basic::$cfg['template']['headers'] as $key => $value) {
				header($value);
			}
		}

		if(is_array($this->filePlanContentLoaded)) {
			foreach($this->filePlanContentLoaded as $key => $value) {   // output
				if(isset($themePlanFunctionReturn[$key])) {
					echo $themePlanFunctionReturn[$key];
				}    // echo the return of the theme function - at the right point

				echo $this->filePlanContentLoaded[$key];
			}
		}

    if(!empty($this->layout) && (!myHelpers::isAjax())) {        // check if we have any layout defined
			$this->putLayout();                // we don't use this anymore because html minify don't work well
		}
		else {
      $this->bodyContent = ob_get_contents();        // get the body contents

			$auth      = basic::load("auth","core");
			$authAddOn = $auth->AddOnBody();

      if(basic::$cfg['errorControl']['dbUi']['allow'] && dbInformant::$disableUi==false) {
				if(in_array("Content-type: text/html; charset=UTF-8",headers_list())) {     // if image sent DbUi wont show

          if($auth->isDevUser()) {
						$authAddOn                     .= dbInformant::showDbUiContainer();
						dbInformant::$dbUiContainerRun = true;
						$authAddOn                     .= dbInformant::$keepForOutBurst;
						dbInformant::$keepForOutBurst  = "";
					}
				}
			}

			if($notif || $authAddOn) {      // insert the alert before </body> // if alert is available show it
				$bodyContentAdd = stringManipulation::lastlReplace($search = "</body>",$replace = $authAddOn.$notif."</body>",$subject = $this->bodyContent);
				if($bodyContentAdd==$this->bodyContent) {      // if we didn't succeed to add the AddOn we'll still add them
					$this->bodyContent .= $authAddOn.$notif;
				}
				else {                                         // add AddOn before closing tags
					$this->bodyContent = $bodyContentAdd;
				}
			}

			if(ob_get_contents()) {
				ob_end_clean();
			}                                                  // clean the buffer
			//	ob_start('myHelpers::minifyContentJs');		   // don't know why this is here
			echo $this->bodyContent;                           // just output the content
		}    // end buffer

		$fileContent = ob_get_contents();
		//	ob_end_clean();		// removed lately

		if(self::$cfg['minifier']['html']['allow']==1) {
			$output = myHelpers::minifyContent($fileContent);
			echo $output;
		}
		else {
			echo $fileContent;
		}
		//	ob_end_flush();
	}

	/**
	 * Operators begins - getting information (cfg) about operators
	 */
	public function operatorsBegins() {

		foreach($this->operatorsToUnset as $key => $value) {
			if($value=="all") {
				unset(self::$cfg['operator']['set']);
			}    // remove all operators from page
			elseif(isset(self::$cfg['operator']['set']) && is_array(self::$cfg['operator']['set'])) {
				foreach(self::$cfg['operator']['set'] as $key2 => $value2) {
					if((isset(self::$cfg['operator']['set'][$key2]['file']) && self::$cfg['operator']['set'][$key2]['file']==$value) || (isset(self::$cfg['operator']['set'][$key2][1]) && self::$cfg['operator']['set'][$key2][1]==$value)) {
						unset(self::$cfg['operator']['set'][$key2]);
					}
				}
			}    // remove operators from page - by file name
		}    // remove chosen operators

		if(isset(self::$cfg['operator']['set']) && is_array(self::$cfg['operator']['set']) && !empty($this->operatorsToActivate['operators'])) {
			$operatorsToActivate = array_merge(self::$cfg['operator']['set'],$this->operatorsToActivate['operators']);
		}    // merging operators cfg and operators from controller
		elseif(isset(self::$cfg['operator']['set']) && is_array(self::$cfg['operator']['set'])) {
			$operatorsToActivate = self::$cfg['operator']['set'];
		}    // if no operators on controller
		elseif(!empty($this->operatorsToActivate['operators'])) {
			$operatorsToActivate = $this->operatorsToActivate['operators'];
		}

		if(isset(self::$cfg['operator']['defaultPosition'])) {
			$defaultSide = array('default' => self::$cfg['operator']['defaultPosition']);
		}
		else {
			$defaultSide = array('default' => 'right');
		}    // selecting default side that must be given for it to operate well

		$operatorsSides = array('right','center','left','default');
		$operatorsPro[] = array('attribute' => 'position','valid' => $operatorsSides,'occurrence' => $defaultSide);
		$operatorsPro[] = array('attribute' => 'file','valid' => 'string','occurrence' => 'required');
		$operatorsPro[] = array('attribute' => 'title','valid' => 'string','occurrence' => 'optional');
		$operatorsPro[] = array('attribute' => 'allowMoreThenOnce','valid' => 'boolean','occurrence' => 'optional');
		$operatorsPro[] = array('attribute' => 'mustShow','valid' => 'boolean','occurrence' => 'optional');
		$operatorsPro[] = array('attribute' => 'authLevelNeeded','valid' => 'int','occurrence' => 'optional');

		if(isset($this->operatorsToActivate) && isset($operatorsToActivate)) {
			$this->operatorsToActivate = $this->cfgAlter($operatorsPro,$operatorsToActivate);
		}

		if(isset($this->operatorsToActivate['authLevelNeeded'])) {
			foreach($this->operatorsToActivate['authLevelNeeded'] as $key => $value) {
				if(!$this->operatorsToActivate['authLevelNeeded'][$key]) {    // if authLevelNeeded is 0
					//	$authIsOk = 1;
				}
				else {    // if authLevelNeeded is 1-5
					$auth = basic::load("auth","core");
					//	$rrr = $auth->checkIfLogged();
					$userLoggedAuthLevel = $auth->getUserLoggedAuthLevel();

					if($userLoggedAuthLevel<$this->operatorsToActivate['authLevelNeeded'][$key]) {
						unset($this->operatorsToActivate['position'][$key]);
					}    // remove the unauthorized operator
				}
			}
		}    // removes unauthorized operators

		$this->loadOperators();
	}

	/**
	 * Load Operators - from it's own controller/model/view
	 */
	public function loadOperators() {

		if(is_array($this->operatorsToActivate) && !empty($this->operatorsToActivate['position'])) {
			foreach($this->operatorsToActivate['position'] as $key => $value) {

				if(lang::getCurrentLanguage()=="he" || strtolower(lang::getCurrentLanguage())=="he-il") {  // switch operator sides - unique language function
					if($this->operatorsToActivate['position'][$key]=="left") {
						$languageSwitchSides[$key] = "right";
					}
					if($this->operatorsToActivate['position'][$key]=="right") {
						$languageSwitchSides[$key] = "left";
					}

					$this->operatorsToActivate['position'][$key] = $languageSwitchSides[$key];
				}

				$missingOperatorClass = array();

				$operatorModelFile      = $this->operatorsPath."models".DS.$this->operatorsToActivate['file'][$key]."Model.php";
				$operatorControllerFile = $this->operatorsPath."controllers".DS.$this->operatorsToActivate['file'][$key]."Controller.php";
				$operatorViewFile       = $this->operatorsPath.$this->viewsPath.$this->operatorsToActivate['file'][$key].'.php';

				if($this->operatorsToActivate['file'][$key]) {
					if(isset($this->operatorsToActivate['title'][$key])) {
						$title = $this->operatorsToActivate['title'][$key];
					}
					else {
						$title = $this->operatorsToActivate['file'][$key];
					}

					$operatorName = $this->operatorsToActivate['file'][$key];

					if(!file_exists($operatorViewFile)) {
						$missingOperatorClass[] = "view";
					}    // if file is missing
				}

				$modelToLoad   = $operatorName."Model";
				$operatorModel = "";    // just a reset

				if(file_exists($operatorModelFile)) {    // loading the model class
					require_once($operatorModelFile);

					if(class_exists($modelToLoad)) {
						$operatorModel = new $modelToLoad();
					}
					else {
						$missingOperatorClass[] = "model";
					}    // if class is missing - maybe mistype the class name
				}
				else {
					$missingOperatorClass[] = "model";
				}    // if file is missing

				$controllerToLoad = $operatorName."Controller";

				if(file_exists($operatorControllerFile)) {    // loading the controller class
					require_once($operatorControllerFile);

					if(class_exists($controllerToLoad)) {
						if(!is_object($this->loadedOperators)) {
							$this->loadedOperators = new stdClass();
						}
						$this->loadedOperators->$operatorName = $operatorController = new $controllerToLoad($operatorModel);

						if(isset($operatorController->addCssIncludes)) {
							foreach($operatorController->addCssIncludes as $key => $value) {    // from operator controller
								if(filter_var($value,FILTER_VALIDATE_URL)===false) {    // url is valid
                  $cssFile = basic::filePathOfPublicDirectory($value);

									if(file_exists($cssFile)) {
										$this->addCssIncludes[] = $value;
									}
									else {
										$this->notif->genNotifByNum($file = "core",$num = 81540,array(
											$cssFile,
											$operatorName
										));
									}
								}    // if the css file is on the server we have to add the main directory
							}
						}

						if(isset($operatorController->addJsIncludes)) {
							foreach($operatorController->addJsIncludes as $key => $value) {    // from operator controller
								if(filter_var($value,FILTER_VALIDATE_URL)===false) {    // url is valid
                  $jsFile = basic::filePathOfPublicDirectory($value);

									if(file_exists($jsFile)) {
										$this->addJsIncludes[] = $value;
									}
									else {
										$this->notif->genNotifByNum($file = "core",$num = 81550,array(
											$jsFile,
											$operatorName
										));
									}
								}    // if the js file is on the server we have to add the main directory
							}
						}
					}
					else {
						$missingOperatorClass[] = "controller";
					}    // if class is missing - maybe mistype the class name
				}
				else {
					$missingOperatorClass[] = "controller";
				}    // if file is missing

				if(count($missingOperatorClass)>0) {
					$missingClasses = implode(", ",$missingOperatorClass);
					$this->trans->loadTrans("attribution");     // loads translations

					if(in_array("view",$missingOperatorClass)) {
						$this->notif->genNotifByNum($file = "core",$num = 81110,$title."/".$operatorName.", classes: ".$missingClasses." is");
					}    // if "view" is missing that's a red alert
					else {
						$this->notif->genNotifByNum($file = "core",$num = 81100,$title."/".$operatorName.", classes: ".$missingClasses." is");
					}    // if "view" is missing that's not a red alert
				}
			}
		}
	}

	/**
	 * Put layout - old way for putting fragments...
	 */
	public function putLayout() {        // we don't use this anymore because html minify don't work well

		$layoutInOtherLanguage = stringManipulation::lastlReplace("/","/".lang::getCurrentLanguage()."/",$this->layout);
		$layoutInOtherLanguage = myHelpers::UrlContent($layoutInOtherLanguage);

		if(file_exists($layoutInOtherLanguage)) {        // gets the language file
			$layoutOutput = $layoutInOtherLanguage;
		}
		else {        // gets the default language file
			$layoutOutput = myHelpers::UrlContent($this->layout);    // we need to check the path contains app prefix (~)
		}

		if(file_exists($layoutOutput)) {
			$this->bodyContent = ob_get_contents();    // get the body contents
			ob_end_clean();                                // clean the buffer
			ob_start('myHelpers::minifyContent');        // start buffer (minify pages)
			include($layoutOutput);                    // include the template
		}
		else {
			$this->notif->genNotifByNum($file = "core",81280,$this->layout);
		}
	}

	/**
	 * Load "mainEvent" into memory
	 * @param bool|false $isAjax
   * @throws ReflectionException
   */
	public function loadMainEvent($isAjax = false) {

		extract($this->variables);        // extract the variables for view pages

		//	ob_start();		// removed lately

    $auth = basic::load("auth","core");
    $this->auth = $auth;

		if(empty($this->viewPath)) {        // render page content
			//	if(lang::getCurrentLanguage()!=lang::getDefaultLanguage()) {
			$languageFileAction = $this->path.$this->controller.DS.lang::getCurrentLanguage().DS.$this->action.".php";
			$languageAction     = lang::getCurrentLanguage()."/".$this->action;

			if(file_exists($languageFileAction)) {
				$this->action = $languageAction;
			}
			//	}    // for loading an action language view file on it's language directory - even if no controller name

			if(file_exists($this->path.$this->controller.DS.$this->action.".php")) {
				ob_start();
				include($this->path.$this->controller.DS.$this->action.".php");
				$this->mainEvantLoaded = ob_get_contents();
				ob_end_clean();
			}
			else {
				//	print_r("** Action file not found on controller directory");
				$this->notif->genNotifByNum($file = "core",81120,array($this->action,$this->controller));
			}
		}
		else {
			if(file_exists($this->viewPath)) {
				ob_start();
				include($this->viewPath);
				$this->mainEvantLoaded = ob_get_contents();
				ob_end_clean();
				//	echo $this->mainEvantLoaded;
			}
			else {
			}    // currently output nothing..
		}    // takes the index.php in the "architecture/application/views/($controllerName)

		$this->mainEvantLoadedVars = get_defined_vars();
	}

	/**
	 * Put subject data - putting the page "mainEvent"
	 * @param bool|false $isAjax
   * @throws ReflectionException
   */
	public function putSubjectData($isAjax = false) {

		if(!$isAjax) {
			if(basic::$cfg['site']['environment']=="dev") {
				//	echo "**** mainEvent starts here ****";     // echo $this->authLevel;
			}   // for dev purposes only

			echo "<div id=\"mainEvent\">";
		}

		if($isAjax || basic::$cfg['template']['loadMainEventB4']==false) {       // in not ajax mainEvent loads from "render" function with all the template
			$this->loadMainEvent($isAjax);
		}

		echo $this->mainEvantLoaded;

		if(!$isAjax) {
			echo "</div>";

			if(basic::$cfg['site']['environment']=="dev") {
				//	echo "<br>**** mainEvent ends here ****<br>";
			}   // for dev purposes only
		}
	}

	/**
	 * Load action from view
	 * @param string $controller - the controller name
	 * @param string $action - the action name
	 * @example $this->loadActionFromView("sil","index");    // from view    // like in "logout" that adds "login" view
	 */
	public function loadActionFromView($controller,$action) {

		$loadView = new controller();
		$loadView->loadActionFromView($controller,$action);
	}

	/**
	 * Include fragment - putting the fragments in place
	 * @param string $path - the path to load from - starting directory is $dir
	 * @param string $dir - the path to load from - (under which architecture)
   * @param bool $renderAtOnce - if true will be rendered immediately
   * @return false|string
	 * @example - $this->includeFragment("views/shared/_defaultLayout");    // from view
	 * @example - $this->includeFragment("themes/rim/fragments/navBar");    // from view
	 */
	public function includeFragment($path,$dir="",$renderAtOnce=false) {    // can include fragments from the application directory

    $dir = !empty($dir) ? $dir : APPLICATIONPATH;

		if(strpos($path,"/")===false) {
			$pathWithLanguage = $this->controller.DS."fragments".DS.lang::getCurrentLanguage()."/".$path;
			$path             = $this->controller."/"."fragments"."/".$path;
		}    // if we did not contain the controller name in $path
		else {
			$pathWithLanguage = stringManipulation::lastlReplace("/","/".lang::getCurrentLanguage()."/",$path);
		}

		$finalPathWithLanguage = $dir.$pathWithLanguage.".php";
		$finalPath             = $dir.$path.".php";

		if(lang::getCurrentLanguage() && lang::getCurrentLanguage()!=lang::getDefaultLanguage()) {
			if(file_exists($finalPathWithLanguage)) {
			  if($renderAtOnce==true) {
          return $this->extractRenderedFragment($finalPathWithLanguage);
        }
			  else {
          include($finalPathWithLanguage);
        }
			}
			else {
				if(file_exists($finalPath)) {
          if($renderAtOnce==true) {
            return $this->extractRenderedFragment($finalPath);
          }
          else {
            include($finalPath);
          }
				}
				else {
					$this->notif->genNotifByNum($file = "core",$num = 81290,$externalVar = array(
						$path,
						$this->controller
					));
				}
			}
		}
		else {
			if(file_exists($finalPath)) {
        if($renderAtOnce==true) {
          return $this->extractRenderedFragment($finalPath);
        }
        else {
          include($finalPath);
        }
			}
			else {
				$this->notif->genNotifByNum($file = "core",$num = 81290,$externalVar = array(
					$path,
					$this->controller
				));
			}
		}
	}

  private function extractRenderedFragment($path) {

    ob_start(); // Start output buffer capture.
    include($path); // Include your template.
    $output = ob_get_contents(); // This contains the output of yourtemplate.php
// Manipulate $output...
    ob_end_clean(); // Clear the buffer.

    return $output; // Print everything.
  }

	/**
	 * Put operators - putting the operators in place
	 * @param string $side - the side that is chosen - right/left/center
	 * @example - Template::putOperators("left");    // from the template plan file (application\themes\most\operatorLeft.php)
	 * @return the file to include
	 */
	public function putOperators($side = "left") {

		if($side!="left" && $side!="right") {
			//	alert($text="** Operators \"side\" Error (code:81090) **",$notifType="warning",$link="",$linkText="",$more="",$breakOut="");
			$this->notif->genNotifByNum($file = "core",$num = 81090);

			return false;
		}

		$num = "";

		if(is_array($this->operatorsToActivate) && !empty($this->operatorsToActivate['position'])) {
			foreach($this->operatorsToActivate['position'] as $key => $value) {
				$operatorViewFile = "";

				if($side==$this->operatorsToActivate['position'][$key]) {
					$operatorViewFile = $this->operatorsPath.$this->viewsPath.$this->operatorsToActivate['file'][$key].'.php';
				}    // settings files that have to load for operators

				if($operatorViewFile) {
					if(file_exists($operatorViewFile)) {
						if(isset($this->operatorsToActivate['title'][$key])) {
							$title = $this->operatorsToActivate['title'][$key];
						}
						else {
							$title = $this->operatorsToActivate['file'][$key];
						}

						$num++;
						$this->operatorBox($title,$this->operatorsToActivate['file'][$key],$operatorViewFile,$num,$side);
					}
					else {
						$missingOperatorDisplay = myHelpers::UrlContent("~/views/shared/_operatorMissing.php");
						include($missingOperatorDisplay);
					}
				}
			}
		}
	}

	/**
	 * Cfg alter - changing cfg by request
	 * @param array $properties - the properties
	 * @param array $cfgOfSubject - the stuff need to be change in the original cfg
	 * @example - $this->operatorsToActivate = $cfgAlter($operatorsPro,$operatorsToActivate);
	 * @return array - the altered cfg
	 */
	public function cfgAlter($properties,$cfgOfSubject) {

		$newCfg          = array();
		$unsetAttributes = array();

		if(isset($cfgOfSubject)) {
			foreach($cfgOfSubject as $key => $value) {
				foreach($cfgOfSubject[$key] as $key2 => $value2) {
					$changeKeyTo = "";
					if(is_int($key2)) {
						if($key2==0) {
							$changeKeyTo = "position";
						}
						if($key2==1) {
							$changeKeyTo = "file";
						}
						if($key2==2) {
							$changeKeyTo = "title";
						}
						if($key2==3) {
							$changeKeyTo = "allowMoreThenOnce";
						}
						if($key2==4) {
							$changeKeyTo = "mustShow";
						}
						if($key2==5) {
							$changeKeyTo = "authLevelNeeded";
						}

						if($changeKeyTo) {
							$cfgOfSubject[$key][$changeKeyTo] = $cfgOfSubject[$key][$key2];
							unset($cfgOfSubject[$key][$key2]);
						}
					}
				}
			}
		}    // convert $cfg that have no keys like: attributes

		if(isset($cfgOfSubject)) {
			foreach($properties as $key => $value) {
				foreach($cfgOfSubject as $key2 => $value2) {
					if($properties[$key]['valid']=="string") {
						if(isset($cfgOfSubject[$key2][$properties[$key]['attribute']])) {
							if(!is_string($cfgOfSubject[$key2][$properties[$key]['attribute']])) {
								$unsetAttributes[] = $key2;    // for removing this array values
								$this->notif->genNotifByNum($file = "core",$num = 81070,$externalVar = $properties[$key]['attribute']."($key2)");
							}
							elseif(!$cfgOfSubject[$key2][$properties[$key]['attribute']]) {
								$unsetAttributes[] = $key2;    // for removing this array values
								$this->notif->genNotifByNum($file = "core",$num = 81030,$externalVar = $properties[$key]['attribute']."($key2)");
							}
						}
					}
					elseif($properties[$key]['valid']=="int") {
						if(isset($cfgOfSubject[$key2][$properties[$key]['attribute']])) {
							if(!is_int($cfgOfSubject[$key2][$properties[$key]['attribute']])) {
								$unsetAttributes[] = $key2;    // for removing this array values
								$this->notif->genNotifByNum($file = "core",$num = 81040,$externalVar = $properties[$key]['attribute']."($key2)");
							}
						}
					}
					elseif($properties[$key]['valid']=="boolean") {
						if(isset($cfgOfSubject[$key2][$properties[$key]['attribute']])) {
							$cfgOfSubject[$key2][$properties[$key]['attribute']] = (bool)$cfgOfSubject[$key2][$properties[$key]['attribute']];
						}
					}    // turning to boolean whatever entered
					elseif(is_array($properties[$key]['valid'])) {
						if(!isset($cfgOfSubject[$key2][$properties[$key]['attribute']])) {
							$this->notif->genNotifByNum($file = "core",$num = 81080,$externalVar = $properties[$key]['attribute']."($key2)");
						}
						elseif(!in_array($cfgOfSubject[$key2][$properties[$key]['attribute']],$properties[$key]['valid'])) {
							$unsetAttributes[] = $key2;    // for removing this array values
							$this->notif->genNotifByNum($file = "core",$num = 81050,$externalVar = $properties[$key]['attribute']."($key2)");
						}
					}    // checking if array contain the entered string
					else {
						$unsetAttributes[] = $key2;    // for removing this array values
						$this->notif->genNotifByNum($file = "core",$num = 81060,$externalVar = $properties[$key]['attribute']."($key2)");
					}
/////////////////////////////// validating attributes end

					if(!isset($properties[$key]['occurrence'])) {
						$properties[$key]['occurrence'] = "optional";
					}
					if(!in_array($properties[$key]['occurrence'],array(
							'optional',
							'required'
						)) && !is_array($properties[$key]['occurrence'])) {
						$this->notif->genNotifByNum($file = "core",$num = 81000,$externalVar = $properties[$key]['attribute']."($key2)");
						break;
					}    // if occurrence is not configured correct: if it's not optional/required/default array

					if($properties[$key]['occurrence']=="optional") {
						if(isset($cfgOfSubject[$key2][$properties[$key]['attribute']])) {
							$newCfg[$properties[$key]['attribute']][$key2] = $cfgOfSubject[$key2][$properties[$key]['attribute']];
							continue;
						}
					}    // if occurrence is optional
					elseif($properties[$key]['occurrence']=="required") {
						if(isset($cfgOfSubject[$key2][$properties[$key]['attribute']])) {
							$newCfg[$properties[$key]['attribute']][$key2] = $cfgOfSubject[$key2][$properties[$key]['attribute']];
						}
						else {
							$unsetAttributes[] = $key2;    // for removing this array values
							$this->notif->genNotifByNum($file = "core",$num = 81100,$externalVar = $properties[$key]['attribute']."($key2)");
						}
						continue;
					}    // if occurrence is required
					elseif(is_array($properties[$key]['occurrence'])) {
						if(isset($properties[$key]['occurrence']['default'])) {
							if(!isset($cfgOfSubject[$key2][$properties[$key]['attribute']])) {
								$newCfg[$properties[$key]['attribute']][$key2] = $properties[$key]['occurrence']['default'];
							}    // replacing missing attribute with default value
							else {
								if($cfgOfSubject[$key2][$properties[$key]['attribute']]=="default") {
									$newCfg[$properties[$key]['attribute']][$key2] = $properties[$key]['occurrence']['default'];
								}    // replacing "default" with default value
								else {
									$newCfg[$properties[$key]['attribute']][$key2] = $cfgOfSubject[$key2][$properties[$key]['attribute']];
								}
							}
						}
						else {
							$unsetAttributes[] = $key2;    // for removing this array values
							$this->notif->genNotifByNum($file = "core",$num = 81020,$externalVar = $properties[$key]['attribute']."($key2)");
						}
						continue;
					}    // if occurrence have default value
				}
			}

			foreach($unsetAttributes as $key => $value) {
				foreach($properties as $key2 => $value2) {
					unset($newCfg[$properties[$key2]['attribute']][$value]);
				}
			}    // removing this array values
		}

		return $newCfg;
	}

  /**
   * Merge script N style files
   * @param $type - must be css or js
   */
  public function mergeScriptNStyleFiles($type) {

    if($type=="css" || $type=="js") {
      $includes         = $type."Includes";
      $addIncludes      = "add".ucfirst($type)."Includes";
      $IncludesToMinify = $type."IncludesToMinify";
      $templateNum = "";
      $controllerNum = "";

      if($type=="css") {
        $templateNum = "81500";
        $controllerNum = "81520";
      }

      if($type=="js") {
        $templateNum = "81510";
        $controllerNum = "81530";
      }

      if(!empty($this->$includes)) {
        foreach($this->$includes as $key => $value) {        // from template
          if(filter_var($value,FILTER_VALIDATE_URL)===false) {    // url is valid
            $fileToAdd = basic::filePathOfPublicDirectory($value);

            if(file_exists($fileToAdd)) {
              $this->$includes[$key] = $value;
            }
            else {
              unset($this->$includes[$key]);
              $this->notif->genNotifByNum($file = "core",$templateNum,array(
                $fileToAdd,
                $this->currentTemplate
              ));
            }
          }    // if the css/js file is on the server we have to add the main directory
        }
      }

      if(!empty($this->$addIncludes)) {
        foreach($this->$addIncludes as $key => $value) {    // from controller
          $fileToAdd = basic::filePathOfPublicDirectory($value);

          if(filter_var($fileToAdd,FILTER_VALIDATE_URL)===false) {    // url is valid
            if(file_exists($fileToAdd)) {
              $this->$includes[]         = $value;
              $this->$IncludesToMinify[] = $value;
            }
            else {
              unset($this->$addIncludes[$key]);
              $this->notif->genNotifByNum($file = "core",$controllerNum,array($fileToAdd,$this->controller));
            }
          }    // if the css/js file is on the server we have to add the main directory
        }
      }
    }
  }

  /**
	 * Css/Js minify - making the css smaller, $cssIncludesToMinify/$jsIncludesToMinify - have all the data about the files
	 * @param string $file - the path to the file we want it to be saved
	 * @param string $type - css/js
	 * @example - $this->cssJsMinify($cssFile,$type="css");
	 * @return string - the name of the file that was created and can be loaded
	 */
	public function cssJsMinify($file,$type) {

		$fileToMerge = $file;
		if(router::isLocalhost()) {
			$fileToMerge = ROOT.DS.$file;
		}

		$type = strtolower($type);
		if($type!="css" && $type!="js") {
			$this->notif->genNotifByNum("core",81600,array($fileToMerge,$type));

			return false;
		}

		if(!isset(self::$cfg['minifier']['css']['gzip'])) {
			self::$cfg['minifier']['css']['gzip'] = 0;
		}
		if(!isset(self::$cfg['minifier']['css']['timer'])) {
			self::$cfg['minifier']['css']['timer'] = 0;
		}
		if(!isset(self::$cfg['minifier']['css']['changedFileTimer'])) {
			self::$cfg['minifier']['css']['changedFileTimer'] = 9999999 * 9999999;
		}

		$currentTime = time();
		$fileTime    = 0;
		//	$fileTime = 1419739624;

		if(file_exists($fileToMerge)) {
			$fileTime = filemtime($fileToMerge);
		}

		$toMinify             = array();
		$oneOfTheFilesIsNewer = "";

		$including = $type."IncludesToMinify";

		foreach($this->$including as $key => $value) {
      $file = basic::filePathOfPublicDirectory($value);

			if(file_exists($file)) {    // if file is not on the server lets forget about it...
				$toMinify[]       = $file;
				$toMinifyFileDate = filemtime($file);

				if($toMinifyFileDate>=$fileTime && $currentTime - self::$cfg['minifier'][$type]['changedFileTimer']>=$toMinifyFileDate) {
					$oneOfTheFilesIsNewer = 1;
				}
			}
			else {                        // if we minify also out side source files
				if(self::$cfg['minifier'][$type]['outSideSource']==1) {
					$toMinify[] = $file;
				}
			}
		}

		if(!file_exists($fileToMerge) || $fileTime + (self::$cfg['minifier'][$type]['timer'])<=$currentTime || $oneOfTheFilesIsNewer==1) {
			require_once(OUTSOURCEPATH."minimizers".DS."magicMinifier".DS."class.magic-min.php");

			$gzip = false;
			if(self::$cfg['minifier'][$type]['gzip']==1) {
				$gzip = true;
			}

			$loadersPath = ROOT.DS."public".DS."loaders";
			$cssPath     = $loadersPath.DS."css";
			$jsPath      = $loadersPath.DS."js";
			If(!is_dir($loadersPath)) {
				mkdir($loadersPath);
			}
			If(!is_dir($cssPath)) {
				mkdir($cssPath);
			}
			If(!is_dir($jsPath)) {
				mkdir($jsPath);
			}

			$minify = new Minifier(array(
				'echo'             => false,
				'gzip'             => $gzip,
				'closure'          => false,
				// make it "true" if you want to use external (http) optimizer
				'hashed_filenames' => true,
				'output_log'       => true
			));

			if(file_exists($fileToMerge)) {
				unlink($fileToMerge);    // must delete the old mini file so it can be recreated
			}

			foreach($toMinify as $key => $value) {
				if(!file_exists($value)) {
					if(!myHelpers::remoteFileExists($url = $value)) {
						unset($toMinify[$key]);
					}
				}
			}    // checks if out side source file exists - if not - remove it

			$outcome = $minify->merge($fileToMerge,"themes/".$this->currentTemplate."/".$type,$toMinify);

			if($type=="css") {
				$this->notif->genNotifByNum("core",81580,addslashes($outcome));
			}
			if($type=="js") {
				$this->notif->genNotifByNum("core",81590,addslashes($outcome));
			}
		}

		if(lang::getCurrentLanguage()) {
			$goBackDir = "/../";
		}
		if(!lang::getCurrentLanguage()) {
			$goBackDir = "/";
		}

		if(router::isLocalhost()) {
			if(stringManipulation::stringEndsWith($haystack = $_SERVER['CONTEXT_DOCUMENT_ROOT'],$needle = router::$localHostNextDir)) {
				$fileToLoad="/".$file;
			}
			else {
				$fileToLoad = router::$localHostNextDir."/".$file;
			}
		}
		else {
			$fileToLoad = $goBackDir.str_replace(ROOT.DS,"",$fileToMerge);
		}

    return $fileToLoad;
	}

	/**
	 * Css/Js - making array of possible cssJs injections
	 * @return array - all the possible Css/Js file and path to load from
	 */
	public function cssJsInjections() {

		$jsIncludesAll            = "";
		$cssIncludesAll           = "";
		$jsIncludesOutSideSource  = "";
		$cssIncludesOutSideSource = "";
		$jsIncludesInSideSource   = "";
		$cssIncludesInSideSource  = "";
		$js                       = "";
		$css                      = "";
		$miniCss                  = "";
		$miniJs                   = "";

		$this->cssIncludes = array_unique($this->cssIncludes);        // Remove duplicates from include css
		$this->jsIncludes  = array_unique($this->jsIncludes);        // Remove duplicates from include js

    if(!isset(self::$cfg['minifier']['css']['allow'])) {
			self::$cfg['minifier']['css']['allow'] = 0;
		}
		if(!isset(self::$cfg['minifier']['js']['allow'])) {
			self::$cfg['minifier']['js']['allow'] = 0;
		}

		if(self::$cfg['minifier']['css']['allow']==1) {
			$filesName = "";

			foreach($this->cssIncludes as $key => $value) {
				if(strpos($value,"http://")!==false) {
					if(self::$cfg['minifier']['css']['outSideSource']==1) {
						$filesName .= "-http-";
					}
					else {
						continue;
					}
				}    // if we have outSideSource file

				$langDid = false;

				if(strpos($value,DS.lang::getCurrentLanguage().DS)!==false) {
					$filesName .= "-L-".lang::getCurrentLanguage()."-";   // -lang-
					$langDid   = true;
				}    // we have css language file here

				$file = basename($value,".css");

				if($langDid) {      // to make file name shorter
					$file = substr($file,0,3);
				}

				$filesName .= $file.".";
			}    // gets the files names to put in a long string
			$cssFile = basic::getPublicDirectory()."loaders/css/".$this->currentTemplate.".theme.".$filesName."mini.css";

//			if(!router::isLocalhost()) {
//				$cssFile = ROOT.DS.$cssFile;
//			}

			$miniCss = $this->cssJsMinify($cssFile,$type = "css");
		}

		if(self::$cfg['minifier']['js']['allow']==1) {
			$filesName = "";
			foreach($this->jsIncludes as $key => $value) {
				if(strpos($value,"http://")!==false) {
					if(self::$cfg['minifier']['css']['outSideSource']==1) {
						$filesName .= "-http-";
					}
					else {
						continue;
					}
				}    // if we have outSideSource file

				if(strpos($value,DS.lang::getCurrentLanguage().DS)!==false) {
					$filesName .= "-L-".lang::getCurrentLanguage()."-";   // -lang-
				}    // we have js language file here

				$file      = basename($value,".js");
				$filesName .= $file.".";
			}    // gets the files names to put in a long string

			//	$jsFile = ROOT.DS."public/themes/".$this->currentTemplate."/js/".$this->currentTemplate.".theme.mini.js";
			$jsFile = "public/loaders/js/".$this->currentTemplate.".theme.".$filesName."mini.js";

			if(!router::isLocalhost()) {
				$jsFile = ROOT.DS.$jsFile;
			}

			$miniJs = $this->cssJsMinify($jsFile,$type = "js");
		}

		foreach($this->jsIncludes as $key => $value) {
			$jsIncludesAll .= "<script src=\"".$value."\" type=\"text/javascript\"></script>\n";

			if(strpos($value,"http://")!==false) {
				$jsIncludesOutSideSource .= "<script src=\"".$value."\" type=\"text/javascript\"></script>\n";
			}
			else {
				$jsIncludesInSideSource .= "<script src=\"".$value."\" type=\"text/javascript\"></script>\n";
			}
		}

		foreach($this->cssIncludes as $key => $value) {
			$cssIncludesAll .= "<link rel=\"stylesheet\" type=\"text/css\" href=\"".$value."\">\n";

			if(strpos($value,"http://")!==false) {
				$cssIncludesOutSideSource .= "<link rel=\"stylesheet\" type=\"text/css\" href=\"".$value."\">\n";
			}
			else {
				$cssIncludesInSideSource .= "<link rel=\"stylesheet\" type=\"text/css\" href=\"".$value."\">\n";
			}
		}

		if($this->js) {
			$js .= "<script type=\"text/javascript\">\n".$this->js."\n</script>\n";
		}

		if($this->css) {
			$css .= "<style type=\"text/css\">\n".$this->css."\n</style>\n";
		}

		return array(
			"js"  => array(
				"jsIncludesAll"           => $jsIncludesAll,
				"jsIncludesInSideSource"  => $jsIncludesInSideSource,
				"jsIncludesOutSideSource" => $jsIncludesOutSideSource,
				"js"                      => $js,
				"miniJs"                  => $miniJs
			),
			"css" => array(
				"cssIncludesAll"           => $cssIncludesAll,
				"cssIncludesInSideSource"  => $cssIncludesInSideSource,
				"cssIncludesOutSideSource" => $cssIncludesOutSideSource,
				"css"                      => $css,
				"miniCss"                  => $miniCss
			)
		);
	}

	/**
	 * Attach file - attaching the file plan/sector to the page
	 * @param string $filePlanSector - the selector plan - like: head/footer/mainEvent
	 * @param string $attachJsCss - must remain for later use in the metaHead
	 * @example - $this->attachFile($filePlanSector);    // from this class only
	 */
	public function attachFile($filePlanSector,$attachJsCss) {

		global $notif;    // must remain like that to work

		$themeFile = $this->themePath.$filePlanSector.'.php';
		if(!empty($this->mainEvantLoadedVars)) {
			extract($this->mainEvantLoadedVars);
		}

		if(file_exists($themeFile)) {    // if we want header from themes
			ob_start();
			include($themeFile);
			$themeFileContent = ob_get_contents();
			ob_end_clean();

			if(self::$cfg['minifier']['html']['allow']==1) {
				$themeFileContent = myHelpers::minifyContent($themeFileContent);
			}

			$this->filePlanLoaded[]                       = $filePlanSector;
			$this->filePlanContentLoaded[$filePlanSector] = $themeFileContent;

			if($filePlanSector=="footer") {
			}    // want to put something in end of footer - have to del this crap
		}
		else {
			//	throw new Exception("** ".$filePlanSector." file not found on themes directory");
			echo "** ".$filePlanSector." file not found on themes directory";
		}

		if(file_exists(APPLICATIONPATH.$this->controllerPath.$filePlanSector.'.php')) {
			include(APPLICATIONPATH.$this->controllerPath.$filePlanSector.'.php');
		}    // if header.php file exist on controller path
	}    // if metaHead.php file not exist on controller path, execute on theme path

	/**
	 * Canonical - adds default canonical
	 * @return string - the default canonical
	 */
	public function canonical() {

		if($this->viewPath) {
			$languageAddress = "";
			if(lang::getCurrentLanguage()!=lang::getDefaultLanguage()) {
				$languageAddress = "/".lang::getCurrentLanguage();
			}    // set the language linker

			if(stringManipulation::stringEndsWith($haystack = $this->viewPath,$needle = "views".DS."shared".DS."_404.php")) {
				return basic::$cfg['site']['address'].$languageAddress."/404";
			}    // 404 page

			if(stringManipulation::stringEndsWith($haystack = $this->viewPath,$needle = "views".DS."index".DS."index.php")) {
				return basic::$cfg['site']['address'].$languageAddress;
			}    // home page
		}

		//	else {
		return basic::$cfg['site']['address'].$_SERVER['REQUEST_URI'];
		//	}
	}

  /**
   * jsonLd - adds default jsonLd
   * @return array - the default jsonLd
   */
  public function jsonLd() {

    $jsonLd = array(
      "@context"        => "http://www.schema.org",
      "@type"           => "WebSite",
      "name"            => basic::$cfg['site']['name'],
      "alternateName"   => "מה קונים",
      "url"             => basic::$cfg['site']['address'],
//      "potentialAction" => array(
//        "@type" => "SearchAction",
//        "target" => basic::$cfg['site']['address']."/search?q={search_term}",
//        "query-input" => "required name=search_term",
//      ),
    );

    return $jsonLd;
  }

	/**
	 * How to render - decide if to return the mainEvent (page content) with or without layout
	 */
	public function howToRender() {

    if(myHelpers::isAjax()) {
			$this->path = myHelpers::UrlContent('~/views/');    // the view path
			template::putSubjectData($isAjax = true);
    }
		else {
			$this->render();
		}
	}

	/**
	 * Return the rendered html string
	 * @return string
	 */
	public function __toString() {

		try {
			$this->howToRender();

			return "";
		}
		catch(Exception $exception) {    // it can be pdo db error, it's important to catch here
			trigger_error("Exception from __toString: ".$exception);

			if(basic::$cfg['site']['environment']!="dev" && basic::$cfg['site']['environment']!="tech") {
				echo "<html><head></head><body>";
				echo "Error on page layout - please contact the site manager...";
				echo "</body></html>";
			}

			return "";
		}
	}

	public static function putReportsUi() {

		$auth = basic::load("auth","core");
		if($auth->isDevUser()) {
			if(basic::$cfg['errorControl']['errorReportUi']['afterHtml']==1) {
				$errorControl = basic::load("errorControl","core");
				echo $errorControl->errorControlContainer;
			}
			if(basic::$cfg['errorControl']['dbUi']['afterHtml']==1) {
				//	$dbInformant = basic::load("dbInformant","core");
				//	echo $dbInformant->dbInformantContainer;
			}
			if(basic::$cfg['log']['infoUi']['afterHtml']==1) {
				$infoUi = basic::load("log","core");
				echo $infoUi->infoUiContainer;
				$infoUi->showPreDefinedPrints();
			}
		}
		if(basic::$cfg['notif']['afterHtml']==1) {
			$notif = basic::load("notif","core");
			echo $notif->notifContainer;
		}
	}

  public static function p($content,$title = false,$align = false,$output = true,$logOnFile = true,$backtrace = false) {
    log::printit($content,$title,$align,$output,$logOnFile,$backtrace);
  }
  public static function printit($content,$title = false,$align = false,$output = true,$logOnFile = true,$backtrace = false) {
    log::printit($content,$title,$align,$output,$logOnFile,$backtrace);
  }
}

